C
C     Benchmark program for calculating time of frank matrix
C
      use ddcommunication_sx, only : eigen_init
     &                             , eigen_free
     &                             , translate_l2g

      implicit double precision (a-h,o-z)

      real(8),allocatable :: a(:,:)
      real(8), pointer ::
     &           b(:),z(:),d(:),e(:),w(:)
      real(8),allocatable :: ah(:,:), al(:,:)
      real(8), pointer ::
     &           bh(:),zh(:),dh(:),eh(:),wh(:),
     &           bl(:),zl(:),dl(:),el(:),wl(:)


      real(8) temp1,temp2
      real(8) time_eigen_s_start, time_eigen_s_finish
      real(8) time_ddeigen_s_start, time_ddeigen_s_finish


      integer, parameter :: NDIM = 2
      logical iexist

      include 'mpif.h'
      include 'trd.h'

C     Initialize MPI
      call mpi_init(ierr)
      call mpi_comm_rank(mpi_comm_world,i$inod,ierr)
      call mpi_comm_size(mpi_comm_world,i$nnod,ierr)

C     Initialize Eigen and Memoris of matrix
      n=500
      m=32
      allocate(a(n,n))
      allocate(ah(n,n),al(n,n))
      call eigen_init(NDIM)
      NPROW = size_of_col
      NPCOL = size_of_row
      nx = ((n-1)/NPROW+1)
      call CSTAB_get_optdim(nx, 6, 16*2, 16*4, nm)
      call eigen_free(0)

      NB  = 64+32
      nmz = ((n-1)/NPROW+1)
      nmz = ((nmz-1)/NB+1)*NB+1
      nmw = ((n-1)/NPCOL+1)
      nmw = ((nmw-1)/NB+1)*NB+1
      nme = ((n-1)/2+1)*2

      nh = (n-1)/4+1
      call CSTAB_get_optdim(nh, 4, 16*2, 16*4, nnh)
      nnh = 4 * nnh

      n1x = ((n-1)/nprocs+1)
      larray = MAX(MAX(nmz,nm,nnh)*nmw, n*n1x)
      allocate(
     &         b(larray), z(larray),
     &         d(nme), e(2*nme), w(nme),
     &         stat=istat)
      allocate(
     &         bh(larray), zh(larray),
     &         dh(nme), eh(2*nme), wh(nme),
     &         bl(larray), zl(larray),
     &         dl(nme), el(2*nme), wl(nme),
     &         stat=istat)
      if(istat.ne.0) then
         print*,"Memory exhausted"
         call flush(6)
         stop
      endif

      call matrix_set(n,a)
      call matrix_adjust_sx(n,a,b,nm)

C     Calculating eigen values and vectors using Double precision routine
      time_eigen_s_start = mpi_wtime()
        call eigen_sx(n,b,nm,w,z,d,e,nme,m,0)
      time_eigen_s_finish = mpi_wtime()

      call ddmatrix_set(n,ah,al)
      call ddmatrix_adjust_sx(n,ah,al,bh,bl,nm)

C     Calculating eigen values and vectors using Double-Double precision routine
      time_ddeigen_s_start = mpi_wtime()
        call ddeigen_sx(n,bh,bl,nm,wh,wl,zh,zl,dh,dl,eh,el,nme,m,0)
      time_ddeigen_s_finish = mpi_wtime()

C     Show results
      if(myrank==1)then 
        write(*,'(A15 I5 A3 I5)') 
     &  "Elapse time   ",n,"  x",n
        write(*,'(A21 F20.16 A9)') '  Eigen_s : Elapse = ',
     &              time_eigen_s_finish - time_eigen_s_start,
     &              "    (sec)"
        write(*,'(A21 F20.16 A9)') 'PQEigen_s : Elapse = ',
     &              time_ddeigen_s_finish - time_ddeigen_s_start,
     &              "    (sec)"
      endif

c     Free memory
      deallocate(a)
      deallocate(z)
      deallocate(d)
      deallocate(e)
      deallocate(w)
      deallocate(ah,al)
      deallocate(zh,zl)
      deallocate(dh,dl)
      deallocate(eh,el)
      deallocate(wh,wl)

C     Finalizing MPI
      call MPI_Finalize(ierr)
      end

