      module ddblas

      implicit none
      interface

***   LEVEL 1

        subroutine DDSWAP(N, XH, XL, INCX, YH, YL, INCY)
          integer, intent(in) :: n, incx, incy
          double precision, intent(inout) :: xh(*), xl(*)
          double precision, intent(inout) :: yh(*), yl(*)
        end subroutine

        subroutine DDCOPY(N, XH, XL, INCX, YH, YL, INCY)
          integer, intent(in) :: n, incx, incy
          double precision, intent(in) :: xh(*), xl(*)
          double precision, intent(out) :: yh(*), yl(*)
        end subroutine

        subroutine DDSCAL(N, AH, AL, XH, XL, INCX)
          integer, intent(in) :: n, incx
          double precision, intent(in) :: ah, al
          double precision, intent(inout) :: xh(*), xl(*)
        end subroutine

        subroutine DDAXPY(N, AH, AL, XH, XL, INCX, YH, YL, INCY)
          integer, intent(in) :: n, incx, incy
          double precision, intent(in) :: ah, al
          double precision, intent(in) :: xh(*), xl(*)
          double precision, intent(inout) :: yh(*), yl(*)
        end subroutine

        subroutine DDDOT(N, XH, XL, INCX, YH, YL, INCY, DDH, DDL)
          integer, intent(in) :: n, incx, incy
          double precision, intent(in) :: xh(*), xl(*), yh(*), yl(*)
          double precision, intent(out) :: ddh, ddl
        end subroutine

        subroutine DDNRM2(N, XH, XL, INCX, DDH, DDL)
          integer, intent(in) :: n, incx
          double precision, intent(in) :: xh(*), xl(*)
          double precision, intent(out) :: ddh, ddl
        end subroutine

        subroutine DDASUM(N, XH, XL, INCX, DDH, DDL)
          integer, intent(in) :: n, incx
          double precision, intent(in) :: xh(*), xl(*)
          double precision, intent(out) :: ddh, ddl
        end subroutine

        integer function IDDAMAX(N, XH, XL, INCX)
          integer, intent(in) :: n, incx
          double precision, intent(in) :: xh(*), xl(*)
        end function

        subroutine DDROTG(AH, AL, BH, BL, CH, CL, SH, SL)
          double precision, intent(inout) :: ah, al, bh, bl
          double precision, intent(out) :: ch, cl, sh, sl
        end subroutine

        subroutine DDROT(N, XH, XL, INCX, YH, YL, INCY, CH, CL, SH, SL)
          integer, intent(in) :: n, incx, incy
          double precision, intent(in) :: ch, cl, sh, sl
          double precision, intent(inout) :: xh(*), xl(*), yh(*), yl(*)
        end subroutine

        subroutine DDROTMG(D1H, D1L, D2H, D2L, X1H, X1L, Y1H, Y1L,
     +      PARAMH, PARAML)
          double precision, intent(inout) :: d1h, d1l, d2h, d2l,
     +        x1h, x1l, paramh(5), paraml(5)
          double precision, intent(in) :: y1h, y1l
        end subroutine

        subroutine DDROTM(N, XH, XL, INCX, YH, YL, INCY, PARAMH, PARAML)
          integer, intent(in) :: n, incx, incy
          double precision, intent(inout) :: xh(*), xl(*), yh(*), yl(*)
          double precision, intent(in) :: paramh(5), paraml(5)
        end subroutine

        subroutine ZZDOTC(N, XH, XL, INCX, YH, YL, INCY, ZZH, ZZL)
          integer, intent(in) :: n, incx, incy
          double complex, intent(in) :: xh(*), xl(*), yh(*), yl(*)
          double complex, intent(out) :: zzh, zzl
        end subroutine

        subroutine ZZDOTU(N, XH, XL, INCX, YH, YL, INCY, ZZH, ZZL)
          integer, intent(in) :: n, incx, incy
          double complex, intent(in) :: xh(*), xl(*), yh(*), yl(*)
          double complex, intent(out) :: zzh, zzl
        end subroutine


***   LEVEL 2

        subroutine DDGEMV(TRANS, M, N, ALPHAH, ALPHAL, AH, AL, LDA,
     +      XH, XL, INCX, BETAH, BETAL, YH, YL, INCY)
          character(1), intent(in) :: trans
          integer, intent(in) :: m, n, lda, incx, incy
          double precision, intent(in) :: alphah, alphal
          double precision, intent(in) :: ah(lda,*), al(lda,*)
          double precision, intent(in) :: xh(*), xl(*)
          double precision, intent(in) :: betah, betal
          double precision, intent(inout) :: yh(*), yl(*)
        end subroutine

        subroutine DDGBMV(TRANS, M, N, KL, KU, ALPHAH, ALPHAL,
     +      AH, AL, LDA, XH, XL, INCX, BETAH, BETAL, YH, YL, INCY)
          character(1), intent(in) :: trans
          integer, intent(in) :: m, n, kl, ku, lda, incx, incy
          double precision, intent(in) :: alphah, alphal
          double precision, intent(in) :: ah(lda,*), al(lda,*)
          double precision, intent(in) :: xh(*), xl(*)
          double precision, intent(in) :: betah, betal
          double precision, intent(inout) :: yh(*), yl(*)
        end subroutine

        subroutine DDGER(M, N, ALPHAH, ALPHAL, XH, XL, INCX,
     +      YH, YL, INCY, AH, AL, LDA)
          integer, intent(in) :: m, n, incx, incy, lda
          double precision, intent(in) :: alphah, alphal
          double precision, intent(in) :: xh(*), xl(*)
          double precision, intent(in) :: yh(*), yl(*)
          double precision, intent(inout) :: ah(lda,*), al(lda,*)
        end subroutine

        subroutine DDSYMV(UPLO, N, ALPHAH, ALPHAL, AH, AL, LDA,
     +      XH, XL, INCX, BETAH, BETAL, YH, YL, INCY)
          character(1), intent(in) :: uplo
          integer, intent(in) :: n, lda, incx, incy
          double precision, intent(in) :: alphah, alphal
          double precision, intent(in) :: ah(lda,*), al(lda,*)
          double precision, intent(in) :: xh(*), xl(*)
          double precision, intent(in) :: betah, betal
          double precision, intent(inout) :: yh(*), yl(*)
        end subroutine

        subroutine DDSBMV(UPLO, N, K, ALPHAH, ALPHAL, AH, AL, LDA,
     +      XH, XL, INCX, BETAH, BETAL, YH, YL, INCY)
          character(1), intent(in) :: uplo
          integer, intent(in) :: n, k, lda, incx, incy
          double precision, intent(in) :: alphah, alphal
          double precision, intent(in) :: ah(lda,*), al(lda,*)
          double precision, intent(in) :: xh(*), xl(*)
          double precision, intent(in) :: betah, betal
          double precision, intent(inout) :: yh(*), yl(*)
        end subroutine

        subroutine DDSYR(UPLO, N, ALPHAH, ALPHAL, XH, XL, INCX,
     +      AH, AL, LDA)
          character(1), intent(in) :: uplo
          integer, intent(in) :: n, incx, lda
          double precision, intent(in) :: alphah, alphal
          double precision, intent(in) :: xh(*), xl(*)
          double precision, intent(inout) :: ah(lda,*), al(lda,*)
        end subroutine

        subroutine DDSYR2(UPLO, N, ALPHAH, ALPHAL, XH, XL, INCX,
     +      YH, YL, INCY, AH, AL, LDA)
          character(1), intent(in) :: uplo
          integer, intent(in) :: n, incx, incy, lda
          double precision, intent(in) :: alphah, alphal
          double precision, intent(in) :: xh(*), xl(*)
          double precision, intent(in) :: yh(*), yl(*)
          double precision, intent(inout) :: ah(lda,*), al(lda,*)
        end subroutine

        subroutine DDTRMV(UPLO, TRANS, DIAG, N, AH, AL, LDA,
     +      XH, XL, INCX)
          character(1), intent(in) :: uplo, trans, diag
          integer, intent(in) :: n, lda, incx
          double precision, intent(in) :: ah(lda,*), al(lda,*)
          double precision, intent(inout) :: xh(*), xl(*)
        end subroutine

        subroutine DDTBMV(UPLO, TRANS, DIAG, N, K, AH, AL, LDA,
     +      XH, XL, INCX)
          character(1), intent(in) :: uplo, trans, diag
          integer, intent(in) :: n, k, lda, incx
          double precision, intent(in) :: ah(lda,*), al(lda,*)
          double precision, intent(inout) :: xh(*), xl(*)
        end subroutine

        subroutine DDTRSV(UPLO, TRANS, DIAG, N, AH, AL, LDA,
     +      XH, XL, INCX)
          character(1), intent(in) :: uplo, trans, diag
          integer, intent(in) :: n, lda, incx
          double precision, intent(in) :: ah(lda,*), al(lda,*)
          double precision, intent(inout) :: xh(*), xl(*)
        end subroutine

        subroutine DDTBSV(UPLO, TRANS, DIAG, N, K, AH, AL, LDA,
     +      XH, XL, INCX)
          character(1), intent(in) :: uplo, trans, diag
          integer, intent(in) :: n, k, lda, incx
          double precision, intent(in) :: ah(lda,*), al(lda,*)
          double precision, intent(inout) :: xh(*), xl(*)
        end subroutine

        subroutine ZZGERC(M, N, ALPHAH, ALPHAL, XH, XL, INCX,
     +      YH, YL, INCY, AH, AL, LDA)
          integer, intent(in) :: m, n, incx, incy, lda
          double complex, intent(in) :: alphah, alphal
          double complex, intent(in) :: xh(*), xl(*)
          double complex, intent(in) :: yh(*), yl(*)
          double complex, intent(inout) :: ah(lda,*), al(lda,*)
        end subroutine

        subroutine ZZGERU(M, N, ALPHAH, ALPHAL, XH, XL, INCX,
     +      YH, YL, INCY, AH, AL, LDA)
          integer, intent(in) :: m, n, incx, incy, lda
          double complex, intent(in) :: alphah, alphal
          double complex, intent(in) :: xh(*), xl(*)
          double complex, intent(in) :: yh(*), yl(*)
          double complex, intent(inout) :: ah(lda,*), al(lda,*)
        end subroutine

        subroutine ZZHEMV(UPLO, N, ALPHAH, ALPHAL, AH, AL, LDA,
     +      XH, XL, INCX, BETAH, BETAL, YH, YL, INCY)
          character(1), intent(in) :: uplo
          integer, intent(in) :: n, lda, incx, incy
          double complex, intent(in) :: alphah, alphal
          double complex, intent(in) :: ah(lda,*), al(lda,*)
          double complex, intent(in) :: xh(*), xl(*)
          double complex, intent(in) :: betah, betal
          double complex, intent(inout) :: yh(*), yl(*)
        end subroutine

        subroutine ZZHBMV(UPLO, N, K, ALPHAH, ALPHAL, AH, AL, LDA,
     +      XH, XL, INCX, BETAH, BETAL, YH, YL, INCY)
          character(1), intent(in) :: uplo
          integer, intent(in) :: n, k, lda, incx, incy
          double complex, intent(in) :: alphah, alphal
          double complex, intent(in) :: ah(lda,*), al(lda,*)
          double complex, intent(in) :: xh(*), xl(*)
          double complex, intent(in) :: betah, betal
          double complex, intent(inout) :: yh(*), yl(*)
        end subroutine

        subroutine ZZHER(UPLO, N, ALPHAH, ALPHAL, XH, XL, INCX,
     +      AH, AL, LDA)
          character(1), intent(in) :: uplo
          integer, intent(in) :: n, incx, lda
          double precision, intent(in) :: alphah, alphal
          double complex, intent(in) :: xh(*), xl(*)
          double complex, intent(inout) :: ah(lda,*), al(lda,*)
        end subroutine

        subroutine ZZHER2(UPLO, N, ALPHAH, ALPHAL, XH, XL, INCX,
     +      YH, YL, INCY, AH, AL, LDA)
          character(1), intent(in) :: uplo
          integer, intent(in) :: n, incx, incy, lda
          double complex, intent(in) :: alphah, alphal
          double complex, intent(in) :: xh(*), xl(*)
          double complex, intent(in) :: yh(*), yl(*)
          double complex, intent(inout) :: ah(lda,*), al(lda,*)
        end subroutine

       
***   LEVEL 3

        subroutine DDGEMM(TRANSA, TRANSB, M, N, K, ALPHAH, ALPHAL,
     +      AH, AL, LDA, BH, BL, LDB, BETAH, BETAL, CH, CL, LDC)
          character(1), intent(in) :: transa, transb
          integer, intent(in) :: m, n, k, lda, ldb, ldc
          double precision, intent(in) :: alphah, alphal
          double precision, intent(in) :: ah(lda,*), al(lda,*)
          double precision, intent(in) :: bh(ldb,*), bl(ldb,*)
          double precision, intent(in) :: betah, betal
          double precision, intent(inout) :: ch(ldc,*), cl(ldc,*)
        end subroutine

        subroutine DDSYMM(SIDE, UPLO, M, N, ALPHAH, ALPHAL,
     +      AH, AL, LDA, BH, BL, LDB, BETAH, BETAL, CH, CL, LDC)
          character(1), intent(in) :: side, uplo
          integer, intent(in) :: m, n, lda, ldb, ldc
          double precision, intent(in) :: alphah, alphal
          double precision, intent(in) :: ah(lda,*), al(lda,*)
          double precision, intent(in) :: bh(ldb,*), bl(ldb,*)
          double precision, intent(in) :: betah, betal
          double precision, intent(inout) :: ch(ldc,*), cl(ldc,*)
        end subroutine

        subroutine DDSYRK(SIDE, UPLO, N, K, ALPHAH, ALPHAL,
     +      AH, AL, LDA, BETAH, BETAL, CH, CL, LDC)
          character(1), intent(in) :: side, uplo
          integer, intent(in) :: n, k, lda, ldc
          double precision, intent(in) :: alphah, alphal
          double precision, intent(in) :: ah(lda,*), al(lda,*)
          double precision, intent(in) :: betah, betal
          double precision, intent(inout) :: ch(ldc,*), cl(ldc,*)
        end subroutine

        subroutine DDSYR2K(UPLO, TRANS, N, K, ALPHAH, ALPHAL,
     +      AH, AL, LDA, BH, BL, LDB, BETAH, BETAL, CH, CL, LDC)
          character(1), intent(in) :: uplo, trans
          integer, intent(in) :: n, k, lda, ldb, ldc
          double precision, intent(in) :: alphah, alphal
          double precision, intent(in) :: ah(lda,*), al(lda,*)
          double precision, intent(in) :: bh(ldb,*), bl(ldb,*)
          double precision, intent(in) :: betah, betal
          double precision, intent(inout) :: ch(ldc,*), cl(ldc,*)
        end subroutine

        subroutine DDTRMM(SIDE, UPLO, TRANSA, DIAG, M, N,
     +      ALPHAH, ALPHAL, AH, AL, LDA, BH, BL, LDB)
          character(1), intent(in) :: side, uplo, transa, diag
          integer, intent(in) :: m, n, lda, ldb
          double precision, intent(in) :: alphah, alphal
          double precision, intent(in) :: ah(lda,*), al(lda,*)
          double precision, intent(inout) :: bh(ldb,*), bl(ldb,*)
        end subroutine

        subroutine DDTRSM(SIDE, UPLO, TRANSA, DIAG, M, N,
     +      ALPHAH, ALPHAL, AH, AL, LDA, BH, BL, LDB)
          character(1), intent(in) :: side, uplo, transa, diag
          integer, intent(in) :: m, n, lda, ldb
          double precision, intent(in) :: alphah, alphal
          double precision, intent(in) :: ah(lda,*), al(lda,*)
          double precision, intent(inout) :: bh(ldb,*), bl(ldb,*)
        end subroutine

        subroutine ZZHEMM(SIDE, UPLO, M, N, ALPHAH, ALPHAL,
     +      AH, AL, LDA, BH, BL, LDB, BETAH, BETAL, CH, CL, LDC)
          character(1), intent(in) :: side, uplo
          integer, intent(in) :: m, n, lda, ldb, ldc
          double complex, intent(in) :: alphah, alphal
          double complex, intent(in) :: ah(lda,*), al(lda,*)
          double complex, intent(in) :: bh(ldb,*), bl(ldb,*)
          double complex, intent(in) :: betah, betal
          double complex, intent(inout) :: ch(ldc,*), cl(ldc,*)
        end subroutine

        subroutine ZZHERK(UPLO, TRANS, N, K, ALPHAH, ALPHAL,
     +      AH, AL, LDA, BETAH, BETAL, CH, CL, LDC)
          character(1), intent(in) :: uplo, trans
          integer, intent(in) :: n, k, lda, ldc
          double precision, intent(in) :: alphah, alphal
          double complex, intent(in) :: ah(lda,*), al(lda,*)
          double precision, intent(in) :: betah, betal
          double complex, intent(inout) :: ch(ldc,*), cl(ldc,*)
        end subroutine

        subroutine ZZHER2K(UPLO, TRANS, N, K, ALPHAH, ALPHAL,
     +      AH, AL, LDA, BH, BL, LDB, BETAH, BETAL, CH, CL, LDC)
          character(1), intent(in) :: uplo, trans
          integer, intent(in) :: n, k, lda, ldb, ldc
          double complex, intent(in) :: alphah, alphal
          double complex, intent(in) :: ah(lda,*), al(lda,*)
          double complex, intent(in) :: bh(ldb,*), bl(ldb,*)
          double precision, intent(in) :: betah, betal
          double complex, intent(inout) :: ch(ldc,*), cl(ldc,*)
        end subroutine

      end interface

      end module
