!     SUBROUTINE DGER(M,N,ALPHA,X,INCX,Y,INCY,A,LDA)
      subroutine ddger(m, n,
     +                alphah, alphal,
     +                xh, xl, incx,
     +                yh, yl, incy,
     +                ah, al, lda)
      implicit none
*     .. Scalar Arguments ..
!     DOUBLE PRECISION ALPHA
      double precision alphah, alphal
      INTEGER INCX,INCY,LDA,M,N
*     ..
*     .. Array Arguments ..
!     DOUBLE PRECISION A(LDA,*),X(*),Y(*)
      double precision ah(lda,*), al(lda,*)
      double precision xh(*), xl(*)
      double precision yh(*), yl(*)
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
*     ..
*
*  Purpose
*  =======
*
*  DGER   performs the rank 1 operation
*
*     A := alpha*x*y' + A,
*
*  where alpha is a scalar, x is an m element vector, y is an n element
*  vector and A is an m by n matrix.
*
*  Arguments
*  ==========
*
*  M      - INTEGER.
*           On entry, M specifies the number of rows of the matrix A.
*           M must be at least zero.
*           Unchanged on exit.
*
*  N      - INTEGER.
*           On entry, N specifies the number of columns of the matrix A.
*           N must be at least zero.
*           Unchanged on exit.
*
*  ALPHA  - DOUBLE PRECISION.
*           On entry, ALPHA specifies the scalar alpha.
*           Unchanged on exit.
*
*  X      - DOUBLE PRECISION array of dimension at least
*           ( 1 + ( m - 1 )*abs( INCX ) ).
*           Before entry, the incremented array X must contain the m
*           element vector x.
*           Unchanged on exit.
*
*  INCX   - INTEGER.
*           On entry, INCX specifies the increment for the elements of
*           X. INCX must not be zero.
*           Unchanged on exit.
*
*  Y      - DOUBLE PRECISION array of dimension at least
*           ( 1 + ( n - 1 )*abs( INCY ) ).
*           Before entry, the incremented array Y must contain the n
*           element vector y.
*           Unchanged on exit.
*
*  INCY   - INTEGER.
*           On entry, INCY specifies the increment for the elements of
*           Y. INCY must not be zero.
*           Unchanged on exit.
*
*  A      - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
*           Before entry, the leading m by n part of the array A must
*           contain the matrix of coefficients. On exit, A is
*           overwritten by the updated matrix.
*
*  LDA    - INTEGER.
*           On entry, LDA specifies the first dimension of A as declared
*           in the calling (sub) program. LDA must be at least
*           max( 1, m ).
*           Unchanged on exit.
*
*
*  Level 2 Blas routine.
*
*  -- Written on 22-October-1986.
*     Jack Dongarra, Argonne National Lab.
*     Jeremy Du Croz, Nag Central Office.
*     Sven Hammarling, Nag Central Office.
*     Richard Hanson, Sandia National Labs.
*
*
*     .. Parameters ..
!     DOUBLE PRECISION ZERO
!     PARAMETER (ZERO=0.0D+0)
      double precision zeroh, zerol
      double precision ddmh, ddml
!     tmp vars
      double precision t_ch, t_cl
      double precision t_p1, t_p2
      double precision t_q1, t_q2
      double precision t_r1, t_r2
      double precision t_s1, t_s2
      double precision t_t1, t_t2
      double precision t_u1, t_u2
      double precision t_x1, t_x2
      double precision t_y1, t_y2
      double precision t_z1, t_z2
      double precision t_x, t_ax
      double precision t_t1h, t_t1l
      double precision t_t2h, t_t2l
*     ..
*     .. Local Scalars ..
!     DOUBLE PRECISION TEMP
      double precision temph, templ
      INTEGER I,INFO,IX,J,JY,KX
*     ..
*     .. External Subroutines ..
      EXTERNAL XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC MAX
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "ddger"
#endif
      zeroh = 0.0d+0
      zerol = 0.0d+0
*     ..
*
*     Test the input parameters.
*
      INFO = 0
      IF (M.LT.0) THEN
          INFO = 1
      ELSE IF (N.LT.0) THEN
          INFO = 2
      ELSE IF (INCX.EQ.0) THEN
          INFO = 5
      ELSE IF (INCY.EQ.0) THEN
          INFO = 7
      ELSE IF (LDA.LT.MAX(1,M)) THEN
          INFO = 9
      END IF
      IF (INFO.NE.0) THEN
          CALL XERBLA('DGER  ',INFO)
          RETURN
      END IF
*
*     Quick return if possible.
*
!     IF ((M.EQ.0) .OR. (N.EQ.0) .OR. (ALPHA.EQ.ZERO)) RETURN
      if((m .eq. 0) .or. (n.eq.0) .or.
!       (alpha .eq. zero)
     + ((alphah .eq. zeroh) .and.
     +  (alphal .eq. zerol))
     +) goto 9999
*
*     Start the operations. In this version the elements of A are
*     accessed sequentially with one pass through A.
*
      IF (INCY.GT.0) THEN
          JY = 1
      ELSE
          JY = 1 - (N-1)*INCY
      END IF
      IF (INCX.EQ.1) THEN
          DO 20 J = 1,N
!             IF (Y(JY).NE.ZERO) THEN
              if(
!               (y(jy) .ne. zero)
     +         ((yh(jy) .ne. zeroh) .or.
     +          (yl(jy) .ne. zerol))
     +        ) then
!                 TEMP = ALPHA*Y(JY)
!<<<              temp = alpha * y(jy)
                  temph = alphah * yh(jy)
                  t_u1 = 134217729.0 * alphah
                  t_p1 = t_u1
     +                     - (t_u1 - alphah)
                  t_p2 = alphah - t_p1
                  t_u1 = 134217729.0 * yh(jy)
                  t_r1 = t_u1
     +                     - (t_u1 - yh(jy))
                  t_r2 = yh(jy) - t_r1
                  templ = ((t_p1*t_r1
     +                    - temph)
     +                    + t_p1 * t_r2
     +                    + t_p2 * t_r1)
     +                    + t_p2 * t_r2
                  templ = templ
     +                  + (alphah*yl(jy)
     +                  + alphal*yh(jy))
                  t_s1 = temph + templ
                  templ = templ - (t_s1 - temph)
                  temph = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              temp = alpha * y(jy)
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                  DO 10 I = 1,M
!                     A(I,J) = A(I,J) + X(I)*TEMP
!<<<                  ddm = x(i) * temp
                      ddmh = xh(i) * temph
                      t_u1 = 134217729.0 * xh(i)
                      t_p1 = t_u1
     +                         - (t_u1 - xh(i))
                      t_p2 = xh(i) - t_p1
                      t_u1 = 134217729.0 * temph
                      t_r1 = t_u1
     +                         - (t_u1 - temph)
                      t_r2 = temph - t_r1
                      ddml = ((t_p1*t_r1
     +                        - ddmh)
     +                        + t_p1 * t_r2
     +                        + t_p2 * t_r1)
     +                        + t_p2 * t_r2
                      ddml = ddml
     +                      + (xh(i)*templ
     +                      + xl(i)*temph)
                      t_s1 = ddmh + ddml
                      ddml = ddml - (t_s1 - ddmh)
                      ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddm = x(i) * temp
!<<<                  a(i,j) = a(i,j) + ddm
                      t_p1 = ah(i,j) + ddmh
                      t_p2 = t_p1 - ah(i,j)
                      t_q1 = t_p1
                      t_q2 = (ah(i,j) - (t_p1 - t_p2))
     +                         + (ddmh - t_p2)
                      t_q2 = t_q2 + al(i,j) + ddml
                      ah(i,j) = t_q1 + t_q2
                      al(i,j) = t_q2 - (ah(i,j) - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  a(i,j) = a(i,j) + ddm
   10             CONTINUE
!$omp end do
!$omp end parallel
              END IF
              JY = JY + INCY
   20     CONTINUE
      ELSE
          IF (INCX.GT.0) THEN
              KX = 1
          ELSE
              KX = 1 - (M-1)*INCX
          END IF
          DO 40 J = 1,N
!             IF (Y(JY).NE.ZERO) THEN
              if(
!               (y(jy) .ne. zero)
     +         ((yh(jy) .ne. zeroh) .or.
     +          (yl(jy) .ne. zerol))
     +        ) then
!                 TEMP = ALPHA*Y(JY)
!<<<              temp = alpha * y(jy)
                  temph = alphah * yh(jy)
                  t_u1 = 134217729.0 * alphah
                  t_p1 = t_u1
     +                     - (t_u1 - alphah)
                  t_p2 = alphah - t_p1
                  t_u1 = 134217729.0 * yh(jy)
                  t_r1 = t_u1
     +                     - (t_u1 - yh(jy))
                  t_r2 = yh(jy) - t_r1
                  templ = ((t_p1*t_r1
     +                    - temph)
     +                    + t_p1 * t_r2
     +                    + t_p2 * t_r1)
     +                    + t_p2 * t_r2
                  templ = templ
     +                  + (alphah*yl(jy)
     +                  + alphal*yh(jy))
                  t_s1 = temph + templ
                  templ = templ - (t_s1 - temph)
                  temph = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              temp = alpha * y(jy)
                  IX = KX
!$omp parallel
!$omp do
!$omp+ private(I,ix)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                  DO 30 I = 1,M
                      ix = kx + (i - 1)*incx
!                     A(I,J) = A(I,J) + X(IX)*TEMP
!<<<                  ddm = x(ix) * temp
                      ddmh = xh(ix) * temph
                      t_u1 = 134217729.0 * xh(ix)
                      t_p1 = t_u1
     +                         - (t_u1 - xh(ix))
                      t_p2 = xh(ix) - t_p1
                      t_u1 = 134217729.0 * temph
                      t_r1 = t_u1
     +                         - (t_u1 - temph)
                      t_r2 = temph - t_r1
                      ddml = ((t_p1*t_r1
     +                        - ddmh)
     +                        + t_p1 * t_r2
     +                        + t_p2 * t_r1)
     +                        + t_p2 * t_r2
                      ddml = ddml
     +                      + (xh(ix)*templ
     +                      + xl(ix)*temph)
                      t_s1 = ddmh + ddml
                      ddml = ddml - (t_s1 - ddmh)
                      ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddm = x(ix) * temp
!<<<                  a(i,j) = a(i,j) + ddm
                      t_p1 = ah(i,j) + ddmh
                      t_p2 = t_p1 - ah(i,j)
                      t_q1 = t_p1
                      t_q2 = (ah(i,j) - (t_p1 - t_p2))
     +                         + (ddmh - t_p2)
                      t_q2 = t_q2 + al(i,j) + ddml
                      ah(i,j) = t_q1 + t_q2
                      al(i,j) = t_q2 - (ah(i,j) - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  a(i,j) = a(i,j) + ddm
!!                    IX = IX + INCX
   30             CONTINUE
!$omp end do
!$omp end parallel
              END IF
              JY = JY + INCY
   40     CONTINUE
      END IF
*
 9999 continue
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = ddger"
#include "fout2.h"
#include "fformat.h"
#endif
      RETURN
*
*     End of DGER  .
*
      END
