!     SUBROUTINE DROTM(N,DX,INCX,DY,INCY,DPARAM)
      subroutine ddrotm(n,dxh,dxl,incx,dyh,dyl,incy,dparamh,dparaml)
      implicit none
*     .. Scalar Arguments ..
      INTEGER INCX,INCY,N
*     ..
*     .. Array Arguments ..
!     DOUBLE PRECISION DPARAM(5),DX(1),DY(1)
      double precision dparamh(5), dparaml(5)
      double precision dxh(1), dxl(1)
      double precision dyh(1), dyl(1)
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
*     ..
*
*  Purpose
*  =======
*
*     APPLY THE MODIFIED GIVENS TRANSFORMATION, H, TO THE 2 BY N MATRIX
*
*     (DX**T) , WHERE **T INDICATES TRANSPOSE. THE ELEMENTS OF DX ARE IN
*     (DY**T)
*
*     DX(LX+I*INCX), I = 0 TO N-1, WHERE LX = 1 IF INCX .GE. 0, ELSE
*     LX = (-INCX)*N, AND SIMILARLY FOR SY USING LY AND INCY.
*     WITH DPARAM(1)=DFLAG, H HAS ONE OF THE FOLLOWING FORMS..
*
*     DFLAG=-1.D0     DFLAG=0.D0        DFLAG=1.D0     DFLAG=-2.D0
*
*       (DH11  DH12)    (1.D0  DH12)    (DH11  1.D0)    (1.D0  0.D0)
*     H=(          )    (          )    (          )    (          )
*       (DH21  DH22),   (DH21  1.D0),   (-1.D0 DH22),   (0.D0  1.D0).
*     SEE DROTMG FOR A DESCRIPTION OF DATA STORAGE IN DPARAM.
*
*  Arguments
*  =========
*
*  N      (input) INTEGER
*         number of elements in input vector(s)
*
*  DX     (input/output) DOUBLE PRECISION array, dimension N
*         double precision vector with 5 elements
*
*  INCX   (input) INTEGER
*         storage spacing between elements of DX
*
*  DY     (input/output) DOUBLE PRECISION array, dimension N
*         double precision vector with N elements
*
*  INCY   (input) INTEGER
*         storage spacing between elements of DY
*
*  DPARAM (input/output)  DOUBLE PRECISION array, dimension 5 
*     DPARAM(1)=DFLAG
*     DPARAM(2)=DH11
*     DPARAM(3)=DH21
*     DPARAM(4)=DH12
*     DPARAM(5)=DH22
*
*  =====================================================================
*
*     .. Local Scalars ..
!     DOUBLE PRECISION DFLAG,DH11,DH12,DH21,DH22,TWO,W,Z,ZERO
      double precision dflagh, dflagl
      double precision dh11h, dh11l
      double precision dh12h, dh12l
      double precision dh21h, dh21l
      double precision dh22h, dh22l
      double precision twoh, twol
      double precision wh, wl
      double precision zh, zl
      double precision zeroh, zerol
      double precision df2h, df2l
      double precision ddmh, ddml
      double precision ddah, ddal
!     tmp vars
      double precision t_ch, t_cl
      double precision t_p1, t_p2
      double precision t_q1, t_q2
      double precision t_r1, t_r2
      double precision t_s1, t_s2
      double precision t_t1, t_t2
      double precision t_u1, t_u2
      double precision t_x1, t_x2
      double precision t_y1, t_y2
      double precision t_z1, t_z2
      double precision t_x, t_ax
      double precision t_t1h, t_t1l
      double precision t_t2h, t_t2l
      INTEGER I,KX,KY,NSTEPS
*     ..
*     .. Data statements ..
!     DATA ZERO,TWO/0.D0,2.D0/
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "ddrotm"
#endif
      zeroh = 0.0d+0
      zerol = 0.0d+0
      twoh = 2.0d0
      twol = 0.0d+0
*     ..
*
!     DFLAG = DPARAM(1)
      dflagh = dparamh(1)
      dflagl = dparaml(1)
!     IF (N.LE.0 .OR. (DFLAG+TWO.EQ.ZERO)) GO TO 140
!<<<  df2 = dflag + two
      t_p1 = dflagh + twoh
      t_p2 = t_p1 - dflagh
      df2h = t_p1
      df2l = (dflagh - (t_p1 - t_p2))
     +      + (twoh - t_p2)
      df2l = df2l + dflagl + twol
      t_s1 = df2h + df2l
      df2l = df2l - (t_s1 - df2h)
      df2h = t_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  df2 = dflag + two
      if(n .le. 0 .or.
!       (df2 .eq. zero)
     + ((df2h .eq. zeroh) .and.
     +  (df2l .eq. zerol))
     +) goto 140
      IF (.NOT. (INCX.EQ.INCY.AND.INCX.GT.0)) GO TO 70
*
      NSTEPS = N*INCX
!     IF (DFLAG) 50,10,30
      if(
!       (dflag .lt. zero)
     + ((dflagh .lt. zeroh) .or.
     +  ((dflagh .eq. zeroh) .and.
     +   (dflagl .lt. zerol)))
     +) goto 50
      if(
!       (dflag .eq. zero)
     + ((dflagh .eq. zeroh) .and.
     +  (dflagl .eq. zerol))
     +) goto 10
      if(
!       (dflag .gt. zero)
     + ((dflagh .gt. zeroh) .or.
     +  ((dflagh .eq. zeroh) .and.
     +   (dflagl .gt. zerol)))
     +) goto 30
   10 CONTINUE
!     DH12 = DPARAM(4)
      dh12h = dparamh(4)
      dh12l = dparaml(4)
!     DH21 = DPARAM(3)
      dh21h = dparamh(3)
      dh21l = dparaml(3)
      DO 20 I = 1,NSTEPS,INCX
!         W = DX(I)
          wh = dxh(i)
          wl = dxl(i)
!         Z = DY(I)
          zh = dyh(i)
          zl = dyl(i)
!         DX(I) = W + Z*DH12
!<<<      ddm = z * dh12
          ddmh = zh * dh12h
          t_u1 = 134217729.0 * zh
          t_p1 = t_u1
     +             - (t_u1 - zh)
          t_p2 = zh - t_p1
          t_u1 = 134217729.0 * dh12h
          t_r1 = t_u1
     +             - (t_u1 - dh12h)
          t_r2 = dh12h - t_r1
          ddml = ((t_p1*t_r1
     +            - ddmh)
     +            + t_p1 * t_r2
     +            + t_p2 * t_r1)
     +            + t_p2 * t_r2
          ddml = ddml
     +          + (zh*dh12l
     +          + zl*dh12h)
          t_s1 = ddmh + ddml
          ddml = ddml - (t_s1 - ddmh)
          ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>      ddm = z * dh12
!<<<      dx(i) = w + ddm
          t_p1 = wh + ddmh
          t_p2 = t_p1 - wh
          dxh(i) = t_p1
          dxl(i) = (wh - (t_p1 - t_p2))
     +          + (ddmh - t_p2)
          dxl(i) = dxl(i) + wl + ddml
          t_s1 = dxh(i) + dxl(i)
          dxl(i) = dxl(i) - (t_s1 - dxh(i))
          dxh(i) = t_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>      dx(i) = w + ddm
!         DY(I) = W*DH21 + Z
!<<<      ddm = w * dh21
          ddmh = wh * dh21h
          t_u1 = 134217729.0 * wh
          t_p1 = t_u1
     +             - (t_u1 - wh)
          t_p2 = wh - t_p1
          t_u1 = 134217729.0 * dh21h
          t_r1 = t_u1
     +             - (t_u1 - dh21h)
          t_r2 = dh21h - t_r1
          ddml = ((t_p1*t_r1
     +            - ddmh)
     +            + t_p1 * t_r2
     +            + t_p2 * t_r1)
     +            + t_p2 * t_r2
          ddml = ddml
     +          + (wh*dh21l
     +          + wl*dh21h)
          t_s1 = ddmh + ddml
          ddml = ddml - (t_s1 - ddmh)
          ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>      ddm = w * dh21
!<<<      dy(i) = ddm + z
          t_p1 = ddmh + zh
          t_p2 = t_p1 - ddmh
          dyh(i) = t_p1
          dyl(i) = (ddmh - (t_p1 - t_p2))
     +          + (zh - t_p2)
          dyl(i) = dyl(i) + ddml + zl
          t_s1 = dyh(i) + dyl(i)
          dyl(i) = dyl(i) - (t_s1 - dyh(i))
          dyh(i) = t_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>      dy(i) = ddm + z
   20 CONTINUE
      GO TO 140
   30 CONTINUE
!     DH11 = DPARAM(2)
      dh11h = dparamh(2)
      dh11l = dparaml(2)
!     DH22 = DPARAM(5)
      dh22h = dparamh(5)
      dh22l = dparaml(5)
      DO 40 I = 1,NSTEPS,INCX
!         W = DX(I)
          wh = dxh(i)
          wl = dxl(i)
!         Z = DY(I)
          zh = dyh(i)
          zl = dyl(i)
!         DX(I) = W*DH11 + Z
!<<<      ddm = w * dh11
          ddmh = wh * dh11h
          t_u1 = 134217729.0 * wh
          t_p1 = t_u1
     +             - (t_u1 - wh)
          t_p2 = wh - t_p1
          t_u1 = 134217729.0 * dh11h
          t_r1 = t_u1
     +             - (t_u1 - dh11h)
          t_r2 = dh11h - t_r1
          ddml = ((t_p1*t_r1
     +            - ddmh)
     +            + t_p1 * t_r2
     +            + t_p2 * t_r1)
     +            + t_p2 * t_r2
          ddml = ddml
     +          + (wh*dh11l
     +          + wl*dh11h)
          t_s1 = ddmh + ddml
          ddml = ddml - (t_s1 - ddmh)
          ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>      ddm = w * dh11
!<<<      dx(i) = ddm + z
          t_p1 = ddmh + zh
          t_p2 = t_p1 - ddmh
          dxh(i) = t_p1
          dxl(i) = (ddmh - (t_p1 - t_p2))
     +          + (zh - t_p2)
          dxl(i) = dxl(i) + ddml + zl
          t_s1 = dxh(i) + dxl(i)
          dxl(i) = dxl(i) - (t_s1 - dxh(i))
          dxh(i) = t_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>      dx(i) = ddm + z
!         DY(I) = -W + DH22*Z
!<<<      ddm = dh22 * z
          ddmh = dh22h * zh
          t_u1 = 134217729.0 * dh22h
          t_p1 = t_u1
     +             - (t_u1 - dh22h)
          t_p2 = dh22h - t_p1
          t_u1 = 134217729.0 * zh
          t_r1 = t_u1
     +             - (t_u1 - zh)
          t_r2 = zh - t_r1
          ddml = ((t_p1*t_r1
     +            - ddmh)
     +            + t_p1 * t_r2
     +            + t_p2 * t_r1)
     +            + t_p2 * t_r2
          ddml = ddml
     +          + (dh22h*zl
     +          + dh22l*zh)
          t_s1 = ddmh + ddml
          ddml = ddml - (t_s1 - ddmh)
          ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>      ddm = dh22 * z
!<<<      dda = ddm - w
          t_p1 = ddmh - wh
          t_p2 = t_p1 - ddmh
          ddah = t_p1
          ddal = (ddmh - (t_p1 - t_p2))
     +             - (wh + t_p2)
          ddal = ddal + ddml - wl
          t_s1 = ddah + ddal
          ddal = ddal - (t_s1 - ddah)
          ddah = t_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>      dda = ddm - w
          dyh(i) = ddah
          dyl(i) = ddal
   40 CONTINUE
      GO TO 140
   50 CONTINUE
!     DH11 = DPARAM(2)
      dh11h = dparamh(2)
      dh11l = dparaml(2)
!     DH12 = DPARAM(4)
      dh12h = dparamh(4)
      dh12l = dparaml(4)
!     DH21 = DPARAM(3)
      dh21h = dparamh(3)
      dh21l = dparaml(3)
!     DH22 = DPARAM(5)
      dh22h = dparamh(5)
      dh22l = dparaml(5)
      DO 60 I = 1,NSTEPS,INCX
!         W = DX(I)
          wh = dxh(i)
          wl = dxl(i)
!         Z = DY(I)
          zh = dyh(i)
          zl = dyl(i)
!         DX(I) = W*DH11 + Z*DH12
!<<<      dda = w * dh11
          ddah = wh * dh11h
          t_u1 = 134217729.0 * wh
          t_p1 = t_u1
     +             - (t_u1 - wh)
          t_p2 = wh - t_p1
          t_u1 = 134217729.0 * dh11h
          t_r1 = t_u1
     +             - (t_u1 - dh11h)
          t_r2 = dh11h - t_r1
          ddal = ((t_p1*t_r1
     +            - ddah)
     +            + t_p1 * t_r2
     +            + t_p2 * t_r1)
     +            + t_p2 * t_r2
          ddal = ddal
     +          + (wh*dh11l
     +          + wl*dh11h)
          t_s1 = ddah + ddal
          ddal = ddal - (t_s1 - ddah)
          ddah = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>      dda = w * dh11
!<<<      ddm = z * dh12
          ddmh = zh * dh12h
          t_u1 = 134217729.0 * zh
          t_p1 = t_u1
     +             - (t_u1 - zh)
          t_p2 = zh - t_p1
          t_u1 = 134217729.0 * dh12h
          t_r1 = t_u1
     +             - (t_u1 - dh12h)
          t_r2 = dh12h - t_r1
          ddml = ((t_p1*t_r1
     +            - ddmh)
     +            + t_p1 * t_r2
     +            + t_p2 * t_r1)
     +            + t_p2 * t_r2
          ddml = ddml
     +          + (zh*dh12l
     +          + zl*dh12h)
          t_s1 = ddmh + ddml
          ddml = ddml - (t_s1 - ddmh)
          ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>      ddm = z * dh12
!<<<      dx(i) = dda + ddm
          t_p1 = ddah + ddmh
          t_p2 = t_p1 - ddah
          dxh(i) = t_p1
          dxl(i) = (ddah - (t_p1 - t_p2))
     +          + (ddmh - t_p2)
          dxl(i) = dxl(i) + ddal + ddml
          t_s1 = dxh(i) + dxl(i)
          dxl(i) = dxl(i) - (t_s1 - dxh(i))
          dxh(i) = t_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>      dx(i) = dda + ddm
!         DY(I) = W*DH21 + Z*DH22
!<<<      dda = w * dh21
          ddah = wh * dh21h
          t_u1 = 134217729.0 * wh
          t_p1 = t_u1
     +             - (t_u1 - wh)
          t_p2 = wh - t_p1
          t_u1 = 134217729.0 * dh21h
          t_r1 = t_u1
     +             - (t_u1 - dh21h)
          t_r2 = dh21h - t_r1
          ddal = ((t_p1*t_r1
     +            - ddah)
     +            + t_p1 * t_r2
     +            + t_p2 * t_r1)
     +            + t_p2 * t_r2
          ddal = ddal
     +          + (wh*dh21l
     +          + wl*dh21h)
          t_s1 = ddah + ddal
          ddal = ddal - (t_s1 - ddah)
          ddah = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>      dda = w * dh21
!<<<      ddm = z * dh22
          ddmh = zh * dh22h
          t_u1 = 134217729.0 * zh
          t_p1 = t_u1
     +             - (t_u1 - zh)
          t_p2 = zh - t_p1
          t_u1 = 134217729.0 * dh22h
          t_r1 = t_u1
     +             - (t_u1 - dh22h)
          t_r2 = dh22h - t_r1
          ddml = ((t_p1*t_r1
     +            - ddmh)
     +            + t_p1 * t_r2
     +            + t_p2 * t_r1)
     +            + t_p2 * t_r2
          ddml = ddml
     +          + (zh*dh22l
     +          + zl*dh22h)
          t_s1 = ddmh + ddml
          ddml = ddml - (t_s1 - ddmh)
          ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>      ddm = z * dh22
!<<<      dy(i) = dda + ddm
          t_p1 = ddah + ddmh
          t_p2 = t_p1 - ddah
          dyh(i) = t_p1
          dyl(i) = (ddah - (t_p1 - t_p2))
     +          + (ddmh - t_p2)
          dyl(i) = dyl(i) + ddal + ddml
          t_s1 = dyh(i) + dyl(i)
          dyl(i) = dyl(i) - (t_s1 - dyh(i))
          dyh(i) = t_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>      dy(i) = dda + ddm
   60 CONTINUE
      GO TO 140
   70 CONTINUE
      KX = 1
      KY = 1
      IF (INCX.LT.0) KX = 1 + (1-N)*INCX
      IF (INCY.LT.0) KY = 1 + (1-N)*INCY
*
!     IF (DFLAG) 120,80,100
      if(
!       (dflag .lt. 0.0)
     + ((dflagh .lt. 0.0d+0) .or.
     +  ((dflagh .eq. 0.0d+0) .and.
     +   (dflagl .lt. 0.0d+0)))
     +) goto 120
      if(
!       (dflag .eq. 0.0)
     + ((dflagh .eq. 0.0d+0) .and.
     +  (dflagl .eq. 0.0d+0))
     +) goto 80
      if(
!       (dflag .gt. 0.0)
     + ((dflagh .gt. 0.0d+0) .or.
     +  ((dflagh .eq. 0.0d+0) .and.
     +   (dflagl .gt. 0.0d+0)))
     +) goto 100
   80 CONTINUE
!     DH12 = DPARAM(4)
      dh12h = dparamh(4)
      dh12l = dparaml(4)
!     DH21 = DPARAM(3)
      dh21h = dparamh(3)
      dh21l = dparaml(3)
      DO 90 I = 1,N
!         W = DX(KX)
          wh = dxh(kx)
          wl = dxl(kx)
!         Z = DY(KY)
          zh = dyh(ky)
          zl = dyl(ky)
!         DX(KX) = W + Z*DH12
!<<<      ddm = z * dh12
          ddmh = zh * dh12h
          t_u1 = 134217729.0 * zh
          t_p1 = t_u1
     +             - (t_u1 - zh)
          t_p2 = zh - t_p1
          t_u1 = 134217729.0 * dh12h
          t_r1 = t_u1
     +             - (t_u1 - dh12h)
          t_r2 = dh12h - t_r1
          ddml = ((t_p1*t_r1
     +            - ddmh)
     +            + t_p1 * t_r2
     +            + t_p2 * t_r1)
     +            + t_p2 * t_r2
          ddml = ddml
     +          + (zh*dh12l
     +          + zl*dh12h)
          t_s1 = ddmh + ddml
          ddml = ddml - (t_s1 - ddmh)
          ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>      ddm = z * dh12
!<<<      dx(kx) = w + ddm
          t_p1 = wh + ddmh
          t_p2 = t_p1 - wh
          dxh(kx) = t_p1
          dxl(kx) = (wh - (t_p1 - t_p2))
     +          + (ddmh - t_p2)
          dxl(kx) = dxl(kx) + wl + ddml
          t_s1 = dxh(kx) + dxl(kx)
          dxl(kx) = dxl(kx) - (t_s1 - dxh(kx))
          dxh(kx) = t_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>      dx(kx) = w + ddm
!         DY(KY) = W*DH21 + Z
!<<<      ddm = w * dh21
          ddmh = wh * dh21h
          t_u1 = 134217729.0 * wh
          t_p1 = t_u1
     +             - (t_u1 - wh)
          t_p2 = wh - t_p1
          t_u1 = 134217729.0 * dh21h
          t_r1 = t_u1
     +             - (t_u1 - dh21h)
          t_r2 = dh21h - t_r1
          ddml = ((t_p1*t_r1
     +            - ddmh)
     +            + t_p1 * t_r2
     +            + t_p2 * t_r1)
     +            + t_p2 * t_r2
          ddml = ddml
     +          + (wh*dh21l
     +          + wl*dh21h)
          t_s1 = ddmh + ddml
          ddml = ddml - (t_s1 - ddmh)
          ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>      ddm = w * dh21
!<<<      dy(ky) = ddm + z
          t_p1 = ddmh + zh
          t_p2 = t_p1 - ddmh
          dyh(ky) = t_p1
          dyl(ky) = (ddmh - (t_p1 - t_p2))
     +          + (zh - t_p2)
          dyl(ky) = dyl(ky) + ddml + zl
          t_s1 = dyh(ky) + dyl(ky)
          dyl(ky) = dyl(ky) - (t_s1 - dyh(ky))
          dyh(ky) = t_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>      dy(ky) = ddm + z
          KX = KX + INCX
          KY = KY + INCY
   90 CONTINUE
      GO TO 140
  100 CONTINUE
!     DH11 = DPARAM(2)
      dh11h = dparamh(2)
      dh11l = dparaml(2)
!     DH22 = DPARAM(5)
      dh22h = dparamh(5)
      dh22l = dparaml(5)
      DO 110 I = 1,N
!         W = DX(KX)
          wh = dxh(kx)
          wl = dxl(kx)
!         Z = DY(KY)
          zh = dyh(ky)
          zl = dyl(ky)
!         DX(KX) = W*DH11 + Z
!<<<      ddm = w * dh11
          ddmh = wh * dh11h
          t_u1 = 134217729.0 * wh
          t_p1 = t_u1
     +             - (t_u1 - wh)
          t_p2 = wh - t_p1
          t_u1 = 134217729.0 * dh11h
          t_r1 = t_u1
     +             - (t_u1 - dh11h)
          t_r2 = dh11h - t_r1
          ddml = ((t_p1*t_r1
     +            - ddmh)
     +            + t_p1 * t_r2
     +            + t_p2 * t_r1)
     +            + t_p2 * t_r2
          ddml = ddml
     +          + (wh*dh11l
     +          + wl*dh11h)
          t_s1 = ddmh + ddml
          ddml = ddml - (t_s1 - ddmh)
          ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>      ddm = w * dh11
!<<<      dx(kx) = ddm + z
          t_p1 = ddmh + zh
          t_p2 = t_p1 - ddmh
          dxh(kx) = t_p1
          dxl(kx) = (ddmh - (t_p1 - t_p2))
     +          + (zh - t_p2)
          dxl(kx) = dxl(kx) + ddml + zl
          t_s1 = dxh(kx) + dxl(kx)
          dxl(kx) = dxl(kx) - (t_s1 - dxh(kx))
          dxh(kx) = t_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>      dx(kx) = ddm + z
!         DY(KY) = -W + DH22*Z
!<<<      ddm = dh22 * z
          ddmh = dh22h * zh
          t_u1 = 134217729.0 * dh22h
          t_p1 = t_u1
     +             - (t_u1 - dh22h)
          t_p2 = dh22h - t_p1
          t_u1 = 134217729.0 * zh
          t_r1 = t_u1
     +             - (t_u1 - zh)
          t_r2 = zh - t_r1
          ddml = ((t_p1*t_r1
     +            - ddmh)
     +            + t_p1 * t_r2
     +            + t_p2 * t_r1)
     +            + t_p2 * t_r2
          ddml = ddml
     +          + (dh22h*zl
     +          + dh22l*zh)
          t_s1 = ddmh + ddml
          ddml = ddml - (t_s1 - ddmh)
          ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>      ddm = dh22 * z
!<<<      dy(ky) = ddm - w
          t_p1 = ddmh - wh
          t_p2 = t_p1 - ddmh
          dyh(ky) = t_p1
          dyl(ky) = (ddmh - (t_p1 - t_p2))
     +             - (wh + t_p2)
          dyl(ky) = dyl(ky) + ddml - wl
          t_s1 = dyh(ky) + dyl(ky)
          dyl(ky) = dyl(ky) - (t_s1 - dyh(ky))
          dyh(ky) = t_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>      dy(ky) = ddm - w
          KX = KX + INCX
          KY = KY + INCY
  110 CONTINUE
      GO TO 140
  120 CONTINUE
!     DH11 = DPARAM(2)
      dh11h = dparamh(2)
      dh11l = dparaml(2)
!     DH12 = DPARAM(4)
      dh12h = dparamh(4)
      dh12l = dparaml(4)
!     DH21 = DPARAM(3)
      dh21h = dparamh(3)
      dh21l = dparaml(3)
!     DH22 = DPARAM(5)
      dh22h = dparamh(5)
      dh22l = dparaml(5)
      DO 130 I = 1,N
!         W = DX(KX)
          wh = dxh(kx)
          wl = dxl(kx)
!         Z = DY(KY)
          zh = dyh(ky)
          zl = dyl(ky)
!         DX(KX) = W*DH11 + Z*DH12
!<<<      dda = w * dh11
          ddah = wh * dh11h
          t_u1 = 134217729.0 * wh
          t_p1 = t_u1
     +             - (t_u1 - wh)
          t_p2 = wh - t_p1
          t_u1 = 134217729.0 * dh11h
          t_r1 = t_u1
     +             - (t_u1 - dh11h)
          t_r2 = dh11h - t_r1
          ddal = ((t_p1*t_r1
     +            - ddah)
     +            + t_p1 * t_r2
     +            + t_p2 * t_r1)
     +            + t_p2 * t_r2
          ddal = ddal
     +          + (wh*dh11l
     +          + wl*dh11h)
          t_s1 = ddah + ddal
          ddal = ddal - (t_s1 - ddah)
          ddah = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>      dda = w * dh11
!<<<      ddm = z * dh12
          ddmh = zh * dh12h
          t_u1 = 134217729.0 * zh
          t_p1 = t_u1
     +             - (t_u1 - zh)
          t_p2 = zh - t_p1
          t_u1 = 134217729.0 * dh12h
          t_r1 = t_u1
     +             - (t_u1 - dh12h)
          t_r2 = dh12h - t_r1
          ddml = ((t_p1*t_r1
     +            - ddmh)
     +            + t_p1 * t_r2
     +            + t_p2 * t_r1)
     +            + t_p2 * t_r2
          ddml = ddml
     +          + (zh*dh12l
     +          + zl*dh12h)
          t_s1 = ddmh + ddml
          ddml = ddml - (t_s1 - ddmh)
          ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>      ddm = z * dh12
!<<<      dx(kx) = dda + ddm
          t_p1 = ddah + ddmh
          t_p2 = t_p1 - ddah
          dxh(kx) = t_p1
          dxl(kx) = (ddah - (t_p1 - t_p2))
     +          + (ddmh - t_p2)
          dxl(kx) = dxl(kx) + ddal + ddml
          t_s1 = dxh(kx) + dxl(kx)
          dxl(kx) = dxl(kx) - (t_s1 - dxh(kx))
          dxh(kx) = t_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>      dx(kx) = dda + ddm
!         DY(KY) = W*DH21 + Z*DH22
!<<<      dda = w * dh21
          ddah = wh * dh21h
          t_u1 = 134217729.0 * wh
          t_p1 = t_u1
     +             - (t_u1 - wh)
          t_p2 = wh - t_p1
          t_u1 = 134217729.0 * dh21h
          t_r1 = t_u1
     +             - (t_u1 - dh21h)
          t_r2 = dh21h - t_r1
          ddal = ((t_p1*t_r1
     +            - ddah)
     +            + t_p1 * t_r2
     +            + t_p2 * t_r1)
     +            + t_p2 * t_r2
          ddal = ddal
     +          + (wh*dh21l
     +          + wl*dh21h)
          t_s1 = ddah + ddal
          ddal = ddal - (t_s1 - ddah)
          ddah = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>      dda = w * dh21
!<<<      ddm = z * dh22
          ddmh = zh * dh22h
          t_u1 = 134217729.0 * zh
          t_p1 = t_u1
     +             - (t_u1 - zh)
          t_p2 = zh - t_p1
          t_u1 = 134217729.0 * dh22h
          t_r1 = t_u1
     +             - (t_u1 - dh22h)
          t_r2 = dh22h - t_r1
          ddml = ((t_p1*t_r1
     +            - ddmh)
     +            + t_p1 * t_r2
     +            + t_p2 * t_r1)
     +            + t_p2 * t_r2
          ddml = ddml
     +          + (zh*dh22l
     +          + zl*dh22h)
          t_s1 = ddmh + ddml
          ddml = ddml - (t_s1 - ddmh)
          ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>      ddm = z * dh22
!<<<      dy(ky) = dda + ddm
          t_p1 = ddah + ddmh
          t_p2 = t_p1 - ddah
          dyh(ky) = t_p1
          dyl(ky) = (ddah - (t_p1 - t_p2))
     +          + (ddmh - t_p2)
          dyl(ky) = dyl(ky) + ddal + ddml
          t_s1 = dyh(ky) + dyl(ky)
          dyl(ky) = dyl(ky) - (t_s1 - dyh(ky))
          dyh(ky) = t_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>      dy(ky) = dda + ddm
          KX = KX + INCX
          KY = KY + INCY
  130 CONTINUE
  140 CONTINUE
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = ddrotm"
#include "fout2.h"
#include "fformat.h"
#endif
      RETURN
      END
