      SUBROUTINE DDSYR2(UPLO,N,ALPHAH,ALPHAL,XH,XL,INCX,YH,YL,INCY,
     *            AH,AL,LDA)
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
*     .. Scalar Arguments ..
      DOUBLE PRECISION ALPHAH,ALPHAL
      INTEGER INCX,INCY,LDA,N
      CHARACTER UPLO
*     ..
*     .. Array Arguments ..
      DOUBLE PRECISION AH(LDA,*),AL(LDA,*),XH(*),XL(*),YH(*),YL(*)
*     ..
*
*  Purpose
*  =======
*
*  DDSYR2  performs the symmetric rank 2 operation
*
*     A := alpha*x*y' + alpha*y*x' + A,
*
*  where alpha is a scalar, x and y are n element vectors and A is an n
*  by n symmetric matrix.
*
*  Arguments
*  ==========
*
*  UPLO   - CHARACTER*1.
*           On entry, UPLO specifies whether the upper or lower
*           triangular part of the array A is to be referenced as
*           follows:
*
*              UPLO = 'U' or 'u'   Only the upper triangular part of A
*                                  is to be referenced.
*
*              UPLO = 'L' or 'l'   Only the lower triangular part of A
*                                  is to be referenced.
*
*           Unchanged on exit.
*
*  N      - INTEGER.
*           On entry, N specifies the order of the matrix A.
*           N must be at least zero.
*           Unchanged on exit.
*
*  ALPHAH - DOUBLE PRECISION.
*           On entry, ALPHA specifies the high word of scalar alpha.
*           Unchanged on exit.
*  ALPHAL - DOUBLE PRECISION.
*           On entry, ALPHA specifies the low word of scalar alpha.
*           Unchanged on exit.
*
*  XH     - DOUBLE PRECISION array of dimension at least
*           ( 1 + ( n - 1 )*abs( INCX ) ).
*           Before entry, the incremented array XH must contain the 
*           high word of the n element vector x.
*           Unchanged on exit.
*  XL     - DOUBLE PRECISION array of dimension at least
*           ( 1 + ( n - 1 )*abs( INCX ) ).
*           Before entry, the incremented array XL must contain the 
*           low word of the n element vector x.
*           Unchanged on exit.
*
*  INCX   - INTEGER.
*           On entry, INCX specifies the increment for the elements of
*           X. INCX must not be zero.
*           Unchanged on exit.
*
*  YH     - DOUBLE PRECISION array of dimension at least
*           ( 1 + ( n - 1 )*abs( INCY ) ).
*           Before entry, the incremented array YH must contain the 
*           high word of the n element vector y.
*           Unchanged on exit.
*  YL     - DOUBLE PRECISION array of dimension at least
*           ( 1 + ( n - 1 )*abs( INCY ) ).
*           Before entry, the incremented array YL must contain the 
*           low word of the n element vector y.
*           Unchanged on exit.
*
*  INCY   - INTEGER.
*           On entry, INCY specifies the increment for the elements of
*           Y. INCY must not be zero.
*           Unchanged on exit.
*
*  AH     - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
*           Before entry with  UPLO = 'U' or 'u', the leading n by n
*           upper triangular part of the array AH must contain the upper
*           triangular part of the symmetric matrix and the strictly
*           lower triangular part of AH is not referenced. On exit, the
*           upper triangular part of the array AH is overwritten by the
*           upper triangular part of the updated matrix.
*           Before entry with UPLO = 'L' or 'l', the leading n by n
*           lower triangular part of the array AH must contain the lower
*           triangular part of the symmetric matrix and the strictly
*           upper triangular part of AH is not referenced. On exit, the
*           lower triangular part of the array AH is overwritten by the
*           lower triangular part of the updated matrix.
*           AH contains the high words of the matrix A.
*  AL     - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
*           Before entry with  UPLO = 'U' or 'u', the leading n by n
*           upper triangular part of the array AL must contain the upper
*           triangular part of the symmetric matrix and the strictly
*           lower triangular part of AL is not referenced. On exit, the
*           upper triangular part of the array AL is overwritten by the
*           upper triangular part of the updated matrix.
*           Before entry with UPLO = 'L' or 'l', the leading n by n
*           lower triangular part of the array AL must contain the lower
*           triangular part of the symmetric matrix and the strictly
*           upper triangular part of AL is not referenced. On exit, the
*           lower triangular part of the array AL is overwritten by the
*           lower triangular part of the updated matrix.
*           AL contains the low words of the matrix A.
*
*  LDA    - INTEGER.
*           On entry, LDA specifies the first dimension of A as declared
*           in the calling (sub) program. LDA must be at least
*           max( 1, n ).
*           Unchanged on exit.
*
*  Further Details
*  ===============
*
*  Level 2 DDBlas routine.
*
*  Advancesoft corp.
*  =================
*
*     First written by Osamu Hino 01/04/2010
*  =====================================================================
*
*     .. Parameters ..
      DOUBLE PRECISION ZERO
      PARAMETER (ZERO=0.0D+0)
*     ..
*     .. Local Scalars ..
!      DOUBLE PRECISION TEMP1,TEMP2
      DOUBLE PRECISION T1H,T1L,T2H,T2L,T3H,T3L,T4H,T4L
      double precision s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2
      INTEGER I,INFO,IX,IY,J,JX,JY,KX,KY
*     ..
*     .. External Functions ..
      LOGICAL LSAME
      EXTERNAL LSAME
*     ..
*     .. External Subroutines ..
      EXTERNAL XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC MAX
*     ..
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "DDSYR2"
#endif
*
*     Test the input parameters.
*
      INFO = 0
      IF (.NOT.LSAME(UPLO,'U') .AND. .NOT.LSAME(UPLO,'L')) THEN
          INFO = 1
      ELSE IF (N.LT.0) THEN
          INFO = 2
      ELSE IF (INCX.EQ.0) THEN
          INFO = 5
      ELSE IF (INCY.EQ.0) THEN
          INFO = 7
      ELSE IF (LDA.LT.MAX(1,N)) THEN
          INFO = 9
      END IF
      IF (INFO.NE.0) THEN
          CALL XERBLA('DDSYR2',INFO)
          RETURN
      END IF
*
*     Quick return if possible.
*
!     IF ((N.EQ.0) .OR. (ALPHA.EQ.ZERO)) RETURN
      IF ((N.EQ.0) .OR. (ALPHAH.EQ.ZERO.AND.ALPHAL.EQ.ZERO)) RETURN
*
*     Set up the start points in X and Y if the increments are not both
*     unity.
*
      IF ((INCX.NE.1) .OR. (INCY.NE.1)) THEN
          IF (INCX.GT.0) THEN
              KX = 1
          ELSE
              KX = 1 - (N-1)*INCX
          END IF
          IF (INCY.GT.0) THEN
              KY = 1
          ELSE
              KY = 1 - (N-1)*INCY
          END IF
          JX = KX
          JY = KY
      END IF
*
*     Start the operations. In this version the elements of A are
*     accessed sequentially with one pass through the triangular part
*     of A.
*
      IF (LSAME(UPLO,'U')) THEN
*
*        Form  A  when A is stored in the upper triangle.
*
          IF ((INCX.EQ.1) .AND. (INCY.EQ.1)) THEN
!$omp parallel
!$omp do
!$omp& private(I,J,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T1H,T1L,T2H,T2L,T3H,T3L,T4H,T4L)
              DO 20 J = 1,N
!                 IF ((X(J).NE.ZERO) .OR. (Y(J).NE.ZERO)) THEN
                  IF ((XH(J).NE.ZERO) .OR. (YH(J).NE.ZERO) .OR.
     *                (XL(J).NE.ZERO) .OR. (YL(J).NE.ZERO)) THEN
!                     TEMP1 = ALPHA*Y(J)
!                     TEMP2 = ALPHA*X(J)
!                     call dd_mul(T1H,T1L,ALPHAH,ALPHAL,YH(J),YL(J))
          T1H = ALPHAH * YH(J)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*YH(J)
          r1 = u1 - (u1-YH(J))
          r2 = YH(J) - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (ALPHAH * YL(J) + ALPHAL * YH(J))
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                     call dd_mul(T2H,T2L,ALPHAH,ALPHAL,XH(J),XL(J))
          T2H = ALPHAH * XH(J)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*XH(J)
          r1 = u1 - (u1-XH(J))
          r2 = XH(J) - r1
          T2L = ((p1*r1-T2H) + p1*r2 + p2*r1) + p2*r2
          T2L = T2L + (ALPHAH * XL(J) + ALPHAL * XH(J))
          s1 = T2H + T2L
          T2L = T2L - (s1 - T2H)
          T2H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
                      DO 10 I = 1,J
!                         A(I,J) = A(I,J) + X(I)*TEMP1 + Y(I)*TEMP2
!                         call dd_mul(T3H,T3L,XH(I),XL(I),T1H,T1L)
          T3H = XH(I) * T1H
          u1 = 134217729.0*XH(I)
          p1 = u1 - (u1-XH(I))
          p2 = XH(I) - p1
          u1 = 134217729.0*T1H
          r1 = u1 - (u1-T1H)
          r2 = T1H - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (XH(I) * T1L + XL(I) * T1H)
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                         call dd_mul(T4H,T4L,YH(I),YL(I),T2H,T2L)
          T4H = YH(I) * T2H
          u1 = 134217729.0*YH(I)
          p1 = u1 - (u1-YH(I))
          p2 = YH(I) - p1
          u1 = 134217729.0*T2H
          r1 = u1 - (u1-T2H)
          r2 = T2H - r1
          T4L = ((p1*r1-T4H) + p1*r2 + p2*r1) + p2*r2
          T4L = T4L + (YH(I) * T2L + YL(I) * T2H)
          s1 = T4H + T4L
          T4L = T4L - (s1 - T4H)
          T4H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                         call dd_selfadd(T3H,T3L,T4H,T4L)
          p1 = T3H + T4H
          p2 = p1 - T3H
          q2 = (T3H - (p1 - p2)) + (T4H - p2)
          q1 = p1
          q2 = q2 + T3L
          q2 = q2 + T4L
          s1 = q1 + q2
          T3L = q2 - (s1 - q1)
          T3H = s1
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                         call dd_selfadd(AH(I,J),AL(I,J),T3H,T3L)
          p1 = AH(I,J) + T3H
          p2 = p1 - AH(I,J)
          q2 = (AH(I,J) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + AL(I,J)
          q2 = q2 + T3L
          s1 = q1 + q2
          AL(I,J) = q2 - (s1 - q1)
          AH(I,J) = s1
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
   10                 CONTINUE
                  END IF
   20         CONTINUE
!$omp end parallel
          ELSE
!$omp parallel
!$omp do
!$omp& private(I,J,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T1H,T1L,T2H,T2L,T3H,T3L,T4H,T4L)
!$omp& private(IX,JX,IY,JY)
              DO 40 J = 1,N
                  JX = KX + INCX*(J-1)
                  JY = KY + INCY*(J-1)
!                 IF ((X(JX).NE.ZERO) .OR. (Y(JY).NE.ZERO)) THEN
                  IF ((XH(JX).NE.ZERO) .OR. (YH(JY).NE.ZERO) .OR.
     *                (XL(JX).NE.ZERO) .OR. (YL(JY).NE.ZERO)) THEN
!                     TEMP1 = ALPHA*Y(JY)
!                     TEMP2 = ALPHA*X(JX)
!                     call dd_mul(T1H,T1L,ALPHAH,ALPHAL,YH(JY),YL(JY))
          T1H = ALPHAH * YH(JY)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*YH(JY)
          r1 = u1 - (u1-YH(JY))
          r2 = YH(JY) - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (ALPHAH * YL(JY) + ALPHAL * YH(JY))
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                     call dd_mul(T2H,T2L,ALPHAH,ALPHAL,XH(JX),XL(JX))
          T2H = ALPHAH * XH(JX)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*XH(JX)
          r1 = u1 - (u1-XH(JX))
          r2 = XH(JX) - r1
          T2L = ((p1*r1-T2H) + p1*r2 + p2*r1) + p2*r2
          T2L = T2L + (ALPHAH * XL(JX) + ALPHAL * XH(JX))
          s1 = T2H + T2L
          T2L = T2L - (s1 - T2H)
          T2H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                      IX = KX
!                      IY = KY
                      DO 30 I = 1,J
                  IX = KX + INCX*(I-1)
                  IY = KY + INCY*(I-1)
!                         A(I,J) = A(I,J) + X(IX)*TEMP1 + Y(IY)*TEMP2
!                         call dd_mul(T3H,T3L,XH(IX),XL(IX),T1H,T1L)
          T3H = XH(IX) * T1H
          u1 = 134217729.0*XH(IX)
          p1 = u1 - (u1-XH(IX))
          p2 = XH(IX) - p1
          u1 = 134217729.0*T1H
          r1 = u1 - (u1-T1H)
          r2 = T1H - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (XH(IX) * T1L + XL(IX) * T1H)
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                         call dd_mul(T4H,T4L,YH(IY),YL(IY),T2H,T2L)
          T4H = YH(IY) * T2H
          u1 = 134217729.0*YH(IY)
          p1 = u1 - (u1-YH(IY))
          p2 = YH(IY) - p1
          u1 = 134217729.0*T2H
          r1 = u1 - (u1-T2H)
          r2 = T2H - r1
          T4L = ((p1*r1-T4H) + p1*r2 + p2*r1) + p2*r2
          T4L = T4L + (YH(IY) * T2L + YL(IY) * T2H)
          s1 = T4H + T4L
          T4L = T4L - (s1 - T4H)
          T4H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                         call dd_selfadd(T3H,T3L,T4H,T4L)
          p1 = T3H + T4H
          p2 = p1 - T3H
          q2 = (T3H - (p1 - p2)) + (T4H - p2)
          q1 = p1
          q2 = q2 + T3L
          q2 = q2 + T4L
          s1 = q1 + q2
          T3L = q2 - (s1 - q1)
          T3H = s1
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                         call dd_selfadd(AH(I,J),AL(I,J),T3H,T3L)
          p1 = AH(I,J) + T3H
          p2 = p1 - AH(I,J)
          q2 = (AH(I,J) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + AL(I,J)
          q2 = q2 + T3L
          s1 = q1 + q2
          AL(I,J) = q2 - (s1 - q1)
          AH(I,J) = s1
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                          IX = IX + INCX
!                          IY = IY + INCY
   30                 CONTINUE
                  END IF
!                  JX = JX + INCX
!                  JY = JY + INCY
   40         CONTINUE
!$omp end parallel
          END IF
      ELSE
*
*        Form  A  when A is stored in the lower triangle.
*
          IF ((INCX.EQ.1) .AND. (INCY.EQ.1)) THEN
!$omp parallel
!$omp do
!$omp& private(I,J,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T1H,T1L,T2H,T2L,T3H,T3L,T4H,T4L)
              DO 60 J = 1,N
!                 IF ((X(J).NE.ZERO) .OR. (Y(J).NE.ZERO)) THEN
                  IF ((XH(J).NE.ZERO) .OR. (YH(J).NE.ZERO) .OR.
     *                (XL(J).NE.ZERO) .OR. (YL(J).NE.ZERO)) THEN
!                     TEMP1 = ALPHA*Y(J)
!                     TEMP2 = ALPHA*X(J)
!                     call dd_mul(T1H,T1L,ALPHAH,ALPHAL,YH(J),YL(J))
          T1H = ALPHAH * YH(J)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*YH(J)
          r1 = u1 - (u1-YH(J))
          r2 = YH(J) - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (ALPHAH * YL(J) + ALPHAL * YH(J))
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                     call dd_mul(T2H,T2L,ALPHAH,ALPHAL,XH(J),XL(J))
          T2H = ALPHAH * XH(J)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*XH(J)
          r1 = u1 - (u1-XH(J))
          r2 = XH(J) - r1
          T2L = ((p1*r1-T2H) + p1*r2 + p2*r1) + p2*r2
          T2L = T2L + (ALPHAH * XL(J) + ALPHAL * XH(J))
          s1 = T2H + T2L
          T2L = T2L - (s1 - T2H)
          T2H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
                      DO 50 I = J,N
!                         A(I,J) = A(I,J) + X(I)*TEMP1 + Y(I)*TEMP2
!                         call dd_mul(T3H,T3L,XH(I),XL(I),T1H,T1L)
          T3H = XH(I) * T1H
          u1 = 134217729.0*XH(I)
          p1 = u1 - (u1-XH(I))
          p2 = XH(I) - p1
          u1 = 134217729.0*T1H
          r1 = u1 - (u1-T1H)
          r2 = T1H - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (XH(I) * T1L + XL(I) * T1H)
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                         call dd_mul(T4H,T4L,YH(I),YL(I),T2H,T2L)
          T4H = YH(I) * T2H
          u1 = 134217729.0*YH(I)
          p1 = u1 - (u1-YH(I))
          p2 = YH(I) - p1
          u1 = 134217729.0*T2H
          r1 = u1 - (u1-T2H)
          r2 = T2H - r1
          T4L = ((p1*r1-T4H) + p1*r2 + p2*r1) + p2*r2
          T4L = T4L + (YH(I) * T2L + YL(I) * T2H)
          s1 = T4H + T4L
          T4L = T4L - (s1 - T4H)
          T4H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                         call dd_selfadd(T3H,T3L,T4H,T4L)
          p1 = T3H + T4H
          p2 = p1 - T3H
          q2 = (T3H - (p1 - p2)) + (T4H - p2)
          q1 = p1
          q2 = q2 + T3L
          q2 = q2 + T4L
          s1 = q1 + q2
          T3L = q2 - (s1 - q1)
          T3H = s1
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                         call dd_selfadd(AH(I,J),AL(I,J),T3H,T3L)
          p1 = AH(I,J) + T3H
          p2 = p1 - AH(I,J)
          q2 = (AH(I,J) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + AL(I,J)
          q2 = q2 + T3L
          s1 = q1 + q2
          AL(I,J) = q2 - (s1 - q1)
          AH(I,J) = s1
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
   50                 CONTINUE
                  END IF
   60         CONTINUE
!$omp end parallel
          ELSE
!$omp parallel
!$omp do
!$omp& private(I,J,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T1H,T1L,T2H,T2L,T3H,T3L,T4H,T4L)
!$omp& private(JX,JY)
!$omp& private(IX,IY)
              DO 80 J = 1,N
                  JX = KX + INCX*(J-1)
                  JY = KY + INCY*(J-1)
!                 IF ((X(JX).NE.ZERO) .OR. (Y(JY).NE.ZERO)) THEN
                  IF ((XH(JX).NE.ZERO) .OR. (YH(JY).NE.ZERO) .OR.
     *                (XL(JX).NE.ZERO) .OR. (YL(JY).NE.ZERO)) THEN
!                     TEMP1 = ALPHA*Y(JY)
!                     TEMP2 = ALPHA*X(JX)
!                     call dd_mul(T1H,T1L,ALPHAH,ALPHAL,YH(JY),YL(JY))
          T1H = ALPHAH * YH(JY)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*YH(JY)
          r1 = u1 - (u1-YH(JY))
          r2 = YH(JY) - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (ALPHAH * YL(JY) + ALPHAL * YH(JY))
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                     call dd_mul(T2H,T2L,ALPHAH,ALPHAL,XH(JX),XL(JX))
          T2H = ALPHAH * XH(JX)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*XH(JX)
          r1 = u1 - (u1-XH(JX))
          r2 = XH(JX) - r1
          T2L = ((p1*r1-T2H) + p1*r2 + p2*r1) + p2*r2
          T2L = T2L + (ALPHAH * XL(JX) + ALPHAL * XH(JX))
          s1 = T2H + T2L
          T2L = T2L - (s1 - T2H)
          T2H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
                      IX = JX
                      IY = JY
                      DO 70 I = J,N
                          IX = JX + INCX*(I-J)
                          IY = JY + INCY*(I-J)
!                          if(INFO.ne.9999) then
!               if(IX.gt.N .or. IX.lt.1 ) print *,' ERROR**** IX=',IX
!               if(IY.gt.N .or. IY.lt.1 ) print *,' ERROR**** IY=',IY
!               if(IX.gt.N .or. IX.lt.1 .or. IY.gt.N .or. IY.lt.1) then
!                  	print *,'I=',I,' J=',J,' JX=',JX,' JY=',JY
!                  	endif
!                  	INFO = 9999
!                  	endif
!                  if(IX.gt.N .or. IX.lt.1 .or. IY.gt.N .or. IY.lt.1)
!     *             goto 70
!                         A(I,J) = A(I,J) + X(IX)*TEMP1 + Y(IY)*TEMP2
!                         call dd_mul(T3H,T3L,XH(IX),XL(IX),T1H,T1L)
          T3H = XH(IX) * T1H
          u1 = 134217729.0*XH(IX)
          p1 = u1 - (u1-XH(IX))
          p2 = XH(IX) - p1
          u1 = 134217729.0*T1H
          r1 = u1 - (u1-T1H)
          r2 = T1H - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (XH(IX) * T1L + XL(IX) * T1H)
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                         call dd_mul(T4H,T4L,YH(IY),YL(IY),T2H,T2L)
          T4H = YH(IY) * T2H
          u1 = 134217729.0*YH(IY)
          p1 = u1 - (u1-YH(IY))
          p2 = YH(IY) - p1
          u1 = 134217729.0*T2H
          r1 = u1 - (u1-T2H)
          r2 = T2H - r1
          T4L = ((p1*r1-T4H) + p1*r2 + p2*r1) + p2*r2
          T4L = T4L + (YH(IY) * T2L + YL(IY) * T2H)
          s1 = T4H + T4L
          T4L = T4L - (s1 - T4H)
          T4H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                         call dd_selfadd(T3H,T3L,T4H,T4L)
          p1 = T3H + T4H
          p2 = p1 - T3H
          q2 = (T3H - (p1 - p2)) + (T4H - p2)
          q1 = p1
          q2 = q2 + T3L
          q2 = q2 + T4L
          s1 = q1 + q2
          T3L = q2 - (s1 - q1)
          T3H = s1
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                         call dd_selfadd(AH(I,J),AL(I,J),T3H,T3L)
          p1 = AH(I,J) + T3H
          p2 = p1 - AH(I,J)
          q2 = (AH(I,J) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + AL(I,J)
          q2 = q2 + T3L
          s1 = q1 + q2
          AL(I,J) = q2 - (s1 - q1)
          AH(I,J) = s1
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                          IX = IX + INCX
!                          IY = IY + INCY
   70                 CONTINUE
                  END IF
!                  JX = JX + INCX
!                  JY = JY + INCY
   80         CONTINUE
!$omp end parallel
          END IF
      END IF
*
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = DDSYR2"
      WRITE(6,'(2A)') "UPLO      = ",UPLO
#include "fout2.h"
#include "fformat.h"
#endif
      RETURN
*
*     End of DDSYR2 .
*
      END
