      SUBROUTINE DDSYR2K(UPLO,TRANS,N,K,ALPHAH,ALPHAL,AH,AL,LDA,
     *                    BH,BL,LDB,BETAH,BETAL,CH,CL,LDC)
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
*     .. Scalar Arguments ..
      DOUBLE PRECISION ALPHAH,BETAH
      DOUBLE PRECISION ALPHAL,BETAL
      INTEGER K,LDA,LDB,LDC,N
      CHARACTER TRANS,UPLO
*     ..
*     .. Array Arguments ..
      DOUBLE PRECISION AH(LDA,*),BH(LDB,*),CH(LDC,*)
      DOUBLE PRECISION AL(LDA,*),BL(LDB,*),CL(LDC,*)
*     ..
*
*  Purpose
*  =======
*
*  DDSYR2K  performs one of the symmetric rank 2k operations
*
*     C := alpha*A*B' + alpha*B*A' + beta*C,
*
*  or
*
*     C := alpha*A'*B + alpha*B'*A + beta*C,
*
*  where  alpha and beta  are scalars, C is an  n by n  symmetric matrix
*  and  A and B  are  n by k  matrices  in the  first  case  and  k by n
*  matrices in the second case.
*
*  Arguments
*  ==========
*
*  UPLO   - CHARACTER*1.
*           On  entry,   UPLO  specifies  whether  the  upper  or  lower
*           triangular  part  of the  array  C  is to be  referenced  as
*           follows:
*
*              UPLO = 'U' or 'u'   Only the  upper triangular part of  C
*                                  is to be referenced.
*
*              UPLO = 'L' or 'l'   Only the  lower triangular part of  C
*                                  is to be referenced.
*
*           Unchanged on exit.
*
*  TRANS  - CHARACTER*1.
*           On entry,  TRANS  specifies the operation to be performed as
*           follows:
*
*              TRANS = 'N' or 'n'   C := alpha*A*B' + alpha*B*A' +
*                                        beta*C.
*
*              TRANS = 'T' or 't'   C := alpha*A'*B + alpha*B'*A +
*                                        beta*C.
*
*              TRANS = 'C' or 'c'   C := alpha*A'*B + alpha*B'*A +
*                                        beta*C.
*
*           Unchanged on exit.
*
*  N      - INTEGER.
*           On entry,  N specifies the order of the matrix C.  N must be
*           at least zero.
*           Unchanged on exit.
*
*  K      - INTEGER.
*           On entry with  TRANS = 'N' or 'n',  K  specifies  the number
*           of  columns  of the  matrices  A and B,  and on  entry  with
*           TRANS = 'T' or 't' or 'C' or 'c',  K  specifies  the  number
*           of rows of the matrices  A and B.  K must be at least  zero.
*           Unchanged on exit.
*
*  ALPHAH - DOUBLE PRECISION.
*           On entry, ALPHAH specifies the high word of scalar alpha.
*           Unchanged on exit.
*  ALPHAL - DOUBLE PRECISION.
*           On entry, ALPHAL specifies the low word of scalar alpha.
*           Unchanged on exit.
*
*  AH     - DOUBLE PRECISION array of DIMENSION ( LDA, ka ), where ka is
*           k  when  TRANS = 'N' or 'n',  and is  n  otherwise.
*           Before entry with  TRANS = 'N' or 'n',  the  leading  n by k
*           part of the array AH must contain the high word of matrix A, 
*           otherwise the leading  k by n  part of the array AH must contain 
*           the high word of matrix A.
*           Unchanged on exit.
*  AL     - DOUBLE PRECISION array of DIMENSION ( LDA, ka ), where ka is
*           k  when  TRANS = 'N' or 'n',  and is  n  otherwise.
*           Before entry with  TRANS = 'N' or 'n',  the  leading  n by k
*           part of the array AL must contain the low word of matrix A, 
*           otherwise the leading  k by n  part of the array AL must contain 
*           the low word of matrix A.
*           Unchanged on exit.
*
*  LDA    - INTEGER.
*           On entry, LDA specifies the first dimension of A as declared
*           in  the  calling  (sub)  program.   When  TRANS = 'N' or 'n'
*           then  LDA must be at least  max( 1, n ), otherwise  LDA must
*           be at least  max( 1, k ).
*           Unchanged on exit.
*
*  BH     - DOUBLE PRECISION array of DIMENSION ( LDB, kb ), where kb is
*           k  when  TRANS = 'N' or 'n',  and is  n  otherwise.
*           Before entry with  TRANS = 'N' or 'n',  the  leading  n by k
*           part of the array  BH must contain the high word of matrix  B,  
*           otherwise the leading  k by n  part of the array  BH must contain
*           the high word of matrix B.
*           Unchanged on exit.
*  BL     - DOUBLE PRECISION array of DIMENSION ( LDB, kb ), where kb is
*           k  when  TRANS = 'N' or 'n',  and is  n  otherwise.
*           Before entry with  TRANS = 'N' or 'n',  the  leading  n by k
*           part of the array BL must contain the high word of matrix B,  
*           otherwise the leading  k by n  part of the array BL must contain
*           the high word of matrix B.
*           Unchanged on exit.
*
*  LDB    - INTEGER.
*           On entry, LDB specifies the first dimension of B as declared
*           in  the  calling  (sub)  program.   When  TRANS = 'N' or 'n'
*           then  LDB must be at least  max( 1, n ), otherwise  LDB must
*           be at least  max( 1, k ).
*           Unchanged on exit.
*
*  BETAH  - DOUBLE PRECISION.
*           On entry, BETAH specifies the high word of scalar beta.
*           Unchanged on exit.
*  BETAL  - DOUBLE PRECISION.
*           On entry, BETAL specifies the low word of scalar beta.
*           Unchanged on exit.
*
*  CH     - DOUBLE PRECISION array of DIMENSION ( LDC, n ).
*           Before entry  with  UPLO = 'U' or 'u',  the leading  n by n
*           upper triangular part of the array CH must contain the high word of 
*           upper triangular part  of the  symmetric matrix  and the strictly
*           lower triangular part of CH is not referenced.  On exit, the
*           upper triangular part of the array CH is overwritten by the
*           high word of upper triangular part of the updated matrix.
*           Before entry  with  UPLO = 'L' or 'l',  the leading  n by n
*           lower triangular part of the array CH must contain the high word of 
*           lower triangular part  of the  symmetric matrix  and the strictly
*           upper triangular part of CH is not referenced.  On exit, the
*           lower triangular part of the array CH is overwritten by the
*           high word of lower triangular part of the updated matrix.
*  CL     - DOUBLE PRECISION array of DIMENSION ( LDC, n ).
*           Before entry  with  UPLO = 'U' or 'u',  the leading  n by n
*           upper triangular part of the array CL must contain the low word of 
*           upper triangular part  of the  symmetric matrix  and the strictly
*           lower triangular part of CL is not referenced.  On exit, the
*           upper triangular part of the array CL is overwritten by the
*           low word of upper triangular part of the updated matrix.
*           Before entry  with  UPLO = 'L' or 'l',  the leading  n by n
*           lower triangular part of the array CL must contain the low word of 
*           lower triangular part  of the  symmetric matrix  and the strictly
*           upper triangular part of CL is not referenced.  On exit, the
*           lower triangular part of the array CL is overwritten by the
*           low word of lower triangular part of the updated matrix.
*
*  LDC    - INTEGER.
*           On entry, LDC specifies the first dimension of C as declared
*           in  the  calling  (sub)  program.   LDC  must  be  at  least
*           max( 1, n ).
*           Unchanged on exit.
*
*  Further Details
*  ===============
*
*  Level 3 DDBlas routine.
*
*  Advancesoft corp.
*  =================
*
*     First written by Osamu Hino 01/04/2010
*  =====================================================================
*
*     .. External Functions ..
      LOGICAL LSAME
      EXTERNAL LSAME
*     ..
*     .. External Subroutines ..
      EXTERNAL XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC MAX
*     ..
*     .. Local Scalars ..
!     DOUBLE PRECISION TEMP1,TEMP2
      DOUBLE PRECISION T1H,T2H,T3H,T4H
      DOUBLE PRECISION T1L,T2L,T3L,T4L
      double precision s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2
      LOGICAL LALPHA,LBETA,LBETAZ,LTMP1,LTMP2
      INTEGER I,INFO,J,L,NROWA
      LOGICAL UPPER
*     ..
*     .. Parameters ..
      DOUBLE PRECISION ONE,ZERO
      PARAMETER (ONE=1.0D+0,ZERO=0.0D+0)
*     ..
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "DDSYR2K"
#endif
*
*     Test the input parameters.
*
      IF (LSAME(TRANS,'N')) THEN
          NROWA = N
      ELSE
          NROWA = K
      END IF
      UPPER = LSAME(UPLO,'U')
*
      INFO = 0
      IF ((.NOT.UPPER) .AND. (.NOT.LSAME(UPLO,'L'))) THEN
          INFO = 1
      ELSE IF ((.NOT.LSAME(TRANS,'N')) .AND.
     +         (.NOT.LSAME(TRANS,'T')) .AND.
     +         (.NOT.LSAME(TRANS,'C'))) THEN
          INFO = 2
      ELSE IF (N.LT.0) THEN
          INFO = 3
      ELSE IF (K.LT.0) THEN
          INFO = 4
      ELSE IF (LDA.LT.MAX(1,NROWA)) THEN
          INFO = 7
      ELSE IF (LDB.LT.MAX(1,NROWA)) THEN
          INFO = 9
      ELSE IF (LDC.LT.MAX(1,N)) THEN
          INFO = 12
      END IF
      IF (INFO.NE.0) THEN
          CALL XERBLA('DDSYR2K',INFO)
          RETURN
      END IF
*
*     Quick return if possible.
*
      LALPHA = (ALPHAH.EQ.ZERO) .AND. (ALPHAL.EQ.ZERO)
      LBETA = (BETAH.EQ.ONE) .AND. (BETAL.EQ.ZERO)
!     IF ((N.EQ.0) .OR. (((ALPHA.EQ.ZERO).OR.
!    +    (K.EQ.0)).AND. (BETA.EQ.ONE))) RETURN
      IF ((N.EQ.0) .OR. ((LALPHA.OR.
     +    (K.EQ.0)).AND. LBETA)) RETURN
      LBETAZ = (BETAH.EQ.ZERO) .AND. (BETAL.EQ.ZERO)
*
*     And when  alpha.eq.zero.
*
!     IF (ALPHA.EQ.ZERO) THEN
      IF (LALPHA) THEN
          IF (UPPER) THEN
!             IF (BETA.EQ.ZERO) THEN
              IF (LBETAZ) THEN
                  DO 20 J = 1,N
!$omp parallel
!$omp do
!$omp& private(I)
                      DO 10 I = 1,J
!                         C(I,J) = ZERO
                          CH(I,J) = ZERO
                          CL(I,J) = ZERO
   10                 CONTINUE
!$omp end do
!$omp end parallel
   20             CONTINUE
              ELSE
                  DO 40 J = 1,N
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
                      DO 30 I = 1,J
!                         C(I,J) = BETA*C(I,J)
!         call dd_selfmul(CH(I,J),CL(I,J),BETAH,BETAL)
          q1 = CH(I,J)*BETAH
          u1 = 134217729.0*CH(I,J)
          p1 = u1 - (u1-CH(I,J))
          p2 = CH(I,J) - p1
          u1 = 134217729.0*BETAH
          r1 = u1 - (u1-BETAH)
          r2 = BETAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + BETAL * CH(I,J)
          q2 = q2 + BETAH * CL(I,J)
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
   30                 CONTINUE
!$omp end do
!$omp end parallel
   40             CONTINUE
              END IF
          ELSE
!             IF (BETA.EQ.ZERO) THEN
              IF (LBETAZ) THEN
                  DO 60 J = 1,N
!$omp parallel
!$omp do
!$omp& private(I)
                      DO 50 I = J,N
!                         C(I,J) = ZERO
                          CH(I,J) = ZERO
                          CL(I,J) = ZERO
   50                 CONTINUE
!$omp end do
!$omp end parallel
   60             CONTINUE
              ELSE
                  DO 80 J = 1,N
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
                      DO 70 I = J,N
!                         C(I,J) = BETA*C(I,J)
!         call dd_selfmul(CH(I,J),CL(I,J),BETAH,BETAL)
          q1 = CH(I,J)*BETAH
          u1 = 134217729.0*CH(I,J)
          p1 = u1 - (u1-CH(I,J))
          p2 = CH(I,J) - p1
          u1 = 134217729.0*BETAH
          r1 = u1 - (u1-BETAH)
          r2 = BETAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + BETAL * CH(I,J)
          q2 = q2 + BETAH * CL(I,J)
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
   70                 CONTINUE
!$omp end do
!$omp end parallel
   80             CONTINUE
              END IF
          END IF
          RETURN
      END IF
*
*     Start the operations.
*
      IF (LSAME(TRANS,'N')) THEN
*
*        Form  C := alpha*A*B' + alpha*B*A' + C.
*
          IF (UPPER) THEN
              DO 130 J = 1,N
!                 IF (BETA.EQ.ZERO) THEN
                  IF (LBETAZ) THEN
!$omp parallel
!$omp do
!$omp& private(I)
                      DO 90 I = 1,J
!                         C(I,J) = ZERO
                          CH(I,J) = ZERO
                          CL(I,J) = ZERO
   90                 CONTINUE
!$omp end do
!$omp end parallel
!                 ELSE IF (BETA.NE.ONE) THEN
                  ELSE IF (.NOT.LBETA) THEN
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
                      DO 100 I = 1,J
!                         C(I,J) = BETA*C(I,J)
!         call dd_selfmul(CH(I,J),CL(I,J),BETAH,BETAL)
          q1 = CH(I,J)*BETAH
          u1 = 134217729.0*CH(I,J)
          p1 = u1 - (u1-CH(I,J))
          p2 = CH(I,J) - p1
          u1 = 134217729.0*BETAH
          r1 = u1 - (u1-BETAH)
          r2 = BETAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + BETAL * CH(I,J)
          q2 = q2 + BETAH * CL(I,J)
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
  100                 CONTINUE
!$omp end do
!$omp end parallel
                  END IF
                  DO 120 L = 1,K
!                     IF ((A(J,L).NE.ZERO) .OR. (B(J,L).NE.ZERO)) THEN
                      LTMP1 = (AH(J,L).NE.ZERO).OR.(AL(J,L).NE.ZERO)
                      LTMP2 = (BH(J,L).NE.ZERO).OR.(BL(J,L).NE.ZERO)
                      IF (LTMP1 .OR. LTMP2) THEN
!                         TEMP1 = ALPHA*B(J,L)
!         call dd_mul(T1H,T1L,ALPHAH,ALPHAL,BH(J,L),BL(J,L))
          T1H = ALPHAH * BH(J,L)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*BH(J,L)
          r1 = u1 - (u1-BH(J,L))
          r2 = BH(J,L) - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (ALPHAH * BL(J,L) + ALPHAL * BH(J,L))
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                         TEMP2 = ALPHA*A(J,L)
!         call dd_mul(T2H,T2L,ALPHAH,ALPHAL,AH(J,L),AL(J,L))
          T2H = ALPHAH * AH(J,L)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*AH(J,L)
          r1 = u1 - (u1-AH(J,L))
          r2 = AH(J,L) - r1
          T2L = ((p1*r1-T2H) + p1*r2 + p2*r1) + p2*r2
          T2L = T2L + (ALPHAH * AL(J,L) + ALPHAL * AH(J,L))
          s1 = T2H + T2L
          T2L = T2L - (s1 - T2H)
          T2H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T3H,T3L,T4H,T4L)
                          DO 110 I = 1,J
!                             C(I,J) = C(I,J) + A(I,L)*TEMP1 +
!    +                                 B(I,L)*TEMP2
!         call dd_mul(T3H,T3L,AH(I,L),AL(I,L),T1H,T1L)
          T3H = AH(I,L) * T1H
          u1 = 134217729.0*AH(I,L)
          p1 = u1 - (u1-AH(I,L))
          p2 = AH(I,L) - p1
          u1 = 134217729.0*T1H
          r1 = u1 - (u1-T1H)
          r2 = T1H - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (AH(I,L) * T1L + AL(I,L) * T1H)
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T4H,T4L,BH(I,L),BL(I,L),T2H,T2L)
          T4H = BH(I,L) * T2H
          u1 = 134217729.0*BH(I,L)
          p1 = u1 - (u1-BH(I,L))
          p2 = BH(I,L) - p1
          u1 = 134217729.0*T2H
          r1 = u1 - (u1-T2H)
          r2 = T2H - r1
          T4L = ((p1*r1-T4H) + p1*r2 + p2*r1) + p2*r2
          T4L = T4L + (BH(I,L) * T2L + BL(I,L) * T2H)
          s1 = T4H + T4L
          T4L = T4L - (s1 - T4H)
          T4H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T3H,T3L,T4H,T4L)
          p1 = T3H + T4H
          p2 = p1 - T3H
          q2 = (T3H - (p1 - p2)) + (T4H - p2)
          q1 = p1
          q2 = q2 + T3L
          q2 = q2 + T4L
          T3H = q1 + q2
          T3L = q2 - (T3H - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!         call dd_selfadd(CH(I,J),CL(I,J),T3H,T3L)
          p1 = CH(I,J) + T3H
          p2 = p1 - CH(I,J)
          q2 = (CH(I,J) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + CL(I,J)
          q2 = q2 + T3L
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
  110                     CONTINUE
!$omp end do
!$omp end parallel
                      END IF
  120             CONTINUE
  130         CONTINUE
          ELSE
              DO 180 J = 1,N
!                 IF (BETA.EQ.ZERO) THEN
                  IF (LBETAZ) THEN
!$omp parallel
!$omp do
!$omp& private(I)
                      DO 140 I = J,N
!                         C(I,J) = ZERO
                          CH(I,J) = ZERO
                          CL(I,J) = ZERO
  140                 CONTINUE
!$omp end do
!$omp end parallel
!                 ELSE IF (BETA.NE.ONE) THEN
                  ELSE IF (.NOT.LBETA) THEN
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
                      DO 150 I = J,N
!                         C(I,J) = BETA*C(I,J)
!         call dd_selfmul(CH(I,J),CL(I,J),BETAH,BETAL)
          q1 = CH(I,J)*BETAH
          u1 = 134217729.0*CH(I,J)
          p1 = u1 - (u1-CH(I,J))
          p2 = CH(I,J) - p1
          u1 = 134217729.0*BETAH
          r1 = u1 - (u1-BETAH)
          r2 = BETAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + BETAL * CH(I,J)
          q2 = q2 + BETAH * CL(I,J)
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
  150                 CONTINUE
!$omp end do
!$omp end parallel
                  END IF
                  DO 170 L = 1,K
!                     IF ((A(J,L).NE.ZERO) .OR. (B(J,L).NE.ZERO)) THEN
                      LTMP1 = (AH(J,L).NE.ZERO).OR.(AL(J,L).NE.ZERO)
                      LTMP2 = (BH(J,L).NE.ZERO).OR.(BL(J,L).NE.ZERO)
!                         TEMP1 = ALPHA*B(J,L)
!         call dd_mul(T1H,T1L,ALPHAH,ALPHAL,BH(J,L),BL(J,L))
          T1H = ALPHAH * BH(J,L)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*BH(J,L)
          r1 = u1 - (u1-BH(J,L))
          r2 = BH(J,L) - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (ALPHAH * BL(J,L) + ALPHAL * BH(J,L))
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                         TEMP2 = ALPHA*A(J,L)
!         call dd_mul(T2H,T2L,ALPHAH,ALPHAL,AH(J,L),AL(J,L))
          T2H = ALPHAH * AH(J,L)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*AH(J,L)
          r1 = u1 - (u1-AH(J,L))
          r2 = AH(J,L) - r1
          T2L = ((p1*r1-T2H) + p1*r2 + p2*r1) + p2*r2
          T2L = T2L + (ALPHAH * AL(J,L) + ALPHAL * AH(J,L))
          s1 = T2H + T2L
          T2L = T2L - (s1 - T2H)
          T2H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
                      IF (LTMP1 .OR. LTMP2) THEN
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T3H,T3L,T4H,T4L)
                          DO 160 I = J,N
!                             C(I,J) = C(I,J) + A(I,L)*TEMP1 +
!    +                                 B(I,L)*TEMP2
!         call dd_mul(T3H,T3L,AH(I,L),AL(I,L),T1H,T1L)
          T3H = AH(I,L) * T1H
          u1 = 134217729.0*AH(I,L)
          p1 = u1 - (u1-AH(I,L))
          p2 = AH(I,L) - p1
          u1 = 134217729.0*T1H
          r1 = u1 - (u1-T1H)
          r2 = T1H - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (AH(I,L) * T1L + AL(I,L) * T1H)
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T4H,T4L,BH(I,L),BL(I,L),T2H,T2L)
          T4H = BH(I,L) * T2H
          u1 = 134217729.0*BH(I,L)
          p1 = u1 - (u1-BH(I,L))
          p2 = BH(I,L) - p1
          u1 = 134217729.0*T2H
          r1 = u1 - (u1-T2H)
          r2 = T2H - r1
          T4L = ((p1*r1-T4H) + p1*r2 + p2*r1) + p2*r2
          T4L = T4L + (BH(I,L) * T2L + BL(I,L) * T2H)
          s1 = T4H + T4L
          T4L = T4L - (s1 - T4H)
          T4H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T3H,T3L,T4H,T4L)
          p1 = T3H + T4H
          p2 = p1 - T3H
          q2 = (T3H - (p1 - p2)) + (T4H - p2)
          q1 = p1
          q2 = q2 + T3L
          q2 = q2 + T4L
          T3H = q1 + q2
          T3L = q2 - (T3H - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!         call dd_selfadd(CH(I,J),CL(I,J),T3H,T3L)
          p1 = CH(I,J) + T3H
          p2 = p1 - CH(I,J)
          q2 = (CH(I,J) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + CL(I,J)
          q2 = q2 + T3L
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
  160                     CONTINUE
!$omp end do
!$omp end parallel
                      END IF
  170             CONTINUE
  180         CONTINUE
          END IF
      ELSE
*
*        Form  C := alpha*A'*B + alpha*B'*A + C.
*
          IF (UPPER) THEN
              DO 210 J = 1,N
                  DO 200 I = 1,J
!                     TEMP1 = ZERO
                      T1H = ZERO
                      T1L = ZERO
!                     TEMP2 = ZERO
                      T2H = ZERO
                      T2L = ZERO
!$omp parallel
!$omp do
!$omp& private(L,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T3H,T3L,T4H,T4L)
                      DO 190 L = 1,K
!                         TEMP1 = TEMP1 + A(L,I)*B(L,J)
!         call dd_mul(T3H,T3L,AH(L,I),AL(L,I),BH(L,J),BL(L,J))
          T3H = AH(L,I) * BH(L,J)
          u1 = 134217729.0*AH(L,I)
          p1 = u1 - (u1-AH(L,I))
          p2 = AH(L,I) - p1
          u1 = 134217729.0*BH(L,J)
          r1 = u1 - (u1-BH(L,J))
          r2 = BH(L,J) - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (AH(L,I) * BL(L,J) + AL(L,I) * BH(L,J))
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T1H,T1L,T3H,T3L)
!$omp critical (name0)
          p1 = T1H + T3H
          p2 = p1 - T1H
          q2 = (T1H - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + T1L
          q2 = q2 + T3L
          T1H = q1 + q2
          T1L = q2 - (T1H - q1)
!$omp end critical (name0)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                         TEMP2 = TEMP2 + B(L,I)*A(L,J)
!         call dd_mul(T4H,T4L,BH(L,I),BL(L,I),AH(L,J),AL(L,J))
          T4H = BH(L,I) * AH(L,J)
          u1 = 134217729.0*BH(L,I)
          p1 = u1 - (u1-BH(L,I))
          p2 = BH(L,I) - p1
          u1 = 134217729.0*AH(L,J)
          r1 = u1 - (u1-AH(L,J))
          r2 = AH(L,J) - r1
          T4L = ((p1*r1-T4H) + p1*r2 + p2*r1) + p2*r2
          T4L = T4L + (BH(L,I) * AL(L,J) + BL(L,I) * AH(L,J))
          s1 = T4H + T4L
          T4L = T4L - (s1 - T4H)
          T4H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T2H,T2L,T4H,T4L)
!$omp critical (name0)
          p1 = T2H + T4H
          p2 = p1 - T2H
          q2 = (T2H - (p1 - p2)) + (T4H - p2)
          q1 = p1
          q2 = q2 + T2L
          q2 = q2 + T4L
          T2H = q1 + q2
          T2L = q2 - (T2H - q1)
!$omp end critical (name0)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
  190                 CONTINUE
!$omp end do
!$omp end parallel
!                     IF (BETA.EQ.ZERO) THEN
                      IF (LBETAZ) THEN
!                         C(I,J) = ALPHA*TEMP1 + ALPHA*TEMP2
!         call dd_mul(T3H,T3L,ALPHAH,ALPHAL,T1H,T1L)
          T3H = ALPHAH * T1H
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*T1H
          r1 = u1 - (u1-T1H)
          r2 = T1H - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (ALPHAH * T1L + ALPHAL * T1H)
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T4H,T4L,ALPHAH,ALPHAL,T2H,T2L)
          T4H = ALPHAH * T2H
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*T2H
          r1 = u1 - (u1-T2H)
          r2 = T2H - r1
          T4L = ((p1*r1-T4H) + p1*r2 + p2*r1) + p2*r2
          T4L = T4L + (ALPHAH * T2L + ALPHAL * T2H)
          s1 = T4H + T4L
          T4L = T4L - (s1 - T4H)
          T4H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_add(CH(I,J),CL(I,J),T3H,T3L,T4H,T4L)
          p1 = T3H + T4H
          p2 = p1 - T3H
          CL(I,J) = (T3H - (p1 - p2)) + (T4H - p2)
          CH(I,J) = p1
          CL(I,J) = CL(I,J) + T3L + T4L
          s1 = CH(I,J) + CL(I,J)
          CL(I,J) = CL(I,J) - (s1 - CH(I,J))
          CH(I,J) = s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
                      ELSE
!                         C(I,J) = BETA*C(I,J) + ALPHA*TEMP1 +
!    +                             ALPHA*TEMP2
!         call dd_mul(T3H,T3L,ALPHAH,ALPHAL,T1H,T1L)
          T3H = ALPHAH * T1H
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*T1H
          r1 = u1 - (u1-T1H)
          r2 = T1H - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (ALPHAH * T1L + ALPHAL * T1H)
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T4H,T4L,ALPHAH,ALPHAL,T2H,T2L)
          T4H = ALPHAH * T2H
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*T2H
          r1 = u1 - (u1-T2H)
          r2 = T2H - r1
          T4L = ((p1*r1-T4H) + p1*r2 + p2*r1) + p2*r2
          T4L = T4L + (ALPHAH * T2L + ALPHAL * T2H)
          s1 = T4H + T4L
          T4L = T4L - (s1 - T4H)
          T4H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfmul(CH(I,J),CL(I,J),BETAH,BETAL)
          q1 = CH(I,J)*BETAH
          u1 = 134217729.0*CH(I,J)
          p1 = u1 - (u1-CH(I,J))
          p2 = CH(I,J) - p1
          u1 = 134217729.0*BETAH
          r1 = u1 - (u1-BETAH)
          r2 = BETAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + BETAL * CH(I,J)
          q2 = q2 + BETAH * CL(I,J)
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!         call dd_selfadd(T3H,T3L,T4H,T4L)
          p1 = T3H + T4H
          p2 = p1 - T3H
          q2 = (T3H - (p1 - p2)) + (T4H - p2)
          q1 = p1
          q2 = q2 + T3L
          q2 = q2 + T4L
          T3H = q1 + q2
          T3L = q2 - (T3H - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!         call dd_selfadd(CH(I,J),CL(I,J),T3H,T3L)
          p1 = CH(I,J) + T3H
          p2 = p1 - CH(I,J)
          q2 = (CH(I,J) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + CL(I,J)
          q2 = q2 + T3L
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
                      END IF
  200             CONTINUE
  210         CONTINUE
          ELSE
              DO 240 J = 1,N
                  DO 230 I = J,N
!                     TEMP1 = ZERO
                      T1H = ZERO
                      T1L = ZERO
!                     TEMP2 = ZERO
                      T2H = ZERO
                      T2L = ZERO
!$omp parallel
!$omp do
!$omp& private(L,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T3H,T3L,T4H,T4L)
                      DO 220 L = 1,K
!                         TEMP1 = TEMP1 + A(L,I)*B(L,J)
!         call dd_mul(T3H,T3L,AH(L,I),AL(L,I),BH(L,J),BL(L,J))
          T3H = AH(L,I) * BH(L,J)
          u1 = 134217729.0*AH(L,I)
          p1 = u1 - (u1-AH(L,I))
          p2 = AH(L,I) - p1
          u1 = 134217729.0*BH(L,J)
          r1 = u1 - (u1-BH(L,J))
          r2 = BH(L,J) - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (AH(L,I) * BL(L,J) + AL(L,I) * BH(L,J))
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T1H,T1L,T3H,T3L)
!$omp critical (name0)
          p1 = T1H + T3H
          p2 = p1 - T1H
          q2 = (T1H - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + T1L
          q2 = q2 + T3L
          T1H = q1 + q2
          T1L = q2 - (T1H - q1)
!$omp end critical (name0)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                         TEMP2 = TEMP2 + B(L,I)*A(L,J)
!         call dd_mul(T4H,T4L,BH(L,I),BL(L,I),AH(L,J),AL(L,J))
          T4H = BH(L,I) * AH(L,J)
          u1 = 134217729.0*BH(L,I)
          p1 = u1 - (u1-BH(L,I))
          p2 = BH(L,I) - p1
          u1 = 134217729.0*AH(L,J)
          r1 = u1 - (u1-AH(L,J))
          r2 = AH(L,J) - r1
          T4L = ((p1*r1-T4H) + p1*r2 + p2*r1) + p2*r2
          T4L = T4L + (BH(L,I) * AL(L,J) + BL(L,I) * AH(L,J))
          s1 = T4H + T4L
          T4L = T4L - (s1 - T4H)
          T4H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T2H,T2L,T4H,T4L)
!$omp critical (name0)
          p1 = T2H + T4H
          p2 = p1 - T2H
          q2 = (T2H - (p1 - p2)) + (T4H - p2)
          q1 = p1
          q2 = q2 + T2L
          q2 = q2 + T4L
          T2H = q1 + q2
          T2L = q2 - (T2H - q1)
!$omp end critical (name0)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
  220                 CONTINUE
!$omp end do
!$omp end parallel
!                     IF (BETA.EQ.ZERO) THEN
                      IF (LBETAZ) THEN
!                         C(I,J) = ALPHA*TEMP1 + ALPHA*TEMP2
!         call dd_mul(T3H,T3L,ALPHAH,ALPHAL,T1H,T1L)
          T3H = ALPHAH * T1H
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*T1H
          r1 = u1 - (u1-T1H)
          r2 = T1H - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (ALPHAH * T1L + ALPHAL * T1H)
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T4H,T4L,ALPHAH,ALPHAL,T2H,T2L)
          T4H = ALPHAH * T2H
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*T2H
          r1 = u1 - (u1-T2H)
          r2 = T2H - r1
          T4L = ((p1*r1-T4H) + p1*r2 + p2*r1) + p2*r2
          T4L = T4L + (ALPHAH * T2L + ALPHAL * T2H)
          s1 = T4H + T4L
          T4L = T4L - (s1 - T4H)
          T4H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_add(CH(I,J),CL(I,J),T3H,T3L,T4H,T4L)
          p1 = T3H + T4H
          p2 = p1 - T3H
          CL(I,J) = (T3H - (p1 - p2)) + (T4H - p2)
          CH(I,J) = p1
          CL(I,J) = CL(I,J) + T3L + T4L
          s1 = CH(I,J) + CL(I,J)
          CL(I,J) = CL(I,J) - (s1 - CH(I,J))
          CH(I,J) = s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
                      ELSE
!                         C(I,J) = BETA*C(I,J) + ALPHA*TEMP1 +
!    +                             ALPHA*TEMP2
!         call dd_mul(T3H,T3L,ALPHAH,ALPHAL,T1H,T1L)
          T3H = ALPHAH * T1H
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*T1H
          r1 = u1 - (u1-T1H)
          r2 = T1H - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (ALPHAH * T1L + ALPHAL * T1H)
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T4H,T4L,ALPHAH,ALPHAL,T2H,T2L)
          T4H = ALPHAH * T2H
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*T2H
          r1 = u1 - (u1-T2H)
          r2 = T2H - r1
          T4L = ((p1*r1-T4H) + p1*r2 + p2*r1) + p2*r2
          T4L = T4L + (ALPHAH * T2L + ALPHAL * T2H)
          s1 = T4H + T4L
          T4L = T4L - (s1 - T4H)
          T4H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfmul(CH(I,J),CL(I,J),BETAH,BETAL)
          q1 = CH(I,J)*BETAH
          u1 = 134217729.0*CH(I,J)
          p1 = u1 - (u1-CH(I,J))
          p2 = CH(I,J) - p1
          u1 = 134217729.0*BETAH
          r1 = u1 - (u1-BETAH)
          r2 = BETAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + BETAL * CH(I,J)
          q2 = q2 + BETAH * CL(I,J)
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!         call dd_selfadd(T3H,T3L,T4H,T4L)
          p1 = T3H + T4H
          p2 = p1 - T3H
          q2 = (T3H - (p1 - p2)) + (T4H - p2)
          q1 = p1
          q2 = q2 + T3L
          q2 = q2 + T4L
          T3H = q1 + q2
          T3L = q2 - (T3H - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!         call dd_selfadd(CH(I,J),CL(I,J),T3H,T3L)
          p1 = CH(I,J) + T3H
          p2 = p1 - CH(I,J)
          q2 = (CH(I,J) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + CL(I,J)
          q2 = q2 + T3L
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
                      END IF
  230             CONTINUE
  240         CONTINUE
          END IF
      END IF
*
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = DDSYR2K"
      WRITE(6,1005) SUBNAME,"DIM N     = ",N
      WRITE(6,1005) SUBNAME,"DIM K     = ",K
      WRITE(6,'(2A)') "UPLO      = ",UPLO
      WRITE(6,'(2A)') "TRANS     = ",TRANS
#include "fout3.h"
#include "fformat.h"
#endif
      RETURN
*
*     End of DDSYR2K.
*
      END
