!     SUBROUTINE ZGERC(M,N,ALPHA,X,INCX,Y,INCY,A,LDA)
      subroutine zzgerc(m, n, alphah, alphal, xh, xl, incx,
     +                   yh, yl, incy, ah, al, lda)
      use bailey
      implicit none
*     .. Scalar Arguments ..
!     DOUBLE COMPLEX ALPHA
      double complex alphah, alphal
      INTEGER INCX,INCY,LDA,M,N
*     ..
*     .. Array Arguments ..
!     DOUBLE COMPLEX A(LDA,*),X(*),Y(*)
      double complex ah(lda,*), al(lda,*)
      double complex xh(*), xl(*)
      double complex yh(*), yl(*)
*     ..
*
*  Purpose
*  =======
*
*  ZGERC  performs the rank 1 operation
*
*     A := alpha*x*conjg( y' ) + A,
*
*  where alpha is a scalar, x is an m element vector, y is an n element
*  vector and A is an m by n matrix.
*
*  Arguments
*  ==========
*
*  M      - INTEGER.
*           On entry, M specifies the number of rows of the matrix A.
*           M must be at least zero.
*           Unchanged on exit.
*
*  N      - INTEGER.
*           On entry, N specifies the number of columns of the matrix A.
*           N must be at least zero.
*           Unchanged on exit.
*
*  ALPHA  - COMPLEX*16      .
*           On entry, ALPHA specifies the scalar alpha.
*           Unchanged on exit.
*
*  X      - COMPLEX*16       array of dimension at least
*           ( 1 + ( m - 1 )*abs( INCX ) ).
*           Before entry, the incremented array X must contain the m
*           element vector x.
*           Unchanged on exit.
*
*  INCX   - INTEGER.
*           On entry, INCX specifies the increment for the elements of
*           X. INCX must not be zero.
*           Unchanged on exit.
*
*  Y      - COMPLEX*16       array of dimension at least
*           ( 1 + ( n - 1 )*abs( INCY ) ).
*           Before entry, the incremented array Y must contain the n
*           element vector y.
*           Unchanged on exit.
*
*  INCY   - INTEGER.
*           On entry, INCY specifies the increment for the elements of
*           Y. INCY must not be zero.
*           Unchanged on exit.
*
*  A      - COMPLEX*16       array of DIMENSION ( LDA, n ).
*           Before entry, the leading m by n part of the array A must
*           contain the matrix of coefficients. On exit, A is
*           overwritten by the updated matrix.
*
*  LDA    - INTEGER.
*           On entry, LDA specifies the first dimension of A as declared
*           in the calling (sub) program. LDA must be at least
*           max( 1, m ).
*           Unchanged on exit.
*
*
*  Level 2 Blas routine.
*
*  -- Written on 22-October-1986.
*     Jack Dongarra, Argonne National Lab.
*     Jeremy Du Croz, Nag Central Office.
*     Sven Hammarling, Nag Central Office.
*     Richard Hanson, Sandia National Labs.
*
*
!  double double modefication (f.y.)
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
*     .. Parameters ..
!     DOUBLE COMPLEX ZERO
      double complex zeroh, zerol
!     PARAMETER (ZERO= (0.0D+0,0.0D+0))
*     ..
*     .. Local Scalars ..
!     DOUBLE COMPLEX TEMP
      double complex temph, templ
      double complex ddmxh, ddmxl
      double complex ddaxh, ddaxl
!     tmp vars
      double complex x_ch, x_cl
      double complex x_p1, x_p2
      double complex x_q1, x_q2
      double complex x_r1, x_r2
      double complex x_s1, x_s2
      double complex x_t1, x_t2
      double complex x_u1, x_u2
      double complex x_x1, x_x2
      double complex x_y1, x_y2
      double complex x_z1, x_z2
      double complex x_x, x_ax
      double complex x_t1h, x_t1l
      double complex x_t2h, x_t2l
      INTEGER I,INFO,IX,J,JY,KX
*     ..
*     .. External Subroutines ..
      EXTERNAL XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC DCONJG,MAX
*     ..
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "zzgerc"
#endif
*
*     Test the input parameters.
*
      zeroh = 0.0d+0
      zerol = 0.0d+0
      INFO = 0
      IF (M.LT.0) THEN
          INFO = 1
      ELSE IF (N.LT.0) THEN
          INFO = 2
      ELSE IF (INCX.EQ.0) THEN
          INFO = 5
      ELSE IF (INCY.EQ.0) THEN
          INFO = 7
      ELSE IF (LDA.LT.MAX(1,M)) THEN
          INFO = 9
      END IF
      IF (INFO.NE.0) THEN
          CALL XERBLA('ZGERC ',INFO)
          RETURN
      END IF
*
*     Quick return if possible.
*
!     IF ((M.EQ.0) .OR. (N.EQ.0) .OR. (ALPHA.EQ.ZERO)) RETURN
      IF ((M.EQ.0) .OR. (N.EQ.0) .OR.
!       (ALPHA .eq. ZERO)
     + ((ALPHAh .eq. ZEROh) .and.
     +  (ALPHAl .eq. ZEROl))
     +) goto 9999
*
*     Start the operations. In this version the elements of A are
*     accessed sequentially with one pass through A.
*
      IF (INCY.GT.0) THEN
          JY = 1
      ELSE
          JY = 1 - (N-1)*INCY
      END IF
      IF (INCX.EQ.1) THEN
          DO 20 J = 1,N
!             IF (Y(JY).NE.ZERO) THEN
              if(
!               (y(jy) .ne. zero)
     +         ((yh(jy) .ne. zeroh) .or.
     +          (yl(jy) .ne. zerol))
     +        ) then
!                 TEMP = ALPHA*DCONJG(Y(JY))
                  ddaxh = dconjg(yh(jy))
                  ddaxl = dconjg(yl(jy))
!<<<              temp = alpha * ddax
                  call bmul( alphah, alphal, ddaxh, ddaxl,
     +             temph, templ )
!                  temph = alphah * ddaxh
!                  x_u1 = 134217729.0 * alphah
!                  x_p1 = x_u1
!     +                     - (x_u1 - alphah)
!                  x_p2 = alphah - x_p1
!                  x_u1 = 134217729.0 * ddaxh
!                  x_r1 = x_u1
!     +                     - (x_u1 - ddaxh)
!                  x_r2 = ddaxh - x_r1
!                  templ = ((x_p1*x_r1
!     +                    - temph)
!     +                    + x_p1 * x_r2
!     +                    + x_p2 * x_r1)
!     +                    + x_p2 * x_r2
!                  templ = templ
!     +                  + (alphah*ddaxl
!     +                  + alphal*ddaxh)
!                  x_s1 = temph + templ
!                  templ = templ - (x_s1 - temph)
!                  temph = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              temp = alpha * ddax
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
                  DO 10 I = 1,M
!                     A(I,J) = A(I,J) + X(I)*TEMP
!<<<                  ddmx = x(i) * temp
                      call bmul( xh(i), xl(i), temph, templ,
     +                  ddmxh, ddmxl )
!                      ddmxh = xh(i) * temph
!                      x_u1 = 134217729.0 * xh(i)
!                      x_p1 = x_u1
!     +                         - (x_u1 - xh(i))
!                      x_p2 = xh(i) - x_p1
!                      x_u1 = 134217729.0 * temph
!                      x_r1 = x_u1
!     +                         - (x_u1 - temph)
!                      x_r2 = temph - x_r1
!                      ddmxl = ((x_p1*x_r1
!     +                        - ddmxh)
!     +                        + x_p1 * x_r2
!     +                        + x_p2 * x_r1)
!     +                        + x_p2 * x_r2
!                      ddmxl = ddmxl
!     +                      + (xh(i)*templ
!     +                      + xl(i)*temph)
!                      x_s1 = ddmxh + ddmxl
!                      ddmxl = ddmxl - (x_s1 - ddmxh)
!                      ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = x(i) * temp
!<<<                  a(i,j) = a(i,j) + ddmx
!                      call bsum( ah(i,j), al(i,j), ddmxh, ddmxl,
!     +                  x_q1, x_q2 )
                      call bsum( ah(i,j), al(i,j), ddmxh, ddmxl,
     +                  ah(i,j), al(i,j) )
!                      ah(i,j) = x_q1
!                      al(i,j) = x_q2
!                      x_p1 = ah(i,j) + ddmxh
!                      x_p2 = x_p1 - ah(i,j)
!                      x_q1 = x_p1
!                      x_q2 = (ah(i,j) - (x_p1 - x_p2))
!     +                         + (ddmxh - x_p2)
!                      x_q2 = x_q2 + al(i,j) + ddmxl
!                      ah(i,j) = x_q1 + x_q2
!                      al(i,j) = x_q2 - (ah(i,j) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  a(i,j) = a(i,j) + ddmx
   10             CONTINUE
!$omp end do
!$omp end parallel
              END IF
              JY = JY + INCY
   20     CONTINUE
      ELSE
          IF (INCX.GT.0) THEN
              KX = 1
          ELSE
              KX = 1 - (M-1)*INCX
          END IF
          DO 40 J = 1,N
!             IF (Y(JY).NE.ZERO) THEN
              if(
!               (y(jy) .ne. 0.0)
     +         ((yh(jy) .ne. 0.0d+0) .or.
     +          (yl(jy) .ne. 0.0d+0))
     +        ) then
!                 TEMP = ALPHA*DCONJG(Y(JY))
                  ddaxh = dconjg(yh(jy))
                  ddaxl = dconjg(yl(jy))
!<<<              temp = alpha * ddax
                  call bmul( alphah, alphal, ddaxh, ddaxl,
     +              temph, templ )
!                  temph = alphah * ddaxh
!                  x_u1 = 134217729.0 * alphah
!                  x_p1 = x_u1
!     +                     - (x_u1 - alphah)
!                  x_p2 = alphah - x_p1
!                  x_u1 = 134217729.0 * ddaxh
!                  x_r1 = x_u1
!     +                     - (x_u1 - ddaxh)
!                  x_r2 = ddaxh - x_r1
!                  templ = ((x_p1*x_r1
!     +                    - temph)
!     +                    + x_p1 * x_r2
!     +                    + x_p2 * x_r1)
!     +                    + x_p2 * x_r2
!                  templ = templ
!     +                  + (alphah*ddaxl
!     +                  + alphal*ddaxh)
!                  x_s1 = temph + templ
!                  templ = templ - (x_s1 - temph)
!                  temph = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              temp = alpha * ddax
                  IX = KX
!$omp parallel
!$omp do
!$omp+ private(I,ix)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
                  DO 30 I = 1,M
                      ix = kx + (i - 1)*incx
!                     A(I,J) = A(I,J) + X(IX)*TEMP
!<<<                  ddmx = x(ix) * temp
                      call bmul( xh(ix), xl(ix), temph, templ,
     +                  ddmxh, ddmxl )
!                      ddmxh = xh(ix) * temph
!                      x_u1 = 134217729.0 * xh(ix)
!                      x_p1 = x_u1
!     +                         - (x_u1 - xh(ix))
!                      x_p2 = xh(ix) - x_p1
!                      x_u1 = 134217729.0 * temph
!                      x_r1 = x_u1
!     +                         - (x_u1 - temph)
!                      x_r2 = temph - x_r1
!                      ddmxl = ((x_p1*x_r1
!     +                        - ddmxh)
!     +                        + x_p1 * x_r2
!     +                        + x_p2 * x_r1)
!     +                        + x_p2 * x_r2
!                      ddmxl = ddmxl
!     +                      + (xh(ix)*templ
!     +                      + xl(ix)*temph)
!                      x_s1 = ddmxh + ddmxl
!                      ddmxl = ddmxl - (x_s1 - ddmxh)
!                      ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = x(ix) * temp
!<<<                  a(i,j) = ddmx + a(i,j)
!                      x_p1 = ah(i,j)
!                      x_p2 = al(i,j)
                      call bsum( ah(i,j), al(i,j), ddmxh, ddmxl,
     +                  ah(i,j), al(i,j) )
!                      ah(i,j) = x_q1
!                      al(i,j) = x_q2
!                      x_p1 = ddmxh + ah(i,j)
!                      x_p2 = x_p1 - ddmxh
!                      x_q1 = x_p1
!                      x_q2 = (ddmxh - (x_p1 - x_p2))
!     +                         + (ah(i,j) - x_p2)
!                      x_q2 = x_q2 + ddmxl + al(i,j)
!                      ah(i,j) = x_q1 + x_q2
!                      al(i,j) = x_q2 - (ah(i,j) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  a(i,j) = ddmx + a(i,j)
!!                    IX = IX + INCX
   30             CONTINUE
!$omp end do
!$omp end parallel
              END IF
              JY = JY + INCY
   40     CONTINUE
      END IF
*
 9999 continue
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = zzgerc"
#include "fout2.h"
#include "fformat.h"
#endif
      RETURN
*
*     End of ZGERC .
*
      END
