!     SUBROUTINE ZHERK(UPLO,TRANS,N,K,ALPHA,A,LDA,BETA,C,LDC)
      subroutine zzherk(uplo, trans, n, k,
     +                   alphah, alphal, ah, al, lda,
     +                   betah, betal, ch, cl, ldc)
      use bailey
      implicit none
*     .. Scalar Arguments ..
!     DOUBLE PRECISION ALPHA,BETA
      double precision alphah, alphal
      double precision betah, betal
      INTEGER K,LDA,LDC,N
      CHARACTER TRANS,UPLO
*     ..
*     .. Array Arguments ..
!     DOUBLE COMPLEX A(LDA,*),C(LDC,*)
      double complex ah(lda,*), al(lda,*)
      double complex ch(ldc,*), cl(ldc,*)
*     ..
*
*  Purpose
*  =======
*
*  ZHERK  performs one of the hermitian rank k operations
*
*     C := alpha*A*conjg( A' ) + beta*C,
*
*  or
*
*     C := alpha*conjg( A' )*A + beta*C,
*
*  where  alpha and beta  are  real scalars,  C is an  n by n  hermitian
*  matrix and  A  is an  n by k  matrix in the  first case and a  k by n
*  matrix in the second case.
*
*  Arguments
*  ==========
*
*  UPLO   - CHARACTER*1.
*           On  entry,   UPLO  specifies  whether  the  upper  or  lower
*           triangular  part  of the  array  C  is to be  referenced  as
*           follows:
*
*              UPLO = 'U' or 'u'   Only the  upper triangular part of  C
*                                  is to be referenced.
*
*              UPLO = 'L' or 'l'   Only the  lower triangular part of  C
*                                  is to be referenced.
*
*           Unchanged on exit.
*
*  TRANS  - CHARACTER*1.
*           On entry,  TRANS  specifies the operation to be performed as
*           follows:
*
*              TRANS = 'N' or 'n'   C := alpha*A*conjg( A' ) + beta*C.
*
*              TRANS = 'C' or 'c'   C := alpha*conjg( A' )*A + beta*C.
*
*           Unchanged on exit.
*
*  N      - INTEGER.
*           On entry,  N specifies the order of the matrix C.  N must be
*           at least zero.
*           Unchanged on exit.
*
*  K      - INTEGER.
*           On entry with  TRANS = 'N' or 'n',  K  specifies  the number
*           of  columns   of  the   matrix   A,   and  on   entry   with
*           TRANS = 'C' or 'c',  K  specifies  the number of rows of the
*           matrix A.  K must be at least zero.
*           Unchanged on exit.
*
*  ALPHA  - DOUBLE PRECISION            .
*           On entry, ALPHA specifies the scalar alpha.
*           Unchanged on exit.
*
*  A      - COMPLEX*16       array of DIMENSION ( LDA, ka ), where ka is
*           k  when  TRANS = 'N' or 'n',  and is  n  otherwise.
*           Before entry with  TRANS = 'N' or 'n',  the  leading  n by k
*           part of the array  A  must contain the matrix  A,  otherwise
*           the leading  k by n  part of the array  A  must contain  the
*           matrix A.
*           Unchanged on exit.
*
*  LDA    - INTEGER.
*           On entry, LDA specifies the first dimension of A as declared
*           in  the  calling  (sub)  program.   When  TRANS = 'N' or 'n'
*           then  LDA must be at least  max( 1, n ), otherwise  LDA must
*           be at least  max( 1, k ).
*           Unchanged on exit.
*
*  BETA   - DOUBLE PRECISION.
*           On entry, BETA specifies the scalar beta.
*           Unchanged on exit.
*
*  C      - COMPLEX*16          array of DIMENSION ( LDC, n ).
*           Before entry  with  UPLO = 'U' or 'u',  the leading  n by n
*           upper triangular part of the array C must contain the upper
*           triangular part  of the  hermitian matrix  and the strictly
*           lower triangular part of C is not referenced.  On exit, the
*           upper triangular part of the array  C is overwritten by the
*           upper triangular part of the updated matrix.
*           Before entry  with  UPLO = 'L' or 'l',  the leading  n by n
*           lower triangular part of the array C must contain the lower
*           triangular part  of the  hermitian matrix  and the strictly
*           upper triangular part of C is not referenced.  On exit, the
*           lower triangular part of the array  C is overwritten by the
*           lower triangular part of the updated matrix.
*           Note that the imaginary parts of the diagonal elements need
*           not be set,  they are assumed to be zero,  and on exit they
*           are set to zero.
*
*  LDC    - INTEGER.
*           On entry, LDC specifies the first dimension of C as declared
*           in  the  calling  (sub)  program.   LDC  must  be  at  least
*           max( 1, n ).
*           Unchanged on exit.
*
*
*  Level 3 Blas routine.
*
*  -- Written on 8-February-1989.
*     Jack Dongarra, Argonne National Laboratory.
*     Iain Duff, AERE Harwell.
*     Jeremy Du Croz, Numerical Algorithms Group Ltd.
*     Sven Hammarling, Numerical Algorithms Group Ltd.
*
*  -- Modified 8-Nov-93 to set C(J,J) to DBLE( C(J,J) ) when BETA = 1.
*     Ed Anderson, Cray Research Inc.
*
*
!  double double modefication
*     .. External Functions ..
      LOGICAL LSAME
      EXTERNAL LSAME
*     ..
*     .. External Subroutines ..
      EXTERNAL XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC DBLE,DCMPLX,DCONJG,MAX
*     ..
*     .. Local Scalars ..
!     DOUBLE COMPLEX TEMP
      double complex temph, templ
!     DOUBLE PRECISION RTEMP
      double precision rtemph, rtempl
      INTEGER I,INFO,J,L,NROWA
      LOGICAL UPPER
*     ..
*     .. Parameters ..
!     DOUBLE PRECISION ONE,ZERO
!     PARAMETER (ONE=1.0D+0,ZERO=0.0D+0)
      double precision oneh, onel
      double precision zeroh, zerol
      double complex ddmxh, ddmxl
      double complex ddaxh, ddaxl
      double complex ddxxh, ddxxl
      double complex zeroxh, zeroxl
!     tmp vars
      double precision t_ch, t_cl
      double precision t_p1, t_p2
      double precision t_q1, t_q2
      double precision t_r1, t_r2
      double precision t_s1, t_s2
      double precision t_t1, t_t2
      double precision t_u1, t_u2
      double precision t_x1, t_x2
      double precision t_y1, t_y2
      double precision t_z1, t_z2
      double precision t_x, t_ax
      double precision t_t1h, t_t1l
      double precision t_t2h, t_t2l
!     tmp vars
      double complex x_ch, x_cl
      double complex x_p1, x_p2
      double complex x_q1, x_q2
      double complex x_r1, x_r2
      double complex x_s1, x_s2
      double complex x_t1, x_t2
      double complex x_u1, x_u2
      double complex x_x1, x_x2
      double complex x_y1, x_y2
      double complex x_z1, x_z2
      double complex x_x, x_ax
      double complex x_t1h, x_t1l
      double complex x_t2h, x_t2l
      double precision ddmh, ddml
      double precision ddah, ddal
      double precision ddxh, ddxl
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "zzherk"
#endif
      zeroh = 0.0d+0
      zerol = 0.0d+0
      oneh = 1.0d+0
      onel = 0.0d+0
*     ..
      zeroxh = dcmplx(zeroh)
      zeroxl = dcmplx(zerol)
*
*     Test the input parameters.
*
      IF (LSAME(TRANS,'N')) THEN
          NROWA = N
      ELSE
          NROWA = K
      END IF
      UPPER = LSAME(UPLO,'U')
*
      INFO = 0
      IF ((.NOT.UPPER) .AND. (.NOT.LSAME(UPLO,'L'))) THEN
          INFO = 1
      ELSE IF ((.NOT.LSAME(TRANS,'N')) .AND.
     +         (.NOT.LSAME(TRANS,'C'))) THEN
          INFO = 2
      ELSE IF (N.LT.0) THEN
          INFO = 3
      ELSE IF (K.LT.0) THEN
          INFO = 4
      ELSE IF (LDA.LT.MAX(1,NROWA)) THEN
          INFO = 7
      ELSE IF (LDC.LT.MAX(1,N)) THEN
          INFO = 10
      END IF
      IF (INFO.NE.0) THEN
          CALL XERBLA('ZHERK ',INFO)
          goto 9999
!         RETURN
      END IF
*
*     Quick return if possible.
*
!     IF ((N.EQ.0) .OR. (((ALPHA.EQ.ZERO).OR.
!    +    (K.EQ.0)).AND. (BETA.EQ.ONE))) RETURN
      if((n.eq.0) .or. ((
!       (alpha .eq. zero)
     + ((alphah .eq. zeroh) .and.
     +  (alphal .eq. zerol))
     +  .or. (K.EQ.0)).and.
!       (beta .eq. one)
     + ((betah .eq. oneh) .and.
     +  (betal .eq. onel))
     +)) goto 9999
*
*     And when  alpha.eq.zero.
*
!     IF (ALPHA.EQ.ZERO) THEN
      if(
!       (alpha .eq. zero)
     + ((alphah .eq. zeroh) .and.
     +  (alphal .eq. zerol))
     +) then
          IF (UPPER) THEN
!             IF (BETA.EQ.ZERO) THEN
              if(
!               (beta .eq. zero)
     +         ((betah .eq. zeroh) .and.
     +          (betal .eq. zerol))
     +        ) then
                  DO 20 J = 1,N
!$omp parallel
!$omp do
!$omp+ private(I)
                      DO 10 I = 1,J
!                         C(I,J) = ZERO
                          ch(i,j) = zeroh
                          cl(i,j) = zerol
   10                 CONTINUE
!$omp end do
!$omp end parallel
   20             CONTINUE
              ELSE
                  DO 40 J = 1,N
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
                      DO 30 I = 1,J - 1
!                         C(I,J) = BETA*C(I,J)
!<<<                      c(i,j) = beta * c(i,j)
c                         x_ch = betah * ch(i,j)
c                         x_u1 = 134217729.0 * betah
c                         x_p1 = x_u1
c    +                             - (x_u1 - betah)
c                         x_p2 = betah - x_p1
c                         x_u1 = 134217729.0 * ch(i,j)
c                         x_r1 = x_u1
c    +                             - (x_u1 - ch(i,j))
c                         x_r2 = ch(i,j) - x_r1
c                         x_cl = ((x_p1*x_r1
c    +                             - x_ch)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         x_cl = x_cl
c    +                             + (betah*cl(i,j)
c    +                             +  betal*ch(i,j))
c                         x_s1 = x_ch + x_cl
c                         cl(i,j) = x_cl - (x_s1 - x_ch)
c                         ch(i,j) = x_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                      c(i,j) = beta * c(i,j)
                          call bmul(betah, betal, ch(i,j), cl(i,j),
     +                        ch(i,j), cl(i,j))
   30                 CONTINUE
!$omp end do
!$omp end parallel
!                     C(J,J) = BETA*DBLE(C(J,J))
                      ddah = dble(ch(j,j))
                      ddal = dble(cl(j,j))
!!!#ddb                 mul c(j,j) beta dda
!<<<                  ddm = beta * dda
c                     ddmh = betah * ddah
c                     t_u1 = 134217729.0 * betah
c                     t_p1 = t_u1
c    +                         - (t_u1 - betah)
c                     t_p2 = betah - t_p1
c                     t_u1 = 134217729.0 * ddah
c                     t_r1 = t_u1
c    +                         - (t_u1 - ddah)
c                     t_r2 = ddah - t_r1
c                     ddml = ((t_p1*t_r1
c    +                        - ddmh)
c    +                        + t_p1 * t_r2
c    +                        + t_p2 * t_r1)
c    +                        + t_p2 * t_r2
c                     ddml = ddml
c    +                      + (betah*ddal
c    +                      + betal*ddah)
c                     t_s1 = ddmh + ddml
c                     ddml = ddml - (t_s1 - ddmh)
c                     ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddm = beta * dda
                      call bmul(betah, betal, ddah, ddal,
     +                    ddmh, ddml)
                      ch(j,j) = ddmh
                      cl(j,j) = ddml
   40             CONTINUE
              END IF
          ELSE
!             IF (BETA.EQ.ZERO) THEN
              if(
!                       (beta .eq. zero)
     +                 ((betah .eq. zeroh) .and.
     +                  (betal .eq. zerol))
     +        ) then
                  DO 60 J = 1,N
!$omp parallel
!$omp do
!$omp+ private(I)
                      DO 50 I = J,N
!                         C(I,J) = ZERO
                          ch(i,j) = zeroh
                          cl(i,j) = zerol
   50                 CONTINUE
!$omp end do
!$omp end parallel
   60             CONTINUE
              ELSE
                  DO 80 J = 1,N
!                     C(J,J) = BETA*DBLE(C(J,J))
                      ddah = dble(ch(j,j))
                      ddal = dble(cl(j,j))
!!!#ddb                 mul c(j,j) beta dda
!<<<                  ddm = beta * dda
c                     ddmh = betah * ddah
c                     t_u1 = 134217729.0 * betah
c                     t_p1 = t_u1
c    +                         - (t_u1 - betah)
c                     t_p2 = betah - t_p1
c                     t_u1 = 134217729.0 * ddah
c                     t_r1 = t_u1
c    +                         - (t_u1 - ddah)
c                     t_r2 = ddah - t_r1
c                     ddml = ((t_p1*t_r1
c    +                        - ddmh)
c    +                        + t_p1 * t_r2
c    +                        + t_p2 * t_r1)
c    +                        + t_p2 * t_r2
c                     ddml = ddml
c    +                      + (betah*ddal
c    +                      + betal*ddah)
c                     t_s1 = ddmh + ddml
c                     ddml = ddml - (t_s1 - ddmh)
c                     ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddm = beta * dda
                      call bmul(betah, betal, ddah, ddal,
     +                    ddmh, ddml)
                      ch(j,j) = ddmh
                      cl(j,j) = ddml
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
                      DO 70 I = J + 1,N
!                         C(I,J) = BETA*C(I,J)
!<<<                      c(i,j) = beta * c(i,j)
c                         x_ch = betah * ch(i,j)
c                         x_u1 = 134217729.0 * betah
c                         x_p1 = x_u1
c    +                             - (x_u1 - betah)
c                         x_p2 = betah - x_p1
c                         x_u1 = 134217729.0 * ch(i,j)
c                         x_r1 = x_u1
c    +                             - (x_u1 - ch(i,j))
c                         x_r2 = ch(i,j) - x_r1
c                         x_cl = ((x_p1*x_r1
c    +                             - x_ch)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         x_cl = x_cl
c    +                             + (betah*cl(i,j)
c    +                             +  betal*ch(i,j))
c                         x_s1 = x_ch + x_cl
c                         cl(i,j) = x_cl - (x_s1 - x_ch)
c                         ch(i,j) = x_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                      c(i,j) = beta * c(i,j)
                          call bmul(betah, betal, ch(i,j), cl(i,j),
     +                        ch(i,j), cl(i,j))
   70                 CONTINUE
!$omp end do
!$omp end parallel
   80             CONTINUE
              END IF
          END IF
          RETURN
      END IF
*
*     Start the operations.
*
      IF (LSAME(TRANS,'N')) THEN
*
*        Form  C := alpha*A*conjg( A' ) + beta*C.
*
          IF (UPPER) THEN
              DO 130 J = 1,N
!                 IF (BETA.EQ.ZERO) THEN
                  if(
!                   (beta .eq. zero)
     +             ((betah .eq. zeroh) .and.
     +              (betal .eq. zerol))
     +            ) then
!$omp parallel
!$omp do
!$omp+ private(I)
                      DO 90 I = 1,J
!                         C(I,J) = ZERO
                          ch(i,j) = zeroh
                          cl(i,j) = zerol
   90                 CONTINUE
!$omp end do
!$omp end parallel
!                 ELSE IF (BETA.NE.ONE) THEN
                  else if(
!                   (beta .ne. one)
     +             ((betah .ne. oneh) .or.
     +              (betal .ne. onel))
     +            ) then
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
                      DO 100 I = 1,J - 1
!                         C(I,J) = BETA*C(I,J)
!<<<                      c(i,j) = beta * c(i,j)
c                         x_ch = betah * ch(i,j)
c                         x_u1 = 134217729.0 * betah
c                         x_p1 = x_u1
c    +                             - (x_u1 - betah)
c                         x_p2 = betah - x_p1
c                         x_u1 = 134217729.0 * ch(i,j)
c                         x_r1 = x_u1
c    +                             - (x_u1 - ch(i,j))
c                         x_r2 = ch(i,j) - x_r1
c                         x_cl = ((x_p1*x_r1
c    +                             - x_ch)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         x_cl = x_cl
c    +                             + (betah*cl(i,j)
c    +                             +  betal*ch(i,j))
c                         x_s1 = x_ch + x_cl
c                         cl(i,j) = x_cl - (x_s1 - x_ch)
c                         ch(i,j) = x_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                      c(i,j) = beta * c(i,j)
                          call bmul(betah, betal, ch(i,j), cl(i,j),
     +                        ch(i,j), cl(i,j))
  100                 CONTINUE
!$omp end do
!$omp end parallel
!                     C(J,J) = BETA*DBLE(C(J,J))
                      ddah = dble(ch(j,j))
                      ddal = dble(cl(j,j))
!!!#ddb                 mul c(j,j) beta dda
!<<<                  ddm = beta * dda
c                     ddmh = betah * ddah
c                     t_u1 = 134217729.0 * betah
c                     t_p1 = t_u1
c    +                         - (t_u1 - betah)
c                     t_p2 = betah - t_p1
c                     t_u1 = 134217729.0 * ddah
c                     t_r1 = t_u1
c    +                         - (t_u1 - ddah)
c                     t_r2 = ddah - t_r1
c                     ddml = ((t_p1*t_r1
c    +                        - ddmh)
c    +                        + t_p1 * t_r2
c    +                        + t_p2 * t_r1)
c    +                        + t_p2 * t_r2
c                     ddml = ddml
c    +                      + (betah*ddal
c    +                      + betal*ddah)
c                     t_s1 = ddmh + ddml
c                     ddml = ddml - (t_s1 - ddmh)
c                     ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddm = beta * dda
                      call bmul(betah, betal, ddah, ddal,
     +                    ddmh, ddml)
                      ch(j,j) = ddmh
                      cl(j,j) = ddml
                  ELSE
!                     C(J,J) = DBLE(C(J,J))
                      ch(j,j) = dble(ch(j,j))
                      cl(j,j) = dble(cl(j,j))
                  END IF
                  DO 120 L = 1,K
!                     IF (A(J,L).NE.DCMPLX(ZERO)) THEN
                      if(
!                       (a(j,l) .ne. zerox)
     +                 ((ah(j,l) .ne. zeroxh) .or.
     +                  (al(j,l) .ne. zeroxl))
     +                ) then
!                         TEMP = ALPHA*DCONJG(A(J,L))
                          ddaxh = dconjg(ah(j,l))
                          ddaxl = dconjg(al(j,l))
!<<<                  temp = alpha * ddax
c                     temph = alphah * ddaxh
c                     x_u1 = 134217729.0 * alphah
c                     x_p1 = x_u1
c    +                         - (x_u1 - alphah)
c                     x_p2 = alphah - x_p1
c                     x_u1 = 134217729.0 * ddaxh
c                     x_r1 = x_u1
c    +                         - (x_u1 - ddaxh)
c                     x_r2 = ddaxh - x_r1
c                     templ = ((x_p1*x_r1
c    +                        - temph)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     templ = templ
c    +                      + (alphah*ddaxl
c    +                      + alphal*ddaxh)
c                     x_s1 = temph + templ
c                     templ = templ - (x_s1 - temph)
c                     temph = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  temp = alpha * ddax
                      call bmul(alphah, alphal, ddaxh, ddaxl,
     +                    temph, templ)
** Here OMP fail with reason unknown
** (gcc version 4.3.2 20081105 (Red Hat 4.3.2-7) (GCC))
**!$omp parallel
**!$omp do
**!$omp+ private(I)
**!#ddb omp tdef x_
**!#ddb omp def ddmx
                          DO 110 I = 1,J - 1
!                             C(I,J) = C(I,J) + TEMP*A(I,L)
!<<<                          ddmx = temp * a(i,l)
c                             ddmxh = temph * ah(i,l)
c                             x_u1 = 134217729.0 * temph
c                             x_p1 = x_u1
c    +                                 - (x_u1 - temph)
c                             x_p2 = temph - x_p1
c                             x_u1 = 134217729.0 * ah(i,l)
c                             x_r1 = x_u1
c    +                                 - (x_u1 - ah(i,l))
c                             x_r2 = ah(i,l) - x_r1
c                             ddmxl = ((x_p1*x_r1
c    +                                - ddmxh)
c    +                                + x_p1 * x_r2
c    +                                + x_p2 * x_r1)
c    +                                + x_p2 * x_r2
c                             ddmxl = ddmxl
c    +                              + (temph*al(i,l)
c    +                              + templ*ah(i,l))
c                             x_s1 = ddmxh + ddmxl
c                             ddmxl = ddmxl - (x_s1 - ddmxh)
c                             ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddmx = temp * a(i,l)
                              call bmul(temph, templ, ah(i,l), al(i,l),
     +                            ddmxh, ddmxl)
!<<<                          c(i,j) = c(i,j) + ddmx
c                             x_p1 = ch(i,j) + ddmxh
c                             x_p2 = x_p1 - ch(i,j)
c                             x_q1 = x_p1
c                             x_q2 = (ch(i,j) - (x_p1 - x_p2))
c    +                                 + (ddmxh - x_p2)
c                             x_q2 = x_q2 + cl(i,j) + ddmxl
c                             ch(i,j) = x_q1 + x_q2
c                             cl(i,j) = x_q2 - (ch(i,j) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                          c(i,j) = c(i,j) + ddmx
                              call bsum(ch(i,j), cl(i,j), ddmxh, ddmxl,
     +                            ch(i,j), cl(i,j))
  110                     CONTINUE
**!$omp end do
**!$omp end parallel
!                         C(J,J) = DBLE(C(J,J)) + DBLE(TEMP*A(I,L))
!<<<                      ddmx = temp * a(i,l)
c                         ddmxh = temph * ah(i,l)
c                         x_u1 = 134217729.0 * temph
c                         x_p1 = x_u1
c    +                             - (x_u1 - temph)
c                         x_p2 = temph - x_p1
c                         x_u1 = 134217729.0 * ah(i,l)
c                         x_r1 = x_u1
c    +                             - (x_u1 - ah(i,l))
c                         x_r2 = ah(i,l) - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (temph*al(i,l)
c    +                          + templ*ah(i,l))
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = temp * a(i,l)
                          call bmul(temph, templ, ah(i,l), al(i,l),
     +                        ddmxh, ddmxl)
                          ddmh = dble(ddmxh)
                          ddml = dble(ddmxl)
                          ddah = dble(ch(j,j))
                          ddal = dble(cl(j,j))
!!!#ddb                     add c(j,j) dda ddm
!<<<                      dda = dda + ddm
c                         t_p1 = ddah + ddmh
c                         t_p2 = t_p1 - ddah
c                         t_q1 = t_p1
c                         t_q2 = (ddah - (t_p1 - t_p2))
c    +                             + (ddmh - t_p2)
c                         t_q2 = t_q2 + ddal + ddml
c                         ddah = t_q1 + t_q2
c                         ddal = t_q2 - (ddah - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      dda = dda + ddm
                          call bsum(ddah, ddal, ddmh, ddml,
     +                        ddah, ddal)
                          ch(j,j) = ddah
                          cl(j,j) = ddal
                      END IF
  120             CONTINUE
  130         CONTINUE
          ELSE
              DO 180 J = 1,N
!                 IF (BETA.EQ.ZERO) THEN
                  if(
!                   (beta .eq. zero)
     +             ((betah .eq. zeroh) .and.
     +              (betal .eq. zerol))
     +            ) then
!$omp parallel
!$omp do
!$omp+ private(I)
                      DO 140 I = J,N
!                         C(I,J) = ZERO
                          ch(i,j) = zeroh
                          cl(i,j) = zerol
  140                 CONTINUE
!$omp end do
!$omp end parallel
!                 ELSE IF (BETA.NE.ONE) THEN
                  else if(
!                           (beta .ne. one)
     +                     ((betah .ne. oneh) .or.
     +                      (betal .ne. onel))
     +            ) then
!                     C(J,J) = BETA*DBLE(C(J,J))
                      ddah = dble(ch(j,j))
                      ddal = dble(cl(j,j))
!!!#ddb                 mul c(j,j) beta dda
!<<<                      ddm = beta * dda
c                         ddmh = betah * ddah
c                         t_u1 = 134217729.0 * betah
c                         t_p1 = t_u1
c    +                             - (t_u1 - betah)
c                         t_p2 = betah - t_p1
c                         t_u1 = 134217729.0 * ddah
c                         t_r1 = t_u1
c    +                             - (t_u1 - ddah)
c                         t_r2 = ddah - t_r1
c                         ddml = ((t_p1*t_r1
c    +                            - ddmh)
c    +                            + t_p1 * t_r2
c    +                            + t_p2 * t_r1)
c    +                            + t_p2 * t_r2
c                         ddml = ddml
c    +                          + (betah*ddal
c    +                          + betal*ddah)
c                         t_s1 = ddmh + ddml
c                         ddml = ddml - (t_s1 - ddmh)
c                         ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddm = beta * dda
                          call bmul(betah, betal, ddah, ddal,
     +                        ddmh, ddml)
                          ch(j,j) = ddmh
                          cl(j,j) = ddml
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
                      DO 150 I = J + 1,N
!                         C(I,J) = BETA*C(I,J)
!<<<                      c(i,j) = beta * c(i,j)
c                         x_ch = betah * ch(i,j)
c                         x_u1 = 134217729.0 * betah
c                         x_p1 = x_u1
c    +                             - (x_u1 - betah)
c                         x_p2 = betah - x_p1
c                         x_u1 = 134217729.0 * ch(i,j)
c                         x_r1 = x_u1
c    +                             - (x_u1 - ch(i,j))
c                         x_r2 = ch(i,j) - x_r1
c                         x_cl = ((x_p1*x_r1
c    +                             - x_ch)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         x_cl = x_cl
c    +                             + (betah*cl(i,j)
c    +                             +  betal*ch(i,j))
c                         x_s1 = x_ch + x_cl
c                         cl(i,j) = x_cl - (x_s1 - x_ch)
c                         ch(i,j) = x_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                      c(i,j) = beta * c(i,j)
                          call bmul(betah, betal, ch(i,j), cl(i,j),
     +                        ch(i,j), cl(i,j))
  150                 CONTINUE
!$omp end do
!$omp end parallel
                  ELSE
!                     C(J,J) = DBLE(C(J,J))
                      ch(j,j) = dble(ch(j,j))
                      cl(j,j) = dble(cl(j,j))
                  END IF
                  DO 170 L = 1,K
!                     IF (A(J,L).NE.DCMPLX(ZERO)) THEN
                      if(
!                       (a(j,l) .ne. zerox)
     +                 ((ah(j,l) .ne. zeroxh) .or.
     +                  (al(j,l) .ne. zeroxl))
     +                ) then
!                         TEMP = ALPHA*DCONJG(A(J,L))
                          ddaxh = dconjg(ah(j,l))
                          ddaxl = dconjg(al(j,l))
!<<<                      temp = alpha * ddax
c                         temph = alphah * ddaxh
c                         x_u1 = 134217729.0 * alphah
c                         x_p1 = x_u1
c    +                             - (x_u1 - alphah)
c                         x_p2 = alphah - x_p1
c                         x_u1 = 134217729.0 * ddaxh
c                         x_r1 = x_u1
c    +                             - (x_u1 - ddaxh)
c                         x_r2 = ddaxh - x_r1
c                         templ = ((x_p1*x_r1
c    +                            - temph)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         templ = templ
c    +                          + (alphah*ddaxl
c    +                          + alphal*ddaxh)
c                         x_s1 = temph + templ
c                         templ = templ - (x_s1 - temph)
c                         temph = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      temp = alpha * ddax
                          call bmul(alphah, alphal, ddaxh, ddaxl,
     +                        temph, templ)
!                         C(J,J) = DBLE(C(J,J)) + DBLE(TEMP*A(J,L))
!<<<                      ddmx = temp * a(j,l)
c                         ddmxh = temph * ah(j,l)
c                         x_u1 = 134217729.0 * temph
c                         x_p1 = x_u1
c    +                             - (x_u1 - temph)
c                         x_p2 = temph - x_p1
c                         x_u1 = 134217729.0 * ah(j,l)
c                         x_r1 = x_u1
c    +                             - (x_u1 - ah(j,l))
c                         x_r2 = ah(j,l) - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (temph*al(j,l)
c    +                          + templ*ah(j,l))
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = temp * a(j,l)
                          call bmul(temph, templ, ah(j,l), al(j,l),
     +                        ddmxh, ddmxl)
                          ddmh = dble(ddmxh)
                          ddml = dble(ddmxl)
                          ddah = dble(ch(j,j))
                          ddal = dble(cl(j,j))
!!!#ddb                     add  c(j,j) dda ddm
!<<<                      dda = dda + ddm
c                         t_p1 = ddah + ddmh
c                         t_p2 = t_p1 - ddah
c                         t_q1 = t_p1
c                         t_q2 = (ddah - (t_p1 - t_p2))
c    +                             + (ddmh - t_p2)
c                         t_q2 = t_q2 + ddal + ddml
c                         ddah = t_q1 + t_q2
c                         ddal = t_q2 - (ddah - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      dda = dda + ddm
                          call bsum(ddah, ddal, ddmh, ddml,
     +                        ddah, ddal)
                          ch(j,j) = ddah
                          cl(j,j) = ddal
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
                          DO 160 I = J + 1,N
!                             C(I,J) = C(I,J) + TEMP*A(I,L)
!<<<                          ddmx = temp * a(i,l)
c                             ddmxh = temph * ah(i,l)
c                             x_u1 = 134217729.0 * temph
c                             x_p1 = x_u1
c    +                                 - (x_u1 - temph)
c                             x_p2 = temph - x_p1
c                             x_u1 = 134217729.0 * ah(i,l)
c                             x_r1 = x_u1
c    +                                 - (x_u1 - ah(i,l))
c                             x_r2 = ah(i,l) - x_r1
c                             ddmxl = ((x_p1*x_r1
c    +                                - ddmxh)
c    +                                + x_p1 * x_r2
c    +                                + x_p2 * x_r1)
c    +                                + x_p2 * x_r2
c                             ddmxl = ddmxl
c    +                              + (temph*al(i,l)
c    +                              + templ*ah(i,l))
c                             x_s1 = ddmxh + ddmxl
c                             ddmxl = ddmxl - (x_s1 - ddmxh)
c                             ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddmx = temp * a(i,l)
                              call bmul(temph, templ, ah(i,l), al(i,l),
     +                            ddmxh, ddmxl)
!<<<                          c(i,j) = c(i,j) + ddmx
c                             x_p1 = ch(i,j) + ddmxh
c                             x_p2 = x_p1 - ch(i,j)
c                             x_q1 = x_p1
c                             x_q2 = (ch(i,j) - (x_p1 - x_p2))
c    +                                 + (ddmxh - x_p2)
c                             x_q2 = x_q2 + cl(i,j) + ddmxl
c                             ch(i,j) = x_q1 + x_q2
c                             cl(i,j) = x_q2 - (ch(i,j) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                          c(i,j) = c(i,j) + ddmx
                              call bsum(ch(i,j), cl(i,j), ddmxh, ddmxl,
     +                            ch(i,j), cl(i,j))
  160                     CONTINUE
!$omp end do
!$omp end parallel
                      END IF
  170             CONTINUE
  180         CONTINUE
          END IF
      ELSE
*
*        Form  C := alpha*conjg( A' )*A + beta*C.
*
          IF (UPPER) THEN
              DO 220 J = 1,N
                  DO 200 I = 1,J - 1
!                     TEMP = ZERO
                      temph = zeroh
                      templ = zerol
!$omp parallel
!$omp do
!$omp+ private(L)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
!$omp+ private(ddaxh, ddaxl)
                      DO 190 L = 1,K
!                         TEMP = TEMP + DCONJG(A(L,I))*A(L,J)
                          ddaxh = dconjg(ah(l,i))
                          ddaxl = dconjg(al(l,i))
!<<<                      ddmx = ddax * a(l,j)
c                         ddmxh = ddaxh * ah(l,j)
c                         x_u1 = 134217729.0 * ddaxh
c                         x_p1 = x_u1
c    +                             - (x_u1 - ddaxh)
c                         x_p2 = ddaxh - x_p1
c                         x_u1 = 134217729.0 * ah(l,j)
c                         x_r1 = x_u1
c    +                             - (x_u1 - ah(l,j))
c                         x_r2 = ah(l,j) - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (ddaxh*al(l,j)
c    +                          + ddaxl*ah(l,j))
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = ddax * a(l,j)
                          call bmul(ddaxh, ddaxl, ah(l,j), al(l,j),
     +                        ddmxh, ddmxl)
!$omp critical
!<<<                      temp = temp + ddmx
c                         x_p1 = temph + ddmxh
c                         x_p2 = x_p1 - temph
c                         x_q1 = x_p1
c                         x_q2 = (temph - (x_p1 - x_p2))
c    +                             + (ddmxh - x_p2)
c                         x_q2 = x_q2 + templ + ddmxl
c                         temph = x_q1 + x_q2
c                         templ = x_q2 - (temph - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      temp = temp + ddmx
                          call bsum(temph, templ, ddmxh, ddmxl,
     +                        temph, templ)
!$omp end critical
  190                 CONTINUE
!$omp end do
!$omp end parallel
!                     IF (BETA.EQ.ZERO) THEN
                      if(
!                       (beta .eq. zero)
     +                 ((betah .eq. zeroh) .and.
     +                  (betal .eq. zerol))
     +                ) then
!                         C(I,J) = ALPHA*TEMP
!<<<                      c(i,j) = alpha * temp
c                         ch(i,j) = alphah * temph
c                         x_u1 = 134217729.0 * alphah
c                         x_p1 = x_u1
c    +                             - (x_u1 - alphah)
c                         x_p2 = alphah - x_p1
c                         x_u1 = 134217729.0 * temph
c                         x_r1 = x_u1
c    +                             - (x_u1 - temph)
c                         x_r2 = temph - x_r1
c                         cl(i,j) = ((x_p1*x_r1
c    +                            - ch(i,j))
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         cl(i,j) = cl(i,j)
c    +                          + (alphah*templ
c    +                          + alphal*temph)
c                         x_s1 = ch(i,j) + cl(i,j)
c                         cl(i,j) = cl(i,j) - (x_s1 - ch(i,j))
c                         ch(i,j) = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      c(i,j) = alpha * temp
                          call bmul(alphah, alphal, temph, templ,
     +                        ch(i,j), cl(i,j))
                      ELSE
!                         C(I,J) = ALPHA*TEMP + BETA*C(I,J)
!<<<                      ddmx = beta * c(i,j)
c                         ddmxh = betah * ch(i,j)
c                         x_u1 = 134217729.0 * betah
c                         x_p1 = x_u1
c    +                             - (x_u1 - betah)
c                         x_p2 = betah - x_p1
c                         x_u1 = 134217729.0 * ch(i,j)
c                         x_r1 = x_u1
c    +                             - (x_u1 - ch(i,j))
c                         x_r2 = ch(i,j) - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (betah*cl(i,j)
c    +                          + betal*ch(i,j))
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = beta * c(i,j)
                          call bmul(betah, betal, ch(i,j), cl(i,j),
     +                        ddmxh, ddmxl)
!<<<                      ddax = alpha * temp
c                         ddaxh = alphah * temph
c                         x_u1 = 134217729.0 * alphah
c                         x_p1 = x_u1
c    +                             - (x_u1 - alphah)
c                         x_p2 = alphah - x_p1
c                         x_u1 = 134217729.0 * temph
c                         x_r1 = x_u1
c    +                             - (x_u1 - temph)
c                         x_r2 = temph - x_r1
c                         ddaxl = ((x_p1*x_r1
c    +                            - ddaxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddaxl = ddaxl
c    +                          + (alphah*templ
c    +                          + alphal*temph)
c                         x_s1 = ddaxh + ddaxl
c                         ddaxl = ddaxl - (x_s1 - ddaxh)
c                         ddaxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddax = alpha * temp
                          call bmul(alphah, alphal, temph, templ,
     +                        ddaxh, ddaxl)
!<<<                      c(i,j) = ddax + ddmx
c                         x_p1 = ddaxh + ddmxh
c                         x_p2 = x_p1 - ddaxh
c                         ch(i,j) = x_p1
c                         cl(i,j) = (ddaxh - (x_p1 - x_p2))
c    +                          + (ddmxh - x_p2)
c                         cl(i,j) = cl(i,j) + ddaxl + ddmxl
c                         x_s1 = ch(i,j) + cl(i,j)
c                         cl(i,j) = cl(i,j) - (x_s1 - ch(i,j))
c                         ch(i,j) = x_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                      c(i,j) = ddax + ddmx
                          call bsum(ddaxh, ddaxl, ddmxh, ddmxl,
     +                        ch(i,j), cl(i,j))
                      END IF
  200             CONTINUE
!                 RTEMP = ZERO
                  rtemph = zeroh
                  rtempl = zerol
!$omp parallel
!$omp do
!$omp+ private(L)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
!$omp+ private(ddaxh, ddaxl)
                  DO 210 L = 1,K
!                     RTEMP = RTEMP + DCONJG(A(L,J))*A(L,J)
                      ddaxh = dconjg(ah(l,j))
                      ddaxl = dconjg(al(l,j))
!<<<                  ddmx = ddax * a(l,j)
c                     ddmxh = ddaxh * ah(l,j)
c                     x_u1 = 134217729.0 * ddaxh
c                     x_p1 = x_u1
c    +                         - (x_u1 - ddaxh)
c                     x_p2 = ddaxh - x_p1
c                     x_u1 = 134217729.0 * ah(l,j)
c                     x_r1 = x_u1
c    +                         - (x_u1 - ah(l,j))
c                     x_r2 = ah(l,j) - x_r1
c                     ddmxl = ((x_p1*x_r1
c    +                        - ddmxh)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     ddmxl = ddmxl
c    +                      + (ddaxh*al(l,j)
c    +                      + ddaxl*ah(l,j))
c                     x_s1 = ddmxh + ddmxl
c                     ddmxl = ddmxl - (x_s1 - ddmxh)
c                     ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = ddax * a(l,j)
                      call bmul(ddaxh, ddaxl, ah(l,j), al(l,j),
     +                    ddmxh, ddmxl)
!$omp critical
!<<<                  rtemp = rtemp + ddmx
c                     x_p1 = rtemph + ddmxh
c                     x_p2 = x_p1 - rtemph
c                     x_q1 = x_p1
c                     x_q2 = (rtemph - (x_p1 - x_p2))
c    +                         + (ddmxh - x_p2)
c                     x_q2 = x_q2 + rtempl + ddmxl
c                     rtemph = x_q1 + x_q2
c                     rtempl = x_q2 - (rtemph - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  rtemp = rtemp + ddmx
c                     FIXME
                      call bsum(rtemph, rtempl,
     +                     dble(ddmxh), dble(ddmxl), rtemph, rtempl)
!$omp end critical
  210             CONTINUE
!$omp end do
!$omp end parallel
!                 IF (BETA.EQ.ZERO) THEN
                  if(
!                   (beta .eq. zero)
     +             ((betah .eq. zeroh) .and.
     +              (betal .eq. zerol))
     +            ) then
!                     C(J,J) = ALPHA*RTEMP
!!!#ddb                 mul c(j,j) alpha rtemp
!<<<                  ddm = alpha * rtemp
c                     ddmh = alphah * rtemph
c                     t_u1 = 134217729.0 * alphah
c                     t_p1 = t_u1
c    +                         - (t_u1 - alphah)
c                     t_p2 = alphah - t_p1
c                     t_u1 = 134217729.0 * rtemph
c                     t_r1 = t_u1
c    +                         - (t_u1 - rtemph)
c                     t_r2 = rtemph - t_r1
c                     ddml = ((t_p1*t_r1
c    +                        - ddmh)
c    +                        + t_p1 * t_r2
c    +                        + t_p2 * t_r1)
c    +                        + t_p2 * t_r2
c                     ddml = ddml
c    +                      + (alphah*rtempl
c    +                      + alphal*rtemph)
c                     t_s1 = ddmh + ddml
c                     ddml = ddml - (t_s1 - ddmh)
c                     ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddm = alpha * rtemp
                      call bmul(alphah, alphal, rtemph, rtempl,
     +                    ddmh, ddml)
                      ch(j,j) = ddmh
                      cl(j,j) = ddml
                  ELSE
!                     C(J,J) = ALPHA*RTEMP + BETA*DBLE(C(J,J))
                      ddah = dble(ch(j,j))
                      ddal = dble(cl(j,j))
!<<<                  ddm = beta * dda
c                     ddmh = betah * ddah
c                     t_u1 = 134217729.0 * betah
c                     t_p1 = t_u1
c    +                         - (t_u1 - betah)
c                     t_p2 = betah - t_p1
c                     t_u1 = 134217729.0 * ddah
c                     t_r1 = t_u1
c    +                         - (t_u1 - ddah)
c                     t_r2 = ddah - t_r1
c                     ddml = ((t_p1*t_r1
c    +                        - ddmh)
c    +                        + t_p1 * t_r2
c    +                        + t_p2 * t_r1)
c    +                        + t_p2 * t_r2
c                     ddml = ddml
c    +                      + (betah*ddal
c    +                      + betal*ddah)
c                     t_s1 = ddmh + ddml
c                     ddml = ddml - (t_s1 - ddmh)
c                     ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddm = beta * dda
                      call bmul(betah, betal, ddah, ddal,
     +                    ddmh, ddml)
!<<<                  ddx = alpha * rtemp
c                     ddxh = alphah * rtemph
c                     t_u1 = 134217729.0 * alphah
c                     t_p1 = t_u1
c    +                         - (t_u1 - alphah)
c                     t_p2 = alphah - t_p1
c                     t_u1 = 134217729.0 * rtemph
c                     t_r1 = t_u1
c    +                         - (t_u1 - rtemph)
c                     t_r2 = rtemph - t_r1
c                     ddxl = ((t_p1*t_r1
c    +                        - ddxh)
c    +                        + t_p1 * t_r2
c    +                        + t_p2 * t_r1)
c    +                        + t_p2 * t_r2
c                     ddxl = ddxl
c    +                      + (alphah*rtempl
c    +                      + alphal*rtemph)
c                     t_s1 = ddxh + ddxl
c                     ddxl = ddxl - (t_s1 - ddxh)
c                     ddxh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddx = alpha * rtemp
                      call bmul(alphah, alphal, rtemph, rtempl,
     +                    ddxh, ddxl)
!<<<                  ddx = ddx + ddm
c                     t_p1 = ddxh + ddmh
c                     t_p2 = t_p1 - ddxh
c                     t_q1 = t_p1
c                     t_q2 = (ddxh - (t_p1 - t_p2))
c    +                         + (ddmh - t_p2)
c                     t_q2 = t_q2 + ddxl + ddml
c                     ddxh = t_q1 + t_q2
c                     ddxl = t_q2 - (ddxh - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  ddx = ddx + ddm
                      call bsum(ddxh, ddxl, ddmh, ddml,
     +                    ddxh, ddxl)
                      ch(j,j) = ddxh
                      cl(j,j) = ddxl
                  END IF
  220         CONTINUE
          ELSE
              DO 260 J = 1,N
!                 RTEMP = ZERO
                  rtemph = zeroh
                  rtempl = zerol
!$omp parallel
!$omp do
!$omp+ private(L)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
!$omp+ private(ddaxh, ddaxl)
                  DO 230 L = 1,K
!                     RTEMP = RTEMP + DCONJG(A(L,J))*A(L,J)
                      ddaxh = dconjg(ah(l,j))
                      ddaxl = dconjg(al(l,j))
!<<<                  ddmx = ddax * a(l,j)
c                     ddmxh = ddaxh * ah(l,j)
c                     x_u1 = 134217729.0 * ddaxh
c                     x_p1 = x_u1
c    +                         - (x_u1 - ddaxh)
c                     x_p2 = ddaxh - x_p1
c                     x_u1 = 134217729.0 * ah(l,j)
c                     x_r1 = x_u1
c    +                         - (x_u1 - ah(l,j))
c                     x_r2 = ah(l,j) - x_r1
c                     ddmxl = ((x_p1*x_r1
c    +                        - ddmxh)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     ddmxl = ddmxl
c    +                      + (ddaxh*al(l,j)
c    +                      + ddaxl*ah(l,j))
c                     x_s1 = ddmxh + ddmxl
c                     ddmxl = ddmxl - (x_s1 - ddmxh)
c                     ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = ddax * a(l,j)
                      call bmul(ddaxh, ddaxl, ah(l,j), al(l,j),
     +                    ddmxh, ddmxl)
!$omp critical
!<<<                  rtemp = rtemp + ddmx
c                     x_p1 = rtemph + ddmxh
c                     x_p2 = x_p1 - rtemph
c                     x_q1 = x_p1
c                     x_q2 = (rtemph - (x_p1 - x_p2))
c    +                         + (ddmxh - x_p2)
c                     x_q2 = x_q2 + rtempl + ddmxl
c                     rtemph = x_q1 + x_q2
c                     rtempl = x_q2 - (rtemph - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  rtemp = rtemp + ddmx
c                     FIXME
                      call bsum(rtemph, rtempl,
     +                     dble(ddmxh), dble(ddmxl), rtemph, rtempl)
!$omp end critical
  230             CONTINUE
!$omp end do
!$omp end parallel
!                 IF (BETA.EQ.ZERO) THEN
                  if(
!                   (beta .eq. zero)
     +             ((betah .eq. zeroh) .and.
     +              (betal .eq. zerol))
     +            ) then
!                     C(J,J) = ALPHA*RTEMP
!!!#ddb               mul c(j,j) alpha rtemp
!<<<                  ddmx = alpha * rtemp
c                     ddmxh = alphah * rtemph
c                     t_u1 = 134217729.0 * alphah
c                     t_p1 = t_u1
c    +                         - (t_u1 - alphah)
c                     t_p2 = alphah - t_p1
c                     t_u1 = 134217729.0 * rtemph
c                     t_r1 = t_u1
c    +                         - (t_u1 - rtemph)
c                     t_r2 = rtemph - t_r1
c                     ddmxl = ((t_p1*t_r1
c    +                        - ddmxh)
c    +                        + t_p1 * t_r2
c    +                        + t_p2 * t_r1)
c    +                        + t_p2 * t_r2
c                     ddmxl = ddmxl
c    +                      + (alphah*rtempl
c    +                      + alphal*rtemph)
c                     t_s1 = ddmxh + ddmxl
c                     ddmxl = ddmxl - (t_s1 - ddmxh)
c                     ddmxh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = alpha * rtemp
                      call bmul(alphah, alphal, rtemph, rtempl,
     +                    ddmxh, ddmxl)
                      ch(j,j) = ddmxh
                      cl(j,j) = ddmxl
                  ELSE
!                     C(J,J) = ALPHA*RTEMP + BETA*DBLE(C(J,J))
                      ddah = dble(ch(j,j))
                      ddal = dble(cl(j,j))
!<<<                  ddm = beta * dda
c                     ddmh = betah * ddah
c                     t_u1 = 134217729.0 * betah
c                     t_p1 = t_u1
c    +                         - (t_u1 - betah)
c                     t_p2 = betah - t_p1
c                     t_u1 = 134217729.0 * ddah
c                     t_r1 = t_u1
c    +                         - (t_u1 - ddah)
c                     t_r2 = ddah - t_r1
c                     ddml = ((t_p1*t_r1
c    +                        - ddmh)
c    +                        + t_p1 * t_r2
c    +                        + t_p2 * t_r1)
c    +                        + t_p2 * t_r2
c                     ddml = ddml
c    +                      + (betah*ddal
c    +                      + betal*ddah)
c                     t_s1 = ddmh + ddml
c                     ddml = ddml - (t_s1 - ddmh)
c                     ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddm = beta * dda
                      call bmul(betah, betal, ddah, ddal,
     +                    ddmh, ddml)
!<<<                  ddx = alpha * rtemp
c                     ddxh = alphah * rtemph
c                     t_u1 = 134217729.0 * alphah
c                     t_p1 = t_u1
c    +                         - (t_u1 - alphah)
c                     t_p2 = alphah - t_p1
c                     t_u1 = 134217729.0 * rtemph
c                     t_r1 = t_u1
c    +                         - (t_u1 - rtemph)
c                     t_r2 = rtemph - t_r1
c                     ddxl = ((t_p1*t_r1
c    +                        - ddxh)
c    +                        + t_p1 * t_r2
c    +                        + t_p2 * t_r1)
c    +                        + t_p2 * t_r2
c                     ddxl = ddxl
c    +                      + (alphah*rtempl
c    +                      + alphal*rtemph)
c                     t_s1 = ddxh + ddxl
c                     ddxl = ddxl - (t_s1 - ddxh)
c                     ddxh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddx = alpha * rtemp
                      call bmul(alphah, alphal, rtemph, rtempl,
     +                    ddxh, ddxl)
!<<<                  ddx = ddx + ddm
c                     t_p1 = ddxh + ddmh
c                     t_p2 = t_p1 - ddxh
c                     t_q1 = t_p1
c                     t_q2 = (ddxh - (t_p1 - t_p2))
c    +                         + (ddmh - t_p2)
c                     t_q2 = t_q2 + ddxl + ddml
c                     ddxh = t_q1 + t_q2
c                     ddxl = t_q2 - (ddxh - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  ddx = ddx + ddm
                      call bsum(ddxh, ddxl, ddmh, ddml,
     +                    ddxh, ddxl)
                      ch(j,j) = ddxh
                      cl(j,j) = ddxl
                  END IF
                  DO 250 I = J + 1,N
!                     TEMP = ZERO
                      temph = zeroh
                      templ = zerol
!$omp parallel
!$omp do
!$omp+ private(L)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
!$omp+ private(ddaxh, ddaxl)
                      DO 240 L = 1,K
!                         TEMP = TEMP + DCONJG(A(L,I))*A(L,J)
                          ddaxh = dconjg(ah(l,i))
                          ddaxl = dconjg(al(l,i))
!<<<                      ddmx = ddax * a(l,j)
c                         ddmxh = ddaxh * ah(l,j)
c                         x_u1 = 134217729.0 * ddaxh
c                         x_p1 = x_u1
c    +                             - (x_u1 - ddaxh)
c                         x_p2 = ddaxh - x_p1
c                         x_u1 = 134217729.0 * ah(l,j)
c                         x_r1 = x_u1
c    +                             - (x_u1 - ah(l,j))
c                         x_r2 = ah(l,j) - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (ddaxh*al(l,j)
c    +                          + ddaxl*ah(l,j))
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = ddax * a(l,j)
                          call bmul(ddaxh, ddaxl, ah(l,j), al(l,j),
     +                        ddmxh, ddmxl)
!$omp critical
!<<<                      temp = temp + ddmx
c                         x_p1 = temph + ddmxh
c                         x_p2 = x_p1 - temph
c                         x_q1 = x_p1
c                         x_q2 = (temph - (x_p1 - x_p2))
c    +                             + (ddmxh - x_p2)
c                         x_q2 = x_q2 + templ + ddmxl
c                         temph = x_q1 + x_q2
c                         templ = x_q2 - (temph - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      temp = temp + ddmx
                          call bsum(temph, templ, ddmxh, ddmxl,
     +                        temph, templ)
!$omp end critical
  240                 CONTINUE
!$omp end do
!$omp end parallel
!                     IF (BETA.EQ.ZERO) THEN
                      if(
!                       (beta .eq. zero)
     +                 ((betah .eq. zeroh) .and.
     +                  (betal .eq. zerol))
     +                ) then
!                         C(I,J) = ALPHA*TEMP
!<<<                      c(i,j) = alpha * temp
c                         ch(i,j) = alphah * temph
c                         x_u1 = 134217729.0 * alphah
c                         x_p1 = x_u1
c    +                             - (x_u1 - alphah)
c                         x_p2 = alphah - x_p1
c                         x_u1 = 134217729.0 * temph
c                         x_r1 = x_u1
c    +                             - (x_u1 - temph)
c                         x_r2 = temph - x_r1
c                         cl(i,j) = ((x_p1*x_r1
c    +                            - ch(i,j))
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         cl(i,j) = cl(i,j)
c    +                          + (alphah*templ
c    +                          + alphal*temph)
c                         x_s1 = ch(i,j) + cl(i,j)
c                         cl(i,j) = cl(i,j) - (x_s1 - ch(i,j))
c                         ch(i,j) = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      c(i,j) = alpha * temp
                          call bmul(alphah, alphal, temph, templ,
     +                        ch(i,j), cl(i,j))
                      ELSE
!                         C(I,J) = ALPHA*TEMP + BETA*C(I,J)
!<<<                      ddax = beta * c(i,j)
c                         ddaxh = betah * ch(i,j)
c                         x_u1 = 134217729.0 * betah
c                         x_p1 = x_u1
c    +                             - (x_u1 - betah)
c                         x_p2 = betah - x_p1
c                         x_u1 = 134217729.0 * ch(i,j)
c                         x_r1 = x_u1
c    +                             - (x_u1 - ch(i,j))
c                         x_r2 = ch(i,j) - x_r1
c                         ddaxl = ((x_p1*x_r1
c    +                            - ddaxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddaxl = ddaxl
c    +                          + (betah*cl(i,j)
c    +                          + betal*ch(i,j))
c                         x_s1 = ddaxh + ddaxl
c                         ddaxl = ddaxl - (x_s1 - ddaxh)
c                         ddaxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddax = beta * c(i,j)
                          call bmul(betah, betal, ch(i,j), cl(i,j),
     +                        ddaxh, ddaxl)
!<<<                      ddmx = alpha * temp
c                         ddmxh = alphah * temph
c                         x_u1 = 134217729.0 * alphah
c                         x_p1 = x_u1
c    +                             - (x_u1 - alphah)
c                         x_p2 = alphah - x_p1
c                         x_u1 = 134217729.0 * temph
c                         x_r1 = x_u1
c    +                             - (x_u1 - temph)
c                         x_r2 = temph - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (alphah*templ
c    +                          + alphal*temph)
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = alpha * temp
                          call bmul(alphah, alphal, temph, templ,
     +                        ddmxh, ddmxl)
!<<<                      c(i,j) = ddmx + ddax
c                         x_p1 = ddmxh + ddaxh
c                         x_p2 = x_p1 - ddmxh
c                         ch(i,j) = x_p1
c                         cl(i,j) = (ddmxh - (x_p1 - x_p2))
c    +                          + (ddaxh - x_p2)
c                         cl(i,j) = cl(i,j) + ddmxl + ddaxl
c                         x_s1 = ch(i,j) + cl(i,j)
c                         cl(i,j) = cl(i,j) - (x_s1 - ch(i,j))
c                         ch(i,j) = x_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                      c(i,j) = ddmx + ddax
                          call bsum(ddmxh, ddmxl, ddaxh, ddaxl,
     +                        ch(i,j), cl(i,j))
                      END IF
  250             CONTINUE
  260         CONTINUE
          END IF
      END IF
*
 9999 continue
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = trans"
!     WRITE(6,1005) SUBNAME,"DIM M     = ",M
      WRITE(6,1005) SUBNAME,"DIM N     = ",N
      WRITE(6,1005) SUBNAME,"DIM K     = ",K
#include "fout3.h"
#include "fformat.h"
#endif
      RETURN
*
*     End of ZHERK .
*
      END
