*> \brief \b DLAED5
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DLAED5 + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dlaed5.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dlaed5.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dlaed5.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DLAED5( I, D, Z, DELTA, RHO, DLAM )
* 
*       .. Scalar Arguments ..
*       INTEGER            I
*       DOUBLE PRECISION   DLAM, RHO
*       ..
*       .. Array Arguments ..
*       DOUBLE PRECISION   D( 2 ), DELTA( 2 ), Z( 2 )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> This subroutine computes the I-th eigenvalue of a symmetric rank-one
*> modification of a 2-by-2 diagonal matrix
*>
*>            diag( D )  +  RHO * Z * transpose(Z) .
*>
*> The diagonal elements in the array D are assumed to satisfy
*>
*>            D(i) < D(j)  for  i < j .
*>
*> We also assume RHO > 0 and that the Euclidean norm of the vector
*> Z is one.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] I
*> \verbatim
*>          I is INTEGER
*>         The index of the eigenvalue to be computed.  I = 1 or I = 2.
*> \endverbatim
*>
*> \param[in] D
*> \verbatim
*>          D is DOUBLE PRECISION array, dimension (2)
*>         The original eigenvalues.  We assume D(1) < D(2).
*> \endverbatim
*>
*> \param[in] Z
*> \verbatim
*>          Z is DOUBLE PRECISION array, dimension (2)
*>         The components of the updating vector.
*> \endverbatim
*>
*> \param[out] DELTA
*> \verbatim
*>          DELTA is DOUBLE PRECISION array, dimension (2)
*>         The vector DELTA contains the information necessary
*>         to construct the eigenvectors.
*> \endverbatim
*>
*> \param[in] RHO
*> \verbatim
*>          RHO is DOUBLE PRECISION
*>         The scalar in the symmetric updating formula.
*> \endverbatim
*>
*> \param[out] DLAM
*> \verbatim
*>          DLAM is DOUBLE PRECISION
*>         The computed lambda_I, the I-th updated eigenvalue.
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERcomputational
*
*> \par Contributors:
*  ==================
*>
*>     Ren-Cang Li, Computer Science Division, University of California
*>     at Berkeley, USA
*>
*  =====================================================================
!      SUBROUTINE DLAED5( I, D, Z, DELTA, RHO, DLAM )
      SUBROUTINE DDLAED5( I, Dh, Dl, Zh, Zl, DELTAh, DELTAl, 
     $                    RHOh, RHOl, DLAMh, DLAMl )
*
*  -- LAPACK computational routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      INTEGER            I
!      DOUBLE PRECISION   DLAM, RHO
      double precision DLAMh, DLAMl
      double precision RHOh, RHOl
*     ..
*     .. Array Arguments ..
!      DOUBLE PRECISION   D( 2 ), DELTA( 2 ), Z( 2 )
      double precision Dh(2), Dl(2)
      double precision DELTAh(2), DELTAl(2)
      double precision Zh(2), Zl(2)
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
!      DOUBLE PRECISION   ZERO, ONE, TWO, FOUR
!      PARAMETER          ( ZERO = 0.0D0, ONE = 1.0D0, TWO = 2.0D0,
!     $                   FOUR = 4.0D0 )
      double precision ZEROh, ZEROl
      double precision ONEh, ONEl
      double precision TWOh, TWOl
      double precision FOURh, FOURl
*     ..
*     .. Local Scalars ..
!      DOUBLE PRECISION   B, C, DEL, TAU, TEMP, W
      double precision Bh, Bl
      double precision Ch, Cl
      double precision DELh, DELl
      double precision TAUh, TAUl
      double precision TEMPh, TEMPl
      double precision Wh, Wl
*     ..
      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision tmp3h, tmp3l
      double precision tmp4h, tmp4l
      double precision tmp5h, tmp5l
      double precision tmp6h, tmp6l
      double precision tmp7h, tmp7l
      double precision tmp8h, tmp8l
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l

*     .. Intrinsic Functions ..
      INTRINSIC          ABS, SQRT
*     ..

      ZEROh = 0.0d+0
      ZEROl = 0.0d+0
      ONEh = 1.0d+0
      ONEl = 0.0d+0
      TWOh = 2.0D0
      TWOl = 0.0d+0
      FOURh = 4.0D0
      FOURl = 0.0d+0
*     .. Executable Statements ..
*
!      DEL = D( 2 ) - D( 1 )
!<<<  DEL = D(2) - D(1)
      T_p1 = Dh(2) - Dh(1)
      T_p2 = T_p1 - Dh(2)
      DELh = T_p1
      DELl = (Dh(2) - (T_p1 - T_p2))
     +         - (Dh(1) + T_p2)
      DELl = DELl + Dl(2) - Dl(1)
      T_s1 = DELh + DELl
      DELl = DELl - (T_s1 - DELh)
      DELh = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  DEL = D(2) - D(1)
      IF( I.EQ.1 ) THEN
!         W = ONE + TWO*RHO*( Z( 2 )*Z( 2 )-Z( 1 )*Z( 1 ) ) / DEL
!<<<  tmp1 = Z(2) * Z(2)
      tmp1h = Zh(2) * Zh(2)
      T_u1 = 134217729.0 * Zh(2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(2))
      T_p2 = Zh(2) - T_p1
      T_u1 = 134217729.0 * Zh(2)
      T_r1 = T_u1
     +         - (T_u1 - Zh(2))
      T_r2 = Zh(2) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(2)*Zl(2)
     +      + Zl(2)*Zh(2))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(2) * Z(2)
!<<<  tmp2 = Z(1) * Z(1)
      tmp2h = Zh(1) * Zh(1)
      T_u1 = 134217729.0 * Zh(1)
      T_p1 = T_u1
     +         - (T_u1 - Zh(1))
      T_p2 = Zh(1) - T_p1
      T_u1 = 134217729.0 * Zh(1)
      T_r1 = T_u1
     +         - (T_u1 - Zh(1))
      T_r2 = Zh(1) - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Zh(1)*Zl(1)
     +      + Zl(1)*Zh(1))
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = Z(1) * Z(1)
!<<<  tmp3 = tmp1 - tmp2
      T_p1 = tmp1h - tmp2h
      T_p2 = T_p1 - tmp1h
      tmp3h = T_p1
      tmp3l = (tmp1h - (T_p1 - T_p2))
     +         - (tmp2h + T_p2)
      tmp3l = tmp3l + tmp1l - tmp2l
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp3 = tmp1 - tmp2
!<<<  tmp4 = tmp3 / DEL
      T_q1 = tmp3h / DELh
      tmp4h = DELh * T_q1
      T_u1 = 134217729.0 * DELh
      T_p1 = T_u1
     +         - (T_u1 - DELh)
      T_p2 = DELh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp4l = ((T_p1*T_r1
     +       - tmp4h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp4l = tmp4l + (DELl*T_q1)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
      T_s1 = tmp3h - tmp4h
      T_r1 = T_s1 - tmp3h
      T_s2 = (tmp3h 
     +         - (T_s1 - T_r1))
     +         - (tmp4h + T_r1)
      T_s2 = T_s2 - tmp4l
      T_s2 = T_s2 + tmp3l
      T_q2 = (T_s1 + T_s2)
     +         / DELh
      tmp4h = T_q1 + T_q2
      tmp4l = T_q2
     +      - (tmp4h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp4 = tmp3 / DEL
!<<<  tmp5 = TWO * RHO
      tmp5h = TWOh * RHOh
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * RHOh
      T_r1 = T_u1
     +         - (T_u1 - RHOh)
      T_r2 = RHOh - T_r1
      tmp5l = ((T_p1*T_r1
     +        - tmp5h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp5l = tmp5l
     +      + (TWOh*RHOl
     +      + TWOl*RHOh)
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp5 = TWO * RHO
!<<<  tmp5 = tmp5 * tmp4
      T_ch = tmp5h * tmp4h
      T_u1 = 134217729.0 * tmp5h
      T_p1 = T_u1
     +         - (T_u1 - tmp5h)
      T_p2 = tmp5h - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp5h*tmp4l
     +         +  tmp5l*tmp4h)
      T_s1 = T_ch + T_cl
      tmp5l = T_cl - (T_s1 - T_ch)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp5 = tmp5 * tmp4
!<<<  W = ONE + tmp5
      T_p1 = ONEh + tmp5h
      T_p2 = T_p1 - ONEh
      Wh = T_p1
      Wl = (ONEh - (T_p1 - T_p2))
     +      + (tmp5h - T_p2)
      Wl = Wl + ONEl + tmp5l
      T_s1 = Wh + Wl
      Wl = Wl - (T_s1 - Wh)
      Wh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  W = ONE + tmp5

         IF( 
!            W.GT.ZERO
!       (W .gt. ZERO)
     + ((Wh .gt. ZEROh) .or.
     +  ((Wh .eq. ZEROh) .and.
     +   (Wl .gt. ZEROl)))
     $      ) THEN
!            B = DEL + RHO*( Z( 1 )*Z( 1 )+Z( 2 )*Z( 2 ) )
!<<<  tmp1 = Z(1) * Z(1)
      tmp1h = Zh(1) * Zh(1)
      T_u1 = 134217729.0 * Zh(1)
      T_p1 = T_u1
     +         - (T_u1 - Zh(1))
      T_p2 = Zh(1) - T_p1
      T_u1 = 134217729.0 * Zh(1)
      T_r1 = T_u1
     +         - (T_u1 - Zh(1))
      T_r2 = Zh(1) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(1)*Zl(1)
     +      + Zl(1)*Zh(1))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(1) * Z(1)
!<<<  tmp2 = Z(2) * Z(2)
      tmp2h = Zh(2) * Zh(2)
      T_u1 = 134217729.0 * Zh(2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(2))
      T_p2 = Zh(2) - T_p1
      T_u1 = 134217729.0 * Zh(2)
      T_r1 = T_u1
     +         - (T_u1 - Zh(2))
      T_r2 = Zh(2) - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Zh(2)*Zl(2)
     +      + Zl(2)*Zh(2))
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = Z(2) * Z(2)
!<<<  tmp3 = tmp1 + tmp2
      T_p1 = tmp1h + tmp2h
      T_p2 = T_p1 - tmp1h
      tmp3h = T_p1
      tmp3l = (tmp1h - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      tmp3l = tmp3l + tmp1l + tmp2l
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp3 = tmp1 + tmp2
!<<<  tmp4 = RHO * tmp3
      tmp4h = RHOh * tmp3h
      T_u1 = 134217729.0 * RHOh
      T_p1 = T_u1
     +         - (T_u1 - RHOh)
      T_p2 = RHOh - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (RHOh*tmp3l
     +      + RHOl*tmp3h)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = RHO * tmp3
!<<<  B = DEL + tmp4
      T_p1 = DELh + tmp4h
      T_p2 = T_p1 - DELh
      Bh = T_p1
      Bl = (DELh - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      Bl = Bl + DELl + tmp4l
      T_s1 = Bh + Bl
      Bl = Bl - (T_s1 - Bh)
      Bh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  B = DEL + tmp4
!            C = RHO*Z( 1 )*Z( 1 )*DEL
!<<<  C = RHO * Z(1)
      Ch = RHOh * Zh(1)
      T_u1 = 134217729.0 * RHOh
      T_p1 = T_u1
     +         - (T_u1 - RHOh)
      T_p2 = RHOh - T_p1
      T_u1 = 134217729.0 * Zh(1)
      T_r1 = T_u1
     +         - (T_u1 - Zh(1))
      T_r2 = Zh(1) - T_r1
      Cl = ((T_p1*T_r1
     +        - Ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      Cl = Cl
     +      + (RHOh*Zl(1)
     +      + RHOl*Zh(1))
      T_s1 = Ch + Cl
      Cl = Cl - (T_s1 - Ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  C = RHO * Z(1)
!<<<  C = C * Z(1)
      T_ch = Ch * Zh(1)
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * Zh(1)
      T_r1 = T_u1
     +         - (T_u1 - Zh(1))
      T_r2 = Zh(1) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Ch*Zl(1)
     +         +  Cl*Zh(1))
      T_s1 = T_ch + T_cl
      Cl = T_cl - (T_s1 - T_ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  C = C * Z(1)
!<<<  C = C * DEL
      T_ch = Ch * DELh
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * DELh
      T_r1 = T_u1
     +         - (T_u1 - DELh)
      T_r2 = DELh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Ch*DELl
     +         +  Cl*DELh)
      T_s1 = T_ch + T_cl
      Cl = T_cl - (T_s1 - T_ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  C = C * DEL
*
*           B > ZERO, always
*
!            TAU = TWO*C / ( B+SQRT( ABS( B*B-FOUR*C ) ) )
!<<<  tmp1 = TWO * C
      tmp1h = TWOh * Ch
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*Cl
     +      + TWOl*Ch)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * C
!<<<  tmp2 = B * B
      tmp2h = Bh * Bh
      T_u1 = 134217729.0 * Bh
      T_p1 = T_u1
     +         - (T_u1 - Bh)
      T_p2 = Bh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Bh*Bl
     +      + Bl*Bh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = B * B
!<<<  tmp3 = FOUR * C
      tmp3h = FOURh * Ch
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (FOURh*Cl
     +      + FOURl*Ch)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = FOUR * C
!<<<  tmp4 = tmp2 - tmp3
      T_p1 = tmp2h - tmp3h
      T_p2 = T_p1 - tmp2h
      tmp4h = T_p1
      tmp4l = (tmp2h - (T_p1 - T_p2))
     +         - (tmp3h + T_p2)
      tmp4l = tmp4l + tmp2l - tmp3l
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp4 = tmp2 - tmp3
!<<<  tmp5 = abs(tmp4)
      if (tmp4h .lt. 0.0d+0) then
        tmp5h = -tmp4h
        tmp5l = -tmp4l
      elseif (
     +  (tmp4h .eq. 0.0d+0).and.
     +  (tmp4l .lt. 0.0d+0)) then
        tmp5h = tmp4h
        tmp5l = -tmp4l
      else
        tmp5h = tmp4h
        tmp5l = tmp4l
      endif
!>>>  tmp5 = abs(tmp4)
!<<<  tmp6 = sqrt(tmp5)
      if (tmp5h.eq.0.0D+0.and.tmp5l.eq.0.0D+0) then
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
      endif
      if (tmp5h.lt.0.0d+0.or.
     +   (tmp5h.eq.0.0d+0.and.tmp5l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp5h)
      T_ax = tmp5h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp5h,tmp5l,T_z1,T_z2)
          T_p1 = tmp5h - T_z1
          T_p2 = T_p1 - tmp5h
          T_t2l = (tmp5h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp5l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp6h = T_t1h
          tmp6l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp6 = sqrt(tmp5)
!<<<  tmp7 = B + tmp6
      T_p1 = Bh + tmp6h
      T_p2 = T_p1 - Bh
      tmp7h = T_p1
      tmp7l = (Bh - (T_p1 - T_p2))
     +      + (tmp6h - T_p2)
      tmp7l = tmp7l + Bl + tmp6l
      T_s1 = tmp7h + tmp7l
      tmp7l = tmp7l - (T_s1 - tmp7h)
      tmp7h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp7 = B + tmp6
!<<<  TAU = tmp1 / tmp7
      T_q1 = tmp1h / tmp7h
      TAUh = tmp7h * T_q1
      T_u1 = 134217729.0 * tmp7h
      T_p1 = T_u1
     +         - (T_u1 - tmp7h)
      T_p2 = tmp7h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TAUl = ((T_p1*T_r1
     +       - TAUh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TAUl = TAUl + (tmp7l*T_q1)
      T_s1 = TAUh + TAUl
      TAUl = TAUl - (T_s1 - TAUh)
      TAUh = T_s1
      T_s1 = tmp1h - TAUh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (TAUh + T_r1)
      T_s2 = T_s2 - TAUl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / tmp7h
      TAUh = T_q1 + T_q2
      TAUl = T_q2
     +      - (TAUh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TAU = tmp1 / tmp7
!            DLAM = D( 1 ) + TAU
!<<<  DLAM = D(1) + TAU
      T_p1 = Dh(1) + TAUh
      T_p2 = T_p1 - Dh(1)
      DLAMh = T_p1
      DLAMl = (Dh(1) - (T_p1 - T_p2))
     +      + (TAUh - T_p2)
      DLAMl = DLAMl + Dl(1) + TAUl
      T_s1 = DLAMh + DLAMl
      DLAMl = DLAMl - (T_s1 - DLAMh)
      DLAMh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  DLAM = D(1) + TAU
!            DELTA( 1 ) = -Z( 1 ) / TAU
!<<<  DELTA(1) = Z(1) / TAU
      T_q1 = Zh(1) / TAUh
      DELTAh(1) = TAUh * T_q1
      T_u1 = 134217729.0 * TAUh
      T_p1 = T_u1
     +         - (T_u1 - TAUh)
      T_p2 = TAUh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      DELTAl(1) = ((T_p1*T_r1
     +       - DELTAh(1))
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      DELTAl(1) = DELTAl(1) + (TAUl*T_q1)
      T_s1 = DELTAh(1) + DELTAl(1)
      DELTAl(1) = DELTAl(1) - (T_s1 - DELTAh(1))
      DELTAh(1) = T_s1
      T_s1 = Zh(1) - DELTAh(1)
      T_r1 = T_s1 - Zh(1)
      T_s2 = (Zh(1) 
     +         - (T_s1 - T_r1))
     +         - (DELTAh(1) + T_r1)
      T_s2 = T_s2 - DELTAl(1)
      T_s2 = T_s2 + Zl(1)
      T_q2 = (T_s1 + T_s2)
     +         / TAUh
      DELTAh(1) = T_q1 + T_q2
      DELTAl(1) = T_q2
     +      - (DELTAh(1) - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  DELTA(1) = Z(1) / TAU
!<<<  DELTA(1) = -DELTA(1)
      DELTAh(1) = -DELTAh(1)
      DELTAl(1) = -DELTAl(1)
!>>>  DELTA(1) = -DELTA(1)
!            DELTA( 2 ) = Z( 2 ) / ( DEL-TAU )
!<<<  tmp1 = DEL - TAU
      T_p1 = DELh - TAUh
      T_p2 = T_p1 - DELh
      tmp1h = T_p1
      tmp1l = (DELh - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      tmp1l = tmp1l + DELl - TAUl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = DEL - TAU
!<<<  DELTA(2) = Z(2) / tmp1
      T_q1 = Zh(2) / tmp1h
      DELTAh(2) = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      DELTAl(2) = ((T_p1*T_r1
     +       - DELTAh(2))
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      DELTAl(2) = DELTAl(2) + (tmp1l*T_q1)
      T_s1 = DELTAh(2) + DELTAl(2)
      DELTAl(2) = DELTAl(2) - (T_s1 - DELTAh(2))
      DELTAh(2) = T_s1
      T_s1 = Zh(2) - DELTAh(2)
      T_r1 = T_s1 - Zh(2)
      T_s2 = (Zh(2) 
     +         - (T_s1 - T_r1))
     +         - (DELTAh(2) + T_r1)
      T_s2 = T_s2 - DELTAl(2)
      T_s2 = T_s2 + Zl(2)
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      DELTAh(2) = T_q1 + T_q2
      DELTAl(2) = T_q2
     +      - (DELTAh(2) - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  DELTA(2) = Z(2) / tmp1
         ELSE
!            B = -DEL + RHO*( Z( 1 )*Z( 1 )+Z( 2 )*Z( 2 ) )
!<<<  tmp1 = Z(1) * Z(1)
      tmp1h = Zh(1) * Zh(1)
      T_u1 = 134217729.0 * Zh(1)
      T_p1 = T_u1
     +         - (T_u1 - Zh(1))
      T_p2 = Zh(1) - T_p1
      T_u1 = 134217729.0 * Zh(1)
      T_r1 = T_u1
     +         - (T_u1 - Zh(1))
      T_r2 = Zh(1) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(1)*Zl(1)
     +      + Zl(1)*Zh(1))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(1) * Z(1)
!<<<  tmp2 = Z(2) * Z(2)
      tmp2h = Zh(2) * Zh(2)
      T_u1 = 134217729.0 * Zh(2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(2))
      T_p2 = Zh(2) - T_p1
      T_u1 = 134217729.0 * Zh(2)
      T_r1 = T_u1
     +         - (T_u1 - Zh(2))
      T_r2 = Zh(2) - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Zh(2)*Zl(2)
     +      + Zl(2)*Zh(2))
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = Z(2) * Z(2)
!<<<  tmp3 = tmp1 + tmp2
      T_p1 = tmp1h + tmp2h
      T_p2 = T_p1 - tmp1h
      tmp3h = T_p1
      tmp3l = (tmp1h - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      tmp3l = tmp3l + tmp1l + tmp2l
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp3 = tmp1 + tmp2
!<<<  tmp4 = RHO * tmp3
      tmp4h = RHOh * tmp3h
      T_u1 = 134217729.0 * RHOh
      T_p1 = T_u1
     +         - (T_u1 - RHOh)
      T_p2 = RHOh - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (RHOh*tmp3l
     +      + RHOl*tmp3h)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = RHO * tmp3
!<<<  B = tmp4 - DEL
      T_p1 = tmp4h - DELh
      T_p2 = T_p1 - tmp4h
      Bh = T_p1
      Bl = (tmp4h - (T_p1 - T_p2))
     +         - (DELh + T_p2)
      Bl = Bl + tmp4l - DELl
      T_s1 = Bh + Bl
      Bl = Bl - (T_s1 - Bh)
      Bh = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  B = tmp4 - DEL
!            C = RHO*Z( 2 )*Z( 2 )*DEL
!<<<  C = RHO * Z(2)
      Ch = RHOh * Zh(2)
      T_u1 = 134217729.0 * RHOh
      T_p1 = T_u1
     +         - (T_u1 - RHOh)
      T_p2 = RHOh - T_p1
      T_u1 = 134217729.0 * Zh(2)
      T_r1 = T_u1
     +         - (T_u1 - Zh(2))
      T_r2 = Zh(2) - T_r1
      Cl = ((T_p1*T_r1
     +        - Ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      Cl = Cl
     +      + (RHOh*Zl(2)
     +      + RHOl*Zh(2))
      T_s1 = Ch + Cl
      Cl = Cl - (T_s1 - Ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  C = RHO * Z(2)
!<<<  C = C * Z(2)
      T_ch = Ch * Zh(2)
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * Zh(2)
      T_r1 = T_u1
     +         - (T_u1 - Zh(2))
      T_r2 = Zh(2) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Ch*Zl(2)
     +         +  Cl*Zh(2))
      T_s1 = T_ch + T_cl
      Cl = T_cl - (T_s1 - T_ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  C = C * Z(2)
!<<<  C = C * DEL
      T_ch = Ch * DELh
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * DELh
      T_r1 = T_u1
     +         - (T_u1 - DELh)
      T_r2 = DELh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Ch*DELl
     +         +  Cl*DELh)
      T_s1 = T_ch + T_cl
      Cl = T_cl - (T_s1 - T_ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  C = C * DEL
            IF( 
!              B.GT.ZERO
!       (B .gt. ZERO)
     + ((Bh .gt. ZEROh) .or.
     +  ((Bh .eq. ZEROh) .and.
     +   (Bl .gt. ZEROl)))
     $          ) THEN
!               TAU = -TWO*C / ( B+SQRT( B*B+FOUR*C ) )
!<<<  tmp1 = TWO * C
      tmp1h = TWOh * Ch
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*Cl
     +      + TWOl*Ch)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * C
!<<<  tmp2 = B * B
      tmp2h = Bh * Bh
      T_u1 = 134217729.0 * Bh
      T_p1 = T_u1
     +         - (T_u1 - Bh)
      T_p2 = Bh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Bh*Bl
     +      + Bl*Bh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = B * B
!<<<  tmp3 = FOUR * C
      tmp3h = FOURh * Ch
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (FOURh*Cl
     +      + FOURl*Ch)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = FOUR * C
!<<<  tmp4 = tmp2 + tmp3
      T_p1 = tmp2h + tmp3h
      T_p2 = T_p1 - tmp2h
      tmp4h = T_p1
      tmp4l = (tmp2h - (T_p1 - T_p2))
     +      + (tmp3h - T_p2)
      tmp4l = tmp4l + tmp2l + tmp3l
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp4 = tmp2 + tmp3
!<<<  tmp6 = sqrt(tmp4)
      if (tmp4h.eq.0.0D+0.and.tmp4l.eq.0.0D+0) then
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
      endif
      if (tmp4h.lt.0.0d+0.or.
     +   (tmp4h.eq.0.0d+0.and.tmp4l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp4h)
      T_ax = tmp4h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp4h,tmp4l,T_z1,T_z2)
          T_p1 = tmp4h - T_z1
          T_p2 = T_p1 - tmp4h
          T_t2l = (tmp4h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp4l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp6h = T_t1h
          tmp6l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp6 = sqrt(tmp4)
!<<<  tmp7 = B + tmp6
      T_p1 = Bh + tmp6h
      T_p2 = T_p1 - Bh
      tmp7h = T_p1
      tmp7l = (Bh - (T_p1 - T_p2))
     +      + (tmp6h - T_p2)
      tmp7l = tmp7l + Bl + tmp6l
      T_s1 = tmp7h + tmp7l
      tmp7l = tmp7l - (T_s1 - tmp7h)
      tmp7h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp7 = B + tmp6
!<<<  TAU = tmp1 / tmp7
      T_q1 = tmp1h / tmp7h
      TAUh = tmp7h * T_q1
      T_u1 = 134217729.0 * tmp7h
      T_p1 = T_u1
     +         - (T_u1 - tmp7h)
      T_p2 = tmp7h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TAUl = ((T_p1*T_r1
     +       - TAUh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TAUl = TAUl + (tmp7l*T_q1)
      T_s1 = TAUh + TAUl
      TAUl = TAUl - (T_s1 - TAUh)
      TAUh = T_s1
      T_s1 = tmp1h - TAUh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (TAUh + T_r1)
      T_s2 = T_s2 - TAUl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / tmp7h
      TAUh = T_q1 + T_q2
      TAUl = T_q2
     +      - (TAUh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TAU = tmp1 / tmp7
!<<<  TAU = -TAU
      TAUh = -TAUh
      TAUl = -TAUl
!>>>  TAU = -TAU

            ELSE
!               TAU = ( B-SQRT( B*B+FOUR*C ) ) / TWO
!<<<  tmp2 = B * B
      tmp2h = Bh * Bh
      T_u1 = 134217729.0 * Bh
      T_p1 = T_u1
     +         - (T_u1 - Bh)
      T_p2 = Bh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Bh*Bl
     +      + Bl*Bh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = B * B
!<<<  tmp3 = FOUR * C
      tmp3h = FOURh * Ch
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (FOURh*Cl
     +      + FOURl*Ch)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = FOUR * C
!<<<  tmp4 = tmp2 + tmp3
      T_p1 = tmp2h + tmp3h
      T_p2 = T_p1 - tmp2h
      tmp4h = T_p1
      tmp4l = (tmp2h - (T_p1 - T_p2))
     +      + (tmp3h - T_p2)
      tmp4l = tmp4l + tmp2l + tmp3l
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp4 = tmp2 + tmp3
!<<<  tmp5 = sqrt(tmp4)
      if (tmp4h.eq.0.0D+0.and.tmp4l.eq.0.0D+0) then
        tmp5h = 0.0d+0
        tmp5l = 0.0d+0
      endif
      if (tmp4h.lt.0.0d+0.or.
     +   (tmp4h.eq.0.0d+0.and.tmp4l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp5h = 0.0d+0
        tmp5l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp4h)
      T_ax = tmp4h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp4h,tmp4l,T_z1,T_z2)
          T_p1 = tmp4h - T_z1
          T_p2 = T_p1 - tmp4h
          T_t2l = (tmp4h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp4l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp5h = T_t1h
          tmp5l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp5 = sqrt(tmp4)
!<<<  tmp6 = B - tmp5
      T_p1 = Bh - tmp5h
      T_p2 = T_p1 - Bh
      tmp6h = T_p1
      tmp6l = (Bh - (T_p1 - T_p2))
     +         - (tmp5h + T_p2)
      tmp6l = tmp6l + Bl - tmp5l
      T_s1 = tmp6h + tmp6l
      tmp6l = tmp6l - (T_s1 - tmp6h)
      tmp6h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp6 = B - tmp5
!<<<  TAU = tmp6 / TWO
      T_q1 = tmp6h / TWOh
      TAUh = TWOh * T_q1
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TAUl = ((T_p1*T_r1
     +       - TAUh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TAUl = TAUl + (TWOl*T_q1)
      T_s1 = TAUh + TAUl
      TAUl = TAUl - (T_s1 - TAUh)
      TAUh = T_s1
      T_s1 = tmp6h - TAUh
      T_r1 = T_s1 - tmp6h
      T_s2 = (tmp6h 
     +         - (T_s1 - T_r1))
     +         - (TAUh + T_r1)
      T_s2 = T_s2 - TAUl
      T_s2 = T_s2 + tmp6l
      T_q2 = (T_s1 + T_s2)
     +         / TWOh
      TAUh = T_q1 + T_q2
      TAUl = T_q2
     +      - (TAUh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TAU = tmp6 / TWO

            END IF
!            DLAM = D( 2 ) + TAU
!<<<  DLAM = D(2) + TAU
      T_p1 = Dh(2) + TAUh
      T_p2 = T_p1 - Dh(2)
      DLAMh = T_p1
      DLAMl = (Dh(2) - (T_p1 - T_p2))
     +      + (TAUh - T_p2)
      DLAMl = DLAMl + Dl(2) + TAUl
      T_s1 = DLAMh + DLAMl
      DLAMl = DLAMl - (T_s1 - DLAMh)
      DLAMh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  DLAM = D(2) + TAU
!            DELTA( 1 ) = -Z( 1 ) / ( DEL+TAU )
!<<<  tmp1 = DEL + TAU
      T_p1 = DELh + TAUh
      T_p2 = T_p1 - DELh
      tmp1h = T_p1
      tmp1l = (DELh - (T_p1 - T_p2))
     +      + (TAUh - T_p2)
      tmp1l = tmp1l + DELl + TAUl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = DEL + TAU
!<<<  DELTA(1) = Z(1) / tmp1
      T_q1 = Zh(1) / tmp1h
      DELTAh(1) = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      DELTAl(1) = ((T_p1*T_r1
     +       - DELTAh(1))
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      DELTAl(1) = DELTAl(1) + (tmp1l*T_q1)
      T_s1 = DELTAh(1) + DELTAl(1)
      DELTAl(1) = DELTAl(1) - (T_s1 - DELTAh(1))
      DELTAh(1) = T_s1
      T_s1 = Zh(1) - DELTAh(1)
      T_r1 = T_s1 - Zh(1)
      T_s2 = (Zh(1) 
     +         - (T_s1 - T_r1))
     +         - (DELTAh(1) + T_r1)
      T_s2 = T_s2 - DELTAl(1)
      T_s2 = T_s2 + Zl(1)
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      DELTAh(1) = T_q1 + T_q2
      DELTAl(1) = T_q2
     +      - (DELTAh(1) - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  DELTA(1) = Z(1) / tmp1
!<<<  DELTA(1) = -DELTA(1)
      DELTAh(1) = -DELTAh(1)
      DELTAl(1) = -DELTAl(1)
!>>>  DELTA(1) = -DELTA(1)
!            DELTA( 2 ) = -Z( 2 ) / TAU
!<<<  DELTA(2) = Z(2) / TAU
      T_q1 = Zh(2) / TAUh
      DELTAh(2) = TAUh * T_q1
      T_u1 = 134217729.0 * TAUh
      T_p1 = T_u1
     +         - (T_u1 - TAUh)
      T_p2 = TAUh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      DELTAl(2) = ((T_p1*T_r1
     +       - DELTAh(2))
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      DELTAl(2) = DELTAl(2) + (TAUl*T_q1)
      T_s1 = DELTAh(2) + DELTAl(2)
      DELTAl(2) = DELTAl(2) - (T_s1 - DELTAh(2))
      DELTAh(2) = T_s1
      T_s1 = Zh(2) - DELTAh(2)
      T_r1 = T_s1 - Zh(2)
      T_s2 = (Zh(2) 
     +         - (T_s1 - T_r1))
     +         - (DELTAh(2) + T_r1)
      T_s2 = T_s2 - DELTAl(2)
      T_s2 = T_s2 + Zl(2)
      T_q2 = (T_s1 + T_s2)
     +         / TAUh
      DELTAh(2) = T_q1 + T_q2
      DELTAl(2) = T_q2
     +      - (DELTAh(2) - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  DELTA(2) = Z(2) / TAU
!<<<  DELTA(2) = -DELTA(2)
      DELTAh(2) = -DELTAh(2)
      DELTAl(2) = -DELTAl(2)
!>>>  DELTA(2) = -DELTA(2)
         END IF
!         TEMP = SQRT( DELTA( 1 )*DELTA( 1 )+DELTA( 2 )*DELTA( 2 ) )
!<<<  tmp1 = DELTA(1) * DELTA(1)
      tmp1h = DELTAh(1) * DELTAh(1)
      T_u1 = 134217729.0 * DELTAh(1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(1))
      T_p2 = DELTAh(1) - T_p1
      T_u1 = 134217729.0 * DELTAh(1)
      T_r1 = T_u1
     +         - (T_u1 - DELTAh(1))
      T_r2 = DELTAh(1) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (DELTAh(1)*DELTAl(1)
     +      + DELTAl(1)*DELTAh(1))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = DELTA(1) * DELTA(1)
!<<<  tmp2 = DELTA(2) * DELTA(2)
      tmp2h = DELTAh(2) * DELTAh(2)
      T_u1 = 134217729.0 * DELTAh(2)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(2))
      T_p2 = DELTAh(2) - T_p1
      T_u1 = 134217729.0 * DELTAh(2)
      T_r1 = T_u1
     +         - (T_u1 - DELTAh(2))
      T_r2 = DELTAh(2) - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (DELTAh(2)*DELTAl(2)
     +      + DELTAl(2)*DELTAh(2))
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = DELTA(2) * DELTA(2)
!<<<  tmp3 = tmp1 + tmp2
      T_p1 = tmp1h + tmp2h
      T_p2 = T_p1 - tmp1h
      tmp3h = T_p1
      tmp3l = (tmp1h - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      tmp3l = tmp3l + tmp1l + tmp2l
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp3 = tmp1 + tmp2
!<<<  TEMP = sqrt(tmp3)
      if (tmp3h.eq.0.0D+0.and.tmp3l.eq.0.0D+0) then
        TEMPh = 0.0d+0
        TEMPl = 0.0d+0
      endif
      if (tmp3h.lt.0.0d+0.or.
     +   (tmp3h.eq.0.0d+0.and.tmp3l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        TEMPh = 0.0d+0
        TEMPl = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp3h)
      T_ax = tmp3h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp3h,tmp3l,T_z1,T_z2)
          T_p1 = tmp3h - T_z1
          T_p2 = T_p1 - tmp3h
          T_t2l = (tmp3h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp3l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          TEMPh = T_t1h
          TEMPl = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  TEMP = sqrt(tmp3)
!         DELTA( 1 ) = DELTA( 1 ) / TEMP
!<<<  DELTA(1) = DELTA(1) / TEMP
      T_q1 = DELTAh(1) / TEMPh
      T_ch = TEMPh * T_q1
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (TEMPl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = DELTAh(1) - T_ch
      T_r1 = T_s1 - DELTAh(1)
      T_s2 = (DELTAh(1)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + DELTAl(1)
      T_q2 = (T_s1 + T_s2)
     +         / TEMPh
      DELTAh(1) = T_q1 + T_q2
      DELTAl(1) = T_q2
     +      - (DELTAh(1) - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  DELTA(1) = DELTA(1) / TEMP
!         DELTA( 2 ) = DELTA( 2 ) / TEMP
!<<<  DELTA(2) = DELTA(2) / TEMP
      T_q1 = DELTAh(2) / TEMPh
      T_ch = TEMPh * T_q1
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (TEMPl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = DELTAh(2) - T_ch
      T_r1 = T_s1 - DELTAh(2)
      T_s2 = (DELTAh(2)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + DELTAl(2)
      T_q2 = (T_s1 + T_s2)
     +         / TEMPh
      DELTAh(2) = T_q1 + T_q2
      DELTAl(2) = T_q2
     +      - (DELTAh(2) - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  DELTA(2) = DELTA(2) / TEMP
      ELSE
*
*     Now I=2
*
!         B = -DEL + RHO*( Z( 1 )*Z( 1 )+Z( 2 )*Z( 2 ) )
!<<<  tmp1 = Z(1) * Z(1)
      tmp1h = Zh(1) * Zh(1)
      T_u1 = 134217729.0 * Zh(1)
      T_p1 = T_u1
     +         - (T_u1 - Zh(1))
      T_p2 = Zh(1) - T_p1
      T_u1 = 134217729.0 * Zh(1)
      T_r1 = T_u1
     +         - (T_u1 - Zh(1))
      T_r2 = Zh(1) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(1)*Zl(1)
     +      + Zl(1)*Zh(1))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(1) * Z(1)
!<<<  tmp2 = Z(2) * Z(2)
      tmp2h = Zh(2) * Zh(2)
      T_u1 = 134217729.0 * Zh(2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(2))
      T_p2 = Zh(2) - T_p1
      T_u1 = 134217729.0 * Zh(2)
      T_r1 = T_u1
     +         - (T_u1 - Zh(2))
      T_r2 = Zh(2) - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Zh(2)*Zl(2)
     +      + Zl(2)*Zh(2))
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = Z(2) * Z(2)
!<<<  tmp3 = tmp1 + tmp2
      T_p1 = tmp1h + tmp2h
      T_p2 = T_p1 - tmp1h
      tmp3h = T_p1
      tmp3l = (tmp1h - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      tmp3l = tmp3l + tmp1l + tmp2l
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp3 = tmp1 + tmp2
!<<<  tmp4 = RHO * tmp3
      tmp4h = RHOh * tmp3h
      T_u1 = 134217729.0 * RHOh
      T_p1 = T_u1
     +         - (T_u1 - RHOh)
      T_p2 = RHOh - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (RHOh*tmp3l
     +      + RHOl*tmp3h)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = RHO * tmp3
!<<<  B = tmp4 - DEL
      T_p1 = tmp4h - DELh
      T_p2 = T_p1 - tmp4h
      Bh = T_p1
      Bl = (tmp4h - (T_p1 - T_p2))
     +         - (DELh + T_p2)
      Bl = Bl + tmp4l - DELl
      T_s1 = Bh + Bl
      Bl = Bl - (T_s1 - Bh)
      Bh = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  B = tmp4 - DEL
!         C = RHO*Z( 2 )*Z( 2 )*DEL
!<<<  C = RHO * Z(2)
      Ch = RHOh * Zh(2)
      T_u1 = 134217729.0 * RHOh
      T_p1 = T_u1
     +         - (T_u1 - RHOh)
      T_p2 = RHOh - T_p1
      T_u1 = 134217729.0 * Zh(2)
      T_r1 = T_u1
     +         - (T_u1 - Zh(2))
      T_r2 = Zh(2) - T_r1
      Cl = ((T_p1*T_r1
     +        - Ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      Cl = Cl
     +      + (RHOh*Zl(2)
     +      + RHOl*Zh(2))
      T_s1 = Ch + Cl
      Cl = Cl - (T_s1 - Ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  C = RHO * Z(2)
!<<<  C = C * Z(2)
      T_ch = Ch * Zh(2)
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * Zh(2)
      T_r1 = T_u1
     +         - (T_u1 - Zh(2))
      T_r2 = Zh(2) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Ch*Zl(2)
     +         +  Cl*Zh(2))
      T_s1 = T_ch + T_cl
      Cl = T_cl - (T_s1 - T_ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  C = C * Z(2)
!<<<  C = C * DEL
      T_ch = Ch * DELh
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * DELh
      T_r1 = T_u1
     +         - (T_u1 - DELh)
      T_r2 = DELh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Ch*DELl
     +         +  Cl*DELh)
      T_s1 = T_ch + T_cl
      Cl = T_cl - (T_s1 - T_ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  C = C * DEL

         IF( 
!            B.GT.ZERO
!       (B .gt. ZERO)
     + ((Bh .gt. ZEROh) .or.
     +  ((Bh .eq. ZEROh) .and.
     +   (Bl .gt. ZEROl)))
     $      ) THEN
!            TAU = ( B+SQRT( B*B+FOUR*C ) ) / TWO
!<<<  tmp2 = B * B
      tmp2h = Bh * Bh
      T_u1 = 134217729.0 * Bh
      T_p1 = T_u1
     +         - (T_u1 - Bh)
      T_p2 = Bh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Bh*Bl
     +      + Bl*Bh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = B * B
!<<<  tmp3 = FOUR * C
      tmp3h = FOURh * Ch
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (FOURh*Cl
     +      + FOURl*Ch)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = FOUR * C
!<<<  tmp4 = tmp2 + tmp3
      T_p1 = tmp2h + tmp3h
      T_p2 = T_p1 - tmp2h
      tmp4h = T_p1
      tmp4l = (tmp2h - (T_p1 - T_p2))
     +      + (tmp3h - T_p2)
      tmp4l = tmp4l + tmp2l + tmp3l
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp4 = tmp2 + tmp3
!<<<  tmp5 = sqrt(tmp4)
      if (tmp4h.eq.0.0D+0.and.tmp4l.eq.0.0D+0) then
        tmp5h = 0.0d+0
        tmp5l = 0.0d+0
      endif
      if (tmp4h.lt.0.0d+0.or.
     +   (tmp4h.eq.0.0d+0.and.tmp4l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp5h = 0.0d+0
        tmp5l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp4h)
      T_ax = tmp4h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp4h,tmp4l,T_z1,T_z2)
          T_p1 = tmp4h - T_z1
          T_p2 = T_p1 - tmp4h
          T_t2l = (tmp4h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp4l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp5h = T_t1h
          tmp5l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp5 = sqrt(tmp4)
!<<<  tmp6 = B + tmp5
      T_p1 = Bh + tmp5h
      T_p2 = T_p1 - Bh
      tmp6h = T_p1
      tmp6l = (Bh - (T_p1 - T_p2))
     +      + (tmp5h - T_p2)
      tmp6l = tmp6l + Bl + tmp5l
      T_s1 = tmp6h + tmp6l
      tmp6l = tmp6l - (T_s1 - tmp6h)
      tmp6h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp6 = B + tmp5
!<<<  TAU = tmp6 / TWO
      T_q1 = tmp6h / TWOh
      TAUh = TWOh * T_q1
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TAUl = ((T_p1*T_r1
     +       - TAUh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TAUl = TAUl + (TWOl*T_q1)
      T_s1 = TAUh + TAUl
      TAUl = TAUl - (T_s1 - TAUh)
      TAUh = T_s1
      T_s1 = tmp6h - TAUh
      T_r1 = T_s1 - tmp6h
      T_s2 = (tmp6h 
     +         - (T_s1 - T_r1))
     +         - (TAUh + T_r1)
      T_s2 = T_s2 - TAUl
      T_s2 = T_s2 + tmp6l
      T_q2 = (T_s1 + T_s2)
     +         / TWOh
      TAUh = T_q1 + T_q2
      TAUl = T_q2
     +      - (TAUh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TAU = tmp6 / TWO

         ELSE
!            TAU = TWO*C / ( -B+SQRT( B*B+FOUR*C ) )
!<<<  tmp1 = TWO * C
      tmp1h = TWOh * Ch
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*Cl
     +      + TWOl*Ch)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * C
!<<<  tmp2 = B * B
      tmp2h = Bh * Bh
      T_u1 = 134217729.0 * Bh
      T_p1 = T_u1
     +         - (T_u1 - Bh)
      T_p2 = Bh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Bh*Bl
     +      + Bl*Bh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = B * B
!<<<  tmp3 = FOUR * C
      tmp3h = FOURh * Ch
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (FOURh*Cl
     +      + FOURl*Ch)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = FOUR * C
!<<<  tmp4 = tmp2 + tmp3
      T_p1 = tmp2h + tmp3h
      T_p2 = T_p1 - tmp2h
      tmp4h = T_p1
      tmp4l = (tmp2h - (T_p1 - T_p2))
     +      + (tmp3h - T_p2)
      tmp4l = tmp4l + tmp2l + tmp3l
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp4 = tmp2 + tmp3
!<<<  tmp5 = sqrt(tmp4)
      if (tmp4h.eq.0.0D+0.and.tmp4l.eq.0.0D+0) then
        tmp5h = 0.0d+0
        tmp5l = 0.0d+0
      endif
      if (tmp4h.lt.0.0d+0.or.
     +   (tmp4h.eq.0.0d+0.and.tmp4l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp5h = 0.0d+0
        tmp5l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp4h)
      T_ax = tmp4h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp4h,tmp4l,T_z1,T_z2)
          T_p1 = tmp4h - T_z1
          T_p2 = T_p1 - tmp4h
          T_t2l = (tmp4h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp4l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp5h = T_t1h
          tmp5l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp5 = sqrt(tmp4)
!<<<  tmp6 = tmp5 - B
      T_p1 = tmp5h - Bh
      T_p2 = T_p1 - tmp5h
      tmp6h = T_p1
      tmp6l = (tmp5h - (T_p1 - T_p2))
     +         - (Bh + T_p2)
      tmp6l = tmp6l + tmp5l - Bl
      T_s1 = tmp6h + tmp6l
      tmp6l = tmp6l - (T_s1 - tmp6h)
      tmp6h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp6 = tmp5 - B
!<<<  TAU = tmp1 / tmp6
      T_q1 = tmp1h / tmp6h
      TAUh = tmp6h * T_q1
      T_u1 = 134217729.0 * tmp6h
      T_p1 = T_u1
     +         - (T_u1 - tmp6h)
      T_p2 = tmp6h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TAUl = ((T_p1*T_r1
     +       - TAUh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TAUl = TAUl + (tmp6l*T_q1)
      T_s1 = TAUh + TAUl
      TAUl = TAUl - (T_s1 - TAUh)
      TAUh = T_s1
      T_s1 = tmp1h - TAUh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (TAUh + T_r1)
      T_s2 = T_s2 - TAUl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / tmp6h
      TAUh = T_q1 + T_q2
      TAUl = T_q2
     +      - (TAUh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TAU = tmp1 / tmp6

         END IF
!         DLAM = D( 2 ) + TAU
!<<<  DLAM = D(2) + TAU
      T_p1 = Dh(2) + TAUh
      T_p2 = T_p1 - Dh(2)
      DLAMh = T_p1
      DLAMl = (Dh(2) - (T_p1 - T_p2))
     +      + (TAUh - T_p2)
      DLAMl = DLAMl + Dl(2) + TAUl
      T_s1 = DLAMh + DLAMl
      DLAMl = DLAMl - (T_s1 - DLAMh)
      DLAMh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  DLAM = D(2) + TAU
!         DELTA( 1 ) = -Z( 1 ) / ( DEL+TAU )
!<<<  tmp1 = DEL + TAU
      T_p1 = DELh + TAUh
      T_p2 = T_p1 - DELh
      tmp1h = T_p1
      tmp1l = (DELh - (T_p1 - T_p2))
     +      + (TAUh - T_p2)
      tmp1l = tmp1l + DELl + TAUl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = DEL + TAU
!<<<  DELTA(1) = Z(1) / tmp1
      T_q1 = Zh(1) / tmp1h
      DELTAh(1) = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      DELTAl(1) = ((T_p1*T_r1
     +       - DELTAh(1))
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      DELTAl(1) = DELTAl(1) + (tmp1l*T_q1)
      T_s1 = DELTAh(1) + DELTAl(1)
      DELTAl(1) = DELTAl(1) - (T_s1 - DELTAh(1))
      DELTAh(1) = T_s1
      T_s1 = Zh(1) - DELTAh(1)
      T_r1 = T_s1 - Zh(1)
      T_s2 = (Zh(1) 
     +         - (T_s1 - T_r1))
     +         - (DELTAh(1) + T_r1)
      T_s2 = T_s2 - DELTAl(1)
      T_s2 = T_s2 + Zl(1)
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      DELTAh(1) = T_q1 + T_q2
      DELTAl(1) = T_q2
     +      - (DELTAh(1) - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  DELTA(1) = Z(1) / tmp1
!<<<  DELTA(1) = -DELTA(1)
      DELTAh(1) = -DELTAh(1)
      DELTAl(1) = -DELTAl(1)
!>>>  DELTA(1) = -DELTA(1)
!         DELTA( 2 ) = -Z( 2 ) / TAU
!<<<  DELTA(2) = Z(2) / TAU
      T_q1 = Zh(2) / TAUh
      DELTAh(2) = TAUh * T_q1
      T_u1 = 134217729.0 * TAUh
      T_p1 = T_u1
     +         - (T_u1 - TAUh)
      T_p2 = TAUh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      DELTAl(2) = ((T_p1*T_r1
     +       - DELTAh(2))
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      DELTAl(2) = DELTAl(2) + (TAUl*T_q1)
      T_s1 = DELTAh(2) + DELTAl(2)
      DELTAl(2) = DELTAl(2) - (T_s1 - DELTAh(2))
      DELTAh(2) = T_s1
      T_s1 = Zh(2) - DELTAh(2)
      T_r1 = T_s1 - Zh(2)
      T_s2 = (Zh(2) 
     +         - (T_s1 - T_r1))
     +         - (DELTAh(2) + T_r1)
      T_s2 = T_s2 - DELTAl(2)
      T_s2 = T_s2 + Zl(2)
      T_q2 = (T_s1 + T_s2)
     +         / TAUh
      DELTAh(2) = T_q1 + T_q2
      DELTAl(2) = T_q2
     +      - (DELTAh(2) - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  DELTA(2) = Z(2) / TAU
!<<<  DELTA(2) = -DELTA(2)
      DELTAh(2) = -DELTAh(2)
      DELTAl(2) = -DELTAl(2)
!>>>  DELTA(2) = -DELTA(2)
!         TEMP = SQRT( DELTA( 1 )*DELTA( 1 )+DELTA( 2 )*DELTA( 2 ) )
!<<<  tmp1 = DELTA(1) * DELTA(1)
      tmp1h = DELTAh(1) * DELTAh(1)
      T_u1 = 134217729.0 * DELTAh(1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(1))
      T_p2 = DELTAh(1) - T_p1
      T_u1 = 134217729.0 * DELTAh(1)
      T_r1 = T_u1
     +         - (T_u1 - DELTAh(1))
      T_r2 = DELTAh(1) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (DELTAh(1)*DELTAl(1)
     +      + DELTAl(1)*DELTAh(1))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = DELTA(1) * DELTA(1)
!<<<  tmp2 = DELTA(2) * DELTA(2)
      tmp2h = DELTAh(2) * DELTAh(2)
      T_u1 = 134217729.0 * DELTAh(2)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(2))
      T_p2 = DELTAh(2) - T_p1
      T_u1 = 134217729.0 * DELTAh(2)
      T_r1 = T_u1
     +         - (T_u1 - DELTAh(2))
      T_r2 = DELTAh(2) - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (DELTAh(2)*DELTAl(2)
     +      + DELTAl(2)*DELTAh(2))
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = DELTA(2) * DELTA(2)
!<<<  tmp3 = tmp1 + tmp2
      T_p1 = tmp1h + tmp2h
      T_p2 = T_p1 - tmp1h
      tmp3h = T_p1
      tmp3l = (tmp1h - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      tmp3l = tmp3l + tmp1l + tmp2l
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp3 = tmp1 + tmp2
!<<<  TEMP = sqrt(tmp3)
      if (tmp3h.eq.0.0D+0.and.tmp3l.eq.0.0D+0) then
        TEMPh = 0.0d+0
        TEMPl = 0.0d+0
      endif
      if (tmp3h.lt.0.0d+0.or.
     +   (tmp3h.eq.0.0d+0.and.tmp3l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        TEMPh = 0.0d+0
        TEMPl = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp3h)
      T_ax = tmp3h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp3h,tmp3l,T_z1,T_z2)
          T_p1 = tmp3h - T_z1
          T_p2 = T_p1 - tmp3h
          T_t2l = (tmp3h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp3l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          TEMPh = T_t1h
          TEMPl = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  TEMP = sqrt(tmp3)
!         DELTA( 1 ) = DELTA( 1 ) / TEMP
!<<<  DELTA(1) = DELTA(1) / TEMP
      T_q1 = DELTAh(1) / TEMPh
      T_ch = TEMPh * T_q1
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (TEMPl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = DELTAh(1) - T_ch
      T_r1 = T_s1 - DELTAh(1)
      T_s2 = (DELTAh(1)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + DELTAl(1)
      T_q2 = (T_s1 + T_s2)
     +         / TEMPh
      DELTAh(1) = T_q1 + T_q2
      DELTAl(1) = T_q2
     +      - (DELTAh(1) - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  DELTA(1) = DELTA(1) / TEMP
!         DELTA( 2 ) = DELTA( 2 ) / TEMP
!<<<  DELTA(2) = DELTA(2) / TEMP
      T_q1 = DELTAh(2) / TEMPh
      T_ch = TEMPh * T_q1
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (TEMPl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = DELTAh(2) - T_ch
      T_r1 = T_s1 - DELTAh(2)
      T_s2 = (DELTAh(2)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + DELTAl(2)
      T_q2 = (T_s1 + T_s2)
     +         / TEMPh
      DELTAh(2) = T_q1 + T_q2
      DELTAl(2) = T_q2
     +      - (DELTAh(2) - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  DELTA(2) = DELTA(2) / TEMP
      END IF
      RETURN
*
*     End OF DLAED5
*
      END
