*> \brief \b DLAED8
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DLAED8 + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dlaed8.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dlaed8.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dlaed8.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DLAED8( ICOMPQ, K, N, QSIZ, D, Q, LDQ, INDXQ, RHO,
*                          CUTPNT, Z, DLAMDA, Q2, LDQ2, W, PERM, GIVPTR,
*                          GIVCOL, GIVNUM, INDXP, INDX, INFO )
* 
*       .. Scalar Arguments ..
*       INTEGER            CUTPNT, GIVPTR, ICOMPQ, INFO, K, LDQ, LDQ2, N,
*      $                   QSIZ
*       DOUBLE PRECISION   RHO
*       ..
*       .. Array Arguments ..
*       INTEGER            GIVCOL( 2, * ), INDX( * ), INDXP( * ),
*      $                   INDXQ( * ), PERM( * )
*       DOUBLE PRECISION   D( * ), DLAMDA( * ), GIVNUM( 2, * ),
*      $                   Q( LDQ, * ), Q2( LDQ2, * ), W( * ), Z( * )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DLAED8 merges the two sets of eigenvalues together into a single
*> sorted set.  Then it tries to deflate the size of the problem.
*> There are two ways in which deflation can occur:  when two or more
*> eigenvalues are close together or if there is a tiny element in the
*> Z vector.  For each such occurrence the order of the related secular
*> equation problem is reduced by one.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] ICOMPQ
*> \verbatim
*>          ICOMPQ is INTEGER
*>          = 0:  Compute eigenvalues only.
*>          = 1:  Compute eigenvectors of original dense symmetric matrix
*>                also.  On entry, Q contains the orthogonal matrix used
*>                to reduce the original matrix to tridiagonal form.
*> \endverbatim
*>
*> \param[out] K
*> \verbatim
*>          K is INTEGER
*>         The number of non-deflated eigenvalues, and the order of the
*>         related secular equation.
*> \endverbatim
*>
*> \param[in] N
*> \verbatim
*>          N is INTEGER
*>         The dimension of the symmetric tridiagonal matrix.  N >= 0.
*> \endverbatim
*>
*> \param[in] QSIZ
*> \verbatim
*>          QSIZ is INTEGER
*>         The dimension of the orthogonal matrix used to reduce
*>         the full matrix to tridiagonal form.  QSIZ >= N if ICOMPQ = 1.
*> \endverbatim
*>
*> \param[in,out] D
*> \verbatim
*>          D is DOUBLE PRECISION array, dimension (N)
*>         On entry, the eigenvalues of the two submatrices to be
*>         combined.  On exit, the trailing (N-K) updated eigenvalues
*>         (those which were deflated) sorted into increasing order.
*> \endverbatim
*>
*> \param[in,out] Q
*> \verbatim
*>          Q is DOUBLE PRECISION array, dimension (LDQ,N)
*>         If ICOMPQ = 0, Q is not referenced.  Otherwise,
*>         on entry, Q contains the eigenvectors of the partially solved
*>         system which has been previously updated in matrix
*>         multiplies with other partially solved eigensystems.
*>         On exit, Q contains the trailing (N-K) updated eigenvectors
*>         (those which were deflated) in its last N-K columns.
*> \endverbatim
*>
*> \param[in] LDQ
*> \verbatim
*>          LDQ is INTEGER
*>         The leading dimension of the array Q.  LDQ >= max(1,N).
*> \endverbatim
*>
*> \param[in] INDXQ
*> \verbatim
*>          INDXQ is INTEGER array, dimension (N)
*>         The permutation which separately sorts the two sub-problems
*>         in D into ascending order.  Note that elements in the second
*>         half of this permutation must first have CUTPNT added to
*>         their values in order to be accurate.
*> \endverbatim
*>
*> \param[in,out] RHO
*> \verbatim
*>          RHO is DOUBLE PRECISION
*>         On entry, the off-diagonal element associated with the rank-1
*>         cut which originally split the two submatrices which are now
*>         being recombined.
*>         On exit, RHO has been modified to the value required by
*>         DLAED3.
*> \endverbatim
*>
*> \param[in] CUTPNT
*> \verbatim
*>          CUTPNT is INTEGER
*>         The location of the last eigenvalue in the leading
*>         sub-matrix.  min(1,N) <= CUTPNT <= N.
*> \endverbatim
*>
*> \param[in] Z
*> \verbatim
*>          Z is DOUBLE PRECISION array, dimension (N)
*>         On entry, Z contains the updating vector (the last row of
*>         the first sub-eigenvector matrix and the first row of the
*>         second sub-eigenvector matrix).
*>         On exit, the contents of Z are destroyed by the updating
*>         process.
*> \endverbatim
*>
*> \param[out] DLAMDA
*> \verbatim
*>          DLAMDA is DOUBLE PRECISION array, dimension (N)
*>         A copy of the first K eigenvalues which will be used by
*>         DLAED3 to form the secular equation.
*> \endverbatim
*>
*> \param[out] Q2
*> \verbatim
*>          Q2 is DOUBLE PRECISION array, dimension (LDQ2,N)
*>         If ICOMPQ = 0, Q2 is not referenced.  Otherwise,
*>         a copy of the first K eigenvectors which will be used by
*>         DLAED7 in a matrix multiply (DGEMM) to update the new
*>         eigenvectors.
*> \endverbatim
*>
*> \param[in] LDQ2
*> \verbatim
*>          LDQ2 is INTEGER
*>         The leading dimension of the array Q2.  LDQ2 >= max(1,N).
*> \endverbatim
*>
*> \param[out] W
*> \verbatim
*>          W is DOUBLE PRECISION array, dimension (N)
*>         The first k values of the final deflation-altered z-vector and
*>         will be passed to DLAED3.
*> \endverbatim
*>
*> \param[out] PERM
*> \verbatim
*>          PERM is INTEGER array, dimension (N)
*>         The permutations (from deflation and sorting) to be applied
*>         to each eigenblock.
*> \endverbatim
*>
*> \param[out] GIVPTR
*> \verbatim
*>          GIVPTR is INTEGER
*>         The number of Givens rotations which took place in this
*>         subproblem.
*> \endverbatim
*>
*> \param[out] GIVCOL
*> \verbatim
*>          GIVCOL is INTEGER array, dimension (2, N)
*>         Each pair of numbers indicates a pair of columns to take place
*>         in a Givens rotation.
*> \endverbatim
*>
*> \param[out] GIVNUM
*> \verbatim
*>          GIVNUM is DOUBLE PRECISION array, dimension (2, N)
*>         Each number indicates the S value to be used in the
*>         corresponding Givens rotation.
*> \endverbatim
*>
*> \param[out] INDXP
*> \verbatim
*>          INDXP is INTEGER array, dimension (N)
*>         The permutation used to place deflated values of D at the end
*>         of the array.  INDXP(1:K) points to the nondeflated D-values
*>         and INDXP(K+1:N) points to the deflated eigenvalues.
*> \endverbatim
*>
*> \param[out] INDX
*> \verbatim
*>          INDX is INTEGER array, dimension (N)
*>         The permutation used to sort the contents of D into ascending
*>         order.
*> \endverbatim
*>
*> \param[out] INFO
*> \verbatim
*>          INFO is INTEGER
*>          = 0:  successful exit.
*>          < 0:  if INFO = -i, the i-th argument had an illegal value.
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERcomputational
*
*> \par Contributors:
*  ==================
*>
*> Jeff Rutter, Computer Science Division, University of California
*> at Berkeley, USA
*
*  =====================================================================
!      SUBROUTINE DLAED8( ICOMPQ, K, N, QSIZ, D, Q, LDQ, INDXQ, RHO,
!     $                   CUTPNT, Z, DLAMDA, Q2, LDQ2, W, PERM, GIVPTR,
!     $                   GIVCOL, GIVNUM, INDXP, INDX, INFO )
      SUBROUTINE DDLAED8( ICOMPQ, K, N, QSIZ, Dh, Dl, Qh, Ql, LDQ, 
     $                   INDXQ, RHOh, RHOl, 
     $                   CUTPNT, Zh, Zl, DLAMDAh, DLAMDAl, Q2h, Q2l, 
     $                   LDQ2, Wh, Wl, PERM, GIVPTR,
     $                   GIVCOL, GIVNUMh, GIVNUMl, INDXP, INDX, INFO )
*
*  -- LAPACK computational routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      INTEGER            CUTPNT, GIVPTR, ICOMPQ, INFO, K, LDQ, LDQ2, N,
     $                   QSIZ
!      DOUBLE PRECISION   RHO
      double precision RHOh, RHOl
*     ..
*     .. Array Arguments ..
      INTEGER            GIVCOL( 2, * ), INDX( * ), INDXP( * ),
     $                   INDXQ( * ), PERM( * )
!      DOUBLE PRECISION   D( * ), DLAMDA( * ), GIVNUM( 2, * ),
!     $                   Q( LDQ, * ), Q2( LDQ2, * ), W( * ), Z( * )
      double precision Dh(*), Dl(*)
      double precision DLAMDAh(*), DLAMDAl(*)
      double precision GIVNUMh(2,*), GIVNUMl(2,*)
      double precision Qh(LDQ,*), Ql(LDQ,*)
      double precision Q2h(LDQ2,*), Q2l(LDQ2,*)
      double precision Wh(*), Wl(*)
      double precision Zh(*), Zl(*)
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
!      DOUBLE PRECISION   MONE, ZERO, ONE, TWO, EIGHT
!      PARAMETER          ( MONE = -1.0D0, ZERO = 0.0D0, ONE = 1.0D0,
!     $                   TWO = 2.0D0, EIGHT = 8.0D0 )
      double precision MONEh, MONEl
      double precision ZEROh, ZEROl
      double precision ONEh, ONEl
      double precision TWOh, TWOl
      double precision EIGHTh, EIGHTl
*     ..
*     .. Local Scalars ..
*
      INTEGER            I, IMAX, J, JLAM, JMAX, JP, K2, N1, N1P1, N2
!      DOUBLE PRECISION   C, EPS, S, T, TAU, TOL
      double precision Ch, Cl
      double precision EPSh, EPSl
      double precision Sh, Sl
      double precision Th, Tl
      double precision TAUh, TAUl
      double precision TOLh, TOLl
      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision tmp3h, tmp3l
      double precision tmp4h, tmp4l
      double precision tmp5h, tmp5l
      double precision tmp6h, tmp6l
      double precision tmp7h, tmp7l
      double precision tmp8h, tmp8l
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l

*     ..
*     .. External Functions ..
      INTEGER            IDAMAX
!      DOUBLE PRECISION   DLAMCH, DLAPY2
!      EXTERNAL           IDAMAX, DLAMCH, DLAPY2
      EXTERNAL           IDAMAX
*     ..
*     .. External Subroutines ..
      EXTERNAL           DDCOPY, DDLACPY, DDLAMRG, DDROT, DDSCAL, 
     $                   XERBLA, DLAMCH, DLAPY2
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, MAX, MIN, SQRT
*     ..
      MONEh = -1.0d0
      MONEl = 0.0d+0
      ZEROh = 0.0d+0
      ZEROl = 0.0d+0
      ONEh = 1.0d+0
      ONEl = 0.0d+0
      TWOh = 2.0d0
      TWOl = 0.0d+0
      EIGHTh = 8.0d0
      EIGHTl = 0.0d+0

*     .. Executable Statements ..
*
*     Test the input parameters.
*
      INFO = 0
*
      IF( ICOMPQ.LT.0 .OR. ICOMPQ.GT.1 ) THEN
         INFO = -1
      ELSE IF( N.LT.0 ) THEN
         INFO = -3
      ELSE IF( ICOMPQ.EQ.1 .AND. QSIZ.LT.N ) THEN
         INFO = -4
      ELSE IF( LDQ.LT.MAX( 1, N ) ) THEN
         INFO = -7
      ELSE IF( CUTPNT.LT.MIN( 1, N ) .OR. CUTPNT.GT.N ) THEN
         INFO = -10
      ELSE IF( LDQ2.LT.MAX( 1, N ) ) THEN
         INFO = -14
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'DDLAED8', -INFO )
         RETURN
      END IF
*
*     Need to initialize GIVPTR to O here in case of quick exit
*     to prevent an unspecified code behavior (usually sigfault) 
*     when IWORK array on entry to *stedc is not zeroed 
*     (or at least some IWORK entries which used in *laed7 for GIVPTR).
*
      GIVPTR = 0
*
*     Quick return if possible
*
      IF( N.EQ.0 )
     $   RETURN
*
      N1 = CUTPNT
      N2 = N - N1
      N1P1 = N1 + 1
*
      IF(
!         RHO.LT.ZERO
!       (RHO .lt. ZERO)
     + ((RHOh .lt. ZEROh) .or.
     +  ((RHOh .eq. ZEROh) .and.
     +   (RHOl .lt. ZEROl)))
     $    ) THEN
!         CALL DSCAL( N2, MONE, Z( N1P1 ), 1 )
         CALL DDSCAL( N2, MONEh, MONEl, Zh( N1P1 ), Zl( N1P1 ), 1 )
      END IF
*
*     Normalize z so that norm(z) = 1
*
!      T = ONE / SQRT( TWO )
!<<<  tmp1 = sqrt(TWO)
      if (TWOh.eq.0.0D+0.and.TWOl.eq.0.0D+0) then
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
      endif
      if (TWOh.lt.0.0d+0.or.
     +   (TWOh.eq.0.0d+0.and.TWOl.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(TWOh)
      T_ax = TWOh * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           TWOh,TWOl,T_z1,T_z2)
          T_p1 = TWOh - T_z1
          T_p2 = T_p1 - TWOh
          T_t2l = (TWOh
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + TWOl - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp1h = T_t1h
          tmp1l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = sqrt(TWO)
!<<<  T = ONE / tmp1
      T_q1 = ONEh / tmp1h
      Th = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      Tl = ((T_p1*T_r1
     +       - Th)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      Tl = Tl + (tmp1l*T_q1)
      T_s1 = Th + Tl
      Tl = Tl - (T_s1 - Th)
      Th = T_s1
      T_s1 = ONEh - Th
      T_r1 = T_s1 - ONEh
      T_s2 = (ONEh 
     +         - (T_s1 - T_r1))
     +         - (Th + T_r1)
      T_s2 = T_s2 - Tl
      T_s2 = T_s2 + ONEl
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      Th = T_q1 + T_q2
      Tl = T_q2
     +      - (Th - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  T = ONE / tmp1
      DO 10 J = 1, N
         INDX( J ) = J
   10 CONTINUE
!      CALL DSCAL( N, T, Z, 1 )
      CALL DDSCAL( N, Th, Tl, Zh, Zl, 1 )
!      RHO = ABS( TWO*RHO )
!<<<  tmp2 = TWO * RHO
      tmp2h = TWOh * RHOh
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * RHOh
      T_r1 = T_u1
     +         - (T_u1 - RHOh)
      T_r2 = RHOh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (TWOh*RHOl
     +      + TWOl*RHOh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = TWO * RHO
!<<<  RHO = abs(tmp2)
      if (tmp2h .lt. 0.0d+0) then
        RHOh = -tmp2h
        RHOl = -tmp2l
      elseif (
     +  (tmp2h .eq. 0.0d+0).and.
     +  (tmp2l .lt. 0.0d+0)) then
        RHOh = tmp2h
        RHOl = -tmp2l
      else
        RHOh = tmp2h
        RHOl = tmp2l
      endif
!>>>  RHO = abs(tmp2)
*
*     Sort the eigenvalues into increasing order
*
      DO 20 I = CUTPNT + 1, N
         INDXQ( I ) = INDXQ( I ) + CUTPNT
   20 CONTINUE
      DO 30 I = 1, N
!         DLAMDA( I ) = D( INDXQ( I ) )
      DLAMDAh(I) = Dh(INDXQ(I))
      DLAMDAl(I) = Dl(INDXQ(I))
!         W( I ) = Z( INDXQ( I ) )
      Wh(I) = Zh(INDXQ(I))
      Wl(I) = Zl(INDXQ(I))
   30 CONTINUE
      I = 1
      J = CUTPNT + 1
!      CALL DLAMRG( N1, N2, DLAMDA, 1, 1, INDX )
      CALL DDLAMRG( N1, N2, DLAMDAh, DLAMDAl, 1, 1, INDX )
      DO 40 I = 1, N
!         D( I ) = DLAMDA( INDX( I ) )
      Dh(I) = DLAMDAh(INDX(I))
      Dl(I) = DLAMDAl(INDX(I))
!         Z( I ) = W( INDX( I ) )
      Zh(I) = Wh(INDX(I))
      Zl(I) = Wl(INDX(I))
   40 CONTINUE
*
*     Calculate the allowable deflation tolerence
*
      IMAX = IDAMAX( N, Z, 1 )
      JMAX = IDAMAX( N, D, 1 )
!      EPS = DLAMCH( 'Epsilon' )
      CALL DDLAMCH('Epsilon', EPSh, EPSl )
!      TOL = EIGHT*EPS*ABS( D( JMAX ) )
!<<<  tmp1 = abs(D(JMAX))
      if (Dh(JMAX) .lt. 0.0d+0) then
        tmp1h = -Dh(JMAX)
        tmp1l = -Dl(JMAX)
      elseif (
     +  (Dh(JMAX) .eq. 0.0d+0).and.
     +  (Dl(JMAX) .lt. 0.0d+0)) then
        tmp1h = Dh(JMAX)
        tmp1l = -Dl(JMAX)
      else
        tmp1h = Dh(JMAX)
        tmp1l = Dl(JMAX)
      endif
!>>>  tmp1 = abs(D(JMAX))
!<<<  tmp2 = tmp1 * EPS
      tmp2h = tmp1h * EPSh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * EPSh
      T_r1 = T_u1
     +         - (T_u1 - EPSh)
      T_r2 = EPSh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (tmp1h*EPSl
     +      + tmp1l*EPSh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = tmp1 * EPS
!<<<  TOL = EIGHT * tmp2
      TOLh = EIGHTh * tmp2h
      T_u1 = 134217729.0 * EIGHTh
      T_p1 = T_u1
     +         - (T_u1 - EIGHTh)
      T_p2 = EIGHTh - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      TOLl = ((T_p1*T_r1
     +        - TOLh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      TOLl = TOLl
     +      + (EIGHTh*tmp2l
     +      + EIGHTl*tmp2h)
      T_s1 = TOLh + TOLl
      TOLl = TOLl - (T_s1 - TOLh)
      TOLh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  TOL = EIGHT * tmp2
*
*     If the rank-1 modifier is small enough, no more needs to be done
*     except to reorganize Q so that its columns correspond with the
*     elements in D.
*
!<<<  tmp1 = abs(Z(IMAX))
      if (Zh(IMAX) .lt. 0.0d+0) then
        tmp1h = -Zh(IMAX)
        tmp1l = -Zl(IMAX)
      elseif (
     +  (Zh(IMAX) .eq. 0.0d+0).and.
     +  (Zl(IMAX) .lt. 0.0d+0)) then
        tmp1h = Zh(IMAX)
        tmp1l = -Zl(IMAX)
      else
        tmp1h = Zh(IMAX)
        tmp1l = Zl(IMAX)
      endif
!>>>  tmp1 = abs(Z(IMAX))
!<<<  tmp2 = RHO * tmp1
      tmp2h = RHOh * tmp1h
      T_u1 = 134217729.0 * RHOh
      T_p1 = T_u1
     +         - (T_u1 - RHOh)
      T_p2 = RHOh - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (RHOh*tmp1l
     +      + RHOl*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = RHO * tmp1
      IF(
!         RHO*ABS( Z( IMAX ) ).LE.TOL
!       (tmp2 .le. TOL)
     + ((tmp2h .lt. TOLh) .or.
     +  ((tmp2h .eq. TOLh) .and.
     +   (tmp2l .le. TOLl)))
     $   ) THEN
         K = 0
         IF( ICOMPQ.EQ.0 ) THEN
            DO 50 J = 1, N
               PERM( J ) = INDXQ( INDX( J ) )
   50       CONTINUE
         ELSE
            DO 60 J = 1, N
               PERM( J ) = INDXQ( INDX( J ) )
!               CALL DCOPY( QSIZ, Q( 1, PERM( J ) ), 1, Q2( 1, J ), 1 )
               CALL DDCOPY( QSIZ, Qh( 1, PERM( J ) ), 
     $                      Ql( 1, PERM( J ) ), 1, Q2h( 1, J ), 
     $                      Q2l( 1, J ), 1 )
   60       CONTINUE
!            CALL DLACPY( 'A', QSIZ, N, Q2( 1, 1 ), LDQ2, Q( 1, 1 ),
!     $                   LDQ )
            CALL DDLACPY( 'A', QSIZ, N, Q2h( 1, 1 ), Q2l( 1, 1 ), 
     $                   LDQ2, Qh( 1, 1 ), Ql( 1, 1 ), LDQ )
         END IF
         RETURN
      END IF
*
*     If there are multiple eigenvalues then the problem deflates.  Here
*     the number of equal eigenvalues are found.  As each equal
*     eigenvalue is found, an elementary reflector is computed to rotate
*     the corresponding eigensubspace so that the corresponding
*     components of Z are zero in this new basis.
*
      K = 0
      K2 = N + 1
      DO 70 J = 1, N
!<<<  tmp1 = abs(Z(J))
      if (Zh(J) .lt. 0.0d+0) then
        tmp1h = -Zh(J)
        tmp1l = -Zl(J)
      elseif (
     +  (Zh(J) .eq. 0.0d+0).and.
     +  (Zl(J) .lt. 0.0d+0)) then
        tmp1h = Zh(J)
        tmp1l = -Zl(J)
      else
        tmp1h = Zh(J)
        tmp1l = Zl(J)
      endif
!>>>  tmp1 = abs(Z(J))
!<<<  tmp2 = RHO * tmp1
      tmp2h = RHOh * tmp1h
      T_u1 = 134217729.0 * RHOh
      T_p1 = T_u1
     +         - (T_u1 - RHOh)
      T_p2 = RHOh - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (RHOh*tmp1l
     +      + RHOl*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = RHO * tmp1
         IF(
!            RHO*ABS( Z( J ) ).LE.TOL
!       (tmp2 .le. TOL)
     + ((tmp2h .lt. TOLh) .or.
     +  ((tmp2h .eq. TOLh) .and.
     +   (tmp2l .le. TOLl)))
     $       ) THEN
*
*           Deflate due to small z component.
*
            K2 = K2 - 1
            INDXP( K2 ) = J
            IF( J.EQ.N )
     $         GO TO 110
         ELSE
            JLAM = J
            GO TO 80
         END IF
   70 CONTINUE
   80 CONTINUE
      J = J + 1
      IF( J.GT.N )
     $   GO TO 100

!<<<  tmp1 = abs(Z(J))
      if (Zh(J) .lt. 0.0d+0) then
        tmp1h = -Zh(J)
        tmp1l = -Zl(J)
      elseif (
     +  (Zh(J) .eq. 0.0d+0).and.
     +  (Zl(J) .lt. 0.0d+0)) then
        tmp1h = Zh(J)
        tmp1l = -Zl(J)
      else
        tmp1h = Zh(J)
        tmp1l = Zl(J)
      endif
!>>>  tmp1 = abs(Z(J))
!<<<  tmp2 = RHO * tmp1
      tmp2h = RHOh * tmp1h
      T_u1 = 134217729.0 * RHOh
      T_p1 = T_u1
     +         - (T_u1 - RHOh)
      T_p2 = RHOh - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (RHOh*tmp1l
     +      + RHOl*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = RHO * tmp1
         IF(
!            RHO*ABS( Z( J ) ).LE.TOL
!       (tmp2 .le. TOL)
     + ((tmp2h .lt. TOLh) .or.
     +  ((tmp2h .eq. TOLh) .and.
     +   (tmp2l .le. TOLl)))
     $       ) THEN
*
*        Deflate due to small z component.
*
         K2 = K2 - 1
         INDXP( K2 ) = J
      ELSE
*
*        Check if eigenvalues are close enough to allow deflation.
*
!         S = Z( JLAM )
      Sh = Zh(JLAM)
      Sl = Zl(JLAM)
!         C = Z( J )
      Ch = Zh(J)
      Cl = Zl(J)
*
*        Find sqrt(a**2+b**2) without overflow or
*        destructive underflow.
*
!         TAU = DLAPY2( C, S )
         CALL DDLAPY2(Ch, Cl, Sh, Sl, TAUh, TAUl )
!         T = D( J ) - D( JLAM )
!<<<  T = D(J) - D(JLAM)
      T_p1 = Dh(J) - Dh(JLAM)
      T_p2 = T_p1 - Dh(J)
      Th = T_p1
      Tl = (Dh(J) - (T_p1 - T_p2))
     +         - (Dh(JLAM) + T_p2)
      Tl = Tl + Dl(J) - Dl(JLAM)
      T_s1 = Th + Tl
      Tl = Tl - (T_s1 - Th)
      Th = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  T = D(J) - D(JLAM)
!         C = C / TAU
!<<<  C = C / TAU
      T_q1 = Ch / TAUh
      T_ch = TAUh * T_q1
      T_u1 = 134217729.0 * TAUh
      T_p1 = T_u1
     +         - (T_u1 - TAUh)
      T_p2 = TAUh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (TAUl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Ch - T_ch
      T_r1 = T_s1 - Ch
      T_s2 = (Ch
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Cl
      T_q2 = (T_s1 + T_s2)
     +         / TAUh
      Ch = T_q1 + T_q2
      Cl = T_q2
     +      - (Ch - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  C = C / TAU
!         S = -S / TAU
!<<<  S = S / TAU
      T_q1 = Sh / TAUh
      T_ch = TAUh * T_q1
      T_u1 = 134217729.0 * TAUh
      T_p1 = T_u1
     +         - (T_u1 - TAUh)
      T_p2 = TAUh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (TAUl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Sh - T_ch
      T_r1 = T_s1 - Sh
      T_s2 = (Sh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Sl
      T_q2 = (T_s1 + T_s2)
     +         / TAUh
      Sh = T_q1 + T_q2
      Sl = T_q2
     +      - (Sh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  S = S / TAU
!<<<  S = -S
      Sh = -Sh
      Sl = -Sl
!>>>  S = -S
!<<<  tmp1 = T * C
      tmp1h = Th * Ch
      T_u1 = 134217729.0 * Th
      T_p1 = T_u1
     +         - (T_u1 - Th)
      T_p2 = Th - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Th*Cl
     +      + Tl*Ch)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = T * C
!<<<  tmp2 = tmp1 * S
      tmp2h = tmp1h * Sh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * Sh
      T_r1 = T_u1
     +         - (T_u1 - Sh)
      T_r2 = Sh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (tmp1h*Sl
     +      + tmp1l*Sh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = tmp1 * S
!<<<  tmp3 = abs(tmp2)
      if (tmp2h .lt. 0.0d+0) then
        tmp3h = -tmp2h
        tmp3l = -tmp2l
      elseif (
     +  (tmp2h .eq. 0.0d+0).and.
     +  (tmp2l .lt. 0.0d+0)) then
        tmp3h = tmp2h
        tmp3l = -tmp2l
      else
        tmp3h = tmp2h
        tmp3l = tmp2l
      endif
!>>>  tmp3 = abs(tmp2)
         IF(
!            ABS( T*C*S ).LE.TOL
!       (tmp3 .le. TOL)
     + ((tmp3h .lt. TOLh) .or.
     +  ((tmp3h .eq. TOLh) .and.
     +   (tmp3l .le. TOLl)))
     $      ) THEN
*
*           Deflation is possible.
*
!            Z( J ) = TAU
      Zh(J) = TAUh
      Zl(J) = TAUl
!            Z( JLAM ) = ZERO
      Zh(JLAM) = ZEROh
      Zl(JLAM) = ZEROl
*
*           Record the appropriate Givens rotation
*
            GIVPTR = GIVPTR + 1
            GIVCOL( 1, GIVPTR ) = INDXQ( INDX( JLAM ) )
            GIVCOL( 2, GIVPTR ) = INDXQ( INDX( J ) )
!            GIVNUM( 1, GIVPTR ) = C
      GIVNUMh(1,GIVPTR) = Ch
      GIVNUMl(1,GIVPTR) = Cl
!            GIVNUM( 2, GIVPTR ) = S
      GIVNUMh(2,GIVPTR) = Sh
      GIVNUMl(2,GIVPTR) = Sl
            IF( ICOMPQ.EQ.1 ) THEN
!               CALL DROT( QSIZ, Q( 1, INDXQ( INDX( JLAM ) ) ), 1,
!     $                    Q( 1, INDXQ( INDX( J ) ) ), 1, C, S )
               CALL DDROT( QSIZ, Qh( 1, INDXQ( INDX( JLAM ) ) ), 
     $                    Ql( 1, INDXQ( INDX( JLAM ) ) ), 1,
     $                    Qh( 1, INDXQ( INDX( J ) ) ), 
     $                    Ql( 1, INDXQ( INDX( J ) ) ), 1, Ch, Cl,Sh,Sl)
            END IF
!            T = D( JLAM )*C*C + D( J )*S*S
!<<<  tmp1 = D(JLAM) * C
      tmp1h = Dh(JLAM) * Ch
      T_u1 = 134217729.0 * Dh(JLAM)
      T_p1 = T_u1
     +         - (T_u1 - Dh(JLAM))
      T_p2 = Dh(JLAM) - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Dh(JLAM)*Cl
     +      + Dl(JLAM)*Ch)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = D(JLAM) * C
!<<<  tmp2 = tmp1 * C
      tmp2h = tmp1h * Ch
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (tmp1h*Cl
     +      + tmp1l*Ch)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = tmp1 * C
!<<<  tmp3 = D(J) * S
      tmp3h = Dh(J) * Sh
      T_u1 = 134217729.0 * Dh(J)
      T_p1 = T_u1
     +         - (T_u1 - Dh(J))
      T_p2 = Dh(J) - T_p1
      T_u1 = 134217729.0 * Sh
      T_r1 = T_u1
     +         - (T_u1 - Sh)
      T_r2 = Sh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (Dh(J)*Sl
     +      + Dl(J)*Sh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = D(J) * S
!<<<  tmp4 = tmp3 * S
      tmp4h = tmp3h * Sh
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * Sh
      T_r1 = T_u1
     +         - (T_u1 - Sh)
      T_r2 = Sh - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*Sl
     +      + tmp3l*Sh)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * S
!<<<  T = tmp2 + tmp4
      T_p1 = tmp2h + tmp4h
      T_p2 = T_p1 - tmp2h
      Th = T_p1
      Tl = (tmp2h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      Tl = Tl + tmp2l + tmp4l
      T_s1 = Th + Tl
      Tl = Tl - (T_s1 - Th)
      Th = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  T = tmp2 + tmp4
!            D( J ) = D( JLAM )*S*S + D( J )*C*C
!<<<  tmp1 = D(JLAM) * S
      tmp1h = Dh(JLAM) * Sh
      T_u1 = 134217729.0 * Dh(JLAM)
      T_p1 = T_u1
     +         - (T_u1 - Dh(JLAM))
      T_p2 = Dh(JLAM) - T_p1
      T_u1 = 134217729.0 * Sh
      T_r1 = T_u1
     +         - (T_u1 - Sh)
      T_r2 = Sh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Dh(JLAM)*Sl
     +      + Dl(JLAM)*Sh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = D(JLAM) * S
!<<<  tmp2 = tmp1 * S
      tmp2h = tmp1h * Sh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * Sh
      T_r1 = T_u1
     +         - (T_u1 - Sh)
      T_r2 = Sh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (tmp1h*Sl
     +      + tmp1l*Sh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = tmp1 * S
!<<<  tmp3 = D(J) * C
      tmp3h = Dh(J) * Ch
      T_u1 = 134217729.0 * Dh(J)
      T_p1 = T_u1
     +         - (T_u1 - Dh(J))
      T_p2 = Dh(J) - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (Dh(J)*Cl
     +      + Dl(J)*Ch)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = D(J) * C
!<<<  tmp4 = tmp3 * C
      tmp4h = tmp3h * Ch
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*Cl
     +      + tmp3l*Ch)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * C
!<<<  D(J) = tmp2 + tmp4
      T_p1 = tmp2h + tmp4h
      T_p2 = T_p1 - tmp2h
      Dh(J) = T_p1
      Dl(J) = (tmp2h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      Dl(J) = Dl(J) + tmp2l + tmp4l
      T_s1 = Dh(J) + Dl(J)
      Dl(J) = Dl(J) - (T_s1 - Dh(J))
      Dh(J) = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  D(J) = tmp2 + tmp4

!            D( JLAM ) = T
      Dh(JLAM) = Th
      Dl(JLAM) = Tl
            K2 = K2 - 1
            I = 1
   90       CONTINUE
            IF( K2+I.LE.N ) THEN
               IF(
!                  D( JLAM ).LT.D( INDXP( K2+I ) )
!       (D(JLAM) .lt. D(INDXP(K2+I)))
     + ((Dh(JLAM) .lt. Dh(INDXP(K2+I))) .or.
     +  ((Dh(JLAM) .eq. Dh(INDXP(K2+I))) .and.
     +   (Dl(JLAM) .lt. Dl(INDXP(K2+I)))))
     $             ) THEN
                  INDXP( K2+I-1 ) = INDXP( K2+I )
                  INDXP( K2+I ) = JLAM
                  I = I + 1
                  GO TO 90
               ELSE
                  INDXP( K2+I-1 ) = JLAM
               END IF
            ELSE
               INDXP( K2+I-1 ) = JLAM
            END IF
            JLAM = J
         ELSE
            K = K + 1
!            W( K ) = Z( JLAM )
      Wh(K) = Zh(JLAM)
      Wl(K) = Zl(JLAM)
!            DLAMDA( K ) = D( JLAM )
      DLAMDAh(K) = Dh(JLAM)
      DLAMDAl(K) = Dl(JLAM)
            INDXP( K ) = JLAM
            JLAM = J
         END IF
      END IF
      GO TO 80
  100 CONTINUE
*
*     Record the last eigenvalue.
*
      K = K + 1
!      W( K ) = Z( JLAM )
      Wh(K) = Zh(JLAM)
      Wl(K) = Zl(JLAM)
!      DLAMDA( K ) = D( JLAM )
      DLAMDAh(K) = Dh(JLAM)
      DLAMDAl(K) = Dl(JLAM)
      INDXP( K ) = JLAM
*
  110 CONTINUE
*
*     Sort the eigenvalues and corresponding eigenvectors into DLAMDA
*     and Q2 respectively.  The eigenvalues/vectors which were not
*     deflated go into the first K slots of DLAMDA and Q2 respectively,
*     while those which were deflated go into the last N - K slots.
*
      IF( ICOMPQ.EQ.0 ) THEN
         DO 120 J = 1, N
            JP = INDXP( J )
!            DLAMDA( J ) = D( JP )
      DLAMDAh(J) = Dh(JP)
      DLAMDAl(J) = Dl(JP)
            PERM( J ) = INDXQ( INDX( JP ) )
  120    CONTINUE
      ELSE
         DO 130 J = 1, N
            JP = INDXP( J )
!            DLAMDA( J ) = D( JP )
      DLAMDAh(J) = Dh(JP)
      DLAMDAl(J) = Dl(JP)
            PERM( J ) = INDXQ( INDX( JP ) )
!            CALL DCOPY( QSIZ, Q( 1, PERM( J ) ), 1, Q2( 1, J ), 1 )
            CALL DDCOPY( QSIZ, Qh( 1, PERM( J ) ), Ql( 1, PERM( J ) ),
     $                   1, Q2h( 1, J ), Q2l( 1, J ), 1 )
  130    CONTINUE
      END IF
*
*     The deflated eigenvalues and their corresponding vectors go back
*     into the last N - K slots of D and Q respectively.
*
      IF( K.LT.N ) THEN
         IF( ICOMPQ.EQ.0 ) THEN
!            CALL DCOPY( N-K, DLAMDA( K+1 ), 1, D( K+1 ), 1 )
            CALL DDCOPY( N-K, DLAMDAh( K+1 ), DLAMDAl( K+1 ), 1, 
     $                   Dh( K+1 ), Dl( K+1 ), 1 )
         ELSE
!            CALL DCOPY( N-K, DLAMDA( K+1 ), 1, D( K+1 ), 1 )
            CALL DDCOPY( N-K, DLAMDAh( K+1 ), DLAMDAl( K+1 ), 
     $                   1, Dh( K+1 ), Dl( K+1 ), 1 )
!            CALL DLACPY( 'A', QSIZ, N-K, Q2( 1, K+1 ), LDQ2,
!     $                   Q( 1, K+1 ), LDQ )
            CALL DDLACPY( 'A', QSIZ, N-K, Q2h( 1, K+1 ), Q2l( 1, K+1 ),
     $                   LDQ2, Qh( 1, K+1 ), Ql( 1, K+1 ), LDQ )
         END IF
      END IF
*
      RETURN
*
*     End of DLAED8
*
      END
