*> \brief \b DLAED9
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DLAED9 + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dlaed9.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dlaed9.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dlaed9.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DLAED9( K, KSTART, KSTOP, N, D, Q, LDQ, RHO, DLAMDA, W,
*                          S, LDS, INFO )
* 
*       .. Scalar Arguments ..
*       INTEGER            INFO, K, KSTART, KSTOP, LDQ, LDS, N
*       DOUBLE PRECISION   RHO
*       ..
*       .. Array Arguments ..
*       DOUBLE PRECISION   D( * ), DLAMDA( * ), Q( LDQ, * ), S( LDS, * ),
*      $                   W( * )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DLAED9 finds the roots of the secular equation, as defined by the
*> values in D, Z, and RHO, between KSTART and KSTOP.  It makes the
*> appropriate calls to DLAED4 and then stores the new matrix of
*> eigenvectors for use in calculating the next level of Z vectors.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] K
*> \verbatim
*>          K is INTEGER
*>          The number of terms in the rational function to be solved by
*>          DLAED4.  K >= 0.
*> \endverbatim
*>
*> \param[in] KSTART
*> \verbatim
*>          KSTART is INTEGER
*> \endverbatim
*>
*> \param[in] KSTOP
*> \verbatim
*>          KSTOP is INTEGER
*>          The updated eigenvalues Lambda(I), KSTART <= I <= KSTOP
*>          are to be computed.  1 <= KSTART <= KSTOP <= K.
*> \endverbatim
*>
*> \param[in] N
*> \verbatim
*>          N is INTEGER
*>          The number of rows and columns in the Q matrix.
*>          N >= K (delation may result in N > K).
*> \endverbatim
*>
*> \param[out] D
*> \verbatim
*>          D is DOUBLE PRECISION array, dimension (N)
*>          D(I) contains the updated eigenvalues
*>          for KSTART <= I <= KSTOP.
*> \endverbatim
*>
*> \param[out] Q
*> \verbatim
*>          Q is DOUBLE PRECISION array, dimension (LDQ,N)
*> \endverbatim
*>
*> \param[in] LDQ
*> \verbatim
*>          LDQ is INTEGER
*>          The leading dimension of the array Q.  LDQ >= max( 1, N ).
*> \endverbatim
*>
*> \param[in] RHO
*> \verbatim
*>          RHO is DOUBLE PRECISION
*>          The value of the parameter in the rank one update equation.
*>          RHO >= 0 required.
*> \endverbatim
*>
*> \param[in] DLAMDA
*> \verbatim
*>          DLAMDA is DOUBLE PRECISION array, dimension (K)
*>          The first K elements of this array contain the old roots
*>          of the deflated updating problem.  These are the poles
*>          of the secular equation.
*> \endverbatim
*>
*> \param[in] W
*> \verbatim
*>          W is DOUBLE PRECISION array, dimension (K)
*>          The first K elements of this array contain the components
*>          of the deflation-adjusted updating vector.
*> \endverbatim
*>
*> \param[out] S
*> \verbatim
*>          S is DOUBLE PRECISION array, dimension (LDS, K)
*>          Will contain the eigenvectors of the repaired matrix which
*>          will be stored for subsequent Z vector calculation and
*>          multiplied by the previously accumulated eigenvectors
*>          to update the system.
*> \endverbatim
*>
*> \param[in] LDS
*> \verbatim
*>          LDS is INTEGER
*>          The leading dimension of S.  LDS >= max( 1, K ).
*> \endverbatim
*>
*> \param[out] INFO
*> \verbatim
*>          INFO is INTEGER
*>          = 0:  successful exit.
*>          < 0:  if INFO = -i, the i-th argument had an illegal value.
*>          > 0:  if INFO = 1, an eigenvalue did not converge
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERcomputational
*
*> \par Contributors:
*  ==================
*>
*> Jeff Rutter, Computer Science Division, University of California
*> at Berkeley, USA
*
*  =====================================================================
!      SUBROUTINE DLAED9( K, KSTART, KSTOP, N, D, Q, LDQ, RHO, DLAMDA, W,
!     $                   S, LDS, INFO )
      SUBROUTINE DDLAED9( K, KSTART, KSTOP, N, Dh, Dl, Qh, Ql, LDQ, 
     $                   RHOh, RHOl, DLAMDAh, DLAMDAl, Wh, Wl, 
     $                   Sh, Sl, LDS, INFO )
*
*  -- LAPACK computational routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      INTEGER            INFO, K, KSTART, KSTOP, LDQ, LDS, N
!      DOUBLE PRECISION   RHO
      double precision RHOh, RHOl
*     ..
*     .. Array Arguments ..
!      DOUBLE PRECISION   D( * ), DLAMDA( * ), Q( LDQ, * ), S( LDS, * ),
!     $                   W( * )
      double precision Dh(*), Dl(*)
      double precision DLAMDAh(*), DLAMDAl(*)
      double precision Qh(LDQ,*), Ql(LDQ,*)
      double precision Sh(LDS,*), Sl(LDS,*)
      double precision Wh(*), Wl(*)

      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision tmp3h, tmp3l
      double precision tmp4h, tmp4l
      double precision tmp5h, tmp5l
      double precision tmp6h, tmp6l
      double precision tmp7h, tmp7l
      double precision tmp8h, tmp8l
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l
*     ..
*
*  =====================================================================
*
*     .. Local Scalars ..
      INTEGER            I, J
!      DOUBLE PRECISION   TEMP
      double precision TEMPh, TEMPl
*     ..
*     .. External Functions ..
!      DOUBLE PRECISION   DLAMC3, DNRM2
!      EXTERNAL           DLAMC3, DNRM2
*     ..
*     .. External Subroutines ..
      EXTERNAL           DDCOPY, DDLAED4, XERBLA, DDLAMC3, DDNRM2
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX, SIGN, SQRT
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
      INFO = 0
*
      IF( K.LT.0 ) THEN
         INFO = -1
      ELSE IF( KSTART.LT.1 .OR. KSTART.GT.MAX( 1, K ) ) THEN
         INFO = -2
      ELSE IF( MAX( 1, KSTOP ).LT.KSTART .OR. KSTOP.GT.MAX( 1, K ) )
     $          THEN
         INFO = -3
      ELSE IF( N.LT.K ) THEN
         INFO = -4
      ELSE IF( LDQ.LT.MAX( 1, K ) ) THEN
         INFO = -7
      ELSE IF( LDS.LT.MAX( 1, K ) ) THEN
         INFO = -12
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'DDLAED9', -INFO )
         RETURN
      END IF
*
*     Quick return if possible
*
      IF( K.EQ.0 )
     $   RETURN
*
*     Modify values DLAMDA(i) to make sure all DLAMDA(i)-DLAMDA(j) can
*     be computed with high relative accuracy (barring over/underflow).
*     This is a problem on machines without a guard digit in
*     add/subtract (Cray XMP, Cray YMP, Cray C 90 and Cray 2).
*     The following code replaces DLAMDA(I) by 2*DLAMDA(I)-DLAMDA(I),
*     which on any of these machines zeros out the bottommost
*     bit of DLAMDA(I) if it is 1; this makes the subsequent
*     subtractions DLAMDA(I)-DLAMDA(J) unproblematic when cancellation
*     occurs. On binary machines with a guard digit (almost all
*     machines) it does not change DLAMDA(I) at all. On hexadecimal
*     and decimal machines with a guard digit, it slightly
*     changes the bottommost bits of DLAMDA(I). It does not account
*     for hexadecimal or decimal machines without guard digits
*     (we know of none). We use a subroutine call to compute
*     2*DLAMBDA(I) to prevent optimizing compilers from eliminating
*     this code.
*
      DO 10 I = 1, N
!         DLAMDA( I ) = DLAMC3( DLAMDA( I ), DLAMDA( I ) ) - DLAMDA( I )
          CALL DDLAMC3(DLAMDAh(I),DLAMDAl(I),
     &                 DLAMDAh(I),DLAMDAl(I),tmp1h, tmp1l)
!<<<  DLAMDA(I) = tmp1 - DLAMDA(I)
      T_p1 = tmp1h - DLAMDAh(I)
      T_p2 = T_p1 - tmp1h
      T_ch = T_p1
      T_cl = (tmp1h - (T_p1 - T_p2))
     +         - (DLAMDAh(I) + T_p2)
      T_cl = T_cl + tmp1l - DLAMDAl(I)
      T_s1 = T_ch + T_cl
      DLAMDAl(I) = T_cl - (T_s1 - T_ch)
      DLAMDAh(I) = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  DLAMDA(I) = tmp1 - DLAMDA(I)
   10 CONTINUE
*
      DO 20 J = KSTART, KSTOP
!         CALL DLAED4( K, J, DLAMDA, W, Q( 1, J ), RHO, D( J ), INFO )
         CALL DDLAED4( K, J, DLAMDAh, DLAMDAl, Wh, Wl, Qh( 1, J ), 
     $                 Ql( 1, J ), RHOh, RHOl, Dh( J ), Dl( J ), INFO )
*
*        If the zero finder fails, the computation is terminated.
*
         IF( INFO.NE.0 )
     $      GO TO 120
   20 CONTINUE
*
      IF( K.EQ.1 .OR. K.EQ.2 ) THEN
         DO 40 I = 1, K
            DO 30 J = 1, K
!               S( J, I ) = Q( J, I )
      Sh(J,I) = Qh(J,I)
      Sl(J,I) = Ql(J,I)
   30       CONTINUE
   40    CONTINUE
         GO TO 120
      END IF
*
*     Compute updated W.
*
!      CALL DCOPY( K, W, 1, S, 1 )
      CALL DDCOPY( K, Wh, Wl, 1, Sh, Sl, 1 )
*
*     Initialize W(I) = Q(I,I)
*
!      CALL DCOPY( K, Q, LDQ+1, W, 1 )
      CALL DDCOPY( K, Qh, Ql, LDQ+1, Wh, Wl, 1 )
      DO 70 J = 1, K
         DO 50 I = 1, J - 1
!            W( I ) = W( I )*( Q( I, J ) / ( DLAMDA( I )-DLAMDA( J ) ) )
!<<<  tmp1 = DLAMDA(I) - DLAMDA(J)
      T_p1 = DLAMDAh(I) - DLAMDAh(J)
      T_p2 = T_p1 - DLAMDAh(I)
      tmp1h = T_p1
      tmp1l = (DLAMDAh(I) - (T_p1 - T_p2))
     +         - (DLAMDAh(J) + T_p2)
      tmp1l = tmp1l + DLAMDAl(I) - DLAMDAl(J)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = DLAMDA(I) - DLAMDA(J)
!<<<  tmp2 = Q(I,J) / tmp1
      T_q1 = Qh(I,J) / tmp1h
      tmp2h = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp2l = ((T_p1*T_r1
     +       - tmp2h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp2l = tmp2l + (tmp1l*T_q1)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
      T_s1 = Qh(I,J) - tmp2h
      T_r1 = T_s1 - Qh(I,J)
      T_s2 = (Qh(I,J) 
     +         - (T_s1 - T_r1))
     +         - (tmp2h + T_r1)
      T_s2 = T_s2 - tmp2l
      T_s2 = T_s2 + Ql(I,J)
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2
     +      - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp2 = Q(I,J) / tmp1
!<<<  W(I) = W(I) * tmp2
      T_ch = Wh(I) * tmp2h
      T_u1 = 134217729.0 * Wh(I)
      T_p1 = T_u1
     +         - (T_u1 - Wh(I))
      T_p2 = Wh(I) - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Wh(I)*tmp2l
     +         +  Wl(I)*tmp2h)
      T_s1 = T_ch + T_cl
      Wl(I) = T_cl - (T_s1 - T_ch)
      Wh(I) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  W(I) = W(I) * tmp2
   50    CONTINUE
         DO 60 I = J + 1, K
!            W( I ) = W( I )*( Q( I, J ) / ( DLAMDA( I )-DLAMDA( J ) ) )
!<<<  tmp1 = DLAMDA(I) - DLAMDA(J)
      T_p1 = DLAMDAh(I) - DLAMDAh(J)
      T_p2 = T_p1 - DLAMDAh(I)
      tmp1h = T_p1
      tmp1l = (DLAMDAh(I) - (T_p1 - T_p2))
     +         - (DLAMDAh(J) + T_p2)
      tmp1l = tmp1l + DLAMDAl(I) - DLAMDAl(J)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = DLAMDA(I) - DLAMDA(J)
!<<<  tmp2 = Q(I,J) / tmp1
      T_q1 = Qh(I,J) / tmp1h
      tmp2h = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp2l = ((T_p1*T_r1
     +       - tmp2h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp2l = tmp2l + (tmp1l*T_q1)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
      T_s1 = Qh(I,J) - tmp2h
      T_r1 = T_s1 - Qh(I,J)
      T_s2 = (Qh(I,J) 
     +         - (T_s1 - T_r1))
     +         - (tmp2h + T_r1)
      T_s2 = T_s2 - tmp2l
      T_s2 = T_s2 + Ql(I,J)
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2
     +      - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp2 = Q(I,J) / tmp1
!<<<  W(I) = W(I) * tmp2
      T_ch = Wh(I) * tmp2h
      T_u1 = 134217729.0 * Wh(I)
      T_p1 = T_u1
     +         - (T_u1 - Wh(I))
      T_p2 = Wh(I) - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Wh(I)*tmp2l
     +         +  Wl(I)*tmp2h)
      T_s1 = T_ch + T_cl
      Wl(I) = T_cl - (T_s1 - T_ch)
      Wh(I) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  W(I) = W(I) * tmp2
   60    CONTINUE
   70 CONTINUE
      DO 80 I = 1, K
!<<<  tmp1 = -W(I)
      tmp1h = -Wh(I)
      tmp1l = -Wl(I)
!>>>  tmp1 = -W(I)
!         W( I ) = SIGN( SQRT( -W( I ) ), S( I, 1 ) )
          IF(
!       (S(I,1) .gt. 0.0)
     + ((Sh(I,1) .gt. 0.0d+0) .or.
     +  ((Sh(I,1) .eq. 0.0d+0) .and.
     +   (Sl(I,1) .gt. 0.0d+0)))
     $      ) THEN
!<<<  tmp2 = sqrt(tmp1)
      if (tmp1h.eq.0.0D+0.and.tmp1l.eq.0.0D+0) then
        tmp2h = 0.0d+0
        tmp2l = 0.0d+0
      endif
      if (tmp1h.lt.0.0d+0.or.
     +   (tmp1h.eq.0.0d+0.and.tmp1l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp2h = 0.0d+0
        tmp2l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp1h)
      T_ax = tmp1h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp1h,tmp1l,T_z1,T_z2)
          T_p1 = tmp1h - T_z1
          T_p2 = T_p1 - tmp1h
          T_t2l = (tmp1h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp1l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp2h = T_t1h
          tmp2l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp2 = sqrt(tmp1)
!<<<  W(I) = abs(tmp2)
      if (tmp2h .lt. 0.0d+0) then
        Wh(I) = -tmp2h
        Wl(I) = -tmp2l
      elseif (
     +  (tmp2h .eq. 0.0d+0).and.
     +  (tmp2l .lt. 0.0d+0)) then
        Wh(I) = tmp2h
        Wl(I) = -tmp2l
      else
        Wh(I) = tmp2h
        Wl(I) = tmp2l
      endif
!>>>  W(I) = abs(tmp2)
          ELSE
!<<<  tmp2 = sqrt(tmp1)
      if (tmp1h.eq.0.0D+0.and.tmp1l.eq.0.0D+0) then
        tmp2h = 0.0d+0
        tmp2l = 0.0d+0
      endif
      if (tmp1h.lt.0.0d+0.or.
     +   (tmp1h.eq.0.0d+0.and.tmp1l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp2h = 0.0d+0
        tmp2l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp1h)
      T_ax = tmp1h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp1h,tmp1l,T_z1,T_z2)
          T_p1 = tmp1h - T_z1
          T_p2 = T_p1 - tmp1h
          T_t2l = (tmp1h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp1l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp2h = T_t1h
          tmp2l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp2 = sqrt(tmp1)
!<<<  W(I) = abs(tmp2)
      if (tmp2h .lt. 0.0d+0) then
        Wh(I) = -tmp2h
        Wl(I) = -tmp2l
      elseif (
     +  (tmp2h .eq. 0.0d+0).and.
     +  (tmp2l .lt. 0.0d+0)) then
        Wh(I) = tmp2h
        Wl(I) = -tmp2l
      else
        Wh(I) = tmp2h
        Wl(I) = tmp2l
      endif
!>>>  W(I) = abs(tmp2)
!<<<  W(I) = -W(I)
      Wh(I) = -Wh(I)
      Wl(I) = -Wl(I)
!>>>  W(I) = -W(I)
          END IF
   80 CONTINUE
*
*     Compute eigenvectors of the modified rank-1 modification.
*
      DO 110 J = 1, K
         DO 90 I = 1, K
!            Q( I, J ) = W( I ) / Q( I, J )
!<<<  Q(I,J) = W(I) / Q(I,J)
      T_q1 = Wh(I) / Qh(I,J)
      T_ch = Qh(I,J) * T_q1
      T_u1 = 134217729.0 * Qh(I,J)
      T_p1 = T_u1
     +         - (T_u1 - Qh(I,J))
      T_p2 = Qh(I,J) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Ql(I,J) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Wh(I) - T_ch
      T_r1 = T_s1 - Wh(I)
      T_s2 = (Wh(I)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Wl(I)
      T_q2 = (T_s1 + T_s2)
     +         / Qh(I,J)
      Qh(I,J) = T_q1 + T_q2
      Ql(I,J) = T_q2
     +      - (Qh(I,J) - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  Q(I,J) = W(I) / Q(I,J)
   90    CONTINUE
         TEMPh = DNRM2( K, Qh( 1, J ), 1 )
         TEMPl = 0.0d0
!         CALL DDNRM2(K,Qh( 1, J ), Ql( 1, J ), 1, TEMPh, TEMPl )
         DO 100 I = 1, K
!            S( I, J ) = Q( I, J ) / TEMP
!<<<  S(I,J) = Q(I,J) / TEMP
      T_q1 = Qh(I,J) / TEMPh
      Sh(I,J) = TEMPh * T_q1
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      Sl(I,J) = ((T_p1*T_r1
     +       - Sh(I,J))
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      Sl(I,J) = Sl(I,J) + (TEMPl*T_q1)
      T_s1 = Sh(I,J) + Sl(I,J)
      Sl(I,J) = Sl(I,J) - (T_s1 - Sh(I,J))
      Sh(I,J) = T_s1
      T_s1 = Qh(I,J) - Sh(I,J)
      T_r1 = T_s1 - Qh(I,J)
      T_s2 = (Qh(I,J) 
     +         - (T_s1 - T_r1))
     +         - (Sh(I,J) + T_r1)
      T_s2 = T_s2 - Sl(I,J)
      T_s2 = T_s2 + Ql(I,J)
      T_q2 = (T_s1 + T_s2)
     +         / TEMPh
      Sh(I,J) = T_q1 + T_q2
      Sl(I,J) = T_q2
     +      - (Sh(I,J) - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  S(I,J) = Q(I,J) / TEMP
  100    CONTINUE
  110 CONTINUE
*
  120 CONTINUE
      RETURN
*
*     End of DLAED9
*
      END
