*> \brief \b DLANSY
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DLANSY + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dlansy.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dlansy.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dlansy.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       DOUBLE PRECISION FUNCTION DLANSY( NORM, UPLO, N, A, LDA, WORK )
* 
*       .. Scalar Arguments ..
*       CHARACTER          NORM, UPLO
*       INTEGER            LDA, N
*       ..
*       .. Array Arguments ..
*       DOUBLE PRECISION   A( LDA, * ), WORK( * )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DLANSY  returns the value of the one norm,  or the Frobenius norm, or
*> the  infinity norm,  or the  element of  largest absolute value  of a
*> real symmetric matrix A.
*> \endverbatim
*>
*> \return DLANSY
*> \verbatim
*>
*>    DLANSY = ( max(abs(A(i,j))), NORM = 'M' or 'm'
*>             (
*>             ( norm1(A),         NORM = '1', 'O' or 'o'
*>             (
*>             ( normI(A),         NORM = 'I' or 'i'
*>             (
*>             ( normF(A),         NORM = 'F', 'f', 'E' or 'e'
*>
*> where  norm1  denotes the  one norm of a matrix (maximum column sum),
*> normI  denotes the  infinity norm  of a matrix  (maximum row sum) and
*> normF  denotes the  Frobenius norm of a matrix (square root of sum of
*> squares).  Note that  max(abs(A(i,j)))  is not a consistent matrix norm.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] NORM
*> \verbatim
*>          NORM is CHARACTER*1
*>          Specifies the value to be returned in DLANSY as described
*>          above.
*> \endverbatim
*>
*> \param[in] UPLO
*> \verbatim
*>          UPLO is CHARACTER*1
*>          Specifies whether the upper or lower triangular part of the
*>          symmetric matrix A is to be referenced.
*>          = 'U':  Upper triangular part of A is referenced
*>          = 'L':  Lower triangular part of A is referenced
*> \endverbatim
*>
*> \param[in] N
*> \verbatim
*>          N is INTEGER
*>          The order of the matrix A.  N >= 0.  When N = 0, DLANSY is
*>          set to zero.
*> \endverbatim
*>
*> \param[in] A
*> \verbatim
*>          A is DOUBLE PRECISION array, dimension (LDA,N)
*>          The symmetric matrix A.  If UPLO = 'U', the leading n by n
*>          upper triangular part of A contains the upper triangular part
*>          of the matrix A, and the strictly lower triangular part of A
*>          is not referenced.  If UPLO = 'L', the leading n by n lower
*>          triangular part of A contains the lower triangular part of
*>          the matrix A, and the strictly upper triangular part of A is
*>          not referenced.
*> \endverbatim
*>
*> \param[in] LDA
*> \verbatim
*>          LDA is INTEGER
*>          The leading dimension of the array A.  LDA >= max(N,1).
*> \endverbatim
*>
*> \param[out] WORK
*> \verbatim
*>          WORK is DOUBLE PRECISION array, dimension (MAX(1,LWORK)),
*>          where LWORK >= N when NORM = 'I' or '1' or 'O'; otherwise,
*>          WORK is not referenced.
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup doubleSYauxiliary
*
*  =====================================================================
!!      DOUBLE PRECISION FUNCTION DLANSY( NORM, UPLO, N, A, LDA, WORK )
        subroutine dDLANSY( NORM, UPLO, N, Ah, Al, LDA, WORKh, WORKl,
     $                      dlansyh, dlansyl )

*
*  -- LAPACK auxiliary routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      CHARACTER          NORM, UPLO
      INTEGER            LDA, N
*     ..
*     .. Array Arguments ..
!!      DOUBLE PRECISION   A( LDA, * ), WORK( * )
      double precision Ah(LDA,*), Al(LDA,*)
      double precision WORKh(*), WORKl(*)
      double precision dlansyh, dlansyl
*     ..
*
* =====================================================================
*
*     .. Parameters ..
!!      DOUBLE PRECISION   ONE, ZERO
!!      PARAMETER          ( ONE = 1.0D+0, ZERO = 0.0D+0 )
      double precision ONEh, ONEl
      double precision ZEROh, ZEROl

*     ..
*     .. Local Scalars ..
      INTEGER            I, J
!!      DOUBLE PRECISION   ABSA, SCALE, SUM, VALUE
      double precision ABSAh, ABSAl
      double precision SCALEh, SCALEl
      double precision SUMh, SUMl
      double precision VALUEh, VALUEl
*     ..
*     .. External Subroutines ..
!!      EXTERNAL           DLASSQ
      EXTERNAL           dDLASSQ

*     ..
*     .. External Functions ..
      LOGICAL            LSAME
      EXTERNAL           LSAME
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, MAX, SQRT
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l
      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
*     ..
*     .. Executable Statements ..
      ONEh = 1.0d+0
      ONEl = 0.0d+0
      ZEROh = 0.0d+0
      ZEROl = 0.0d+0
*
      IF( N.EQ.0 ) THEN
!!         VALUE = ZERO
      VALUEh = ZEROh
      VALUEl = ZEROl
      ELSE IF( LSAME( NORM, 'M' ) ) THEN
*
*        Find max(abs(A(i,j))).
*
!!         VALUE = ZERO
      VALUEh = ZEROh
      VALUEl = ZEROl
         IF( LSAME( UPLO, 'U' ) ) THEN
            DO 20 J = 1, N
               DO 10 I = 1, J
!!                  VALUE = MAX( VALUE, ABS( A( I, J ) ) )
!<<<  tmp1 = abs(A(I,J))
      if (Ah(I,J) .lt. 0.0d+0) then
        tmp1h = -Ah(I,J)
        tmp1l = -Al(I,J)
      elseif (
     +  (Ah(I,J) .eq. 0.0d+0).and.
     +  (Al(I,J) .lt. 0.0d+0)) then
        tmp1h = Ah(I,J)
        tmp1l = -Al(I,J)
      else
        tmp1h = Ah(I,J)
        tmp1l = Al(I,J)
      endif
!>>>  tmp1 = abs(A(I,J))
                  IF(
!       (VALUE .lt. tmp1)
     + ((VALUEh .lt. tmp1h) .or.
     +  ((VALUEh .eq. tmp1h) .and.
     +   (VALUEl .lt. tmp1l)))
     $               ) THEN
      VALUEh = tmp1h
      VALUEl = tmp1l
                  END IF
   10          CONTINUE
   20       CONTINUE
         ELSE
            DO 40 J = 1, N
               DO 30 I = J, N
!!                  VALUE = MAX( VALUE, ABS( A( I, J ) ) )
!<<<  tmp1 = abs(A(I,J))
      if (Ah(I,J) .lt. 0.0d+0) then
        tmp1h = -Ah(I,J)
        tmp1l = -Al(I,J)
      elseif (
     +  (Ah(I,J) .eq. 0.0d+0).and.
     +  (Al(I,J) .lt. 0.0d+0)) then
        tmp1h = Ah(I,J)
        tmp1l = -Al(I,J)
      else
        tmp1h = Ah(I,J)
        tmp1l = Al(I,J)
      endif
!>>>  tmp1 = abs(A(I,J))
                  IF(
!       (VALUE .lt. tmp1)
     + ((VALUEh .lt. tmp1h) .or.
     +  ((VALUEh .eq. tmp1h) .and.
     +   (VALUEl .lt. tmp1l)))
     $               ) THEN
      VALUEh = tmp1h
      VALUEl = tmp1l
                  END IF
   30          CONTINUE
   40       CONTINUE
         END IF
      ELSE IF( ( LSAME( NORM, 'I' ) ) .OR. ( LSAME( NORM, 'O' ) ) .OR.
     $         ( NORM.EQ.'1' ) ) THEN
*
*        Find normI(A) ( = norm1(A), since A is symmetric).
*
!!         VALUE = ZERO
      VALUEh = ZEROh
      VALUEl = ZEROl
         IF( LSAME( UPLO, 'U' ) ) THEN
            DO 60 J = 1, N
!!               SUM = ZERO
      SUMh = ZEROh
      SUMl = ZEROl
               DO 50 I = 1, J - 1
!!                  ABSA = ABS( A( I, J ) )
!<<<  ABSA = abs(A(I,J))
      if (Ah(I,J) .lt. 0.0d+0) then
        ABSAh = -Ah(I,J)
        ABSAl = -Al(I,J)
      elseif (
     +  (Ah(I,J) .eq. 0.0d+0).and.
     +  (Al(I,J) .lt. 0.0d+0)) then
        ABSAh = Ah(I,J)
        ABSAl = -Al(I,J)
      else
        ABSAh = Ah(I,J)
        ABSAl = Al(I,J)
      endif
!>>>  ABSA = abs(A(I,J))
!!                  SUM = SUM + ABSA
!<<<  tmp1 = SUM + ABSA
      T_p1 = SUMh + ABSAh
      T_p2 = T_p1 - SUMh
      T_q1 = T_p1
      T_q2 = (SUMh - (T_p1 - T_p2))
     +         + (ABSAh - T_p2)
      T_q2 = T_q2 + SUMl + ABSAl
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = SUM + ABSA
      SUMh = tmp1h
      SUMl = tmp1l
!!                  WORK( I ) = WORK( I ) + ABSA
!<<<  tmp1 = WORK(I) + ABSA
      T_p1 = WORKh(I) + ABSAh
      T_p2 = T_p1 - WORKh(I)
      T_q1 = T_p1
      T_q2 = (WORKh(I) - (T_p1 - T_p2))
     +         + (ABSAh - T_p2)
      T_q2 = T_q2 + WORKl(I) + ABSAl
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = WORK(I) + ABSA
      WORKh(I) = tmp1h
      WORKl(I) = tmp1l
   50          CONTINUE
!!               WORK( J ) = SUM + ABS( A( J, J ) )
!<<<  tmp1 = abs(A(J,J))
      if (Ah(J,J) .lt. 0.0d+0) then
        tmp1h = -Ah(J,J)
        tmp1l = -Al(J,J)
      elseif (
     +  (Ah(J,J) .eq. 0.0d+0).and.
     +  (Al(J,J) .lt. 0.0d+0)) then
        tmp1h = Ah(J,J)
        tmp1l = -Al(J,J)
      else
        tmp1h = Ah(J,J)
        tmp1l = Al(J,J)
      endif
!>>>  tmp1 = abs(A(J,J))
!<<<  tmp2 = SUM + tmp1
      T_p1 = SUMh + tmp1h
      T_p2 = T_p1 - SUMh
      T_q1 = T_p1
      T_q2 = (SUMh - (T_p1 - T_p2))
     +         + (tmp1h - T_p2)
      T_q2 = T_q2 + SUMl + tmp1l
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2 - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp2 = SUM + tmp1
      WORKh(J) = tmp2h
      WORKl(J) = tmp2l
   60       CONTINUE
            DO 70 I = 1, N
!!               VALUE = MAX( VALUE, WORK( I ) )
               IF(
!       (VALUE .lt. WORK(I))
     + ((VALUEh .lt. WORKh(I)) .or.
     +  ((VALUEh .eq. WORKh(I)) .and.
     +   (VALUEl .lt. WORKl(I))))
     $           ) THEN
      VALUEh = WORKh(I)
      VALUEl = WORKl(I)
               END IF
   70       CONTINUE
         ELSE
            DO 80 I = 1, N
!!               WORK( I ) = ZERO
      WORKh(I) = ZEROh
      WORKl(I) = ZEROl
   80       CONTINUE
            DO 100 J = 1, N
!!               SUM = WORK( J ) + ABS( A( J, J ) )
!<<<  tmp1 = abs(A(J,J))
      if (Ah(J,J) .lt. 0.0d+0) then
        tmp1h = -Ah(J,J)
        tmp1l = -Al(J,J)
      elseif (
     +  (Ah(J,J) .eq. 0.0d+0).and.
     +  (Al(J,J) .lt. 0.0d+0)) then
        tmp1h = Ah(J,J)
        tmp1l = -Al(J,J)
      else
        tmp1h = Ah(J,J)
        tmp1l = Al(J,J)
      endif
!>>>  tmp1 = abs(A(J,J))
!<<<  SUM = WORK(J) + tmp1
      T_p1 = WORKh(J) + tmp1h
      T_p2 = T_p1 - WORKh(J)
      T_q1 = T_p1
      T_q2 = (WORKh(J) - (T_p1 - T_p2))
     +         + (tmp1h - T_p2)
      T_q2 = T_q2 + WORKl(J) + tmp1l
      SUMh = T_q1 + T_q2
      SUMl = T_q2 - (SUMh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  SUM = WORK(J) + tmp1
               DO 90 I = J + 1, N
!!                  ABSA = ABS( A( I, J ) )
!<<<  ABSA = abs(A(I,J))
      if (Ah(I,J) .lt. 0.0d+0) then
        ABSAh = -Ah(I,J)
        ABSAl = -Al(I,J)
      elseif (
     +  (Ah(I,J) .eq. 0.0d+0).and.
     +  (Al(I,J) .lt. 0.0d+0)) then
        ABSAh = Ah(I,J)
        ABSAl = -Al(I,J)
      else
        ABSAh = Ah(I,J)
        ABSAl = Al(I,J)
      endif
!>>>  ABSA = abs(A(I,J))
!!                  SUM = SUM + ABSA
!<<<  tmp1 = SUM + ABSA
      T_p1 = SUMh + ABSAh
      T_p2 = T_p1 - SUMh
      T_q1 = T_p1
      T_q2 = (SUMh - (T_p1 - T_p2))
     +         + (ABSAh - T_p2)
      T_q2 = T_q2 + SUMl + ABSAl
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = SUM + ABSA
      SUMh = tmp1h
      SUMl = tmp1l
!!                  WORK( I ) = WORK( I ) + ABSA
!<<<  tmp1 = WORK(I) + ABSA
      T_p1 = WORKh(I) + ABSAh
      T_p2 = T_p1 - WORKh(I)
      T_q1 = T_p1
      T_q2 = (WORKh(I) - (T_p1 - T_p2))
     +         + (ABSAh - T_p2)
      T_q2 = T_q2 + WORKl(I) + ABSAl
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = WORK(I) + ABSA
      WORKh(I) = tmp1h
      WORKl(I) = tmp1l
   90          CONTINUE
!!               VALUE = MAX( VALUE, SUM )
               IF(
!       (VALUE .lt. SUM)
     + ((VALUEh .lt. SUMh) .or.
     +  ((VALUEh .eq. SUMh) .and.
     +   (VALUEl .lt. SUMl)))
     $           ) THEN
      VALUEh = SUMh
      VALUEl = SUMl
               END IF
  100       CONTINUE
         END IF
      ELSE IF( ( LSAME( NORM, 'F' ) ) .OR. ( LSAME( NORM, 'E' ) ) ) THEN
*
*        Find normF(A).
*
!!         SCALE = ZERO
      SCALEh = ZEROh
      SCALEl = ZEROl
!!         SUM = ONE
      SUMh = ONEh
      SUMl = ONEl
         IF( LSAME( UPLO, 'U' ) ) THEN
            DO 110 J = 2, N
!!               CALL DLASSQ( J-1, A( 1, J ), 1, SCALE, SUM )
               CALL dDLASSQ( J-1, Ah( 1, J ), Al( 1, J ), 1, SCALEh,
     $                       SCALEl, SUMh, SUMl )
  110       CONTINUE
         ELSE
            DO 120 J = 1, N - 1
!!               CALL DLASSQ( N-J, A( J+1, J ), 1, SCALE, SUM )
               CALL dDLASSQ( N-J, Ah( J+1, J ), Al( J+1, J ), 1, 
     $                       SCALEh, SCALEl, SUMh, SUMl )
  120       CONTINUE
         END IF
!!         SUM = 2*SUM
      tmp1h = 2.0D0
      tmp1l = 0.0d+0
!<<<  tmp2 = tmp1 * SUM
      T_ch = tmp1h * SUMh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * SUMh
      T_r1 = T_u1
     +         - (T_u1 - SUMh)
      T_r2 = SUMh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*SUMl
     +         +  tmp1l*SUMh)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = tmp1 * SUM
      SUMh = tmp2h
      SUMl = tmp2l
!!         CALL DLASSQ( N, A, LDA+1, SCALE, SUM )
         CALL dDLASSQ( N, A, LDA+1, SCALEh, SCALEl, SUMh, SUMl )
!!         VALUE = SCALE*SQRT( SUM )
!<<<  tmp1 = sqrt(SUM)
      if (SUMh.eq.0.0D+0.and.SUMl.eq.0.0D+0) then
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
      else
      if (SUMh.lt.0.0d+0.or.
     +   (SUMh.eq.0.0d+0.and.SUMl.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(SUMh)
      T_ax = SUMh * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           SUMh,SUMl,T_z1,T_z2)
          T_p1 = SUMh - T_z1
          T_p2 = T_p1 - SUMh
          T_t2l = (SUMh
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + SUMl - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp1h = T_t1h
          tmp1l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  tmp1 = sqrt(SUM)
!<<<  VALUE = SCALE * tmp1
      T_ch = SCALEh * tmp1h
      T_u1 = 134217729.0 * SCALEh
      T_p1 = T_u1
     +         - (T_u1 - SCALEh)
      T_p2 = SCALEh - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (SCALEh*tmp1l
     +         +  SCALEl*tmp1h)
      T_s1 = T_ch + T_cl
      VALUEl = T_cl - (T_s1 - T_ch)
      VALUEh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  VALUE = SCALE * tmp1
      END IF
*
!!      DLANSY = VALUE
      dlansyh = VALUEh
      dlansyl = VALUEl
      RETURN
*
*     End of DLANSY
*
      END
