*> \brief \b DLAPY2
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DLAPY2 + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dlapy2.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dlapy2.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dlapy2.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       DOUBLE PRECISION FUNCTION DLAPY2( X, Y )
* 
*       .. Scalar Arguments ..
*       DOUBLE PRECISION   X, Y
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DLAPY2 returns sqrt(x**2+y**2), taking care not to cause unnecessary
*> overflow.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] X
*> \verbatim
*>          X is DOUBLE PRECISION
*> \endverbatim
*>
*> \param[in] Y
*> \verbatim
*>          Y is DOUBLE PRECISION
*>          X and Y specify the values x and y.
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERauxiliary
*
*  =====================================================================
!      DOUBLE PRECISION FUNCTION DLAPY2( X, Y )
      SUBROUTINE DDLAPY2( Xh,Xl,Yh,Yl,DLAPY2h,DLAPY2l )
*
*  -- LAPACK auxiliary routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
!      DOUBLE PRECISION   X, Y
      double precision Xh, Xl
      double precision Yh, Yl
      double precision DLAPY2h, DLAPY2l
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
!      DOUBLE PRECISION   ZERO
!      PARAMETER          ( ZERO = 0.0D0 )
!      DOUBLE PRECISION   ONE
!      PARAMETER          ( ONE = 1.0D0 )
      double precision ZEROh, ZEROl
      double precision ONEh, ONEl
*     ..
*     .. Local Scalars ..
!      DOUBLE PRECISION   W, XABS, YABS, Z
      double precision Wh, Wl
      double precision XABSh, XABSl
      double precision YABSh, YABSl
      double precision Zh, Zl
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, MAX, MIN, SQRT
*     ..
      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision tmp3h, tmp3l
      double precision tmp4h, tmp4l
      double precision tmp5h, tmp5l
      double precision tmp6h, tmp6l
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l
*
      ZEROh = 0.0d+0
      ZEROl = 0.0d+0
      ONEh = 1.0d+0
      ONEl = 0.0d+0
*     .. Executable Statements ..
*
!      XABS = ABS( X )
!<<<  XABS = abs(X)
      if (Xh .lt. 0.0d+0) then
        XABSh = -Xh
        XABSl = -Xl
      elseif (
     +  (Xh .eq. 0.0d+0).and.
     +  (Xl .lt. 0.0d+0)) then
        XABSh = Xh
        XABSl = -Xl
      else
        XABSh = Xh
        XABSl = Xl
      endif
!>>>  XABS = abs(X)
!      YABS = ABS( Y )
!<<<  YABS = abs(Y)
      if (Yh .lt. 0.0d+0) then
        YABSh = -Yh
        YABSl = -Yl
      elseif (
     +  (Yh .eq. 0.0d+0).and.
     +  (Yl .lt. 0.0d+0)) then
        YABSh = Yh
        YABSl = -Yl
      else
        YABSh = Yh
        YABSl = Yl
      endif
!>>>  YABS = abs(Y)
!      W = MAX( XABS, YABS )
      IF(
!       (XABS .gt. YABS)
     + ((XABSh .gt. YABSh) .or.
     +  ((XABSh .eq. YABSh) .and.
     +   (XABSl .gt. YABSl)))
     &    ) THEN
      Wh = XABSh
      Wl = XABSl
      ELSE
      Wh = YABSh
      Wl = YABSl
      END IF
!      Z = MIN( XABS, YABS )
      IF(
!       (XABS .lt. YABS)
     + ((XABSh .lt. YABSh) .or.
     +  ((XABSh .eq. YABSh) .and.
     +   (XABSl .lt. YABSl)))
     &    ) THEN
      Zh = XABSh
      Zl = XABSl
      ELSE
      Zh = YABSh
      Zl = YABSl
      END IF

      IF( 
!        Z.EQ.ZERO 
!       (Z .eq. ZERO)
     + ((Zh .eq. ZEROh) .and.
     +  (Zl .eq. ZEROl))
     &    ) THEN
!         DLAPY2 = W
      DLAPY2h = Wh
      DLAPY2l = Wl
      ELSE
!         DLAPY2 = W*SQRT( ONE+( Z / W )**2 )
!<<<  tmp1 = Z / W
      T_q1 = Zh / Wh
      tmp1h = Wh * T_q1
      T_u1 = 134217729.0 * Wh
      T_p1 = T_u1
     +         - (T_u1 - Wh)
      T_p2 = Wh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp1l = ((T_p1*T_r1
     +       - tmp1h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp1l = tmp1l + (Wl*T_q1)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
      T_s1 = Zh - tmp1h
      T_r1 = T_s1 - Zh
      T_s2 = (Zh 
     +         - (T_s1 - T_r1))
     +         - (tmp1h + T_r1)
      T_s2 = T_s2 - tmp1l
      T_s2 = T_s2 + Zl
      T_q2 = (T_s1 + T_s2)
     +         / Wh
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = Z / W
!<<<  tmp2 = tmp1 * tmp1
      tmp2h = tmp1h * tmp1h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (tmp1h*tmp1l
     +      + tmp1l*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = tmp1 * tmp1
!<<<  tmp3 = ONE + tmp2
      T_p1 = ONEh + tmp2h
      T_p2 = T_p1 - ONEh
      tmp3h = T_p1
      tmp3l = (ONEh - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      tmp3l = tmp3l + ONEl + tmp2l
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp3 = ONE + tmp2
!<<<  tmp4 = sqrt(tmp3)
      if (tmp3h.eq.0.0D+0.and.tmp3l.eq.0.0D+0) then
        tmp4h = 0.0d+0
        tmp4l = 0.0d+0
      endif
      if (tmp3h.lt.0.0d+0.or.
     +   (tmp3h.eq.0.0d+0.and.tmp3l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp4h = 0.0d+0
        tmp4l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp3h)
      T_ax = tmp3h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp3h,tmp3l,T_z1,T_z2)
          T_p1 = tmp3h - T_z1
          T_p2 = T_p1 - tmp3h
          T_t2l = (tmp3h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp3l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp4h = T_t1h
          tmp4l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp4 = sqrt(tmp3)
!<<<  DLAPY2 = W * tmp4
      DLAPY2h = Wh * tmp4h
      T_u1 = 134217729.0 * Wh
      T_p1 = T_u1
     +         - (T_u1 - Wh)
      T_p2 = Wh - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      DLAPY2l = ((T_p1*T_r1
     +        - DLAPY2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      DLAPY2l = DLAPY2l
     +      + (Wh*tmp4l
     +      + Wl*tmp4h)
      T_s1 = DLAPY2h + DLAPY2l
      DLAPY2l = DLAPY2l - (T_s1 - DLAPY2h)
      DLAPY2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  DLAPY2 = W * tmp4
      END IF
      RETURN
*
*     End of DDLAPY2
*
      END
