*> \brief \b DLASCL
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DLASCL + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dlascl.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dlascl.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dlascl.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DLASCL( TYPE, KL, KU, CFROM, CTO, M, N, A, LDA, INFO )
* 
*       .. Scalar Arguments ..
*       CHARACTER          TYPE
*       INTEGER            INFO, KL, KU, LDA, M, N
*       DOUBLE PRECISION   CFROM, CTO
*       ..
*       .. Array Arguments ..
*       DOUBLE PRECISION   A( LDA, * )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DLASCL multiplies the M by N real matrix A by the real scalar
*> CTO/CFROM.  This is done without over/underflow as long as the final
*> result CTO*A(I,J)/CFROM does not over/underflow. TYPE specifies that
*> A may be full, upper triangular, lower triangular, upper Hessenberg,
*> or banded.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] TYPE
*> \verbatim
*>          TYPE is CHARACTER*1
*>          TYPE indices the storage type of the input matrix.
*>          = 'G':  A is a full matrix.
*>          = 'L':  A is a lower triangular matrix.
*>          = 'U':  A is an upper triangular matrix.
*>          = 'H':  A is an upper Hessenberg matrix.
*>          = 'B':  A is a symmetric band matrix with lower bandwidth KL
*>                  and upper bandwidth KU and with the only the lower
*>                  half stored.
*>          = 'Q':  A is a symmetric band matrix with lower bandwidth KL
*>                  and upper bandwidth KU and with the only the upper
*>                  half stored.
*>          = 'Z':  A is a band matrix with lower bandwidth KL and upper
*>                  bandwidth KU. See DGBTRF for storage details.
*> \endverbatim
*>
*> \param[in] KL
*> \verbatim
*>          KL is INTEGER
*>          The lower bandwidth of A.  Referenced only if TYPE = 'B',
*>          'Q' or 'Z'.
*> \endverbatim
*>
*> \param[in] KU
*> \verbatim
*>          KU is INTEGER
*>          The upper bandwidth of A.  Referenced only if TYPE = 'B',
*>          'Q' or 'Z'.
*> \endverbatim
*>
*> \param[in] CFROM
*> \verbatim
*>          CFROM is DOUBLE PRECISION
*> \endverbatim
*>
*> \param[in] CTO
*> \verbatim
*>          CTO is DOUBLE PRECISION
*>
*>          The matrix A is multiplied by CTO/CFROM. A(I,J) is computed
*>          without over/underflow if the final result CTO*A(I,J)/CFROM
*>          can be represented without over/underflow.  CFROM must be
*>          nonzero.
*> \endverbatim
*>
*> \param[in] M
*> \verbatim
*>          M is INTEGER
*>          The number of rows of the matrix A.  M >= 0.
*> \endverbatim
*>
*> \param[in] N
*> \verbatim
*>          N is INTEGER
*>          The number of columns of the matrix A.  N >= 0.
*> \endverbatim
*>
*> \param[in,out] A
*> \verbatim
*>          A is DOUBLE PRECISION array, dimension (LDA,N)
*>          The matrix to be multiplied by CTO/CFROM.  See TYPE for the
*>          storage type.
*> \endverbatim
*>
*> \param[in] LDA
*> \verbatim
*>          LDA is INTEGER
*>          The leading dimension of the array A.  LDA >= max(1,M).
*> \endverbatim
*>
*> \param[out] INFO
*> \verbatim
*>          INFO is INTEGER
*>          0  - successful exit
*>          <0 - if INFO = -i, the i-th argument had an illegal value.
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERauxiliary
*
*  =====================================================================
!      SUBROUTINE DLASCL( TYPE, KL, KU, CFROM, CTO, M, N, A, LDA, INFO )
      SUBROUTINE DDLASCL( TYPE, KL, KU, CFROMh, CFROMl, CTOh, CTOl, 
     &                   M, N, Ah, Al, LDA, INFO )
*
*  -- LAPACK auxiliary routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      CHARACTER          TYPE
      INTEGER            INFO, KL, KU, LDA, M, N
!      DOUBLE PRECISION   CFROM, CTO
      double precision CFROMh, CFROMl
      double precision CTOh, CTOl
*     ..
*     .. Array Arguments ..
!      DOUBLE PRECISION   A( LDA, * )
      double precision Ah(LDA,*), Al(LDA,*)
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
!      DOUBLE PRECISION   ZERO, ONE
!      PARAMETER          ( ZERO = 0.0D0, ONE = 1.0D0 )
      double precision ZEROh, ZEROl
      double precision ONEh, ONEl
*     ..
*     .. Local Scalars ..
      LOGICAL            DONE
      INTEGER            I, ITYPE, J, K1, K2, K3, K4
!      DOUBLE PRECISION   BIGNUM, CFROM1, CFROMC, CTO1, CTOC, MUL, SMLNUM
      double precision BIGNUMh, BIGNUMl
      double precision CFROM1h, CFROM1l
      double precision CFROMCh, CFROMCl
      double precision CTO1h, CTO1l
      double precision CTOCh, CTOCl
      double precision MULh, MULl
      double precision SMLNUMh, SMLNUMl
*     ..
*     .. External Functions ..
      LOGICAL            LSAME, DDISNAN
!      DOUBLE PRECISION   DLAMCH
!      EXTERNAL           LSAME, DLAMCH, DISNAN
      EXTERNAL           LSAME, DDISNAN
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, MAX, MIN
*     ..
*     .. External Subroutines ..
      EXTERNAL           DDLAMCH,XERBLA
*     ..

      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision tmp3h, tmp3l
      double precision tmp4h, tmp4l
      double precision tmp5h, tmp5l
      double precision tmp6h, tmp6l
      double precision tmp7h, tmp7l
      double precision tmp8h, tmp8l
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l

      ZEROh = 0.0d+0
      ZEROl = 0.0d+0
      ONEh = 1.0d+0
      ONEl = 0.0d+0
      CFROM1l = 0.0d0
*     .. Executable Statements ..
*
*     Test the input arguments
*
      INFO = 0
*
      IF( LSAME( TYPE, 'G' ) ) THEN
         ITYPE = 0
      ELSE IF( LSAME( TYPE, 'L' ) ) THEN
         ITYPE = 1
      ELSE IF( LSAME( TYPE, 'U' ) ) THEN
         ITYPE = 2
      ELSE IF( LSAME( TYPE, 'H' ) ) THEN
         ITYPE = 3
      ELSE IF( LSAME( TYPE, 'B' ) ) THEN
         ITYPE = 4
      ELSE IF( LSAME( TYPE, 'Q' ) ) THEN
         ITYPE = 5
      ELSE IF( LSAME( TYPE, 'Z' ) ) THEN
         ITYPE = 6
      ELSE
         ITYPE = -1
      END IF
*
      IF( ITYPE.EQ.-1 ) THEN
         INFO = -1
      ELSE IF(
!              CFROM.EQ.ZERO 
!       (CFROM .eq. ZERO)
     + ((CFROMh .eq. ZEROh) .and.
     +  (CFROMl .eq. ZEROl))
     $         .OR. 
!              DISNAN(CFROM)
     $         DDISNAN(CFROMh,CFROMl)
     $         ) THEN
         INFO = -4
      ELSE IF(
!              DISNAN(CTO)
     $         DDISNAN(CTOh,CTOl)
     $         ) THEN
         INFO = -5
      ELSE IF( M.LT.0 ) THEN
         INFO = -6
      ELSE IF( N.LT.0 .OR. ( ITYPE.EQ.4 .AND. N.NE.M ) .OR.
     $         ( ITYPE.EQ.5 .AND. N.NE.M ) ) THEN
         INFO = -7
      ELSE IF( ITYPE.LE.3 .AND. LDA.LT.MAX( 1, M ) ) THEN
         INFO = -9
      ELSE IF( ITYPE.GE.4 ) THEN
         IF( KL.LT.0 .OR. KL.GT.MAX( M-1, 0 ) ) THEN
            INFO = -2
         ELSE IF( KU.LT.0 .OR. KU.GT.MAX( N-1, 0 ) .OR.
     $            ( ( ITYPE.EQ.4 .OR. ITYPE.EQ.5 ) .AND. KL.NE.KU ) )
     $             THEN
            INFO = -3
         ELSE IF( ( ITYPE.EQ.4 .AND. LDA.LT.KL+1 ) .OR.
     $            ( ITYPE.EQ.5 .AND. LDA.LT.KU+1 ) .OR.
     $            ( ITYPE.EQ.6 .AND. LDA.LT.2*KL+KU+1 ) ) THEN
            INFO = -9
         END IF
      END IF
*
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'DDLASCL', -INFO )
         RETURN
      END IF
*
*     Quick return if possible
*
      IF( N.EQ.0 .OR. M.EQ.0 )
     $   RETURN
*
*     Get machine parameters
*
!      SMLNUM = DLAMCH( 'S' )
      CALL DDLAMCH('S', SMLNUMh, SMLNUMl )
!      BIGNUM = ONE / SMLNUM
      BIGNUMh = ONEh / SMLNUMh
      BIGNUMl = 0.0d0
*
!      CFROMC = CFROM
      CFROMCh = CFROMh
      CFROMCl = CFROMl
!      CTOC = CTO
      CTOCh = CTOh
      CTOCl = CTOl
*
   10 CONTINUE
!      CFROM1 = CFROMC*SMLNUM
      CFROM1h = CFROMCh*SMLNUMh
      IF( CFROM1h.EQ.CFROMCh ) THEN
!        CFROMC is an inf.  Multiply by a correctly signed zero for
!        finite CTOC, or a NaN if CTOC is infinite.
!         MUL = CTOC / CFROMC
         MULh = CTOCh / CFROMCh
         MULl = 0.0d0
         DONE = .TRUE.
!         CTO1 = CTOC
      CTO1h = CTOCh
      CTO1l = CTOCl
      ELSE
!         CTO1 = CTOC / BIGNUM
         CTO1h = CTOCh / BIGNUMh
         IF( CTO1h.EQ.CTOCh ) THEN
!           CTOC is either 0 or an inf.  In both cases, CTOC itself
!           serves as the correct multiplication factor.
!            MUL = CTOC
            MULh = CTOCh
            MULl = 0.0d0
            DONE = .TRUE.
!            CFROMC = ONE
      CFROMCh = ONEh
      CFROMCl = ONEl
         ELSE IF((ABS( CFROM1h ).GT.ABS( CTOCh )) .AND. 
     &           (CTOCh.NE.ZEROh)
     &           ) THEN
!            MUL = SMLNUM
      MULh = SMLNUMh
      MULl = SMLNUMl
            DONE = .FALSE.
!            CFROMC = CFROM1
      CFROMCh = CFROM1h
      CFROMCl = CFROM1l
         ELSE IF( ABS( CTO1h ).GT.ABS( CFROMCh ) ) THEN
!            MUL = BIGNUM
      MULh = BIGNUMh
      MULl = BIGNUMl
            DONE = .FALSE.
!            CTOC = CTO1
      CTOCh = CTO1h
      CTOCl = CTO1l
         ELSE
!            MUL = CTOC / CFROMC
!<<<  MUL = CTOC / CFROMC
      T_q1 = CTOCh / CFROMCh
      MULh = CFROMCh * T_q1
      T_u1 = 134217729.0 * CFROMCh
      T_p1 = T_u1
     +         - (T_u1 - CFROMCh)
      T_p2 = CFROMCh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      MULl = ((T_p1*T_r1
     +       - MULh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      MULl = MULl + (CFROMCl*T_q1)
      T_s1 = MULh + MULl
      MULl = MULl - (T_s1 - MULh)
      MULh = T_s1
      T_s1 = CTOCh - MULh
      T_r1 = T_s1 - CTOCh
      T_s2 = (CTOCh 
     +         - (T_s1 - T_r1))
     +         - (MULh + T_r1)
      T_s2 = T_s2 - MULl
      T_s2 = T_s2 + CTOCl
      T_q2 = (T_s1 + T_s2)
     +         / CFROMCh
      MULh = T_q1 + T_q2
      MULl = T_q2
     +      - (MULh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  MUL = CTOC / CFROMC
            DONE = .TRUE.
         END IF
      END IF
*
      IF( ITYPE.EQ.0 ) THEN
*
*        Full matrix
*
         DO 30 J = 1, N
            DO 20 I = 1, M
!               A( I, J ) = A( I, J )*MUL
!<<<  A(I,J) = A(I,J) * MUL
      T_ch = Ah(I,J) * MULh
      T_u1 = 134217729.0 * Ah(I,J)
      T_p1 = T_u1
     +         - (T_u1 - Ah(I,J))
      T_p2 = Ah(I,J) - T_p1
      T_u1 = 134217729.0 * MULh
      T_r1 = T_u1
     +         - (T_u1 - MULh)
      T_r2 = MULh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Ah(I,J)*MULl
     +         +  Al(I,J)*MULh)
      T_s1 = T_ch + T_cl
      Al(I,J) = T_cl - (T_s1 - T_ch)
      Ah(I,J) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  A(I,J) = A(I,J) * MUL
   20       CONTINUE
   30    CONTINUE
*
      ELSE IF( ITYPE.EQ.1 ) THEN
*
*        Lower triangular matrix
*
         DO 50 J = 1, N
            DO 40 I = J, M
!               A( I, J ) = A( I, J )*MUL
!<<<  A(I,J) = A(I,J) * MUL
      T_ch = Ah(I,J) * MULh
      T_u1 = 134217729.0 * Ah(I,J)
      T_p1 = T_u1
     +         - (T_u1 - Ah(I,J))
      T_p2 = Ah(I,J) - T_p1
      T_u1 = 134217729.0 * MULh
      T_r1 = T_u1
     +         - (T_u1 - MULh)
      T_r2 = MULh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Ah(I,J)*MULl
     +         +  Al(I,J)*MULh)
      T_s1 = T_ch + T_cl
      Al(I,J) = T_cl - (T_s1 - T_ch)
      Ah(I,J) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  A(I,J) = A(I,J) * MUL
   40       CONTINUE
   50    CONTINUE
*
      ELSE IF( ITYPE.EQ.2 ) THEN
*
*        Upper triangular matrix
*
         DO 70 J = 1, N
            DO 60 I = 1, MIN( J, M )
!               A( I, J ) = A( I, J )*MUL
!<<<  A(I,J) = A(I,J) * MUL
      T_ch = Ah(I,J) * MULh
      T_u1 = 134217729.0 * Ah(I,J)
      T_p1 = T_u1
     +         - (T_u1 - Ah(I,J))
      T_p2 = Ah(I,J) - T_p1
      T_u1 = 134217729.0 * MULh
      T_r1 = T_u1
     +         - (T_u1 - MULh)
      T_r2 = MULh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Ah(I,J)*MULl
     +         +  Al(I,J)*MULh)
      T_s1 = T_ch + T_cl
      Al(I,J) = T_cl - (T_s1 - T_ch)
      Ah(I,J) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  A(I,J) = A(I,J) * MUL
   60       CONTINUE
   70    CONTINUE
*
      ELSE IF( ITYPE.EQ.3 ) THEN
*
*        Upper Hessenberg matrix
*
         DO 90 J = 1, N
            DO 80 I = 1, MIN( J+1, M )
!               A( I, J ) = A( I, J )*MUL
!<<<  A(I,J) = A(I,J) * MUL
      T_ch = Ah(I,J) * MULh
      T_u1 = 134217729.0 * Ah(I,J)
      T_p1 = T_u1
     +         - (T_u1 - Ah(I,J))
      T_p2 = Ah(I,J) - T_p1
      T_u1 = 134217729.0 * MULh
      T_r1 = T_u1
     +         - (T_u1 - MULh)
      T_r2 = MULh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Ah(I,J)*MULl
     +         +  Al(I,J)*MULh)
      T_s1 = T_ch + T_cl
      Al(I,J) = T_cl - (T_s1 - T_ch)
      Ah(I,J) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  A(I,J) = A(I,J) * MUL
   80       CONTINUE
   90    CONTINUE
*
      ELSE IF( ITYPE.EQ.4 ) THEN
*
*        Lower half of a symmetric band matrix
*
         K3 = KL + 1
         K4 = N + 1
         DO 110 J = 1, N
            DO 100 I = 1, MIN( K3, K4-J )
!               A( I, J ) = A( I, J )*MUL
!<<<  A(I,J) = A(I,J) * MUL
      T_ch = Ah(I,J) * MULh
      T_u1 = 134217729.0 * Ah(I,J)
      T_p1 = T_u1
     +         - (T_u1 - Ah(I,J))
      T_p2 = Ah(I,J) - T_p1
      T_u1 = 134217729.0 * MULh
      T_r1 = T_u1
     +         - (T_u1 - MULh)
      T_r2 = MULh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Ah(I,J)*MULl
     +         +  Al(I,J)*MULh)
      T_s1 = T_ch + T_cl
      Al(I,J) = T_cl - (T_s1 - T_ch)
      Ah(I,J) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  A(I,J) = A(I,J) * MUL
  100       CONTINUE
  110    CONTINUE
*
      ELSE IF( ITYPE.EQ.5 ) THEN
*
*        Upper half of a symmetric band matrix
*
         K1 = KU + 2
         K3 = KU + 1
         DO 130 J = 1, N
            DO 120 I = MAX( K1-J, 1 ), K3
!               A( I, J ) = A( I, J )*MUL
!<<<  A(I,J) = A(I,J) * MUL
      T_ch = Ah(I,J) * MULh
      T_u1 = 134217729.0 * Ah(I,J)
      T_p1 = T_u1
     +         - (T_u1 - Ah(I,J))
      T_p2 = Ah(I,J) - T_p1
      T_u1 = 134217729.0 * MULh
      T_r1 = T_u1
     +         - (T_u1 - MULh)
      T_r2 = MULh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Ah(I,J)*MULl
     +         +  Al(I,J)*MULh)
      T_s1 = T_ch + T_cl
      Al(I,J) = T_cl - (T_s1 - T_ch)
      Ah(I,J) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  A(I,J) = A(I,J) * MUL
  120       CONTINUE
  130    CONTINUE
*
      ELSE IF( ITYPE.EQ.6 ) THEN
*
*        Band matrix
*
         K1 = KL + KU + 2
         K2 = KL + 1
         K3 = 2*KL + KU + 1
         K4 = KL + KU + 1 + M
         DO 150 J = 1, N
            DO 140 I = MAX( K1-J, K2 ), MIN( K3, K4-J )
!               A( I, J ) = A( I, J )*MUL
!<<<  A(I,J) = A(I,J) * MUL
      T_ch = Ah(I,J) * MULh
      T_u1 = 134217729.0 * Ah(I,J)
      T_p1 = T_u1
     +         - (T_u1 - Ah(I,J))
      T_p2 = Ah(I,J) - T_p1
      T_u1 = 134217729.0 * MULh
      T_r1 = T_u1
     +         - (T_u1 - MULh)
      T_r2 = MULh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Ah(I,J)*MULl
     +         +  Al(I,J)*MULh)
      T_s1 = T_ch + T_cl
      Al(I,J) = T_cl - (T_s1 - T_ch)
      Ah(I,J) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  A(I,J) = A(I,J) * MUL
  140       CONTINUE
  150    CONTINUE
*
      END IF
*
      IF( .NOT.DONE )
     $   GO TO 10
*
      RETURN
*
*     End of DLASCL
*
      END
