*> \brief \b DLASQ5
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DLASQ5 + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dlasq5.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dlasq5.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dlasq5.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DLASQ5( I0, N0, Z, PP, TAU, DMIN, DMIN1, DMIN2, DN,
*                          DNM1, DNM2, IEEE )
* 
*       .. Scalar Arguments ..
*       LOGICAL            IEEE
*       INTEGER            I0, N0, PP
*       DOUBLE PRECISION   DMIN, DMIN1, DMIN2, DN, DNM1, DNM2, TAU
*       ..
*       .. Array Arguments ..
*       DOUBLE PRECISION   Z( * )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DLASQ5 computes one dqds transform in ping-pong form, one
*> version for IEEE machines another for non IEEE machines.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] I0
*> \verbatim
*>          I0 is INTEGER
*>        First index.
*> \endverbatim
*>
*> \param[in] N0
*> \verbatim
*>          N0 is INTEGER
*>        Last index.
*> \endverbatim
*>
*> \param[in] Z
*> \verbatim
*>          Z is DOUBLE PRECISION array, dimension ( 4*N )
*>        Z holds the qd array. EMIN is stored in Z(4*N0) to avoid
*>        an extra argument.
*> \endverbatim
*>
*> \param[in] PP
*> \verbatim
*>          PP is INTEGER
*>        PP=0 for ping, PP=1 for pong.
*> \endverbatim
*>
*> \param[in] TAU
*> \verbatim
*>          TAU is DOUBLE PRECISION
*>        This is the shift.
*> \endverbatim
*>
*> \param[out] DMIN
*> \verbatim
*>          DMIN is DOUBLE PRECISION
*>        Minimum value of d.
*> \endverbatim
*>
*> \param[out] DMIN1
*> \verbatim
*>          DMIN1 is DOUBLE PRECISION
*>        Minimum value of d, excluding D( N0 ).
*> \endverbatim
*>
*> \param[out] DMIN2
*> \verbatim
*>          DMIN2 is DOUBLE PRECISION
*>        Minimum value of d, excluding D( N0 ) and D( N0-1 ).
*> \endverbatim
*>
*> \param[out] DN
*> \verbatim
*>          DN is DOUBLE PRECISION
*>        d(N0), the last value of d.
*> \endverbatim
*>
*> \param[out] DNM1
*> \verbatim
*>          DNM1 is DOUBLE PRECISION
*>        d(N0-1).
*> \endverbatim
*>
*> \param[out] DNM2
*> \verbatim
*>          DNM2 is DOUBLE PRECISION
*>        d(N0-2).
*> \endverbatim
*>
*> \param[in] IEEE
*> \verbatim
*>          IEEE is LOGICAL
*>        Flag for IEEE or non IEEE arithmetic.
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERcomputational
*
*  =====================================================================
!!      SUBROUTINE DLASQ5( I0, N0, Z, PP, TAU, DMIN, DMIN1, DMIN2, DN,
!!     $                   DNM1, DNM2, IEEE )
      SUBROUTINE dDLASQ5( I0, N0, Zh, Zl, PP, TAUh, TAUl, DMINh, DMINl,
     $                    DMIN1h, DMIN1l, DMIN2h, DMIN2l, DNh, DNl,
     $                    DNM1h, DNM1l, DNM2h, DNM2l, IEEE )
*
*  -- LAPACK computational routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      LOGICAL            IEEE
      INTEGER            I0, N0, PP
!!      DOUBLE PRECISION   DMIN, DMIN1, DMIN2, DN, DNM1, DNM2, TAU
      double precision DMINh, DMINl
      double precision DMIN1h, DMIN1l
      double precision DMIN2h, DMIN2l
      double precision DNh, DNl
      double precision DNM1h, DNM1l
      double precision DNM2h, DNM2l
      double precision TAUh, TAUl
*     ..
*     .. Array Arguments ..
!!      DOUBLE PRECISION   Z( * )
      double precision Zh(*), Zl(*)
*     ..
*
*  =====================================================================
*
*     .. Parameter ..
!!      DOUBLE PRECISION   ZERO
!!      PARAMETER          ( ZERO = 0.0D0 )
      double precision ZEROh, ZEROl
*     ..
*     .. Local Scalars ..
      INTEGER            J4, J4P2
!!      DOUBLE PRECISION   D, EMIN, TEMP
      double precision Dh, Dl
      double precision EMINh, EMINl
      double precision TEMPh, TEMPl
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MIN
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l
      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
*     ..
*     .. Executable Statements ..
      ZEROh = 0.0d+0
      ZEROl = 0.0d+0
*
      IF( ( N0-I0-1 ).LE.0 )
     $   RETURN
*
      J4 = 4*I0 + PP - 3
!!      EMIN = Z( J4+4 ) 
      EMINh = Zh(J4+4)
      EMINl = Zl(J4+4)
!!      D = Z( J4 ) - TAU
!<<<  D = Z(J4) - TAU
      T_p1 = Zh(J4) - TAUh
      T_p2 = T_p1 - Zh(J4)
      T_ch = T_p1
      T_cl = (Zh(J4) - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      T_cl = T_cl + Zl(J4) - TAUl
      T_s1 = T_ch + T_cl
      Dl = T_cl - (T_s1 - T_ch)
      Dh = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  D = Z(J4) - TAU
!!      DMIN = D
      DMINh = Dh
      DMINl = Dl
!!      DMIN1 = -Z( J4 )
!<<<  tmp1 = -Z(J4)
      tmp1h = -Zh(J4)
      tmp1l = -Zl(J4)
!>>>  tmp1 = -Z(J4)
      DMIN1h = tmp1h
      DMIN1l = tmp1l
*
      IF( IEEE ) THEN
*
*        Code for IEEE arithmetic.
*
         IF( PP.EQ.0 ) THEN
            DO 10 J4 = 4*I0, 4*( N0-3 ), 4
!!               Z( J4-2 ) = D + Z( J4-1 ) 
!<<<  Z(J4-2) = D + Z(J4-1)
      T_p1 = Dh + Zh(J4-1)
      T_p2 = T_p1 - Dh
      T_q1 = T_p1
      T_q2 = (Dh - (T_p1 - T_p2))
     +         + (Zh(J4-1) - T_p2)
      T_q2 = T_q2 + Dl + Zl(J4-1)
      Zh(J4-2) = T_q1 + T_q2
      Zl(J4-2) = T_q2 - (Zh(J4-2) - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  Z(J4-2) = D + Z(J4-1)
!!               TEMP = Z( J4+1 ) / Z( J4-2 )
!<<<  TEMP = Z(J4+1) / Z(J4-2)
      T_q1 = Zh(J4+1) / Zh(J4-2)
      T_ch = Zh(J4-2) * T_q1
      T_u1 = 134217729.0 * Zh(J4-2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4-2))
      T_p2 = Zh(J4-2) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Zl(J4-2) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Zh(J4+1) - T_ch
      T_r1 = T_s1 - Zh(J4+1)
      T_s2 = (Zh(J4+1)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Zl(J4+1)
      T_q2 = (T_s1 + T_s2)
     +         / Zh(J4-2)
      TEMPh = T_q1 + T_q2
      TEMPl = T_q2
     +      - (TEMPh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP = Z(J4+1) / Z(J4-2)
!!               D = D*TEMP - TAU
!<<<  tmp1 = D * TEMP
      T_ch = Dh * TEMPh
      T_u1 = 134217729.0 * Dh
      T_p1 = T_u1
     +         - (T_u1 - Dh)
      T_p2 = Dh - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Dh*TEMPl
     +         +  Dl*TEMPh)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = D * TEMP
!<<<  D = tmp1 - TAU
      T_p1 = tmp1h - TAUh
      T_p2 = T_p1 - tmp1h
      T_ch = T_p1
      T_cl = (tmp1h - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      T_cl = T_cl + tmp1l - TAUl
      T_s1 = T_ch + T_cl
      Dl = T_cl - (T_s1 - T_ch)
      Dh = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  D = tmp1 - TAU
!!               DMIN = MIN( DMIN, D )
               IF(
!       (DMIN .gt. D)
     + ((DMINh .gt. Dh) .or.
     +  ((DMINh .eq. Dh) .and.
     +   (DMINl .gt. Dl)))
     $           ) THEN
      DMINh = Dh
      DMINl = Dl
               END IF
!!               Z( J4 ) = Z( J4-1 )*TEMP
!<<<  Z(J4) = Z(J4-1) * TEMP
      T_ch = Zh(J4-1) * TEMPh
      T_u1 = 134217729.0 * Zh(J4-1)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4-1))
      T_p2 = Zh(J4-1) - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(J4-1)*TEMPl
     +         +  Zl(J4-1)*TEMPh)
      T_s1 = T_ch + T_cl
      Zl(J4) = T_cl - (T_s1 - T_ch)
      Zh(J4) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  Z(J4) = Z(J4-1) * TEMP
!!               EMIN = MIN( Z( J4 ), EMIN )
               IF(
!       (Z(J4) .lt. EMIN)
     + ((Zh(J4) .lt. EMINh) .or.
     +  ((Zh(J4) .eq. EMINh) .and.
     +   (Zl(J4) .lt. EMINl)))
     $           ) THEN
      EMINh = Zh(J4)
      EMINl = Zl(J4)
               END IF
   10       CONTINUE
         ELSE
            DO 20 J4 = 4*I0, 4*( N0-3 ), 4
!!               Z( J4-3 ) = D + Z( J4 ) 
!<<<  Z(J4-3) = D + Z(J4)
      T_p1 = Dh + Zh(J4)
      T_p2 = T_p1 - Dh
      T_q1 = T_p1
      T_q2 = (Dh - (T_p1 - T_p2))
     +         + (Zh(J4) - T_p2)
      T_q2 = T_q2 + Dl + Zl(J4)
      Zh(J4-3) = T_q1 + T_q2
      Zl(J4-3) = T_q2 - (Zh(J4-3) - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  Z(J4-3) = D + Z(J4)
!!               TEMP = Z( J4+2 ) / Z( J4-3 )
!<<<  TEMP = Z(J4+2) / Z(J4-3)
      T_q1 = Zh(J4+2) / Zh(J4-3)
      T_ch = Zh(J4-3) * T_q1
      T_u1 = 134217729.0 * Zh(J4-3)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4-3))
      T_p2 = Zh(J4-3) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Zl(J4-3) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Zh(J4+2) - T_ch
      T_r1 = T_s1 - Zh(J4+2)
      T_s2 = (Zh(J4+2)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Zl(J4+2)
      T_q2 = (T_s1 + T_s2)
     +         / Zh(J4-3)
      TEMPh = T_q1 + T_q2
      TEMPl = T_q2
     +      - (TEMPh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP = Z(J4+2) / Z(J4-3)
!!               D = D*TEMP - TAU
!<<<  tmp1 = D * TEMP
      T_ch = Dh * TEMPh
      T_u1 = 134217729.0 * Dh
      T_p1 = T_u1
     +         - (T_u1 - Dh)
      T_p2 = Dh - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Dh*TEMPl
     +         +  Dl*TEMPh)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = D * TEMP
!<<<  D = tmp1 - TAU
      T_p1 = tmp1h - TAUh
      T_p2 = T_p1 - tmp1h
      T_ch = T_p1
      T_cl = (tmp1h - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      T_cl = T_cl + tmp1l - TAUl
      T_s1 = T_ch + T_cl
      Dl = T_cl - (T_s1 - T_ch)
      Dh = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  D = tmp1 - TAU
!!               DMIN = MIN( DMIN, D )
               IF(
!       (DMIN .gt. D)
     + ((DMINh .gt. Dh) .or.
     +  ((DMINh .eq. Dh) .and.
     +   (DMINl .gt. Dl)))
     $           ) THEN
      DMINh = Dh
      DMINl = Dl
               END IF
!!               Z( J4-1 ) = Z( J4 )*TEMP
!<<<  Z(J4-1) = Z(J4) * TEMP
      T_ch = Zh(J4) * TEMPh
      T_u1 = 134217729.0 * Zh(J4)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4))
      T_p2 = Zh(J4) - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(J4)*TEMPl
     +         +  Zl(J4)*TEMPh)
      T_s1 = T_ch + T_cl
      Zl(J4-1) = T_cl - (T_s1 - T_ch)
      Zh(J4-1) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  Z(J4-1) = Z(J4) * TEMP
!!               EMIN = MIN( Z( J4-1 ), EMIN )
               IF(
!       (Z(J4-1) .lt. EMIN)
     + ((Zh(J4-1) .lt. EMINh) .or.
     +  ((Zh(J4-1) .eq. EMINh) .and.
     +   (Zl(J4-1) .lt. EMINl)))
     $           ) THEN
      EMINh = Zh(J4-1)
      EMINl = Zl(J4-1)
               END IF
   20       CONTINUE
         END IF
*
*        Unroll last two steps. 
*
!!         DNM2 = D
      DNM2h = Dh
      DNM2l = Dl
!!         DMIN2 = DMIN
      DMIN2h = DMINh
      DMIN2l = DMINl
         J4 = 4*( N0-2 ) - PP
         J4P2 = J4 + 2*PP - 1
!!         Z( J4-2 ) = DNM2 + Z( J4P2 )
!<<<  Z(J4-2) = DNM2 + Z(J4P2)
      T_p1 = DNM2h + Zh(J4P2)
      T_p2 = T_p1 - DNM2h
      T_q1 = T_p1
      T_q2 = (DNM2h - (T_p1 - T_p2))
     +         + (Zh(J4P2) - T_p2)
      T_q2 = T_q2 + DNM2l + Zl(J4P2)
      Zh(J4-2) = T_q1 + T_q2
      Zl(J4-2) = T_q2 - (Zh(J4-2) - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  Z(J4-2) = DNM2 + Z(J4P2)
!!         Z( J4 ) = Z( J4P2+2 )*( Z( J4P2 ) / Z( J4-2 ) )
!<<<  tmp1 = Z(J4P2) / Z(J4-2)
      T_q1 = Zh(J4P2) / Zh(J4-2)
      T_ch = Zh(J4-2) * T_q1
      T_u1 = 134217729.0 * Zh(J4-2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4-2))
      T_p2 = Zh(J4-2) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Zl(J4-2) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Zh(J4P2) - T_ch
      T_r1 = T_s1 - Zh(J4P2)
      T_s2 = (Zh(J4P2)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Zl(J4P2)
      T_q2 = (T_s1 + T_s2)
     +         / Zh(J4-2)
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = Z(J4P2) / Z(J4-2)
!<<<  Z(J4) = Z(J4P2+2) * tmp1
      T_ch = Zh(J4P2+2) * tmp1h
      T_u1 = 134217729.0 * Zh(J4P2+2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4P2+2))
      T_p2 = Zh(J4P2+2) - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(J4P2+2)*tmp1l
     +         +  Zl(J4P2+2)*tmp1h)
      T_s1 = T_ch + T_cl
      Zl(J4) = T_cl - (T_s1 - T_ch)
      Zh(J4) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  Z(J4) = Z(J4P2+2) * tmp1
!!         DNM1 = Z( J4P2+2 )*( DNM2 / Z( J4-2 ) ) - TAU
!<<<  tmp1 = DNM2 / Z(J4-2)
      T_q1 = DNM2h / Zh(J4-2)
      T_ch = Zh(J4-2) * T_q1
      T_u1 = 134217729.0 * Zh(J4-2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4-2))
      T_p2 = Zh(J4-2) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Zl(J4-2) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = DNM2h - T_ch
      T_r1 = T_s1 - DNM2h
      T_s2 = (DNM2h
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + DNM2l
      T_q2 = (T_s1 + T_s2)
     +         / Zh(J4-2)
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = DNM2 / Z(J4-2)
!<<<  tmp2 = Z(J4P2+2) * tmp1
      T_ch = Zh(J4P2+2) * tmp1h
      T_u1 = 134217729.0 * Zh(J4P2+2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4P2+2))
      T_p2 = Zh(J4P2+2) - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(J4P2+2)*tmp1l
     +         +  Zl(J4P2+2)*tmp1h)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = Z(J4P2+2) * tmp1
!<<<  DNM1 = tmp2 - TAU
      T_p1 = tmp2h - TAUh
      T_p2 = T_p1 - tmp2h
      T_ch = T_p1
      T_cl = (tmp2h - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      T_cl = T_cl + tmp2l - TAUl
      T_s1 = T_ch + T_cl
      DNM1l = T_cl - (T_s1 - T_ch)
      DNM1h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  DNM1 = tmp2 - TAU
!!         DMIN = MIN( DMIN, DNM1 )
         IF(
!       (DMIN .gt. DNM1)
     + ((DMINh .gt. DNM1h) .or.
     +  ((DMINh .eq. DNM1h) .and.
     +   (DMINl .gt. DNM1l)))
     $     ) THEN
      DMINh = DNM1h
      DMINl = DNM1l
         END IF
*
!!         DMIN1 = DMIN
      DMIN1h = DMINh
      DMIN1l = DMINl
         J4 = J4 + 4
         J4P2 = J4 + 2*PP - 1
!!         Z( J4-2 ) = DNM1 + Z( J4P2 )
!<<<  Z(J4-2) = DNM1 + Z(J4P2)
      T_p1 = DNM1h + Zh(J4P2)
      T_p2 = T_p1 - DNM1h
      T_q1 = T_p1
      T_q2 = (DNM1h - (T_p1 - T_p2))
     +         + (Zh(J4P2) - T_p2)
      T_q2 = T_q2 + DNM1l + Zl(J4P2)
      Zh(J4-2) = T_q1 + T_q2
      Zl(J4-2) = T_q2 - (Zh(J4-2) - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  Z(J4-2) = DNM1 + Z(J4P2)
!!         Z( J4 ) = Z( J4P2+2 )*( Z( J4P2 ) / Z( J4-2 ) )
!<<<  tmp1 = Z(J4P2) / Z(J4-2)
      T_q1 = Zh(J4P2) / Zh(J4-2)
      T_ch = Zh(J4-2) * T_q1
      T_u1 = 134217729.0 * Zh(J4-2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4-2))
      T_p2 = Zh(J4-2) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Zl(J4-2) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Zh(J4P2) - T_ch
      T_r1 = T_s1 - Zh(J4P2)
      T_s2 = (Zh(J4P2)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Zl(J4P2)
      T_q2 = (T_s1 + T_s2)
     +         / Zh(J4-2)
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = Z(J4P2) / Z(J4-2)
!<<<  Z(J4) = Z(J4P2+2) * tmp1
      T_ch = Zh(J4P2+2) * tmp1h
      T_u1 = 134217729.0 * Zh(J4P2+2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4P2+2))
      T_p2 = Zh(J4P2+2) - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(J4P2+2)*tmp1l
     +         +  Zl(J4P2+2)*tmp1h)
      T_s1 = T_ch + T_cl
      Zl(J4) = T_cl - (T_s1 - T_ch)
      Zh(J4) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  Z(J4) = Z(J4P2+2) * tmp1
!!         DN = Z( J4P2+2 )*( DNM1 / Z( J4-2 ) ) - TAU
!<<<  tmp1 = DNM1 / Z(J4-2)
      T_q1 = DNM1h / Zh(J4-2)
      T_ch = Zh(J4-2) * T_q1
      T_u1 = 134217729.0 * Zh(J4-2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4-2))
      T_p2 = Zh(J4-2) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Zl(J4-2) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = DNM1h - T_ch
      T_r1 = T_s1 - DNM1h
      T_s2 = (DNM1h
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + DNM1l
      T_q2 = (T_s1 + T_s2)
     +         / Zh(J4-2)
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = DNM1 / Z(J4-2)
!<<<  tmp2 = Z(J4P2+2) * tmp1
      T_ch = Zh(J4P2+2) * tmp1h
      T_u1 = 134217729.0 * Zh(J4P2+2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4P2+2))
      T_p2 = Zh(J4P2+2) - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(J4P2+2)*tmp1l
     +         +  Zl(J4P2+2)*tmp1h)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = Z(J4P2+2) * tmp1
!<<<  DN = tmp2 - TAU
      T_p1 = tmp2h - TAUh
      T_p2 = T_p1 - tmp2h
      T_ch = T_p1
      T_cl = (tmp2h - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      T_cl = T_cl + tmp2l - TAUl
      T_s1 = T_ch + T_cl
      DNl = T_cl - (T_s1 - T_ch)
      DNh = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  DN = tmp2 - TAU
!!         DMIN = MIN( DMIN, DN )
         IF(
!       (DMIN .gt. DN)
     + ((DMINh .gt. DNh) .or.
     +  ((DMINh .eq. DNh) .and.
     +   (DMINl .gt. DNl)))
     $     ) THEN
      DMINh = DNh
      DMINl = DNl
         END IF
*
      ELSE
*
*        Code for non IEEE arithmetic.
*
         IF( PP.EQ.0 ) THEN
            DO 30 J4 = 4*I0, 4*( N0-3 ), 4
!!               Z( J4-2 ) = D + Z( J4-1 ) 
!<<<  Z(J4-2) = D + Z(J4-1)
      T_p1 = Dh + Zh(J4-1)
      T_p2 = T_p1 - Dh
      T_q1 = T_p1
      T_q2 = (Dh - (T_p1 - T_p2))
     +         + (Zh(J4-1) - T_p2)
      T_q2 = T_q2 + Dl + Zl(J4-1)
      Zh(J4-2) = T_q1 + T_q2
      Zl(J4-2) = T_q2 - (Zh(J4-2) - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  Z(J4-2) = D + Z(J4-1)
!!               IF( D.LT.ZERO ) THEN
               IF(
!       (D .lt. ZERO)
     + ((Dh .lt. ZEROh) .or.
     +  ((Dh .eq. ZEROh) .and.
     +   (Dl .lt. ZEROl)))
     $           ) THEN
                  RETURN
               ELSE 
!!                  Z( J4 ) = Z( J4+1 )*( Z( J4-1 ) / Z( J4-2 ) )
!<<<  tmp1 = Z(J4-1) / Z(J4-2)
      T_q1 = Zh(J4-1) / Zh(J4-2)
      T_ch = Zh(J4-2) * T_q1
      T_u1 = 134217729.0 * Zh(J4-2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4-2))
      T_p2 = Zh(J4-2) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Zl(J4-2) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Zh(J4-1) - T_ch
      T_r1 = T_s1 - Zh(J4-1)
      T_s2 = (Zh(J4-1)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Zl(J4-1)
      T_q2 = (T_s1 + T_s2)
     +         / Zh(J4-2)
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = Z(J4-1) / Z(J4-2)
!<<<  Z(J4) = Z(J4+1) * tmp1
      T_ch = Zh(J4+1) * tmp1h
      T_u1 = 134217729.0 * Zh(J4+1)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4+1))
      T_p2 = Zh(J4+1) - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(J4+1)*tmp1l
     +         +  Zl(J4+1)*tmp1h)
      T_s1 = T_ch + T_cl
      Zl(J4) = T_cl - (T_s1 - T_ch)
      Zh(J4) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  Z(J4) = Z(J4+1) * tmp1
!!                  D = Z( J4+1 )*( D / Z( J4-2 ) ) - TAU
!<<<  tmp1 = D / Z(J4-2)
      T_q1 = Dh / Zh(J4-2)
      T_ch = Zh(J4-2) * T_q1
      T_u1 = 134217729.0 * Zh(J4-2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4-2))
      T_p2 = Zh(J4-2) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Zl(J4-2) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Dh - T_ch
      T_r1 = T_s1 - Dh
      T_s2 = (Dh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Dl
      T_q2 = (T_s1 + T_s2)
     +         / Zh(J4-2)
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = D / Z(J4-2)
!<<<  tmp2 = Z(J4+1) * tmp1
      T_ch = Zh(J4+1) * tmp1h
      T_u1 = 134217729.0 * Zh(J4+1)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4+1))
      T_p2 = Zh(J4+1) - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(J4+1)*tmp1l
     +         +  Zl(J4+1)*tmp1h)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = Z(J4+1) * tmp1
!<<<  D = tmp2 - TAU
      T_p1 = tmp2h - TAUh
      T_p2 = T_p1 - tmp2h
      T_ch = T_p1
      T_cl = (tmp2h - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      T_cl = T_cl + tmp2l - TAUl
      T_s1 = T_ch + T_cl
      Dl = T_cl - (T_s1 - T_ch)
      Dh = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  D = tmp2 - TAU
               END IF
!!               DMIN = MIN( DMIN, D )
               IF(
!       (DMIN .gt. D)
     + ((DMINh .gt. Dh) .or.
     +  ((DMINh .eq. Dh) .and.
     +   (DMINl .gt. Dl)))
     $           ) THEN
      DMINh = Dh
      DMINl = Dl
               END IF
!!               EMIN = MIN( EMIN, Z( J4 ) )
               IF(
!       (EMIN .gt. Z(J4))
     + ((EMINh .gt. Zh(J4)) .or.
     +  ((EMINh .eq. Zh(J4)) .and.
     +   (EMINl .gt. Zl(J4))))
     $           ) THEN
      EMINh = Zh(J4)
      EMINl = Zl(J4)
               END IF
   30       CONTINUE
         ELSE
            DO 40 J4 = 4*I0, 4*( N0-3 ), 4
!!               Z( J4-3 ) = D + Z( J4 ) 
!<<<  Z(J4-3) = D + Z(J4)
      T_p1 = Dh + Zh(J4)
      T_p2 = T_p1 - Dh
      T_q1 = T_p1
      T_q2 = (Dh - (T_p1 - T_p2))
     +         + (Zh(J4) - T_p2)
      T_q2 = T_q2 + Dl + Zl(J4)
      Zh(J4-3) = T_q1 + T_q2
      Zl(J4-3) = T_q2 - (Zh(J4-3) - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  Z(J4-3) = D + Z(J4)
!!               IF( D.LT.ZERO ) THEN
               IF(
!       (D .lt. ZERO)
     + ((Dh .lt. ZEROh) .or.
     +  ((Dh .eq. ZEROh) .and.
     +   (Dl .lt. ZEROl)))
     $           ) THEN
                  RETURN
               ELSE 
!!                  Z( J4-1 ) = Z( J4+2 )*( Z( J4 ) / Z( J4-3 ) )
!<<<  tmp1 = Z(J4) / Z(J4-3)
      T_q1 = Zh(J4) / Zh(J4-3)
      T_ch = Zh(J4-3) * T_q1
      T_u1 = 134217729.0 * Zh(J4-3)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4-3))
      T_p2 = Zh(J4-3) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Zl(J4-3) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Zh(J4) - T_ch
      T_r1 = T_s1 - Zh(J4)
      T_s2 = (Zh(J4)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Zl(J4)
      T_q2 = (T_s1 + T_s2)
     +         / Zh(J4-3)
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = Z(J4) / Z(J4-3)
!<<<  Z(J4-1) = Z(J4+2) * tmp1
      T_ch = Zh(J4+2) * tmp1h
      T_u1 = 134217729.0 * Zh(J4+2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4+2))
      T_p2 = Zh(J4+2) - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(J4+2)*tmp1l
     +         +  Zl(J4+2)*tmp1h)
      T_s1 = T_ch + T_cl
      Zl(J4-1) = T_cl - (T_s1 - T_ch)
      Zh(J4-1) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  Z(J4-1) = Z(J4+2) * tmp1
!!                  D = Z( J4+2 )*( D / Z( J4-3 ) ) - TAU
!<<<  tmp1 = D / Z(J4-3)
      T_q1 = Dh / Zh(J4-3)
      T_ch = Zh(J4-3) * T_q1
      T_u1 = 134217729.0 * Zh(J4-3)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4-3))
      T_p2 = Zh(J4-3) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Zl(J4-3) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Dh - T_ch
      T_r1 = T_s1 - Dh
      T_s2 = (Dh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Dl
      T_q2 = (T_s1 + T_s2)
     +         / Zh(J4-3)
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = D / Z(J4-3)
!<<<  tmp2 = Z(J4+2) * tmp1
      T_ch = Zh(J4+2) * tmp1h
      T_u1 = 134217729.0 * Zh(J4+2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4+2))
      T_p2 = Zh(J4+2) - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(J4+2)*tmp1l
     +         +  Zl(J4+2)*tmp1h)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = Z(J4+2) * tmp1
!<<<  D = tmp2 - TAU
      T_p1 = tmp2h - TAUh
      T_p2 = T_p1 - tmp2h
      T_ch = T_p1
      T_cl = (tmp2h - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      T_cl = T_cl + tmp2l - TAUl
      T_s1 = T_ch + T_cl
      Dl = T_cl - (T_s1 - T_ch)
      Dh = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  D = tmp2 - TAU
               END IF
!!               DMIN = MIN( DMIN, D )
               IF(
!       (DMIN .gt. D)
     + ((DMINh .gt. Dh) .or.
     +  ((DMINh .eq. Dh) .and.
     +   (DMINl .gt. Dl)))
     $           ) THEN
      DMINh = Dh
      DMINl = Dl
               END IF
!!               EMIN = MIN( EMIN, Z( J4-1 ) )
               IF(
!       (EMIN .gt. Z(J4-1))
     + ((EMINh .gt. Zh(J4-1)) .or.
     +  ((EMINh .eq. Zh(J4-1)) .and.
     +   (EMINl .gt. Zl(J4-1))))
     $           ) THEN
      EMINh = Zh(J4-1)
      EMINl = Zl(J4-1)
               END IF
   40       CONTINUE
         END IF
*
*        Unroll last two steps. 
*
!!         DNM2 = D
      DNM2h = Dh
      DNM2l = Dl
!!         DMIN2 = DMIN
      DMIN2h = DMINh
      DMIN2l = DMINl
         J4 = 4*( N0-2 ) - PP
         J4P2 = J4 + 2*PP - 1
!!         Z( J4-2 ) = DNM2 + Z( J4P2 )
!<<<  Z(J4-2) = DNM2 + Z(J4P2)
      T_p1 = DNM2h + Zh(J4P2)
      T_p2 = T_p1 - DNM2h
      T_q1 = T_p1
      T_q2 = (DNM2h - (T_p1 - T_p2))
     +         + (Zh(J4P2) - T_p2)
      T_q2 = T_q2 + DNM2l + Zl(J4P2)
      Zh(J4-2) = T_q1 + T_q2
      Zl(J4-2) = T_q2 - (Zh(J4-2) - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  Z(J4-2) = DNM2 + Z(J4P2)
!!         IF( DNM2.LT.ZERO ) THEN
         IF(
!       (DNM2 .lt. ZERO)
     + ((DNM2h .lt. ZEROh) .or.
     +  ((DNM2h .eq. ZEROh) .and.
     +   (DNM2l .lt. ZEROl)))
     $     ) THEN
            RETURN
         ELSE
!!            Z( J4 ) = Z( J4P2+2 )*( Z( J4P2 ) / Z( J4-2 ) )
!<<<  tmp1 = Z(J4P2) / Z(J4-2)
      T_q1 = Zh(J4P2) / Zh(J4-2)
      T_ch = Zh(J4-2) * T_q1
      T_u1 = 134217729.0 * Zh(J4-2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4-2))
      T_p2 = Zh(J4-2) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Zl(J4-2) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Zh(J4P2) - T_ch
      T_r1 = T_s1 - Zh(J4P2)
      T_s2 = (Zh(J4P2)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Zl(J4P2)
      T_q2 = (T_s1 + T_s2)
     +         / Zh(J4-2)
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = Z(J4P2) / Z(J4-2)
!<<<  Z(J4) = Z(J4P2+2) * tmp1
      T_ch = Zh(J4P2+2) * tmp1h
      T_u1 = 134217729.0 * Zh(J4P2+2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4P2+2))
      T_p2 = Zh(J4P2+2) - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(J4P2+2)*tmp1l
     +         +  Zl(J4P2+2)*tmp1h)
      T_s1 = T_ch + T_cl
      Zl(J4) = T_cl - (T_s1 - T_ch)
      Zh(J4) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  Z(J4) = Z(J4P2+2) * tmp1
!!            DNM1 = Z( J4P2+2 )*( DNM2 / Z( J4-2 ) ) - TAU
!<<<  tmp1 = DNM2 / Z(J4-2)
      T_q1 = DNM2h / Zh(J4-2)
      T_ch = Zh(J4-2) * T_q1
      T_u1 = 134217729.0 * Zh(J4-2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4-2))
      T_p2 = Zh(J4-2) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Zl(J4-2) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = DNM2h - T_ch
      T_r1 = T_s1 - DNM2h
      T_s2 = (DNM2h
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + DNM2l
      T_q2 = (T_s1 + T_s2)
     +         / Zh(J4-2)
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = DNM2 / Z(J4-2)
!<<<  tmp2 = Z(J4P2+2) * tmp1
      T_ch = Zh(J4P2+2) * tmp1h
      T_u1 = 134217729.0 * Zh(J4P2+2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4P2+2))
      T_p2 = Zh(J4P2+2) - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(J4P2+2)*tmp1l
     +         +  Zl(J4P2+2)*tmp1h)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = Z(J4P2+2) * tmp1
!<<<  DNM1 = tmp2 - TAU
      T_p1 = tmp2h - TAUh
      T_p2 = T_p1 - tmp2h
      T_ch = T_p1
      T_cl = (tmp2h - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      T_cl = T_cl + tmp2l - TAUl
      T_s1 = T_ch + T_cl
      DNM1l = T_cl - (T_s1 - T_ch)
      DNM1h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  DNM1 = tmp2 - TAU
         END IF
!!         DMIN = MIN( DMIN, DNM1 )
         IF(
!       (DMIN .gt. DNM1)
     + ((DMINh .gt. DNM1h) .or.
     +  ((DMINh .eq. DNM1h) .and.
     +   (DMINl .gt. DNM1l)))
     $     ) THEN
      DMINh = DNM1h
      DMINl = DNM1l
         END IF
*
!!         DMIN1 = DMIN
      DMIN1h = DMINh
      DMIN1l = DMINl
         J4 = J4 + 4
         J4P2 = J4 + 2*PP - 1
!!         Z( J4-2 ) = DNM1 + Z( J4P2 )
!<<<  Z(J4-2) = DNM1 + Z(J4P2)
      T_p1 = DNM1h + Zh(J4P2)
      T_p2 = T_p1 - DNM1h
      T_q1 = T_p1
      T_q2 = (DNM1h - (T_p1 - T_p2))
     +         + (Zh(J4P2) - T_p2)
      T_q2 = T_q2 + DNM1l + Zl(J4P2)
      Zh(J4-2) = T_q1 + T_q2
      Zl(J4-2) = T_q2 - (Zh(J4-2) - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  Z(J4-2) = DNM1 + Z(J4P2)
!!         IF( DNM1.LT.ZERO ) THEN
         IF(
!       (DNM1 .lt. ZERO)
     + ((DNM1h .lt. ZEROh) .or.
     +  ((DNM1h .eq. ZEROh) .and.
     +   (DNM1l .lt. ZEROl)))
     $     ) THEN
            RETURN
         ELSE
!!            Z( J4 ) = Z( J4P2+2 )*( Z( J4P2 ) / Z( J4-2 ) )
!<<<  tmp1 = Z(J4P2) / Z(J4-2)
      T_q1 = Zh(J4P2) / Zh(J4-2)
      T_ch = Zh(J4-2) * T_q1
      T_u1 = 134217729.0 * Zh(J4-2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4-2))
      T_p2 = Zh(J4-2) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Zl(J4-2) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Zh(J4P2) - T_ch
      T_r1 = T_s1 - Zh(J4P2)
      T_s2 = (Zh(J4P2)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Zl(J4P2)
      T_q2 = (T_s1 + T_s2)
     +         / Zh(J4-2)
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = Z(J4P2) / Z(J4-2)
!<<<  Z(J4) = Z(J4P2+2) * tmp1
      T_ch = Zh(J4P2+2) * tmp1h
      T_u1 = 134217729.0 * Zh(J4P2+2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4P2+2))
      T_p2 = Zh(J4P2+2) - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(J4P2+2)*tmp1l
     +         +  Zl(J4P2+2)*tmp1h)
      T_s1 = T_ch + T_cl
      Zl(J4) = T_cl - (T_s1 - T_ch)
      Zh(J4) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  Z(J4) = Z(J4P2+2) * tmp1
!!            DN = Z( J4P2+2 )*( DNM1 / Z( J4-2 ) ) - TAU
!<<<  tmp1 = DNM1 / Z(J4-2)
      T_q1 = DNM1h / Zh(J4-2)
      T_ch = Zh(J4-2) * T_q1
      T_u1 = 134217729.0 * Zh(J4-2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4-2))
      T_p2 = Zh(J4-2) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Zl(J4-2) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = DNM1h - T_ch
      T_r1 = T_s1 - DNM1h
      T_s2 = (DNM1h
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + DNM1l
      T_q2 = (T_s1 + T_s2)
     +         / Zh(J4-2)
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = DNM1 / Z(J4-2)
!<<<  tmp2 = Z(J4P2+2) * tmp1
      T_ch = Zh(J4P2+2) * tmp1h
      T_u1 = 134217729.0 * Zh(J4P2+2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J4P2+2))
      T_p2 = Zh(J4P2+2) - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(J4P2+2)*tmp1l
     +         +  Zl(J4P2+2)*tmp1h)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = Z(J4P2+2) * tmp1
!<<<  DN = tmp2 - TAU
      T_p1 = tmp2h - TAUh
      T_p2 = T_p1 - tmp2h
      T_ch = T_p1
      T_cl = (tmp2h - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      T_cl = T_cl + tmp2l - TAUl
      T_s1 = T_ch + T_cl
      DNl = T_cl - (T_s1 - T_ch)
      DNh = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  DN = tmp2 - TAU
         END IF
!!         DMIN = MIN( DMIN, DN )
         IF(
!       (DMIN .gt. DN)
     + ((DMINh .gt. DNh) .or.
     +  ((DMINh .eq. DNh) .and.
     +   (DMINl .gt. DNl)))
     $     ) THEN
      DMINh = DNh
      DMINl = DNl
         END IF
*
      END IF
*
!!      Z( J4+2 ) = DN
      Zh(J4+2) = DNh
      Zl(J4+2) = DNl
!!      Z( 4*N0-PP ) = EMIN
      Zh(4*N0-PP) = EMINh
      Zl(4*N0-PP) = EMINl
      RETURN
*
*     End of DLASQ5
*
      END
