*> \brief \b DSTEQR
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DSTEQR + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dsteqr.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dsteqr.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dsteqr.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DSTEQR( COMPZ, N, D, E, Z, LDZ, WORK, INFO )
* 
*       .. Scalar Arguments ..
*       CHARACTER          COMPZ
*       INTEGER            INFO, LDZ, N
*       ..
*       .. Array Arguments ..
*       DOUBLE PRECISION   D( * ), E( * ), WORK( * ), Z( LDZ, * )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DSTEQR computes all eigenvalues and, optionally, eigenvectors of a
*> symmetric tridiagonal matrix using the implicit QL or QR method.
*> The eigenvectors of a full or band symmetric matrix can also be found
*> if DSYTRD or DSPTRD or DSBTRD has been used to reduce this matrix to
*> tridiagonal form.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] COMPZ
*> \verbatim
*>          COMPZ is CHARACTER*1
*>          = 'N':  Compute eigenvalues only.
*>          = 'V':  Compute eigenvalues and eigenvectors of the original
*>                  symmetric matrix.  On entry, Z must contain the
*>                  orthogonal matrix used to reduce the original matrix
*>                  to tridiagonal form.
*>          = 'I':  Compute eigenvalues and eigenvectors of the
*>                  tridiagonal matrix.  Z is initialized to the identity
*>                  matrix.
*> \endverbatim
*>
*> \param[in] N
*> \verbatim
*>          N is INTEGER
*>          The order of the matrix.  N >= 0.
*> \endverbatim
*>
*> \param[in,out] D
*> \verbatim
*>          D is DOUBLE PRECISION array, dimension (N)
*>          On entry, the diagonal elements of the tridiagonal matrix.
*>          On exit, if INFO = 0, the eigenvalues in ascending order.
*> \endverbatim
*>
*> \param[in,out] E
*> \verbatim
*>          E is DOUBLE PRECISION array, dimension (N-1)
*>          On entry, the (n-1) subdiagonal elements of the tridiagonal
*>          matrix.
*>          On exit, E has been destroyed.
*> \endverbatim
*>
*> \param[in,out] Z
*> \verbatim
*>          Z is DOUBLE PRECISION array, dimension (LDZ, N)
*>          On entry, if  COMPZ = 'V', then Z contains the orthogonal
*>          matrix used in the reduction to tridiagonal form.
*>          On exit, if INFO = 0, then if  COMPZ = 'V', Z contains the
*>          orthonormal eigenvectors of the original symmetric matrix,
*>          and if COMPZ = 'I', Z contains the orthonormal eigenvectors
*>          of the symmetric tridiagonal matrix.
*>          If COMPZ = 'N', then Z is not referenced.
*> \endverbatim
*>
*> \param[in] LDZ
*> \verbatim
*>          LDZ is INTEGER
*>          The leading dimension of the array Z.  LDZ >= 1, and if
*>          eigenvectors are desired, then  LDZ >= max(1,N).
*> \endverbatim
*>
*> \param[out] WORK
*> \verbatim
*>          WORK is DOUBLE PRECISION array, dimension (max(1,2*N-2))
*>          If COMPZ = 'N', then WORK is not referenced.
*> \endverbatim
*>
*> \param[out] INFO
*> \verbatim
*>          INFO is INTEGER
*>          = 0:  successful exit
*>          < 0:  if INFO = -i, the i-th argument had an illegal value
*>          > 0:  the algorithm has failed to find all the eigenvalues in
*>                a total of 30*N iterations; if INFO = i, then i
*>                elements of E have not converged to zero; on exit, D
*>                and E contain the elements of a symmetric tridiagonal
*>                matrix which is orthogonally similar to the original
*>                matrix.
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERcomputational
*
*  =====================================================================
!      SUBROUTINE DSTEQR( COMPZ, N, D, E, Z, LDZ, WORK, INFO )
      SUBROUTINE DDSTEQR( COMPZ, N, Dh,Dl, Eh,El, Zh,Zl, 
     &                   LDZ, WORKh,WORKl, INFO )
*
*  -- LAPACK computational routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      CHARACTER          COMPZ
      INTEGER            INFO, LDZ, N
*     ..
*     .. Array Arguments ..
!      DOUBLE PRECISION   D( * ), E( * ), WORK( * ), Z( LDZ, * )
      double precision Dh(*), Dl(*)
      double precision Eh(*), El(*)
      double precision WORKh(*), WORKl(*)
      double precision Zh(LDZ,*), Zl(LDZ,*)
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
!      DOUBLE PRECISION   ZERO, ONE, TWO, THREE
      double precision ZEROh, ZEROl
      double precision ONEh, ONEl
      double precision TWOh, TWOl
      double precision THREEh, THREEl
!      PARAMETER          ( ZERO = 0.0D0, ONE = 1.0D0, TWO = 2.0D0,
!     $                   THREE = 3.0D0 )
      INTEGER            MAXIT
      PARAMETER          ( MAXIT = 30 )
*     ..
*     .. Local Scalars ..
      INTEGER            I, ICOMPZ, II, ISCALE, J, JTOT, K, L, L1, LEND,
     $                   LENDM1, LENDP1, LENDSV, LM1, LSV, M, MM, MM1,
     $                   NM1, NMAXIT
!      DOUBLE PRECISION   ANORM, B, C, EPS, EPS2, F, G, P, R, RT1, RT2,
!     $                   S, SAFMAX, SAFMIN, SSFMAX, SSFMIN, TST
      double precision ANORMh, ANORMl
      double precision Bh, Bl
      double precision Ch, Cl
      double precision EPSh, EPSl
      double precision EPS2h, EPS2l
      double precision Fh, Fl
      double precision Gh, Gl
      double precision Ph, Pl
      double precision Rh, Rl
      double precision RT1h, RT1l
      double precision RT2h, RT2l
      double precision Sh, Sl
      double precision SAFMAXh, SAFMAXl
      double precision SAFMINh, SAFMINl
      double precision SSFMAXh, SSFMAXl
      double precision SSFMINh, SSFMINl
      double precision TSTh, TSTl
*     ..
*     .. External Functions ..
      LOGICAL            LSAME
!      DOUBLE PRECISION   DLAMCH, DLANST, DLAPY2
!      DOUBLE PRECISION   DDLAMCH, DDLANST, DDLAPY2
!      EXTERNAL           LSAME, DLAMCH, DLANST, DLAPY2
      EXTERNAL           LSAME
*     ..
*     .. External Subroutines ..
!      EXTERNAL           DLAE2, DLAEV2, DLARTG, DLASCL, DLASET, DLASR,
!     $                   DLASRT, DSWAP, XERBLA
      EXTERNAL           DDLAE2, DDLAEV2, DDLARTG, DDLASCL, DDLASET,
     $                   DDLASR, DDLASRT, DDSWAP, XERBLA, DDLAMCH, 
     $                   DDLANST, DDLAPY2
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, MAX, SIGN, SQRT
*     ..
      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision tmp3h, tmp3l
      double precision tmp4h, tmp4l
      double precision tmp5h, tmp5l
      double precision tmp6h, tmp6l
      double precision tmp7h, tmp7l
      double precision tmp8h, tmp8l
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l


      ZEROh = 0.0d+0
      ZEROl = 0.0d+0
      ONEh = 1.0d+0
      ONEl = 0.0d+0
      TWOh = 2.0D+0
      TWOl = 0.0d+0
      THREEh = 3.0D+0
      THREEl = 0.0d+0
*     .. Executable Statements ..
*
*     Test the input parameters.
*
      INFO = 0
*
      IF( LSAME( COMPZ, 'N' ) ) THEN
         ICOMPZ = 0
      ELSE IF( LSAME( COMPZ, 'V' ) ) THEN
         ICOMPZ = 1
      ELSE IF( LSAME( COMPZ, 'I' ) ) THEN
         ICOMPZ = 2
      ELSE
         ICOMPZ = -1
      END IF
      IF( ICOMPZ.LT.0 ) THEN
         INFO = -1
      ELSE IF( N.LT.0 ) THEN
         INFO = -2
      ELSE IF( ( LDZ.LT.1 ) .OR. ( ICOMPZ.GT.0 .AND. LDZ.LT.MAX( 1,
     $         N ) ) ) THEN
         INFO = -6
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'DDSTEQR', -INFO )
         RETURN
      END IF
*
*     Quick return if possible
*
      IF( N.EQ.0 )
     $   RETURN
*
      IF( N.EQ.1 ) THEN
         IF( ICOMPZ.EQ.2 ) THEN
!     $      Z( 1, 1 ) = ONE
      Zh(1,1) = ONEh
      Zl(1,1) = ONEl
        END IF
         RETURN
      END IF
*
*     Determine the unit roundoff and over/underflow thresholds.
*
!      EPS = DLAMCH( 'E' )
      CALL DDLAMCH('E',EPSh,EPSl)
!      EPS2 = EPS**2
!<<<  EPS2 = EPS * EPS
      EPS2h = EPSh * EPSh
      T_u1 = 134217729.0 * EPSh
      T_p1 = T_u1
     +         - (T_u1 - EPSh)
      T_p2 = EPSh - T_p1
      T_u1 = 134217729.0 * EPSh
      T_r1 = T_u1
     +         - (T_u1 - EPSh)
      T_r2 = EPSh - T_r1
      EPS2l = ((T_p1*T_r1
     +        - EPS2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      EPS2l = EPS2l
     +      + (EPSh*EPSl
     +      + EPSl*EPSh)
      T_s1 = EPS2h + EPS2l
      EPS2l = EPS2l - (T_s1 - EPS2h)
      EPS2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  EPS2 = EPS * EPS
!      SAFMIN = DLAMCH( 'S' )
!      SAFMAX = ONE / SAFMIN
!      SSFMAX = SQRT( SAFMAX ) / THREE
!      SSFMIN = SQRT( SAFMIN ) / EPS2
      CALL DDLAMCH('S', SAFMINh,SAFMINl)
      SAFMAXh = ONEh / SAFMINh
      SAFMAXl = 0.0d0
      SSFMAXh = SQRT( SAFMAXh ) / THREEh
      SSFMAXl = 0.0d0
      SSFMINh = SQRT( SAFMINh ) / EPS2h
      SSFMINl = 0.0d0
*
*     Compute the eigenvalues and eigenvectors of the tridiagonal
*     matrix.
*
      IF( ICOMPZ.EQ.2 )
!     $   CALL DLASET( 'Full', N, N, ZERO, ONE, Z, LDZ )
     &   CALL DDLASET( 'Full', N, N,ZEROh,ZEROl,ONEh,ONEl,Zh,Zl,LDZ)
*
      NMAXIT = N*MAXIT
      JTOT = 0
*
*     Determine where the matrix splits and choose QL or QR iteration
*     for each block, according to whether top or bottom diagonal
*     element is smaller.
*
      L1 = 1
      NM1 = N - 1
*
   10 CONTINUE
      IF( L1.GT.N )
     $   GO TO 160
      IF( L1.GT.1 ) THEN
!     $   E( L1-1 ) = ZERO
      Eh(L1-1) = ZEROh
      El(L1-1) = ZEROl
      END IF
      IF( L1.LE.NM1 ) THEN
         DO 20 M = L1, NM1
!            TST = ABS( E( M ) )
!<<<  TST = abs(E(M))
      if (Eh(M) .lt. 0.0d+0) then
        TSTh = -Eh(M)
        TSTl = -El(M)
      elseif (
     +  (Eh(M) .eq. 0.0d+0).and.
     +  (El(M) .lt. 0.0d+0)) then
        TSTh = Eh(M)
        TSTl = -El(M)
      else
        TSTh = Eh(M)
        TSTl = El(M)
      endif
!>>>  TST = abs(E(M))
            IF(
!               TST.EQ.ZERO
!       (TST .eq. ZERO)
     + ((TSTh .eq. ZEROh) .and.
     +  (TSTl .eq. ZEROl))
     &         )
     $         GO TO 30
!<<<  tmp1 = abs(D(M))
      if (Dh(M) .lt. 0.0d+0) then
        tmp1h = -Dh(M)
        tmp1l = -Dl(M)
      elseif (
     +  (Dh(M) .eq. 0.0d+0).and.
     +  (Dl(M) .lt. 0.0d+0)) then
        tmp1h = Dh(M)
        tmp1l = -Dl(M)
      else
        tmp1h = Dh(M)
        tmp1l = Dl(M)
      endif
!>>>  tmp1 = abs(D(M))
!<<<  tmp2 = abs(D(M+1))
      if (Dh(M+1) .lt. 0.0d+0) then
        tmp2h = -Dh(M+1)
        tmp2l = -Dl(M+1)
      elseif (
     +  (Dh(M+1) .eq. 0.0d+0).and.
     +  (Dl(M+1) .lt. 0.0d+0)) then
        tmp2h = Dh(M+1)
        tmp2l = -Dl(M+1)
      else
        tmp2h = Dh(M+1)
        tmp2l = Dl(M+1)
      endif
!>>>  tmp2 = abs(D(M+1))
!<<<  tmp3 = sqrt(tmp1)
      if (tmp1h.eq.0.0D+0.and.tmp1l.eq.0.0D+0) then
        tmp3h = 0.0d+0
        tmp3l = 0.0d+0
      endif
      if (tmp1h.lt.0.0d+0.or.
     +   (tmp1h.eq.0.0d+0.and.tmp1l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp3h = 0.0d+0
        tmp3l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp1h)
      T_ax = tmp1h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp1h,tmp1l,T_z1,T_z2)
          T_p1 = tmp1h - T_z1
          T_p2 = T_p1 - tmp1h
          T_t2l = (tmp1h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp1l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp3h = T_t1h
          tmp3l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp3 = sqrt(tmp1)
!<<<  tmp4 = sqrt(tmp2)
      if (tmp2h.eq.0.0D+0.and.tmp2l.eq.0.0D+0) then
        tmp4h = 0.0d+0
        tmp4l = 0.0d+0
      endif
      if (tmp2h.lt.0.0d+0.or.
     +   (tmp2h.eq.0.0d+0.and.tmp2l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp4h = 0.0d+0
        tmp4l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp2h)
      T_ax = tmp2h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp2h,tmp2l,T_z1,T_z2)
          T_p1 = tmp2h - T_z1
          T_p2 = T_p1 - tmp2h
          T_t2l = (tmp2h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp2l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp4h = T_t1h
          tmp4l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp4 = sqrt(tmp2)
!<<<  tmp5 = tmp3 * tmp4
      tmp5h = tmp3h * tmp4h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      tmp5l = ((T_p1*T_r1
     +        - tmp5h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp5l = tmp5l
     +      + (tmp3h*tmp4l
     +      + tmp3l*tmp4h)
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp5 = tmp3 * tmp4
!<<<  tmp6 = tmp5 * EPS
      tmp6h = tmp5h * EPSh
      T_u1 = 134217729.0 * tmp5h
      T_p1 = T_u1
     +         - (T_u1 - tmp5h)
      T_p2 = tmp5h - T_p1
      T_u1 = 134217729.0 * EPSh
      T_r1 = T_u1
     +         - (T_u1 - EPSh)
      T_r2 = EPSh - T_r1
      tmp6l = ((T_p1*T_r1
     +        - tmp6h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp6l = tmp6l
     +      + (tmp5h*EPSl
     +      + tmp5l*EPSh)
      T_s1 = tmp6h + tmp6l
      tmp6l = tmp6l - (T_s1 - tmp6h)
      tmp6h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp6 = tmp5 * EPS
            IF(
!              TST.LE.( SQRT( ABS( D( M ) ) )*SQRT( ABS( D( M+
!     $          1 ) ) ) )*EPS 
!       (TST .le. tmp6)
     + ((TSTh .lt. tmp6h) .or.
     +  ((TSTh .eq. tmp6h) .and.
     +   (TSTl .le. tmp6l)))
     &         ) THEN
!               E( M ) = ZERO
      Eh(M) = ZEROh
      El(M) = ZEROl
               GO TO 30
            END IF
   20    CONTINUE
      END IF
      M = N
*
   30 CONTINUE
      L = L1
      LSV = L
      LEND = M
      LENDSV = LEND
      L1 = M + 1
      IF( LEND.EQ.L )
     $   GO TO 10
*
*     Scale submatrix in rows and columns L to LEND
*
!      ANORM = DLANST( 'M', LEND-L+1, D( L ), E( L ) )
      CALL DDLANST('M',LEND-L+1,Dh(L),Dl(L),Eh(L),El(L),ANORMh,ANORMl)
      ISCALE = 0
      IF(
!         ANORM.EQ.ZERO
!       (ANORM .eq. ZERO)
     + ((ANORMh .eq. ZEROh) .and.
     +  (ANORMl .eq. ZEROl))
     &    )
     $   GO TO 10
      IF( 
!         ANORM.GT.SSFMAX
!       (ANORM .gt. SSFMAX)
     + ((ANORMh .gt. SSFMAXh) .or.
     +  ((ANORMh .eq. SSFMAXh) .and.
     +   (ANORMl .gt. SSFMAXl)))
     &   ) THEN
         ISCALE = 1
!         CALL DLASCL( 'G', 0, 0, ANORM, SSFMAX, LEND-L+1, 1, D( L ), N,
!     $                INFO )
         CALL DDLASCL( 'G', 0, 0, ANORMh,ANORMl, SSFMAXh,SSFMAXl, 
     $                LEND-L+1, 1, Dh(L),Dl(L), N,INFO )
!         CALL DLASCL( 'G', 0, 0, ANORM, SSFMAX, LEND-L, 1, E( L ), N,
!     $                INFO )
         CALL DDLASCL( 'G', 0, 0, ANORMh,ANORMl, SSFMAXh,SSFMAXl, 
     $                LEND-L, 1, Eh(L),El(L),N,INFO )
      ELSE IF(
! ANORM.LT.SSFMIN 
!       (ANORM .lt. SSFMIN)
     + ((ANORMh .lt. SSFMINh) .or.
     +  ((ANORMh .eq. SSFMINh) .and.
     +   (ANORMl .lt. SSFMINl)))
     &        ) THEN
         ISCALE = 2
!         CALL DLASCL( 'G', 0, 0, ANORM, SSFMIN, LEND-L+1, 1, D( L ), N,
!     $                INFO )
         CALL DDLASCL( 'G', 0, 0, ANORMh,ANORMl, SSFMINh,SSFMINl,
     $                LEND-L+1, 1, Dh(L), Dl(L), N, INFO )
!         CALL DLASCL( 'G', 0, 0, ANORM, SSFMIN, LEND-L, 1, E( L ), N,
!     $                INFO )
         CALL DDLASCL( 'G', 0, 0, ANORMh,ANORMl, SSFMINh,SSFMINl,
     $                 LEND-L, 1, Eh(L),El(L), N,INFO )
      END IF
*
*     Choose between QL and QR iteration
*
!<<<  tmp1 = abs(D(LEND))
      if (Dh(LEND) .lt. 0.0d+0) then
        tmp1h = -Dh(LEND)
        tmp1l = -Dl(LEND)
      elseif (
     +  (Dh(LEND) .eq. 0.0d+0).and.
     +  (Dl(LEND) .lt. 0.0d+0)) then
        tmp1h = Dh(LEND)
        tmp1l = -Dl(LEND)
      else
        tmp1h = Dh(LEND)
        tmp1l = Dl(LEND)
      endif
!>>>  tmp1 = abs(D(LEND))
!<<<  tmp2 = abs(D(L))
      if (Dh(L) .lt. 0.0d+0) then
        tmp2h = -Dh(L)
        tmp2l = -Dl(L)
      elseif (
     +  (Dh(L) .eq. 0.0d+0).and.
     +  (Dl(L) .lt. 0.0d+0)) then
        tmp2h = Dh(L)
        tmp2l = -Dl(L)
      else
        tmp2h = Dh(L)
        tmp2l = Dl(L)
      endif
!>>>  tmp2 = abs(D(L))
      IF(
!        ABS( D( LEND ) ).LT.ABS( D( L ) )
!       (tmp1 .lt. tmp2)
     + ((tmp1h .lt. tmp2h) .or.
     +  ((tmp1h .eq. tmp2h) .and.
     +   (tmp1l .lt. tmp2l)))
     &    ) THEN
         LEND = LSV
         L = LENDSV
      END IF
*
      IF( LEND.GT.L ) THEN
*
*        QL Iteration
*
*        Look for small subdiagonal element.
*
   40    CONTINUE
         IF( L.NE.LEND ) THEN
            LENDM1 = LEND - 1
            DO 50 M = L, LENDM1
!               TST = ABS( E( M ) )**2
!<<<  tmp1 = abs(E(M))
      if (Eh(M) .lt. 0.0d+0) then
        tmp1h = -Eh(M)
        tmp1l = -El(M)
      elseif (
     +  (Eh(M) .eq. 0.0d+0).and.
     +  (El(M) .lt. 0.0d+0)) then
        tmp1h = Eh(M)
        tmp1l = -El(M)
      else
        tmp1h = Eh(M)
        tmp1l = El(M)
      endif
!>>>  tmp1 = abs(E(M))
!<<<  TST = tmp1 * tmp1
      TSTh = tmp1h * tmp1h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      TSTl = ((T_p1*T_r1
     +        - TSTh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      TSTl = TSTl
     +      + (tmp1h*tmp1l
     +      + tmp1l*tmp1h)
      T_s1 = TSTh + TSTl
      TSTl = TSTl - (T_s1 - TSTh)
      TSTh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  TST = tmp1 * tmp1
!<<<  tmp3 = abs(D(M))
      if (Dh(M) .lt. 0.0d+0) then
        tmp3h = -Dh(M)
        tmp3l = -Dl(M)
      elseif (
     +  (Dh(M) .eq. 0.0d+0).and.
     +  (Dl(M) .lt. 0.0d+0)) then
        tmp3h = Dh(M)
        tmp3l = -Dl(M)
      else
        tmp3h = Dh(M)
        tmp3l = Dl(M)
      endif
!>>>  tmp3 = abs(D(M))
!<<<  tmp4 = abs(D(M+1))
      if (Dh(M+1) .lt. 0.0d+0) then
        tmp4h = -Dh(M+1)
        tmp4l = -Dl(M+1)
      elseif (
     +  (Dh(M+1) .eq. 0.0d+0).and.
     +  (Dl(M+1) .lt. 0.0d+0)) then
        tmp4h = Dh(M+1)
        tmp4l = -Dl(M+1)
      else
        tmp4h = Dh(M+1)
        tmp4l = Dl(M+1)
      endif
!>>>  tmp4 = abs(D(M+1))
!<<<  tmp5 = EPS2 * tmp3
      tmp5h = EPS2h * tmp3h
      T_u1 = 134217729.0 * EPS2h
      T_p1 = T_u1
     +         - (T_u1 - EPS2h)
      T_p2 = EPS2h - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      tmp5l = ((T_p1*T_r1
     +        - tmp5h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp5l = tmp5l
     +      + (EPS2h*tmp3l
     +      + EPS2l*tmp3h)
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp5 = EPS2 * tmp3
!<<<  tmp6 = tmp5 * tmp4
      tmp6h = tmp5h * tmp4h
      T_u1 = 134217729.0 * tmp5h
      T_p1 = T_u1
     +         - (T_u1 - tmp5h)
      T_p2 = tmp5h - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      tmp6l = ((T_p1*T_r1
     +        - tmp6h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp6l = tmp6l
     +      + (tmp5h*tmp4l
     +      + tmp5l*tmp4h)
      T_s1 = tmp6h + tmp6l
      tmp6l = tmp6l - (T_s1 - tmp6h)
      tmp6h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp6 = tmp5 * tmp4
!<<<  tmp7 = tmp6 + SAFMIN
      T_p1 = tmp6h + SAFMINh
      T_p2 = T_p1 - tmp6h
      tmp7h = T_p1
      tmp7l = (tmp6h - (T_p1 - T_p2))
     +      + (SAFMINh - T_p2)
      tmp7l = tmp7l + tmp6l + SAFMINl
      T_s1 = tmp7h + tmp7l
      tmp7l = tmp7l - (T_s1 - tmp7h)
      tmp7h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp7 = tmp6 + SAFMIN
               IF(
!                  TST.LE.(EPS2*ABS(D( M )))*ABS(D( M+1))+SAFMIN
!       (TST .le. tmp7)
     + ((TSTh .lt. tmp7h) .or.
     +  ((TSTh .eq. tmp7h) .and.
     +   (TSTl .le. tmp7l)))
     $             )GO TO 60
   50       CONTINUE
         END IF
*
         M = LEND
*
   60    CONTINUE
         IF( M.LT.LEND )THEN
!     $      E( M ) = ZERO
      Eh(M) = ZEROh
      El(M) = ZEROl
         END IF
!         P = D( L )
      Ph = Dh(L)
      Pl = Dl(L)
         IF( M.EQ.L )
     $      GO TO 80
*
*        If remaining matrix is 2-by-2, use DLAE2 or SLAEV2
*        to compute its eigensystem.
*
         IF( M.EQ.L+1 ) THEN
            IF( ICOMPZ.GT.0 ) THEN
!               CALL DLAEV2( D( L ), E( L ), D( L+1 ), RT1, RT2, C, S )
               CALL DDLAEV2(Dh(L),Dl(L),Eh(L),El(L),Dh(L+1),Dl(L+1),
     &                      RT1h,RT1l,RT2h,RT2l,Ch,Cl,Sh,Sl )
!               WORK( L ) = C
      WORKh(L) = Ch
      WORKl(L) = Cl
!               WORK( N-1+L ) = S
      WORKh(N-1+L) = Sh
      WORKl(N-1+L) = Sl
!               CALL DLASR( 'R', 'V', 'B', N, 2, WORK( L ),
!     $                     WORK( N-1+L ), Z( 1, L ), LDZ )
               CALL DDLASR( 'R', 'V', 'B', N, 2, WORKh(L),WORKl(L),
     $                  WORKh(N-1+L),WORKl(N-1+L),Zh(1,L),Zl(1,L),LDZ)
            ELSE
!               CALL DLAE2( D( L ), E( L ), D( L+1 ), RT1, RT2 )
               CALL DDLAE2( Dh(L),Dl(L), Eh(L),El(L), 
     $                     Dh(L+1),Dl(L+1), RT1h,RT1l, RT2h, RT2l )
            END IF
!            D( L ) = RT1
!            D( L+1 ) = RT2
!            E( L ) = ZERO
      Dh(L) = RT1h
      Dl(L) = RT1l
      Dh(L+1) = RT2h
      Dl(L+1) = RT2l
      Eh(L) = ZEROh
      El(L) = ZEROl
            L = L + 2
            IF( L.LE.LEND )
     $         GO TO 40
            GO TO 140
         END IF
*
         IF( JTOT.EQ.NMAXIT )
     $      GO TO 140
         JTOT = JTOT + 1
*
*        Form shift.
*
!         G = ( D( L+1 )-P ) / ( TWO*E( L ) )
!<<<  tmp1 = TWO * E(L)
      tmp1h = TWOh * Eh(L)
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Eh(L)
      T_r1 = T_u1
     +         - (T_u1 - Eh(L))
      T_r2 = Eh(L) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*El(L)
     +      + TWOl*Eh(L))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * E(L)
!<<<  tmp2 = D(L+1) - P
      T_p1 = Dh(L+1) - Ph
      T_p2 = T_p1 - Dh(L+1)
      tmp2h = T_p1
      tmp2l = (Dh(L+1) - (T_p1 - T_p2))
     +         - (Ph + T_p2)
      tmp2l = tmp2l + Dl(L+1) - Pl
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp2 = D(L+1) - P
!<<<  G = tmp2 / tmp1
      T_q1 = tmp2h / tmp1h
      Gh = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      Gl = ((T_p1*T_r1
     +       - Gh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      Gl = Gl + (tmp1l*T_q1)
      T_s1 = Gh + Gl
      Gl = Gl - (T_s1 - Gh)
      Gh = T_s1
      T_s1 = tmp2h - Gh
      T_r1 = T_s1 - tmp2h
      T_s2 = (tmp2h 
     +         - (T_s1 - T_r1))
     +         - (Gh + T_r1)
      T_s2 = T_s2 - Gl
      T_s2 = T_s2 + tmp2l
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      Gh = T_q1 + T_q2
      Gl = T_q2
     +      - (Gh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  G = tmp2 / tmp1
!         R = DLAPY2( G, ONE )
         CALL DDLAPY2(Gh,Gl,ONEh,ONEl,Rh,Rl)
!         G = D( M ) - P + ( E( L ) / ( G+SIGN( R, G ) ) )
         IF(
!       (G .ge. 0.0)
     + ((Gh .gt. 0.0d+0) .or.
     +  ((Gh .eq. 0.0d+0) .and.
     +   (Gl .ge. 0.0d+0)))
     &     ) THEN
!<<<  tmp1 = abs(R)
      if (Rh .lt. 0.0d+0) then
        tmp1h = -Rh
        tmp1l = -Rl
      elseif (
     +  (Rh .eq. 0.0d+0).and.
     +  (Rl .lt. 0.0d+0)) then
        tmp1h = Rh
        tmp1l = -Rl
      else
        tmp1h = Rh
        tmp1l = Rl
      endif
!>>>  tmp1 = abs(R)
         ELSE
!<<<  tmp2 = abs(R)
      if (Rh .lt. 0.0d+0) then
        tmp2h = -Rh
        tmp2l = -Rl
      elseif (
     +  (Rh .eq. 0.0d+0).and.
     +  (Rl .lt. 0.0d+0)) then
        tmp2h = Rh
        tmp2l = -Rl
      else
        tmp2h = Rh
        tmp2l = Rl
      endif
!>>>  tmp2 = abs(R)
!<<<  tmp1 = -tmp2
      tmp1h = -tmp2h
      tmp1l = -tmp2l
!>>>  tmp1 = -tmp2
         END IF
!<<<  tmp3 = G + tmp1
      T_p1 = Gh + tmp1h
      T_p2 = T_p1 - Gh
      tmp3h = T_p1
      tmp3l = (Gh - (T_p1 - T_p2))
     +      + (tmp1h - T_p2)
      tmp3l = tmp3l + Gl + tmp1l
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp3 = G + tmp1
!<<<  tmp4 = E(L) / tmp3
      T_q1 = Eh(L) / tmp3h
      tmp4h = tmp3h * T_q1
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp4l = ((T_p1*T_r1
     +       - tmp4h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp4l = tmp4l + (tmp3l*T_q1)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
      T_s1 = Eh(L) - tmp4h
      T_r1 = T_s1 - Eh(L)
      T_s2 = (Eh(L) 
     +         - (T_s1 - T_r1))
     +         - (tmp4h + T_r1)
      T_s2 = T_s2 - tmp4l
      T_s2 = T_s2 + El(L)
      T_q2 = (T_s1 + T_s2)
     +         / tmp3h
      tmp4h = T_q1 + T_q2
      tmp4l = T_q2
     +      - (tmp4h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp4 = E(L) / tmp3
!<<<  tmp5 = D(M) - P
      T_p1 = Dh(M) - Ph
      T_p2 = T_p1 - Dh(M)
      tmp5h = T_p1
      tmp5l = (Dh(M) - (T_p1 - T_p2))
     +         - (Ph + T_p2)
      tmp5l = tmp5l + Dl(M) - Pl
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp5 = D(M) - P
!<<<  G = tmp5 + tmp4
      T_p1 = tmp5h + tmp4h
      T_p2 = T_p1 - tmp5h
      Gh = T_p1
      Gl = (tmp5h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      Gl = Gl + tmp5l + tmp4l
      T_s1 = Gh + Gl
      Gl = Gl - (T_s1 - Gh)
      Gh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  G = tmp5 + tmp4
*
!         S = ONE
!         C = ONE
!         P = ZERO
      Sh = ONEh
      Sl = ONEl
      Ch = ONEh
      Cl = ONEl
      Ph = ZEROh
      Pl = ZEROl
*
*        Inner loop
*
         MM1 = M - 1
         DO 70 I = MM1, L, -1
!            F = S*E( I )
!<<<  F = S * E(I)
      Fh = Sh * Eh(I)
      T_u1 = 134217729.0 * Sh
      T_p1 = T_u1
     +         - (T_u1 - Sh)
      T_p2 = Sh - T_p1
      T_u1 = 134217729.0 * Eh(I)
      T_r1 = T_u1
     +         - (T_u1 - Eh(I))
      T_r2 = Eh(I) - T_r1
      Fl = ((T_p1*T_r1
     +        - Fh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      Fl = Fl
     +      + (Sh*El(I)
     +      + Sl*Eh(I))
      T_s1 = Fh + Fl
      Fl = Fl - (T_s1 - Fh)
      Fh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  F = S * E(I)
!            B = C*E( I )
!<<<  B = C * E(I)
      Bh = Ch * Eh(I)
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * Eh(I)
      T_r1 = T_u1
     +         - (T_u1 - Eh(I))
      T_r2 = Eh(I) - T_r1
      Bl = ((T_p1*T_r1
     +        - Bh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      Bl = Bl
     +      + (Ch*El(I)
     +      + Cl*Eh(I))
      T_s1 = Bh + Bl
      Bl = Bl - (T_s1 - Bh)
      Bh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  B = C * E(I)
!            CALL DLARTG( G, F, C, S, R )
            CALL DDLARTG(Gh,Gl,Fh,Fl,Ch,Cl,Sh,Sl,Rh,Rl)
            IF( I.NE.M-1 ) THEN
!     $         E( I+1 ) = R
      Eh(I+1) = Rh
      El(I+1) = Rl
           END IF
!            G = D( I+1 ) - P
!<<<  G = D(I+1) - P
      T_p1 = Dh(I+1) - Ph
      T_p2 = T_p1 - Dh(I+1)
      Gh = T_p1
      Gl = (Dh(I+1) - (T_p1 - T_p2))
     +         - (Ph + T_p2)
      Gl = Gl + Dl(I+1) - Pl
      T_s1 = Gh + Gl
      Gl = Gl - (T_s1 - Gh)
      Gh = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  G = D(I+1) - P
!            R = ( D( I )-G )*S + TWO*C*B
!<<<  tmp1 = D(I) - G
      T_p1 = Dh(I) - Gh
      T_p2 = T_p1 - Dh(I)
      tmp1h = T_p1
      tmp1l = (Dh(I) - (T_p1 - T_p2))
     +         - (Gh + T_p2)
      tmp1l = tmp1l + Dl(I) - Gl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = D(I) - G
!<<<  tmp2 = tmp1 * S
      tmp2h = tmp1h * Sh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * Sh
      T_r1 = T_u1
     +         - (T_u1 - Sh)
      T_r2 = Sh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (tmp1h*Sl
     +      + tmp1l*Sh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = tmp1 * S
!<<<  tmp3 = TWO * C
      tmp3h = TWOh * Ch
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (TWOh*Cl
     +      + TWOl*Ch)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = TWO * C
!<<<  tmp4 = tmp3 * B
      tmp4h = tmp3h * Bh
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*Bl
     +      + tmp3l*Bh)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * B
!<<<  R = tmp2 + tmp4
      T_p1 = tmp2h + tmp4h
      T_p2 = T_p1 - tmp2h
      Rh = T_p1
      Rl = (tmp2h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      Rl = Rl + tmp2l + tmp4l
      T_s1 = Rh + Rl
      Rl = Rl - (T_s1 - Rh)
      Rh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  R = tmp2 + tmp4
!            P = S*R
!<<<  P = S * R
      Ph = Sh * Rh
      T_u1 = 134217729.0 * Sh
      T_p1 = T_u1
     +         - (T_u1 - Sh)
      T_p2 = Sh - T_p1
      T_u1 = 134217729.0 * Rh
      T_r1 = T_u1
     +         - (T_u1 - Rh)
      T_r2 = Rh - T_r1
      Pl = ((T_p1*T_r1
     +        - Ph)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      Pl = Pl
     +      + (Sh*Rl
     +      + Sl*Rh)
      T_s1 = Ph + Pl
      Pl = Pl - (T_s1 - Ph)
      Ph = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  P = S * R
!            D( I+1 ) = G + P
!<<<  D(I+1) = G + P
      T_p1 = Gh + Ph
      T_p2 = T_p1 - Gh
      Dh(I+1) = T_p1
      Dl(I+1) = (Gh - (T_p1 - T_p2))
     +      + (Ph - T_p2)
      Dl(I+1) = Dl(I+1) + Gl + Pl
      T_s1 = Dh(I+1) + Dl(I+1)
      Dl(I+1) = Dl(I+1) - (T_s1 - Dh(I+1))
      Dh(I+1) = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  D(I+1) = G + P
!            G = C*R - B
!<<<  tmp1 = C * R
      tmp1h = Ch * Rh
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * Rh
      T_r1 = T_u1
     +         - (T_u1 - Rh)
      T_r2 = Rh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Ch*Rl
     +      + Cl*Rh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = C * R
!<<<  G = tmp1 - B
      T_p1 = tmp1h - Bh
      T_p2 = T_p1 - tmp1h
      Gh = T_p1
      Gl = (tmp1h - (T_p1 - T_p2))
     +         - (Bh + T_p2)
      Gl = Gl + tmp1l - Bl
      T_s1 = Gh + Gl
      Gl = Gl - (T_s1 - Gh)
      Gh = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  G = tmp1 - B
*
*           If eigenvectors are desired, then save rotations.
*
            IF( ICOMPZ.GT.0 ) THEN
!               WORK( I ) = C
      WORKh(I) = Ch
      WORKl(I) = Cl
!               WORK( N-1+I ) = -S
!<<<  WORK(N-1+I) = -S
      WORKh(N-1+I) = -Sh
      WORKl(N-1+I) = -Sl
!>>>  WORK(N-1+I) = -S
            END IF
*
   70    CONTINUE
*
*        If eigenvectors are desired, then apply saved rotations.
*
         IF( ICOMPZ.GT.0 ) THEN
            MM = M - L + 1
!            CALL DLASR( 'R', 'V', 'B', N, MM, WORK( L ), WORK( N-1+L ),
!     $                  Z( 1, L ), LDZ )
            CALL DDLASR( 'R', 'V', 'B', N, MM, WORKh( L ),WORKl( L ),
     $                 WORKh(N-1+L),WORKl(N-1+L),Zh(1,L),Zl(1,L), LDZ)
         END IF
*
!         D( L ) = D( L ) - P
!<<<  D(L) = D(L) - P
      T_p1 = Dh(L) - Ph
      T_p2 = T_p1 - Dh(L)
      T_ch = T_p1
      T_cl = (Dh(L) - (T_p1 - T_p2))
     +         - (Ph + T_p2)
      T_cl = T_cl + Dl(L) - Pl
      T_s1 = T_ch + T_cl
      Dl(L) = T_cl - (T_s1 - T_ch)
      Dh(L) = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  D(L) = D(L) - P
!         E( L ) = G
      Eh(L) = Gh
      El(L) = Gl
         GO TO 40
*
*        Eigenvalue found.
*
   80    CONTINUE
!         D( L ) = P
      Dh(L) = Ph
      Dl(L) = Pl
*
         L = L + 1
         IF( L.LE.LEND )
     $      GO TO 40
         GO TO 140
*
      ELSE
*
*        QR Iteration
*
*        Look for small superdiagonal element.
*
   90    CONTINUE
         IF( L.NE.LEND ) THEN
            LENDP1 = LEND + 1
            DO 100 M = L, LENDP1, -1
!               TST = ABS( E( M-1 ) )**2
!<<<  tmp1 = abs(E(M-1))
      if (Eh(M-1) .lt. 0.0d+0) then
        tmp1h = -Eh(M-1)
        tmp1l = -El(M-1)
      elseif (
     +  (Eh(M-1) .eq. 0.0d+0).and.
     +  (El(M-1) .lt. 0.0d+0)) then
        tmp1h = Eh(M-1)
        tmp1l = -El(M-1)
      else
        tmp1h = Eh(M-1)
        tmp1l = El(M-1)
      endif
!>>>  tmp1 = abs(E(M-1))
!<<<  TST = tmp1 * tmp1
      TSTh = tmp1h * tmp1h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      TSTl = ((T_p1*T_r1
     +        - TSTh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      TSTl = TSTl
     +      + (tmp1h*tmp1l
     +      + tmp1l*tmp1h)
      T_s1 = TSTh + TSTl
      TSTl = TSTl - (T_s1 - TSTh)
      TSTh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  TST = tmp1 * tmp1
!<<<  tmp2 = abs(D(M))
      if (Dh(M) .lt. 0.0d+0) then
        tmp2h = -Dh(M)
        tmp2l = -Dl(M)
      elseif (
     +  (Dh(M) .eq. 0.0d+0).and.
     +  (Dl(M) .lt. 0.0d+0)) then
        tmp2h = Dh(M)
        tmp2l = -Dl(M)
      else
        tmp2h = Dh(M)
        tmp2l = Dl(M)
      endif
!>>>  tmp2 = abs(D(M))
!<<<  tmp3 = EPS2 * tmp2
      tmp3h = EPS2h * tmp2h
      T_u1 = 134217729.0 * EPS2h
      T_p1 = T_u1
     +         - (T_u1 - EPS2h)
      T_p2 = EPS2h - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (EPS2h*tmp2l
     +      + EPS2l*tmp2h)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = EPS2 * tmp2
!<<<  tmp4 = abs(D(M-1))
      if (Dh(M-1) .lt. 0.0d+0) then
        tmp4h = -Dh(M-1)
        tmp4l = -Dl(M-1)
      elseif (
     +  (Dh(M-1) .eq. 0.0d+0).and.
     +  (Dl(M-1) .lt. 0.0d+0)) then
        tmp4h = Dh(M-1)
        tmp4l = -Dl(M-1)
      else
        tmp4h = Dh(M-1)
        tmp4l = Dl(M-1)
      endif
!>>>  tmp4 = abs(D(M-1))
!<<<  tmp5 = tmp3 * tmp4
      tmp5h = tmp3h * tmp4h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      tmp5l = ((T_p1*T_r1
     +        - tmp5h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp5l = tmp5l
     +      + (tmp3h*tmp4l
     +      + tmp3l*tmp4h)
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp5 = tmp3 * tmp4
!<<<  tmp6 = tmp5 + SAFMIN
      T_p1 = tmp5h + SAFMINh
      T_p2 = T_p1 - tmp5h
      tmp6h = T_p1
      tmp6l = (tmp5h - (T_p1 - T_p2))
     +      + (SAFMINh - T_p2)
      tmp6l = tmp6l + tmp5l + SAFMINl
      T_s1 = tmp6h + tmp6l
      tmp6l = tmp6l - (T_s1 - tmp6h)
      tmp6h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp6 = tmp5 + SAFMIN
               IF(
!                 TST.LE.( EPS2*ABS( D( M ) ) )*ABS( D( M-1 ) )+SAFMIN 
!       (TST .le. tmp6)
     + ((TSTh .lt. tmp6h) .or.
     +  ((TSTh .eq. tmp6h) .and.
     +   (TSTl .le. tmp6l)))
     &           )GO TO 110
  100       CONTINUE
         END IF
*
         M = LEND
*
  110    CONTINUE
         IF( M.GT.LEND )THEN
!     $      E( M-1 ) = ZERO
      Eh(M-1) = ZEROh
      El(M-1) = ZEROl
         END IF
!         P = D( L )
      Ph = Dh(L)
      Pl = Dl(L)
         IF( M.EQ.L )
     $      GO TO 130
*
*        If remaining matrix is 2-by-2, use DLAE2 or SLAEV2
*        to compute its eigensystem.
*
         IF( M.EQ.L-1 ) THEN
            IF( ICOMPZ.GT.0 ) THEN
!               CALL DLAEV2( D( L-1 ), E( L-1 ), D( L ), RT1, RT2, C, S )
               CALL DDLAEV2(Dh(L-1),Dl(L-1),Eh(L-1),El(L-1),Dh(L),Dl(L),
     $                      RT1h,RT1l,RT2h,RT2l, Ch,Cl, Sh,Sl )

!               WORK( M ) = C
      WORKh(M) = Ch
      WORKl(M) = Cl
!               WORK( N-1+M ) = S
      WORKh(N-1+M) = Sh
      WORKl(N-1+M) = Sl
!               CALL DLASR( 'R', 'V', 'F', N, 2, WORK( M ),
!     $                     WORK( N-1+M ), Z( 1, L-1 ), LDZ )
               CALL DDLASR( 'R', 'V', 'F', N, 2, WORKh(M),WORKl(M),
     $             WORKh(N-1+M),WORKl(N-1+M), Zh(1,L-1),Zl(1,L-1),LDZ)
            ELSE
!               CALL DLAE2( D( L-1 ), E( L-1 ), D( L ), RT1, RT2 )
               CALL DDLAE2( Dh( L-1 ),Dl( L-1 ), Eh( L-1 ),El( L-1 ),
     $                      Dh( L ),Dl( L ), RT1h,RT1l, RT2h,RT2l )
            END IF
!            D( L-1 ) = RT1
      Dh(L-1) = RT1h
      Dl(L-1) = RT1l
!            D( L ) = RT2
      Dh(L) = RT2h
      Dl(L) = RT2l
!            E( L-1 ) = ZERO
      Eh(L-1) = ZEROh
      El(L-1) = ZEROl
            L = L - 2
            IF( L.GE.LEND )
     $         GO TO 90
            GO TO 140
         END IF
*
         IF( JTOT.EQ.NMAXIT )
     $      GO TO 140
         JTOT = JTOT + 1
*
*        Form shift.
*
!         G = ( D( L-1 )-P ) / ( TWO*E( L-1 ) )
!<<<  tmp1 = D(L-1) - P
      T_p1 = Dh(L-1) - Ph
      T_p2 = T_p1 - Dh(L-1)
      tmp1h = T_p1
      tmp1l = (Dh(L-1) - (T_p1 - T_p2))
     +         - (Ph + T_p2)
      tmp1l = tmp1l + Dl(L-1) - Pl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = D(L-1) - P
!<<<  tmp2 = TWO * E(L-1)
      tmp2h = TWOh * Eh(L-1)
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Eh(L-1)
      T_r1 = T_u1
     +         - (T_u1 - Eh(L-1))
      T_r2 = Eh(L-1) - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (TWOh*El(L-1)
     +      + TWOl*Eh(L-1))
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = TWO * E(L-1)
!<<<  G = tmp1 / tmp2
      T_q1 = tmp1h / tmp2h
      Gh = tmp2h * T_q1
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      Gl = ((T_p1*T_r1
     +       - Gh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      Gl = Gl + (tmp2l*T_q1)
      T_s1 = Gh + Gl
      Gl = Gl - (T_s1 - Gh)
      Gh = T_s1
      T_s1 = tmp1h - Gh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (Gh + T_r1)
      T_s2 = T_s2 - Gl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / tmp2h
      Gh = T_q1 + T_q2
      Gl = T_q2
     +      - (Gh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  G = tmp1 / tmp2
!         R = DLAPY2( G, ONE )
          CALL DDLAPY2(Gh,Gl,ONEh,ONEl,Rh,Rl)
!         G = D( M ) - P + ( E( L-1 ) / ( G+SIGN( R, G ) ) )
         IF(
!       (G .ge. 0.0)
     + ((Gh .gt. 0.0d+0) .or.
     +  ((Gh .eq. 0.0d+0) .and.
     +   (Gl .ge. 0.0d+0)))
     &     ) THEN
!<<<  tmp1 = abs(R)
      if (Rh .lt. 0.0d+0) then
        tmp1h = -Rh
        tmp1l = -Rl
      elseif (
     +  (Rh .eq. 0.0d+0).and.
     +  (Rl .lt. 0.0d+0)) then
        tmp1h = Rh
        tmp1l = -Rl
      else
        tmp1h = Rh
        tmp1l = Rl
      endif
!>>>  tmp1 = abs(R)
         ELSE
!<<<  tmp2 = abs(R)
      if (Rh .lt. 0.0d+0) then
        tmp2h = -Rh
        tmp2l = -Rl
      elseif (
     +  (Rh .eq. 0.0d+0).and.
     +  (Rl .lt. 0.0d+0)) then
        tmp2h = Rh
        tmp2l = -Rl
      else
        tmp2h = Rh
        tmp2l = Rl
      endif
!>>>  tmp2 = abs(R)
!<<<  tmp1 = -tmp2
      tmp1h = -tmp2h
      tmp1l = -tmp2l
!>>>  tmp1 = -tmp2
         END IF
!<<<  tmp3 = G + tmp1
      T_p1 = Gh + tmp1h
      T_p2 = T_p1 - Gh
      tmp3h = T_p1
      tmp3l = (Gh - (T_p1 - T_p2))
     +      + (tmp1h - T_p2)
      tmp3l = tmp3l + Gl + tmp1l
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp3 = G + tmp1
!<<<  tmp4 = E(L-1) / tmp3
      T_q1 = Eh(L-1) / tmp3h
      tmp4h = tmp3h * T_q1
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp4l = ((T_p1*T_r1
     +       - tmp4h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp4l = tmp4l + (tmp3l*T_q1)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
      T_s1 = Eh(L-1) - tmp4h
      T_r1 = T_s1 - Eh(L-1)
      T_s2 = (Eh(L-1) 
     +         - (T_s1 - T_r1))
     +         - (tmp4h + T_r1)
      T_s2 = T_s2 - tmp4l
      T_s2 = T_s2 + El(L-1)
      T_q2 = (T_s1 + T_s2)
     +         / tmp3h
      tmp4h = T_q1 + T_q2
      tmp4l = T_q2
     +      - (tmp4h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp4 = E(L-1) / tmp3
!<<<  tmp5 = D(M) - P
      T_p1 = Dh(M) - Ph
      T_p2 = T_p1 - Dh(M)
      tmp5h = T_p1
      tmp5l = (Dh(M) - (T_p1 - T_p2))
     +         - (Ph + T_p2)
      tmp5l = tmp5l + Dl(M) - Pl
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp5 = D(M) - P
!<<<  G = tmp5 + tmp4
      T_p1 = tmp5h + tmp4h
      T_p2 = T_p1 - tmp5h
      Gh = T_p1
      Gl = (tmp5h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      Gl = Gl + tmp5l + tmp4l
      T_s1 = Gh + Gl
      Gl = Gl - (T_s1 - Gh)
      Gh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  G = tmp5 + tmp4
*
!         S = ONE
      Sh = ONEh
      Sl = ONEl
!         C = ONE
      Ch = ONEh
      Cl = ONEl
!         P = ZERO
      Ph = ZEROh
      Pl = ZEROl
*
*        Inner loop
*
         LM1 = L - 1
         DO 120 I = M, LM1
!            F = S*E( I )
!<<<  F = S * E(I)
      Fh = Sh * Eh(I)
      T_u1 = 134217729.0 * Sh
      T_p1 = T_u1
     +         - (T_u1 - Sh)
      T_p2 = Sh - T_p1
      T_u1 = 134217729.0 * Eh(I)
      T_r1 = T_u1
     +         - (T_u1 - Eh(I))
      T_r2 = Eh(I) - T_r1
      Fl = ((T_p1*T_r1
     +        - Fh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      Fl = Fl
     +      + (Sh*El(I)
     +      + Sl*Eh(I))
      T_s1 = Fh + Fl
      Fl = Fl - (T_s1 - Fh)
      Fh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  F = S * E(I)
!            B = C*E( I )
!<<<  B = C * E(I)
      Bh = Ch * Eh(I)
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * Eh(I)
      T_r1 = T_u1
     +         - (T_u1 - Eh(I))
      T_r2 = Eh(I) - T_r1
      Bl = ((T_p1*T_r1
     +        - Bh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      Bl = Bl
     +      + (Ch*El(I)
     +      + Cl*Eh(I))
      T_s1 = Bh + Bl
      Bl = Bl - (T_s1 - Bh)
      Bh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  B = C * E(I)
!            CALL DLARTG( G, F, C, S, R )
            CALL DDLARTG( Gh,Gl,Fh,Fl,Ch,Cl,Sh,Sl,Rh,Rl )
            IF( I.NE.M )THEN
!     $         E( I-1 ) = R
      Eh(I-1) = Rh
      El(I-1) = Rl
            END IF
!            G = D( I ) - P
!<<<  G = D(I) - P
      T_p1 = Dh(I) - Ph
      T_p2 = T_p1 - Dh(I)
      Gh = T_p1
      Gl = (Dh(I) - (T_p1 - T_p2))
     +         - (Ph + T_p2)
      Gl = Gl + Dl(I) - Pl
      T_s1 = Gh + Gl
      Gl = Gl - (T_s1 - Gh)
      Gh = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  G = D(I) - P
!            R = ( D( I+1 )-G )*S + TWO*C*B
!<<<  tmp1 = TWO * C
      tmp1h = TWOh * Ch
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*Cl
     +      + TWOl*Ch)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * C
!<<<  tmp2 = tmp1 * B
      tmp2h = tmp1h * Bh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (tmp1h*Bl
     +      + tmp1l*Bh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = tmp1 * B
!<<<  tmp3 = D(I+1) - G
      T_p1 = Dh(I+1) - Gh
      T_p2 = T_p1 - Dh(I+1)
      tmp3h = T_p1
      tmp3l = (Dh(I+1) - (T_p1 - T_p2))
     +         - (Gh + T_p2)
      tmp3l = tmp3l + Dl(I+1) - Gl
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp3 = D(I+1) - G
!<<<  tmp4 = tmp3 * S
      tmp4h = tmp3h * Sh
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * Sh
      T_r1 = T_u1
     +         - (T_u1 - Sh)
      T_r2 = Sh - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*Sl
     +      + tmp3l*Sh)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * S
!<<<  R = tmp2 + tmp4
      T_p1 = tmp2h + tmp4h
      T_p2 = T_p1 - tmp2h
      Rh = T_p1
      Rl = (tmp2h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      Rl = Rl + tmp2l + tmp4l
      T_s1 = Rh + Rl
      Rl = Rl - (T_s1 - Rh)
      Rh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  R = tmp2 + tmp4
!            P = S*R
!<<<  P = S * R
      Ph = Sh * Rh
      T_u1 = 134217729.0 * Sh
      T_p1 = T_u1
     +         - (T_u1 - Sh)
      T_p2 = Sh - T_p1
      T_u1 = 134217729.0 * Rh
      T_r1 = T_u1
     +         - (T_u1 - Rh)
      T_r2 = Rh - T_r1
      Pl = ((T_p1*T_r1
     +        - Ph)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      Pl = Pl
     +      + (Sh*Rl
     +      + Sl*Rh)
      T_s1 = Ph + Pl
      Pl = Pl - (T_s1 - Ph)
      Ph = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  P = S * R
!            D( I ) = G + P
!<<<  D(I) = G + P
      T_p1 = Gh + Ph
      T_p2 = T_p1 - Gh
      Dh(I) = T_p1
      Dl(I) = (Gh - (T_p1 - T_p2))
     +      + (Ph - T_p2)
      Dl(I) = Dl(I) + Gl + Pl
      T_s1 = Dh(I) + Dl(I)
      Dl(I) = Dl(I) - (T_s1 - Dh(I))
      Dh(I) = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  D(I) = G + P
!            G = C*R - B
!<<<  tmp1 = C * R
      tmp1h = Ch * Rh
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * Rh
      T_r1 = T_u1
     +         - (T_u1 - Rh)
      T_r2 = Rh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Ch*Rl
     +      + Cl*Rh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = C * R
!<<<  G = tmp1 - B
      T_p1 = tmp1h - Bh
      T_p2 = T_p1 - tmp1h
      Gh = T_p1
      Gl = (tmp1h - (T_p1 - T_p2))
     +         - (Bh + T_p2)
      Gl = Gl + tmp1l - Bl
      T_s1 = Gh + Gl
      Gl = Gl - (T_s1 - Gh)
      Gh = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  G = tmp1 - B
*
*           If eigenvectors are desired, then save rotations.
*
            IF( ICOMPZ.GT.0 ) THEN
!               WORK( I ) = C
      WORKh(I) = Ch
      WORKl(I) = Cl
!               WORK( N-1+I ) = S
      WORKh(N-1+I) = Sh
      WORKl(N-1+I) = Sl
            END IF
*
  120    CONTINUE
*
*        If eigenvectors are desired, then apply saved rotations.
*
         IF( ICOMPZ.GT.0 ) THEN
            MM = L - M + 1
!            CALL DLASR( 'R', 'V', 'F', N, MM, WORK( M ), WORK( N-1+M ),
!     $                  Z( 1, M ), LDZ )
            CALL DDLASR( 'R', 'V', 'F', N, MM, WORKh(M),WORKl(M),
     $                WORKh(N-1+M),WORKl(N-1+M),Zh(1, M),Zl(1, M),LDZ)
         END IF
*
!         D( L ) = D( L ) - P
!<<<  D(L) = D(L) - P
      T_p1 = Dh(L) - Ph
      T_p2 = T_p1 - Dh(L)
      T_ch = T_p1
      T_cl = (Dh(L) - (T_p1 - T_p2))
     +         - (Ph + T_p2)
      T_cl = T_cl + Dl(L) - Pl
      T_s1 = T_ch + T_cl
      Dl(L) = T_cl - (T_s1 - T_ch)
      Dh(L) = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  D(L) = D(L) - P
!         E( LM1 ) = G
      Eh(LM1) = Gh
      El(LM1) = Gl
         GO TO 90
*
*        Eigenvalue found.
*
  130    CONTINUE
!         D( L ) = P
      Dh(L) = Ph
      Dl(L) = Pl
*
         L = L - 1
         IF( L.GE.LEND )
     $      GO TO 90
         GO TO 140
*
      END IF
*
*     Undo scaling if necessary
*
  140 CONTINUE
      IF( ISCALE.EQ.1 ) THEN
!         CALL DLASCL( 'G', 0, 0, SSFMAX, ANORM, LENDSV-LSV+1, 1,
!     $                D( LSV ), N, INFO )
         CALL DDLASCL( 'G', 0, 0, SSFMAXh,SSFMAXl, ANORMh,ANORMl,
     $                LENDSV-LSV+1, 1, Dh( LSV ),Dl( LSV ), N, INFO )
!         CALL DLASCL( 'G', 0, 0, SSFMAX, ANORM, LENDSV-LSV, 1, E( LSV ),
!     $                N, INFO )
         CALL DDLASCL( 'G', 0, 0, SSFMAXh,SSFMAXl, ANORMh,ANORMl,
     $                 LENDSV-LSV, 1, Eh( LSV ),El( LSV ),N, INFO )
      ELSE IF( ISCALE.EQ.2 ) THEN
!         CALL DLASCL( 'G', 0, 0, SSFMIN, ANORM, LENDSV-LSV+1, 1,
!     $                D( LSV ), N, INFO )
         CALL DDLASCL( 'G', 0, 0, SSFMINh,SSFMINl, ANORMh,ANORMl,
     $                LENDSV-LSV+1, 1, Dh( LSV ),Dl( LSV ), N, INFO )
!         CALL DLASCL( 'G', 0, 0, SSFMIN, ANORM, LENDSV-LSV, 1, E( LSV ),
!     $                N, INFO )
         CALL DDLASCL( 'G', 0, 0, SSFMINh,SSFMINl, ANORMh,ANORMl,
     $                LENDSV-LSV, 1, Eh( LSV ),El( LSV ), N, INFO )
      END IF
*
*     Check for no convergence to an eigenvalue after a total
*     of N*MAXIT iterations.
*
      IF( JTOT.LT.NMAXIT )
     $   GO TO 10
      DO 150 I = 1, N - 1
         IF(
!            E( I ).NE.ZERO
!       (E(I) .ne. ZERO)
     + ((Eh(I) .ne. ZEROh) .or.
     +  (El(I) .ne. ZEROl))
     $       )
     $      INFO = INFO + 1
  150 CONTINUE
      GO TO 190
*
*     Order eigenvalues and eigenvectors.
*
  160 CONTINUE
      IF( ICOMPZ.EQ.0 ) THEN
*
*        Use Quick Sort
*
!         CALL DLASRT( 'I', N, D, INFO )
         CALL DDLASRT( 'I', N, Dh, Dl,INFO )
*
      ELSE
*
*        Use Selection Sort to minimize swaps of eigenvectors
*
         DO 180 II = 2, N
            I = II - 1
            K = I
!            P = D( I )
      Ph = Dh(I)
      Pl = Dl(I)
            DO 170 J = II, N
               IF(
!                  D( J ).LT.P
!       (D(J) .lt. P)
     + ((Dh(J) .lt. Ph) .or.
     +  ((Dh(J) .eq. Ph) .and.
     +   (Dl(J) .lt. Pl)))
     &             ) THEN
                  K = J
!                  P = D( J )
      Ph = Dh(J)
      Pl = Dl(J)
               END IF
  170       CONTINUE
            IF( K.NE.I ) THEN
!               D( K ) = D( I )
      Dh(K) = Dh(I)
      Dl(K) = Dl(I)
!               D( I ) = P
      Dh(I) = Ph
      Dl(I) = Pl
!               CALL DSWAP( N, Z( 1, I ), 1, Z( 1, K ), 1 )
               CALL DDSWAP( N, Zh( 1, I ),Zl( 1, I ), 1, 
     $                      Zh( 1, K ),Zl( 1, K ), 1 )
            END IF
  180    CONTINUE
      END IF
*
  190 CONTINUE
      RETURN
*
*     End of DSTEQR
*
      END
