*> \brief \b DSTERF
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DSTERF + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dsterf.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dsterf.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dsterf.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DSTERF( N, D, E, INFO )
* 
*       .. Scalar Arguments ..
*       INTEGER            INFO, N
*       ..
*       .. Array Arguments ..
*       DOUBLE PRECISION   D( * ), E( * )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DSTERF computes all eigenvalues of a symmetric tridiagonal matrix
*> using the Pal-Walker-Kahan variant of the QL or QR algorithm.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] N
*> \verbatim
*>          N is INTEGER
*>          The order of the matrix.  N >= 0.
*> \endverbatim
*>
*> \param[in,out] D
*> \verbatim
*>          D is DOUBLE PRECISION array, dimension (N)
*>          On entry, the n diagonal elements of the tridiagonal matrix.
*>          On exit, if INFO = 0, the eigenvalues in ascending order.
*> \endverbatim
*>
*> \param[in,out] E
*> \verbatim
*>          E is DOUBLE PRECISION array, dimension (N-1)
*>          On entry, the (n-1) subdiagonal elements of the tridiagonal
*>          matrix.
*>          On exit, E has been destroyed.
*> \endverbatim
*>
*> \param[out] INFO
*> \verbatim
*>          INFO is INTEGER
*>          = 0:  successful exit
*>          < 0:  if INFO = -i, the i-th argument had an illegal value
*>          > 0:  the algorithm failed to find all of the eigenvalues in
*>                a total of 30*N iterations; if INFO = i, then i
*>                elements of E have not converged to zero.
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERcomputational
*
*  =====================================================================
!      SUBROUTINE DSTERF( N, D, E, INFO )
      SUBROUTINE DDSTERF( N, Dh, Dl, Eh, El, INFO )
*
*  -- LAPACK computational routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      INTEGER            INFO, N
*     ..
*     .. Array Arguments ..
!      DOUBLE PRECISION   D( * ), E( * )
      double precision Dh(*), Dl(*)
      double precision Eh(*), El(*)
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
!      DOUBLE PRECISION   ZERO, ONE, TWO, THREE
!      PARAMETER          ( ZERO = 0.0D0, ONE = 1.0D0, TWO = 2.0D0,
!     $                   THREE = 3.0D0 )
      double precision ZEROh, ZEROl
      double precision ONEh, ONEl
      double precision TWOh, TWOl
      double precision THREEh, THREEl
      INTEGER            MAXIT
      PARAMETER          ( MAXIT = 30 )
*     ..
*     .. Local Scalars ..
      INTEGER            I, ISCALE, JTOT, L, L1, LEND, LENDSV, LSV, M,
     $                   NMAXIT
!      DOUBLE PRECISION   ALPHA, ANORM, BB, C, EPS, EPS2, GAMMA, OLDC,
!     $                   OLDGAM, P, R, RT1, RT2, RTE, S, SAFMAX, SAFMIN,
!     $                   SIGMA, SSFMAX, SSFMIN, RMAX
      double precision ALPHAh, ALPHAl
      double precision ANORMh, ANORMl
      double precision BBh, BBl
      double precision Ch, Cl
      double precision EPSh, EPSl
      double precision EPS2h, EPS2l
      double precision GAMMAh, GAMMAl
      double precision OLDCh, OLDCl
      double precision OLDGAMh, OLDGAMl
      double precision Ph, Pl
      double precision Rh, Rl
      double precision RT1h, RT1l
      double precision RT2h, RT2l
      double precision RTEh, RTEl
      double precision Sh, Sl
      double precision SAFMAXh, SAFMAXl
      double precision SAFMINh, SAFMINl
      double precision SIGMAh, SIGMAl
      double precision SSFMAXh, SSFMAXl
      double precision SSFMINh, SSFMINl
      double precision RMAXh, RMAXl
*     ..
*     .. External Functions ..
!      DOUBLE PRECISION   DLAMCH, DLANST, DLAPY2
!      EXTERNAL           DLAMCH, DLANST, DLAPY2
*     ..
*     .. External Subroutines ..
      EXTERNAL           DDLAE2, DDLASCL, DDLASRT, XERBLA, 
     $                   DDLAMCH, DDLANST, DDLAPY2
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, SIGN, SQRT
*     ..
      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision tmp3h, tmp3l
      double precision tmp4h, tmp4l
      double precision tmp5h, tmp5l
      double precision tmp6h, tmp6l
      double precision tmp7h, tmp7l
      double precision tmp8h, tmp8l
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l

      ZEROh = 0.0d+0
      ZEROl = 0.0d+0
      ONEh = 1.0d+0
      ONEl = 0.0d+0
      TWOh = 2.d0
      TWOl = 0.0d+0
      THREEh = 3.d0
      THREEl = 0.0d+0

*     .. Executable Statements ..
*
*     Test the input parameters.
*
      INFO = 0
*
*     Quick return if possible
*
      IF( N.LT.0 ) THEN
         INFO = -1
         CALL XERBLA( 'DDSTERF', -INFO )
         RETURN
      END IF
      IF( N.LE.1 )
     $   RETURN
*
*     Determine the unit roundoff for this environment.
*
!      EPS = DLAMCH( 'E' )
      CALL DDLAMCH('E',EPSh,EPSl)
!      EPS2 = EPS**2
!<<<  EPS2 = EPS * EPS
      EPS2h = EPSh * EPSh
      T_u1 = 134217729.0 * EPSh
      T_p1 = T_u1
     +         - (T_u1 - EPSh)
      T_p2 = EPSh - T_p1
      T_u1 = 134217729.0 * EPSh
      T_r1 = T_u1
     +         - (T_u1 - EPSh)
      T_r2 = EPSh - T_r1
      EPS2l = ((T_p1*T_r1
     +        - EPS2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      EPS2l = EPS2l
     +      + (EPSh*EPSl
     +      + EPSl*EPSh)
      T_s1 = EPS2h + EPS2l
      EPS2l = EPS2l - (T_s1 - EPS2h)
      EPS2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  EPS2 = EPS * EPS
!      SAFMIN = DLAMCH( 'S' )
      CALL DDLAMCH('S',SAFMINh,SAFMINl)
!      SAFMAX = ONE / SAFMIN
      SAFMAXh = ONEh / SAFMINh
      SAFMAXl = 0.0d0
!      SSFMAX = SQRT( SAFMAX ) / THREE
      SSFMAXh = SQRT( SAFMAXh ) / THREEh
      SSFMAXl = 0.0d0
!      SSFMIN = SQRT( SAFMIN ) / EPS2
      SSFMINh = SQRT( SAFMINh ) / EPS2h
      SSFMINl = 0.0d0
!      RMAX = DLAMCH( 'O' )
      CALL DDLAMCH('O',RMAXh,RMAXl)
*
*     Compute the eigenvalues of the tridiagonal matrix.
*
      NMAXIT = N*MAXIT
!      SIGMA = ZERO
      SIGMAh = ZEROh
      SIGMAl = ZEROl
      JTOT = 0
*
*     Determine where the matrix splits and choose QL or QR iteration
*     for each block, according to whether top or bottom diagonal
*     element is smaller.
*
      L1 = 1
*
   10 CONTINUE
      IF( L1.GT.N )
     $   GO TO 170
      IF( L1.GT.1 ) then
!     $   E( L1-1 ) = ZERO
      Eh(L1-1) = ZEROh
      El(L1-1) = ZEROl
      END IF
      DO 20 M = L1, N - 1
!<<<  tmp1 = abs(E(M))
      if (Eh(M) .lt. 0.0d+0) then
        tmp1h = -Eh(M)
        tmp1l = -El(M)
      elseif (
     +  (Eh(M) .eq. 0.0d+0).and.
     +  (El(M) .lt. 0.0d+0)) then
        tmp1h = Eh(M)
        tmp1l = -El(M)
      else
        tmp1h = Eh(M)
        tmp1l = El(M)
      endif
!>>>  tmp1 = abs(E(M))
!<<<  tmp2 = abs(D(M))
      if (Dh(M) .lt. 0.0d+0) then
        tmp2h = -Dh(M)
        tmp2l = -Dl(M)
      elseif (
     +  (Dh(M) .eq. 0.0d+0).and.
     +  (Dl(M) .lt. 0.0d+0)) then
        tmp2h = Dh(M)
        tmp2l = -Dl(M)
      else
        tmp2h = Dh(M)
        tmp2l = Dl(M)
      endif
!>>>  tmp2 = abs(D(M))
!<<<  tmp3 = sqrt(tmp2)
      if (tmp2h.eq.0.0D+0.and.tmp2l.eq.0.0D+0) then
        tmp3h = 0.0d+0
        tmp3l = 0.0d+0
      endif
      if (tmp2h.lt.0.0d+0.or.
     +   (tmp2h.eq.0.0d+0.and.tmp2l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp3h = 0.0d+0
        tmp3l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp2h)
      T_ax = tmp2h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp2h,tmp2l,T_z1,T_z2)
          T_p1 = tmp2h - T_z1
          T_p2 = T_p1 - tmp2h
          T_t2l = (tmp2h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp2l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp3h = T_t1h
          tmp3l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp3 = sqrt(tmp2)
!<<<  tmp4 = abs(D(M+1))
      if (Dh(M+1) .lt. 0.0d+0) then
        tmp4h = -Dh(M+1)
        tmp4l = -Dl(M+1)
      elseif (
     +  (Dh(M+1) .eq. 0.0d+0).and.
     +  (Dl(M+1) .lt. 0.0d+0)) then
        tmp4h = Dh(M+1)
        tmp4l = -Dl(M+1)
      else
        tmp4h = Dh(M+1)
        tmp4l = Dl(M+1)
      endif
!>>>  tmp4 = abs(D(M+1))
!<<<  tmp5 = sqrt(tmp4)
      if (tmp4h.eq.0.0D+0.and.tmp4l.eq.0.0D+0) then
        tmp5h = 0.0d+0
        tmp5l = 0.0d+0
      endif
      if (tmp4h.lt.0.0d+0.or.
     +   (tmp4h.eq.0.0d+0.and.tmp4l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp5h = 0.0d+0
        tmp5l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp4h)
      T_ax = tmp4h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp4h,tmp4l,T_z1,T_z2)
          T_p1 = tmp4h - T_z1
          T_p2 = T_p1 - tmp4h
          T_t2l = (tmp4h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp4l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp5h = T_t1h
          tmp5l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp5 = sqrt(tmp4)
!<<<  tmp6 = tmp3 * tmp5
      tmp6h = tmp3h * tmp5h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * tmp5h
      T_r1 = T_u1
     +         - (T_u1 - tmp5h)
      T_r2 = tmp5h - T_r1
      tmp6l = ((T_p1*T_r1
     +        - tmp6h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp6l = tmp6l
     +      + (tmp3h*tmp5l
     +      + tmp3l*tmp5h)
      T_s1 = tmp6h + tmp6l
      tmp6l = tmp6l - (T_s1 - tmp6h)
      tmp6h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp6 = tmp3 * tmp5
!<<<  tmp7 = tmp6 * EPS
      tmp7h = tmp6h * EPSh
      T_u1 = 134217729.0 * tmp6h
      T_p1 = T_u1
     +         - (T_u1 - tmp6h)
      T_p2 = tmp6h - T_p1
      T_u1 = 134217729.0 * EPSh
      T_r1 = T_u1
     +         - (T_u1 - EPSh)
      T_r2 = EPSh - T_r1
      tmp7l = ((T_p1*T_r1
     +        - tmp7h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp7l = tmp7l
     +      + (tmp6h*EPSl
     +      + tmp6l*EPSh)
      T_s1 = tmp7h + tmp7l
      tmp7l = tmp7l - (T_s1 - tmp7h)
      tmp7h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp7 = tmp6 * EPS
         IF(
!            ABS( E( M ) ).LE.( SQRT( ABS( D( M ) ) )*SQRT( ABS( D( M+
!     $       1 ) ) ) )*EPS
!       (tmp1 .le. tmp7)
     + ((tmp1h .lt. tmp7h) .or.
     +  ((tmp1h .eq. tmp7h) .and.
     +   (tmp1l .le. tmp7l)))
     $       ) THEN
!            E( M ) = ZERO
      Eh(M) = ZEROh
      El(M) = ZEROl
            GO TO 30
         END IF
   20 CONTINUE
      M = N
*
   30 CONTINUE
      L = L1
      LSV = L
      LEND = M
      LENDSV = LEND
      L1 = M + 1
      IF( LEND.EQ.L )
     $   GO TO 10
*
*     Scale submatrix in rows and columns L to LEND
*
!      ANORM = DLANST( 'M', LEND-L+1, D( L ), E( L ) )
      CALL DDLANST('M',LEND-L+1,Dh(L),Dl(L),Eh(L),El(L),ANORMh,ANORMl)
      ISCALE = 0
      IF(
!         ANORM.EQ.ZERO
!       (ANORM .eq. ZERO)
     + ((ANORMh .eq. ZEROh) .and.
     +  (ANORMl .eq. ZEROl))
     $    )
     $   GO TO 10      
      IF(
!         (ANORM.GT.SSFMAX)
!       (ANORM .gt. SSFMAX)
     + ((ANORMh .gt. SSFMAXh) .or.
     +  ((ANORMh .eq. SSFMAXh) .and.
     +   (ANORMl .gt. SSFMAXl)))
     $    ) THEN
         ISCALE = 1
!         CALL DLASCL( 'G', 0, 0, ANORM, SSFMAX, LEND-L+1, 1, D( L ), N,
!     $                INFO )
         CALL DDLASCL( 'G', 0, 0, ANORMh, ANORMl, SSFMAXh, SSFMAXl, 
     $                LEND-L+1, 1, Dh( L ), Dl( L ), N,
     $                INFO )
!         CALL DLASCL( 'G', 0, 0, ANORM, SSFMAX, LEND-L, 1, E( L ), N,
!     $                INFO )
         CALL DDLASCL( 'G', 0, 0, ANORMh, ANORMl, SSFMAXh, SSFMAXl, 
     $                LEND-L, 1, Eh( L ), El( L ), N,
     $                INFO )
      ELSE IF(
!              ANORM.LT.SSFMIN
!       (ANORM .lt. SSFMIN)
     + ((ANORMh .lt. SSFMINh) .or.
     +  ((ANORMh .eq. SSFMINh) .and.
     +   (ANORMl .lt. SSFMINl)))
     $         ) THEN
         ISCALE = 2
!         CALL DLASCL( 'G', 0, 0, ANORM, SSFMIN, LEND-L+1, 1, D( L ), N,
!     $                INFO )
         CALL DDLASCL( 'G', 0, 0, ANORMh, ANORMl, SSFMINh, SSFMINl, 
     $                LEND-L+1, 1, Dh( L ), Dl( L ), N,
     $                INFO )
!         CALL DLASCL( 'G', 0, 0, ANORM, SSFMIN, LEND-L, 1, E( L ), N,
!     $                INFO )
         CALL DDLASCL( 'G', 0, 0, ANORMh, ANORMl, SSFMINh, SSFMINl, 
     $                LEND-L, 1, Eh( L ), El( L ), N,
     $                INFO )
      END IF
*
      DO 40 I = L, LEND - 1
!         E( I ) = E( I )**2
!<<<  E(I) = E(I) * E(I)
      T_ch = Eh(I) * Eh(I)
      T_u1 = 134217729.0 * Eh(I)
      T_p1 = T_u1
     +         - (T_u1 - Eh(I))
      T_p2 = Eh(I) - T_p1
      T_u1 = 134217729.0 * Eh(I)
      T_r1 = T_u1
     +         - (T_u1 - Eh(I))
      T_r2 = Eh(I) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Eh(I)*El(I)
     +         +  El(I)*Eh(I))
      T_s1 = T_ch + T_cl
      El(I) = T_cl - (T_s1 - T_ch)
      Eh(I) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  E(I) = E(I) * E(I)
   40 CONTINUE
*
*     Choose between QL and QR iteration
*
!<<<  tmp1 = abs(D(LEND))
      if (Dh(LEND) .lt. 0.0d+0) then
        tmp1h = -Dh(LEND)
        tmp1l = -Dl(LEND)
      elseif (
     +  (Dh(LEND) .eq. 0.0d+0).and.
     +  (Dl(LEND) .lt. 0.0d+0)) then
        tmp1h = Dh(LEND)
        tmp1l = -Dl(LEND)
      else
        tmp1h = Dh(LEND)
        tmp1l = Dl(LEND)
      endif
!>>>  tmp1 = abs(D(LEND))
!<<<  tmp2 = abs(D(L))
      if (Dh(L) .lt. 0.0d+0) then
        tmp2h = -Dh(L)
        tmp2l = -Dl(L)
      elseif (
     +  (Dh(L) .eq. 0.0d+0).and.
     +  (Dl(L) .lt. 0.0d+0)) then
        tmp2h = Dh(L)
        tmp2l = -Dl(L)
      else
        tmp2h = Dh(L)
        tmp2l = Dl(L)
      endif
!>>>  tmp2 = abs(D(L))
      IF(
!        ABS( D( LEND ) ).LT.ABS( D( L ) )
!       (tmp1 .lt. tmp2)
     + ((tmp1h .lt. tmp2h) .or.
     +  ((tmp1h .eq. tmp2h) .and.
     +   (tmp1l .lt. tmp2l)))
     $    ) THEN
         LEND = LSV
         L = LENDSV
      END IF
*
      IF( LEND.GE.L ) THEN
*
*        QL Iteration
*
*        Look for small subdiagonal element.
*
   50    CONTINUE
         IF( L.NE.LEND ) THEN
            DO 60 M = L, LEND - 1
!<<<  tmp1 = abs(E(M))
      if (Eh(M) .lt. 0.0d+0) then
        tmp1h = -Eh(M)
        tmp1l = -El(M)
      elseif (
     +  (Eh(M) .eq. 0.0d+0).and.
     +  (El(M) .lt. 0.0d+0)) then
        tmp1h = Eh(M)
        tmp1l = -El(M)
      else
        tmp1h = Eh(M)
        tmp1l = El(M)
      endif
!>>>  tmp1 = abs(E(M))
!<<<  tmp2 = D(M) * D(M+1)
      tmp2h = Dh(M) * Dh(M+1)
      T_u1 = 134217729.0 * Dh(M)
      T_p1 = T_u1
     +         - (T_u1 - Dh(M))
      T_p2 = Dh(M) - T_p1
      T_u1 = 134217729.0 * Dh(M+1)
      T_r1 = T_u1
     +         - (T_u1 - Dh(M+1))
      T_r2 = Dh(M+1) - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Dh(M)*Dl(M+1)
     +      + Dl(M)*Dh(M+1))
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = D(M) * D(M+1)
!<<<  tmp3 = abs(tmp2)
      if (tmp2h .lt. 0.0d+0) then
        tmp3h = -tmp2h
        tmp3l = -tmp2l
      elseif (
     +  (tmp2h .eq. 0.0d+0).and.
     +  (tmp2l .lt. 0.0d+0)) then
        tmp3h = tmp2h
        tmp3l = -tmp2l
      else
        tmp3h = tmp2h
        tmp3l = tmp2l
      endif
!>>>  tmp3 = abs(tmp2)
!<<<  tmp4 = EPS2 * tmp3
      tmp4h = EPS2h * tmp3h
      T_u1 = 134217729.0 * EPS2h
      T_p1 = T_u1
     +         - (T_u1 - EPS2h)
      T_p2 = EPS2h - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (EPS2h*tmp3l
     +      + EPS2l*tmp3h)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = EPS2 * tmp3
               IF(
!                  ABS( E( M ) ).LE.EPS2*ABS( D( M )*D( M+1 ) )
!       (tmp1 .le. tmp4)
     + ((tmp1h .lt. tmp4h) .or.
     +  ((tmp1h .eq. tmp4h) .and.
     +   (tmp1l .le. tmp4l)))
     $             )
     $            GO TO 70
   60       CONTINUE
         END IF
         M = LEND
*
   70    CONTINUE
         IF( M.LT.LEND ) THEN
!     $      E( M ) = ZERO
      Eh(M) = ZEROh
      El(M) = ZEROl
!         P = D( L )
      Ph = Dh(L)
      Pl = Dl(L)
         END IF
         IF( M.EQ.L )
     $      GO TO 90
*
*        If remaining matrix is 2 by 2, use DLAE2 to compute its
*        eigenvalues.
*
         IF( M.EQ.L+1 ) THEN
!            RTE = SQRT( E( L ) )
!<<<  RTE = sqrt(E(L))
      if (Eh(L).eq.0.0D+0.and.El(L).eq.0.0D+0) then
        RTEh = 0.0d+0
        RTEl = 0.0d+0
      endif
      if (Eh(L).lt.0.0d+0.or.
     +   (Eh(L).eq.0.0d+0.and.El(L).lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        RTEh = 0.0d+0
        RTEl = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(Eh(L))
      T_ax = Eh(L) * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           Eh(L),El(L),T_z1,T_z2)
          T_p1 = Eh(L) - T_z1
          T_p2 = T_p1 - Eh(L)
          T_t2l = (Eh(L)
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + El(L) - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          RTEh = T_t1h
          RTEl = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  RTE = sqrt(E(L))
!            CALL DLAE2( D( L ), RTE, D( L+1 ), RT1, RT2 )
            CALL DDLAE2( Dh(L), Dl(L), RTEh, RTEl, Dh(L+1), Dl(L+1),
     $                   RT1h, RT1l, RT2h, RT2l )
!            D( L ) = RT1
      Dh(L) = RT1h
      Dl(L) = RT1l
!            D( L+1 ) = RT2
      Dh(L+1) = RT2h
      Dl(L+1) = RT2l
!            E( L ) = ZERO
      Eh(L) = ZEROh
      El(L) = ZEROl
            L = L + 2
            IF( L.LE.LEND )
     $         GO TO 50
            GO TO 150
         END IF
*
         IF( JTOT.EQ.NMAXIT )
     $      GO TO 150
         JTOT = JTOT + 1
*
*        Form shift.
*
!         RTE = SQRT( E( L ) )
!<<<  RTE = sqrt(E(L))
      if (Eh(L).eq.0.0D+0.and.El(L).eq.0.0D+0) then
        RTEh = 0.0d+0
        RTEl = 0.0d+0
      endif
      if (Eh(L).lt.0.0d+0.or.
     +   (Eh(L).eq.0.0d+0.and.El(L).lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        RTEh = 0.0d+0
        RTEl = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(Eh(L))
      T_ax = Eh(L) * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           Eh(L),El(L),T_z1,T_z2)
          T_p1 = Eh(L) - T_z1
          T_p2 = T_p1 - Eh(L)
          T_t2l = (Eh(L)
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + El(L) - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          RTEh = T_t1h
          RTEl = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  RTE = sqrt(E(L))
!         SIGMA = ( D( L+1 )-P ) / ( TWO*RTE )
!<<<  tmp1 = TWO * RTE
      tmp1h = TWOh * RTEh
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * RTEh
      T_r1 = T_u1
     +         - (T_u1 - RTEh)
      T_r2 = RTEh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*RTEl
     +      + TWOl*RTEh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * RTE
!<<<  tmp2 = D(L+1) - P
      T_p1 = Dh(L+1) - Ph
      T_p2 = T_p1 - Dh(L+1)
      tmp2h = T_p1
      tmp2l = (Dh(L+1) - (T_p1 - T_p2))
     +         - (Ph + T_p2)
      tmp2l = tmp2l + Dl(L+1) - Pl
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp2 = D(L+1) - P
!<<<  SIGMA = tmp2 / tmp1
      T_q1 = tmp2h / tmp1h
      SIGMAh = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      SIGMAl = ((T_p1*T_r1
     +       - SIGMAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      SIGMAl = SIGMAl + (tmp1l*T_q1)
      T_s1 = SIGMAh + SIGMAl
      SIGMAl = SIGMAl - (T_s1 - SIGMAh)
      SIGMAh = T_s1
      T_s1 = tmp2h - SIGMAh
      T_r1 = T_s1 - tmp2h
      T_s2 = (tmp2h 
     +         - (T_s1 - T_r1))
     +         - (SIGMAh + T_r1)
      T_s2 = T_s2 - SIGMAl
      T_s2 = T_s2 + tmp2l
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      SIGMAh = T_q1 + T_q2
      SIGMAl = T_q2
     +      - (SIGMAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  SIGMA = tmp2 / tmp1
!         R = DLAPY2( SIGMA, ONE )
         CALL DDLAPY2(SIGMAh, SIGMAl, ONEh, ONEl, Rh, Rl )
!         SIGMA = P - ( RTE / ( SIGMA+SIGN( R, SIGMA ) ) )
!<<<  tmp1 = abs(R)
      if (Rh .lt. 0.0d+0) then
        tmp1h = -Rh
        tmp1l = -Rl
      elseif (
     +  (Rh .eq. 0.0d+0).and.
     +  (Rl .lt. 0.0d+0)) then
        tmp1h = Rh
        tmp1l = -Rl
      else
        tmp1h = Rh
        tmp1l = Rl
      endif
!>>>  tmp1 = abs(R)
          IF(
!       (SIGMA .ge. ZERO)
     + ((SIGMAh .gt. ZEROh) .or.
     +  ((SIGMAh .eq. ZEROh) .and.
     +   (SIGMAl .ge. ZEROl)))
     $       ) then
!<<<  tmp1 = SIGMA + tmp1
      T_p1 = SIGMAh + tmp1h
      T_p2 = T_p1 - SIGMAh
      T_q1 = T_p1
      T_q2 = (SIGMAh - (T_p1 - T_p2))
     +         + (tmp1h - T_p2)
      T_q2 = T_q2 + SIGMAl + tmp1l
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = SIGMA + tmp1
          ELSE
!<<<  tmp1 = SIGMA - tmp1
      T_p1 = SIGMAh - tmp1h
      T_p2 = T_p1 - SIGMAh
      T_ch = T_p1
      T_cl = (SIGMAh - (T_p1 - T_p2))
     +         - (tmp1h + T_p2)
      T_cl = T_cl + SIGMAl - tmp1l
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp1 = SIGMA - tmp1
          END IF
!<<<  tmp2 = RTE / tmp1
      T_q1 = RTEh / tmp1h
      tmp2h = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp2l = ((T_p1*T_r1
     +       - tmp2h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp2l = tmp2l + (tmp1l*T_q1)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
      T_s1 = RTEh - tmp2h
      T_r1 = T_s1 - RTEh
      T_s2 = (RTEh 
     +         - (T_s1 - T_r1))
     +         - (tmp2h + T_r1)
      T_s2 = T_s2 - tmp2l
      T_s2 = T_s2 + RTEl
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2
     +      - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp2 = RTE / tmp1
!<<<  SIGMA = P - tmp2
      T_p1 = Ph - tmp2h
      T_p2 = T_p1 - Ph
      SIGMAh = T_p1
      SIGMAl = (Ph - (T_p1 - T_p2))
     +         - (tmp2h + T_p2)
      SIGMAl = SIGMAl + Pl - tmp2l
      T_s1 = SIGMAh + SIGMAl
      SIGMAl = SIGMAl - (T_s1 - SIGMAh)
      SIGMAh = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  SIGMA = P - tmp2
*
!         C = ONE
      Ch = ONEh
      Cl = ONEl
!         S = ZERO
      Sh = ZEROh
      Sl = ZEROl
!         GAMMA = D( M ) - SIGMA
!<<<  GAMMA = D(M) - SIGMA
      T_p1 = Dh(M) - SIGMAh
      T_p2 = T_p1 - Dh(M)
      GAMMAh = T_p1
      GAMMAl = (Dh(M) - (T_p1 - T_p2))
     +         - (SIGMAh + T_p2)
      GAMMAl = GAMMAl + Dl(M) - SIGMAl
      T_s1 = GAMMAh + GAMMAl
      GAMMAl = GAMMAl - (T_s1 - GAMMAh)
      GAMMAh = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  GAMMA = D(M) - SIGMA
!         P = GAMMA*GAMMA
!<<<  P = GAMMA * GAMMA
      Ph = GAMMAh * GAMMAh
      T_u1 = 134217729.0 * GAMMAh
      T_p1 = T_u1
     +         - (T_u1 - GAMMAh)
      T_p2 = GAMMAh - T_p1
      T_u1 = 134217729.0 * GAMMAh
      T_r1 = T_u1
     +         - (T_u1 - GAMMAh)
      T_r2 = GAMMAh - T_r1
      Pl = ((T_p1*T_r1
     +        - Ph)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      Pl = Pl
     +      + (GAMMAh*GAMMAl
     +      + GAMMAl*GAMMAh)
      T_s1 = Ph + Pl
      Pl = Pl - (T_s1 - Ph)
      Ph = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  P = GAMMA * GAMMA
*
*        Inner loop
*
         DO 80 I = M - 1, L, -1
!            BB = E( I )
      BBh = Eh(I)
      BBl = El(I)
!            R = P + BB
!<<<  R = P + BB
      T_p1 = Ph + BBh
      T_p2 = T_p1 - Ph
      Rh = T_p1
      Rl = (Ph - (T_p1 - T_p2))
     +      + (BBh - T_p2)
      Rl = Rl + Pl + BBl
      T_s1 = Rh + Rl
      Rl = Rl - (T_s1 - Rh)
      Rh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  R = P + BB
            IF( I.NE.M-1 ) THEN
!     $         E( I+1 ) = S*R
!<<<  E(I+1) = S * R
      Eh(I+1) = Sh * Rh
      T_u1 = 134217729.0 * Sh
      T_p1 = T_u1
     +         - (T_u1 - Sh)
      T_p2 = Sh - T_p1
      T_u1 = 134217729.0 * Rh
      T_r1 = T_u1
     +         - (T_u1 - Rh)
      T_r2 = Rh - T_r1
      El(I+1) = ((T_p1*T_r1
     +        - Eh(I+1))
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      El(I+1) = El(I+1)
     +      + (Sh*Rl
     +      + Sl*Rh)
      T_s1 = Eh(I+1) + El(I+1)
      El(I+1) = El(I+1) - (T_s1 - Eh(I+1))
      Eh(I+1) = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  E(I+1) = S * R
            END IF
!            OLDC = C
      OLDCh = Ch
      OLDCl = Cl
!            C = P / R
!<<<  C = P / R
      T_q1 = Ph / Rh
      Ch = Rh * T_q1
      T_u1 = 134217729.0 * Rh
      T_p1 = T_u1
     +         - (T_u1 - Rh)
      T_p2 = Rh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      Cl = ((T_p1*T_r1
     +       - Ch)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      Cl = Cl + (Rl*T_q1)
      T_s1 = Ch + Cl
      Cl = Cl - (T_s1 - Ch)
      Ch = T_s1
      T_s1 = Ph - Ch
      T_r1 = T_s1 - Ph
      T_s2 = (Ph 
     +         - (T_s1 - T_r1))
     +         - (Ch + T_r1)
      T_s2 = T_s2 - Cl
      T_s2 = T_s2 + Pl
      T_q2 = (T_s1 + T_s2)
     +         / Rh
      Ch = T_q1 + T_q2
      Cl = T_q2
     +      - (Ch - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  C = P / R
!            S = BB / R
!<<<  S = BB / R
      T_q1 = BBh / Rh
      Sh = Rh * T_q1
      T_u1 = 134217729.0 * Rh
      T_p1 = T_u1
     +         - (T_u1 - Rh)
      T_p2 = Rh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      Sl = ((T_p1*T_r1
     +       - Sh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      Sl = Sl + (Rl*T_q1)
      T_s1 = Sh + Sl
      Sl = Sl - (T_s1 - Sh)
      Sh = T_s1
      T_s1 = BBh - Sh
      T_r1 = T_s1 - BBh
      T_s2 = (BBh 
     +         - (T_s1 - T_r1))
     +         - (Sh + T_r1)
      T_s2 = T_s2 - Sl
      T_s2 = T_s2 + BBl
      T_q2 = (T_s1 + T_s2)
     +         / Rh
      Sh = T_q1 + T_q2
      Sl = T_q2
     +      - (Sh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  S = BB / R
!            OLDGAM = GAMMA
      OLDGAMh = GAMMAh
      OLDGAMl = GAMMAl
!            ALPHA = D( I )
      ALPHAh = Dh(I)
      ALPHAl = Dl(I)
!            GAMMA = C*( ALPHA-SIGMA ) - S*OLDGAM
!<<<  tmp1 = S * OLDGAM
      tmp1h = Sh * OLDGAMh
      T_u1 = 134217729.0 * Sh
      T_p1 = T_u1
     +         - (T_u1 - Sh)
      T_p2 = Sh - T_p1
      T_u1 = 134217729.0 * OLDGAMh
      T_r1 = T_u1
     +         - (T_u1 - OLDGAMh)
      T_r2 = OLDGAMh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Sh*OLDGAMl
     +      + Sl*OLDGAMh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = S * OLDGAM
!<<<  tmp2 = ALPHA - SIGMA
      T_p1 = ALPHAh - SIGMAh
      T_p2 = T_p1 - ALPHAh
      tmp2h = T_p1
      tmp2l = (ALPHAh - (T_p1 - T_p2))
     +         - (SIGMAh + T_p2)
      tmp2l = tmp2l + ALPHAl - SIGMAl
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp2 = ALPHA - SIGMA
!<<<  tmp3 = C * tmp2
      tmp3h = Ch * tmp2h
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (Ch*tmp2l
     +      + Cl*tmp2h)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = C * tmp2
!<<<  GAMMA = tmp3 - tmp1
      T_p1 = tmp3h - tmp1h
      T_p2 = T_p1 - tmp3h
      GAMMAh = T_p1
      GAMMAl = (tmp3h - (T_p1 - T_p2))
     +         - (tmp1h + T_p2)
      GAMMAl = GAMMAl + tmp3l - tmp1l
      T_s1 = GAMMAh + GAMMAl
      GAMMAl = GAMMAl - (T_s1 - GAMMAh)
      GAMMAh = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  GAMMA = tmp3 - tmp1
!            D( I+1 ) = OLDGAM + ( ALPHA-GAMMA )
!<<<  tmp4 = ALPHA - GAMMA
      T_p1 = ALPHAh - GAMMAh
      T_p2 = T_p1 - ALPHAh
      tmp4h = T_p1
      tmp4l = (ALPHAh - (T_p1 - T_p2))
     +         - (GAMMAh + T_p2)
      tmp4l = tmp4l + ALPHAl - GAMMAl
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp4 = ALPHA - GAMMA
!<<<  D(I+1) = OLDGAM + tmp4
      T_p1 = OLDGAMh + tmp4h
      T_p2 = T_p1 - OLDGAMh
      Dh(I+1) = T_p1
      Dl(I+1) = (OLDGAMh - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      Dl(I+1) = Dl(I+1) + OLDGAMl + tmp4l
      T_s1 = Dh(I+1) + Dl(I+1)
      Dl(I+1) = Dl(I+1) - (T_s1 - Dh(I+1))
      Dh(I+1) = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  D(I+1) = OLDGAM + tmp4
            IF(
!               C.NE.ZERO
!       (C .ne. ZERO)
     + ((Ch .ne. ZEROh) .or.
     +  (Cl .ne. ZEROl))
     $          ) THEN
!               P = ( GAMMA*GAMMA ) / C
!<<<  tmp1 = GAMMA * GAMMA
      tmp1h = GAMMAh * GAMMAh
      T_u1 = 134217729.0 * GAMMAh
      T_p1 = T_u1
     +         - (T_u1 - GAMMAh)
      T_p2 = GAMMAh - T_p1
      T_u1 = 134217729.0 * GAMMAh
      T_r1 = T_u1
     +         - (T_u1 - GAMMAh)
      T_r2 = GAMMAh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (GAMMAh*GAMMAl
     +      + GAMMAl*GAMMAh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = GAMMA * GAMMA
!<<<  P = tmp1 / C
      T_q1 = tmp1h / Ch
      Ph = Ch * T_q1
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      Pl = ((T_p1*T_r1
     +       - Ph)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      Pl = Pl + (Cl*T_q1)
      T_s1 = Ph + Pl
      Pl = Pl - (T_s1 - Ph)
      Ph = T_s1
      T_s1 = tmp1h - Ph
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (Ph + T_r1)
      T_s2 = T_s2 - Pl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / Ch
      Ph = T_q1 + T_q2
      Pl = T_q2
     +      - (Ph - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  P = tmp1 / C
            ELSE
!               P = OLDC*BB
!<<<  P = OLDC * BB
      Ph = OLDCh * BBh
      T_u1 = 134217729.0 * OLDCh
      T_p1 = T_u1
     +         - (T_u1 - OLDCh)
      T_p2 = OLDCh - T_p1
      T_u1 = 134217729.0 * BBh
      T_r1 = T_u1
     +         - (T_u1 - BBh)
      T_r2 = BBh - T_r1
      Pl = ((T_p1*T_r1
     +        - Ph)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      Pl = Pl
     +      + (OLDCh*BBl
     +      + OLDCl*BBh)
      T_s1 = Ph + Pl
      Pl = Pl - (T_s1 - Ph)
      Ph = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  P = OLDC * BB
            END IF
   80    CONTINUE
*
!         E( L ) = S*P
!     P
      Eh(L) = Sh
      El(L) = Sl
!         D( L ) = SIGMA + GAMMA
!<<<  D(L) = SIGMA + GAMMA
      T_p1 = SIGMAh + GAMMAh
      T_p2 = T_p1 - SIGMAh
      Dh(L) = T_p1
      Dl(L) = (SIGMAh - (T_p1 - T_p2))
     +      + (GAMMAh - T_p2)
      Dl(L) = Dl(L) + SIGMAl + GAMMAl
      T_s1 = Dh(L) + Dl(L)
      Dl(L) = Dl(L) - (T_s1 - Dh(L))
      Dh(L) = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  D(L) = SIGMA + GAMMA
         GO TO 50
*
*        Eigenvalue found.
*
   90    CONTINUE
!         D( L ) = P
      Dh(L) = Ph
      Dl(L) = Pl
*
         L = L + 1
         IF( L.LE.LEND )
     $      GO TO 50
         GO TO 150
*
      ELSE
*
*        QR Iteration
*
*        Look for small superdiagonal element.
*
  100    CONTINUE
         DO 110 M = L, LEND + 1, -1
!<<<  tmp1 = abs(E(M-1))
      if (Eh(M-1) .lt. 0.0d+0) then
        tmp1h = -Eh(M-1)
        tmp1l = -El(M-1)
      elseif (
     +  (Eh(M-1) .eq. 0.0d+0).and.
     +  (El(M-1) .lt. 0.0d+0)) then
        tmp1h = Eh(M-1)
        tmp1l = -El(M-1)
      else
        tmp1h = Eh(M-1)
        tmp1l = El(M-1)
      endif
!>>>  tmp1 = abs(E(M-1))
!<<<  tmp2 = D(M) * D(M-1)
      tmp2h = Dh(M) * Dh(M-1)
      T_u1 = 134217729.0 * Dh(M)
      T_p1 = T_u1
     +         - (T_u1 - Dh(M))
      T_p2 = Dh(M) - T_p1
      T_u1 = 134217729.0 * Dh(M-1)
      T_r1 = T_u1
     +         - (T_u1 - Dh(M-1))
      T_r2 = Dh(M-1) - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Dh(M)*Dl(M-1)
     +      + Dl(M)*Dh(M-1))
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = D(M) * D(M-1)
!<<<  tmp3 = abs(tmp2)
      if (tmp2h .lt. 0.0d+0) then
        tmp3h = -tmp2h
        tmp3l = -tmp2l
      elseif (
     +  (tmp2h .eq. 0.0d+0).and.
     +  (tmp2l .lt. 0.0d+0)) then
        tmp3h = tmp2h
        tmp3l = -tmp2l
      else
        tmp3h = tmp2h
        tmp3l = tmp2l
      endif
!>>>  tmp3 = abs(tmp2)
!<<<  tmp4 = EPS2 * tmp3
      tmp4h = EPS2h * tmp3h
      T_u1 = 134217729.0 * EPS2h
      T_p1 = T_u1
     +         - (T_u1 - EPS2h)
      T_p2 = EPS2h - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (EPS2h*tmp3l
     +      + EPS2l*tmp3h)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = EPS2 * tmp3
            IF(
!               ABS( E( M-1 ) ).LE.EPS2*ABS( D( M )*D( M-1 ) )
!       (tmp1 .le. tmp4)
     + ((tmp1h .lt. tmp4h) .or.
     +  ((tmp1h .eq. tmp4h) .and.
     +   (tmp1l .le. tmp4l)))
     $          )
     $         GO TO 120
  110    CONTINUE
         M = LEND
*
  120    CONTINUE
         IF( M.GT.LEND ) THEN
!     $      E( M-1 ) = ZERO
      Eh(M-1) = ZEROh
      El(M-1) = ZEROl
!         P = D( L )
      Ph = Dh(L)
      Pl = Dl(L)
         END IF
         IF( M.EQ.L )
     $      GO TO 140
*
*        If remaining matrix is 2 by 2, use DLAE2 to compute its
*        eigenvalues.
*
         IF( M.EQ.L-1 ) THEN
!            RTE = SQRT( E( L-1 ) )
!<<<  RTE = sqrt(E(L-1))
      if (Eh(L-1).eq.0.0D+0.and.El(L-1).eq.0.0D+0) then
        RTEh = 0.0d+0
        RTEl = 0.0d+0
      endif
      if (Eh(L-1).lt.0.0d+0.or.
     +   (Eh(L-1).eq.0.0d+0.and.El(L-1).lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        RTEh = 0.0d+0
        RTEl = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(Eh(L-1))
      T_ax = Eh(L-1) * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           Eh(L-1),El(L-1),T_z1,T_z2)
          T_p1 = Eh(L-1) - T_z1
          T_p2 = T_p1 - Eh(L-1)
          T_t2l = (Eh(L-1)
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + El(L-1) - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          RTEh = T_t1h
          RTEl = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  RTE = sqrt(E(L-1))
!            CALL DLAE2( D( L ), RTE, D( L-1 ), RT1, RT2 )
            CALL DDLAE2( Dh(L), Dl(L), RTEh, RTEl, Dh( L-1 ), 
     $                   Dl( L-1 ), RT1h, RT1l, RT2h, RT2l )
!            D( L ) = RT1
      Dh(L) = RT1h
      Dl(L) = RT1l
!            D( L-1 ) = RT2
      Dh(L-1) = RT2h
      Dl(L-1) = RT2l
!            E( L-1 ) = ZERO
      Eh(L-1) = ZEROh
      El(L-1) = ZEROl
            L = L - 2
            IF( L.GE.LEND )
     $         GO TO 100
            GO TO 150
         END IF
*
         IF( JTOT.EQ.NMAXIT )
     $      GO TO 150
         JTOT = JTOT + 1
*
*        Form shift.
*
!         RTE = SQRT( E( L-1 ) )
!<<<  RTE = sqrt(E(L-1))
      if (Eh(L-1).eq.0.0D+0.and.El(L-1).eq.0.0D+0) then
        RTEh = 0.0d+0
        RTEl = 0.0d+0
      endif
      if (Eh(L-1).lt.0.0d+0.or.
     +   (Eh(L-1).eq.0.0d+0.and.El(L-1).lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        RTEh = 0.0d+0
        RTEl = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(Eh(L-1))
      T_ax = Eh(L-1) * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           Eh(L-1),El(L-1),T_z1,T_z2)
          T_p1 = Eh(L-1) - T_z1
          T_p2 = T_p1 - Eh(L-1)
          T_t2l = (Eh(L-1)
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + El(L-1) - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          RTEh = T_t1h
          RTEl = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  RTE = sqrt(E(L-1))
!         SIGMA = ( D( L-1 )-P ) / ( TWO*RTE )
!<<<  tmp1 = TWO * RTE
      tmp1h = TWOh * RTEh
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * RTEh
      T_r1 = T_u1
     +         - (T_u1 - RTEh)
      T_r2 = RTEh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*RTEl
     +      + TWOl*RTEh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * RTE
!<<<  tmp2 = D(L-1) - P
      T_p1 = Dh(L-1) - Ph
      T_p2 = T_p1 - Dh(L-1)
      tmp2h = T_p1
      tmp2l = (Dh(L-1) - (T_p1 - T_p2))
     +         - (Ph + T_p2)
      tmp2l = tmp2l + Dl(L-1) - Pl
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp2 = D(L-1) - P
!<<<  SIGMA = tmp2 / tmp1
      T_q1 = tmp2h / tmp1h
      SIGMAh = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      SIGMAl = ((T_p1*T_r1
     +       - SIGMAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      SIGMAl = SIGMAl + (tmp1l*T_q1)
      T_s1 = SIGMAh + SIGMAl
      SIGMAl = SIGMAl - (T_s1 - SIGMAh)
      SIGMAh = T_s1
      T_s1 = tmp2h - SIGMAh
      T_r1 = T_s1 - tmp2h
      T_s2 = (tmp2h 
     +         - (T_s1 - T_r1))
     +         - (SIGMAh + T_r1)
      T_s2 = T_s2 - SIGMAl
      T_s2 = T_s2 + tmp2l
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      SIGMAh = T_q1 + T_q2
      SIGMAl = T_q2
     +      - (SIGMAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  SIGMA = tmp2 / tmp1
!         R = DLAPY2( SIGMA, ONE )
         CALL DDLAPY2( SIGMAh, SIGMAl, ONEh, ONEl, Rh, Rl )
!         SIGMA = P - ( RTE / ( SIGMA+SIGN( R, SIGMA ) ) )
!<<<  tmp1 = abs(R)
      if (Rh .lt. 0.0d+0) then
        tmp1h = -Rh
        tmp1l = -Rl
      elseif (
     +  (Rh .eq. 0.0d+0).and.
     +  (Rl .lt. 0.0d+0)) then
        tmp1h = Rh
        tmp1l = -Rl
      else
        tmp1h = Rh
        tmp1l = Rl
      endif
!>>>  tmp1 = abs(R)
         IF(
!       (SIGMA .gt. ZERO)
     + ((SIGMAh .gt. ZEROh) .or.
     +  ((SIGMAh .eq. ZEROh) .and.
     +   (SIGMAl .gt. ZEROl)))
     $     )THEN
!<<<  tmp1 = SIGMA + tmp1
      T_p1 = SIGMAh + tmp1h
      T_p2 = T_p1 - SIGMAh
      T_q1 = T_p1
      T_q2 = (SIGMAh - (T_p1 - T_p2))
     +         + (tmp1h - T_p2)
      T_q2 = T_q2 + SIGMAl + tmp1l
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = SIGMA + tmp1
         ELSE
!<<<  tmp1 = SIGMA - tmp1
      T_p1 = SIGMAh - tmp1h
      T_p2 = T_p1 - SIGMAh
      T_ch = T_p1
      T_cl = (SIGMAh - (T_p1 - T_p2))
     +         - (tmp1h + T_p2)
      T_cl = T_cl + SIGMAl - tmp1l
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp1 = SIGMA - tmp1
         END IF
!<<<  tmp1 = RTE / tmp1
      T_q1 = RTEh / tmp1h
      T_ch = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (tmp1l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = RTEh - T_ch
      T_r1 = T_s1 - RTEh
      T_s2 = (RTEh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + RTEl
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = RTE / tmp1
!<<<  SIGMA = P - tmp1
      T_p1 = Ph - tmp1h
      T_p2 = T_p1 - Ph
      SIGMAh = T_p1
      SIGMAl = (Ph - (T_p1 - T_p2))
     +         - (tmp1h + T_p2)
      SIGMAl = SIGMAl + Pl - tmp1l
      T_s1 = SIGMAh + SIGMAl
      SIGMAl = SIGMAl - (T_s1 - SIGMAh)
      SIGMAh = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  SIGMA = P - tmp1
*
!         C = ONE
      Ch = ONEh
      Cl = ONEl
!         S = ZERO
      Sh = ZEROh
      Sl = ZEROl
!         GAMMA = D( M ) - SIGMA
!<<<  GAMMA = D(M) - SIGMA
      T_p1 = Dh(M) - SIGMAh
      T_p2 = T_p1 - Dh(M)
      GAMMAh = T_p1
      GAMMAl = (Dh(M) - (T_p1 - T_p2))
     +         - (SIGMAh + T_p2)
      GAMMAl = GAMMAl + Dl(M) - SIGMAl
      T_s1 = GAMMAh + GAMMAl
      GAMMAl = GAMMAl - (T_s1 - GAMMAh)
      GAMMAh = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  GAMMA = D(M) - SIGMA
!         P = GAMMA*GAMMA
!<<<  P = GAMMA * GAMMA
      Ph = GAMMAh * GAMMAh
      T_u1 = 134217729.0 * GAMMAh
      T_p1 = T_u1
     +         - (T_u1 - GAMMAh)
      T_p2 = GAMMAh - T_p1
      T_u1 = 134217729.0 * GAMMAh
      T_r1 = T_u1
     +         - (T_u1 - GAMMAh)
      T_r2 = GAMMAh - T_r1
      Pl = ((T_p1*T_r1
     +        - Ph)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      Pl = Pl
     +      + (GAMMAh*GAMMAl
     +      + GAMMAl*GAMMAh)
      T_s1 = Ph + Pl
      Pl = Pl - (T_s1 - Ph)
      Ph = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  P = GAMMA * GAMMA
*
*        Inner loop
*
         DO 130 I = M, L - 1
!            BB = E( I )
      BBh = Eh(I)
      BBl = El(I)
!            R = P + BB
!<<<  R = P + BB
      T_p1 = Ph + BBh
      T_p2 = T_p1 - Ph
      Rh = T_p1
      Rl = (Ph - (T_p1 - T_p2))
     +      + (BBh - T_p2)
      Rl = Rl + Pl + BBl
      T_s1 = Rh + Rl
      Rl = Rl - (T_s1 - Rh)
      Rh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  R = P + BB
            IF( I.NE.M ) THEN
!     $         E( I-1 ) = S*R
!<<<  E(I-1) = S * R
      Eh(I-1) = Sh * Rh
      T_u1 = 134217729.0 * Sh
      T_p1 = T_u1
     +         - (T_u1 - Sh)
      T_p2 = Sh - T_p1
      T_u1 = 134217729.0 * Rh
      T_r1 = T_u1
     +         - (T_u1 - Rh)
      T_r2 = Rh - T_r1
      El(I-1) = ((T_p1*T_r1
     +        - Eh(I-1))
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      El(I-1) = El(I-1)
     +      + (Sh*Rl
     +      + Sl*Rh)
      T_s1 = Eh(I-1) + El(I-1)
      El(I-1) = El(I-1) - (T_s1 - Eh(I-1))
      Eh(I-1) = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  E(I-1) = S * R
            END IF
!            OLDC = C
      OLDCh = Ch
      OLDCl = Cl
!            C = P / R
!<<<  C = P / R
      T_q1 = Ph / Rh
      Ch = Rh * T_q1
      T_u1 = 134217729.0 * Rh
      T_p1 = T_u1
     +         - (T_u1 - Rh)
      T_p2 = Rh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      Cl = ((T_p1*T_r1
     +       - Ch)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      Cl = Cl + (Rl*T_q1)
      T_s1 = Ch + Cl
      Cl = Cl - (T_s1 - Ch)
      Ch = T_s1
      T_s1 = Ph - Ch
      T_r1 = T_s1 - Ph
      T_s2 = (Ph 
     +         - (T_s1 - T_r1))
     +         - (Ch + T_r1)
      T_s2 = T_s2 - Cl
      T_s2 = T_s2 + Pl
      T_q2 = (T_s1 + T_s2)
     +         / Rh
      Ch = T_q1 + T_q2
      Cl = T_q2
     +      - (Ch - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  C = P / R
!            S = BB / R
!<<<  S = BB / R
      T_q1 = BBh / Rh
      Sh = Rh * T_q1
      T_u1 = 134217729.0 * Rh
      T_p1 = T_u1
     +         - (T_u1 - Rh)
      T_p2 = Rh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      Sl = ((T_p1*T_r1
     +       - Sh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      Sl = Sl + (Rl*T_q1)
      T_s1 = Sh + Sl
      Sl = Sl - (T_s1 - Sh)
      Sh = T_s1
      T_s1 = BBh - Sh
      T_r1 = T_s1 - BBh
      T_s2 = (BBh 
     +         - (T_s1 - T_r1))
     +         - (Sh + T_r1)
      T_s2 = T_s2 - Sl
      T_s2 = T_s2 + BBl
      T_q2 = (T_s1 + T_s2)
     +         / Rh
      Sh = T_q1 + T_q2
      Sl = T_q2
     +      - (Sh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  S = BB / R
!            OLDGAM = GAMMA
      OLDGAMh = GAMMAh
      OLDGAMl = GAMMAl
!            ALPHA = D( I+1 )
      ALPHAh = Dh(I+1)
      ALPHAl = Dl(I+1)
!            GAMMA = C*( ALPHA-SIGMA ) - S*OLDGAM
!<<<  tmp1 = S * OLDGAM
      tmp1h = Sh * OLDGAMh
      T_u1 = 134217729.0 * Sh
      T_p1 = T_u1
     +         - (T_u1 - Sh)
      T_p2 = Sh - T_p1
      T_u1 = 134217729.0 * OLDGAMh
      T_r1 = T_u1
     +         - (T_u1 - OLDGAMh)
      T_r2 = OLDGAMh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Sh*OLDGAMl
     +      + Sl*OLDGAMh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = S * OLDGAM
!<<<  tmp2 = ALPHA - SIGMA
      T_p1 = ALPHAh - SIGMAh
      T_p2 = T_p1 - ALPHAh
      tmp2h = T_p1
      tmp2l = (ALPHAh - (T_p1 - T_p2))
     +         - (SIGMAh + T_p2)
      tmp2l = tmp2l + ALPHAl - SIGMAl
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp2 = ALPHA - SIGMA
!<<<  tmp3 = C * tmp2
      tmp3h = Ch * tmp2h
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (Ch*tmp2l
     +      + Cl*tmp2h)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = C * tmp2
!<<<  GAMMA = tmp3 - tmp1
      T_p1 = tmp3h - tmp1h
      T_p2 = T_p1 - tmp3h
      GAMMAh = T_p1
      GAMMAl = (tmp3h - (T_p1 - T_p2))
     +         - (tmp1h + T_p2)
      GAMMAl = GAMMAl + tmp3l - tmp1l
      T_s1 = GAMMAh + GAMMAl
      GAMMAl = GAMMAl - (T_s1 - GAMMAh)
      GAMMAh = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  GAMMA = tmp3 - tmp1
!            D( I ) = OLDGAM + ( ALPHA-GAMMA )
!<<<  tmp4 = ALPHA - GAMMA
      T_p1 = ALPHAh - GAMMAh
      T_p2 = T_p1 - ALPHAh
      tmp4h = T_p1
      tmp4l = (ALPHAh - (T_p1 - T_p2))
     +         - (GAMMAh + T_p2)
      tmp4l = tmp4l + ALPHAl - GAMMAl
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp4 = ALPHA - GAMMA
!<<<  D(I) = OLDGAM + tmp4
      T_p1 = OLDGAMh + tmp4h
      T_p2 = T_p1 - OLDGAMh
      Dh(I) = T_p1
      Dl(I) = (OLDGAMh - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      Dl(I) = Dl(I) + OLDGAMl + tmp4l
      T_s1 = Dh(I) + Dl(I)
      Dl(I) = Dl(I) - (T_s1 - Dh(I))
      Dh(I) = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  D(I) = OLDGAM + tmp4
            IF(
!               C.NE.ZERO
!       (C .ne. ZERO)
     + ((Ch .ne. ZEROh) .or.
     +  (Cl .ne. ZEROl))
     $          ) THEN
!               P = ( GAMMA*GAMMA ) / C
!<<<  tmp1 = GAMMA * GAMMA
      tmp1h = GAMMAh * GAMMAh
      T_u1 = 134217729.0 * GAMMAh
      T_p1 = T_u1
     +         - (T_u1 - GAMMAh)
      T_p2 = GAMMAh - T_p1
      T_u1 = 134217729.0 * GAMMAh
      T_r1 = T_u1
     +         - (T_u1 - GAMMAh)
      T_r2 = GAMMAh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (GAMMAh*GAMMAl
     +      + GAMMAl*GAMMAh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = GAMMA * GAMMA
!<<<  P = tmp1 / C
      T_q1 = tmp1h / Ch
      Ph = Ch * T_q1
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      Pl = ((T_p1*T_r1
     +       - Ph)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      Pl = Pl + (Cl*T_q1)
      T_s1 = Ph + Pl
      Pl = Pl - (T_s1 - Ph)
      Ph = T_s1
      T_s1 = tmp1h - Ph
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (Ph + T_r1)
      T_s2 = T_s2 - Pl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / Ch
      Ph = T_q1 + T_q2
      Pl = T_q2
     +      - (Ph - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  P = tmp1 / C
            ELSE
!               P = OLDC*BB
!<<<  P = OLDC * BB
      Ph = OLDCh * BBh
      T_u1 = 134217729.0 * OLDCh
      T_p1 = T_u1
     +         - (T_u1 - OLDCh)
      T_p2 = OLDCh - T_p1
      T_u1 = 134217729.0 * BBh
      T_r1 = T_u1
     +         - (T_u1 - BBh)
      T_r2 = BBh - T_r1
      Pl = ((T_p1*T_r1
     +        - Ph)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      Pl = Pl
     +      + (OLDCh*BBl
     +      + OLDCl*BBh)
      T_s1 = Ph + Pl
      Pl = Pl - (T_s1 - Ph)
      Ph = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  P = OLDC * BB
            END IF
  130    CONTINUE
*
!         E( L-1 ) = S*P
!<<<  E(L-1) = S * P
      Eh(L-1) = Sh * Ph
      T_u1 = 134217729.0 * Sh
      T_p1 = T_u1
     +         - (T_u1 - Sh)
      T_p2 = Sh - T_p1
      T_u1 = 134217729.0 * Ph
      T_r1 = T_u1
     +         - (T_u1 - Ph)
      T_r2 = Ph - T_r1
      El(L-1) = ((T_p1*T_r1
     +        - Eh(L-1))
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      El(L-1) = El(L-1)
     +      + (Sh*Pl
     +      + Sl*Ph)
      T_s1 = Eh(L-1) + El(L-1)
      El(L-1) = El(L-1) - (T_s1 - Eh(L-1))
      Eh(L-1) = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  E(L-1) = S * P
!         D( L ) = SIGMA + GAMMA
!<<<  D(L) = SIGMA + GAMMA
      T_p1 = SIGMAh + GAMMAh
      T_p2 = T_p1 - SIGMAh
      Dh(L) = T_p1
      Dl(L) = (SIGMAh - (T_p1 - T_p2))
     +      + (GAMMAh - T_p2)
      Dl(L) = Dl(L) + SIGMAl + GAMMAl
      T_s1 = Dh(L) + Dl(L)
      Dl(L) = Dl(L) - (T_s1 - Dh(L))
      Dh(L) = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  D(L) = SIGMA + GAMMA
         GO TO 100
*
*        Eigenvalue found.
*
  140    CONTINUE
!         D( L ) = P
      Dh(L) = Ph
      Dl(L) = Pl
*
         L = L - 1
         IF( L.GE.LEND )
     $      GO TO 100
         GO TO 150
*
      END IF
*
*     Undo scaling if necessary
*
  150 CONTINUE
      IF( ISCALE.EQ.1 )
!     $   CALL DLASCL( 'G', 0, 0, SSFMAX, ANORM, LENDSV-LSV+1, 1,
!     $                D( LSV ), N, INFO )
     $   CALL DDLASCL( 'G', 0, 0, SSFMAXh, SSFMAXl, ANORMh, ANORMl, 
     $                LENDSV-LSV+1, 1,
     $                Dh( LSV ), Dl( LSV ), N, INFO )
      IF( ISCALE.EQ.2 )
!     $   CALL DLASCL( 'G', 0, 0, SSFMIN, ANORM, LENDSV-LSV+1, 1,
!     $                D( LSV ), N, INFO )
     $   CALL DDLASCL( 'G', 0, 0, SSFMINh, SSFMINl, ANORMh, ANORMl, 
     $                LENDSV-LSV+1, 1,
     $                Dh( LSV ), Dl( LSV ), N, INFO )
*
*     Check for no convergence to an eigenvalue after a total
*     of N*MAXIT iterations.
*
      IF( JTOT.LT.NMAXIT )
     $   GO TO 10
      DO 160 I = 1, N - 1
         IF(
!            E( I ).NE.ZERO
!       (E(I) .ne. ZERO)
     + ((Eh(I) .ne. ZEROh) .or.
     +  (El(I) .ne. ZEROl))
     $       )
     $      INFO = INFO + 1
  160 CONTINUE
      GO TO 180
*
*     Sort eigenvalues in increasing order.
*
  170 CONTINUE
!      CALL DLASRT( 'I', N, D, INFO )
      CALL DDLASRT( 'I', N, Dh, Dl, INFO )
*
  180 CONTINUE
      RETURN
*
*     End of DSTERF
*
      END
