*> \brief <b> DSYEVD computes the eigenvalues and, optionally, the left and/or right eigenvectors for SY matrices</b>
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DSYEVD + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dsyevd.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dsyevd.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dsyevd.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DSYEVD( JOBZ, UPLO, N, A, LDA, W, WORK, LWORK, IWORK,
*                          LIWORK, INFO )
* 
*       .. Scalar Arguments ..
*       CHARACTER          JOBZ, UPLO
*       INTEGER            INFO, LDA, LIWORK, LWORK, N
*       ..
*       .. Array Arguments ..
*       INTEGER            IWORK( * )
*       DOUBLE PRECISION   A( LDA, * ), W( * ), WORK( * )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DSYEVD computes all eigenvalues and, optionally, eigenvectors of a
*> real symmetric matrix A. If eigenvectors are desired, it uses a
*> divide and conquer algorithm.
*>
*> The divide and conquer algorithm makes very mild assumptions about
*> floating point arithmetic. It will work on machines with a guard
*> digit in add/subtract, or on those binary machines without guard
*> digits which subtract like the Cray X-MP, Cray Y-MP, Cray C-90, or
*> Cray-2. It could conceivably fail on hexadecimal or decimal machines
*> without guard digits, but we know of none.
*>
*> Because of large use of BLAS of level 3, DSYEVD needs N**2 more
*> workspace than DSYEVX.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] JOBZ
*> \verbatim
*>          JOBZ is CHARACTER*1
*>          = 'N':  Compute eigenvalues only;
*>          = 'V':  Compute eigenvalues and eigenvectors.
*> \endverbatim
*>
*> \param[in] UPLO
*> \verbatim
*>          UPLO is CHARACTER*1
*>          = 'U':  Upper triangle of A is stored;
*>          = 'L':  Lower triangle of A is stored.
*> \endverbatim
*>
*> \param[in] N
*> \verbatim
*>          N is INTEGER
*>          The order of the matrix A.  N >= 0.
*> \endverbatim
*>
*> \param[in,out] A
*> \verbatim
*>          A is DOUBLE PRECISION array, dimension (LDA, N)
*>          On entry, the symmetric matrix A.  If UPLO = 'U', the
*>          leading N-by-N upper triangular part of A contains the
*>          upper triangular part of the matrix A.  If UPLO = 'L',
*>          the leading N-by-N lower triangular part of A contains
*>          the lower triangular part of the matrix A.
*>          On exit, if JOBZ = 'V', then if INFO = 0, A contains the
*>          orthonormal eigenvectors of the matrix A.
*>          If JOBZ = 'N', then on exit the lower triangle (if UPLO='L')
*>          or the upper triangle (if UPLO='U') of A, including the
*>          diagonal, is destroyed.
*> \endverbatim
*>
*> \param[in] LDA
*> \verbatim
*>          LDA is INTEGER
*>          The leading dimension of the array A.  LDA >= max(1,N).
*> \endverbatim
*>
*> \param[out] W
*> \verbatim
*>          W is DOUBLE PRECISION array, dimension (N)
*>          If INFO = 0, the eigenvalues in ascending order.
*> \endverbatim
*>
*> \param[out] WORK
*> \verbatim
*>          WORK is DOUBLE PRECISION array,
*>                                         dimension (LWORK)
*>          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
*> \endverbatim
*>
*> \param[in] LWORK
*> \verbatim
*>          LWORK is INTEGER
*>          The dimension of the array WORK.
*>          If N <= 1,               LWORK must be at least 1.
*>          If JOBZ = 'N' and N > 1, LWORK must be at least 2*N+1.
*>          If JOBZ = 'V' and N > 1, LWORK must be at least
*>                                                1 + 6*N + 2*N**2.
*>
*>          If LWORK = -1, then a workspace query is assumed; the routine
*>          only calculates the optimal sizes of the WORK and IWORK
*>          arrays, returns these values as the first entries of the WORK
*>          and IWORK arrays, and no error message related to LWORK or
*>          LIWORK is issued by XERBLA.
*> \endverbatim
*>
*> \param[out] IWORK
*> \verbatim
*>          IWORK is INTEGER array, dimension (MAX(1,LIWORK))
*>          On exit, if INFO = 0, IWORK(1) returns the optimal LIWORK.
*> \endverbatim
*>
*> \param[in] LIWORK
*> \verbatim
*>          LIWORK is INTEGER
*>          The dimension of the array IWORK.
*>          If N <= 1,                LIWORK must be at least 1.
*>          If JOBZ  = 'N' and N > 1, LIWORK must be at least 1.
*>          If JOBZ  = 'V' and N > 1, LIWORK must be at least 3 + 5*N.
*>
*>          If LIWORK = -1, then a workspace query is assumed; the
*>          routine only calculates the optimal sizes of the WORK and
*>          IWORK arrays, returns these values as the first entries of
*>          the WORK and IWORK arrays, and no error message related to
*>          LWORK or LIWORK is issued by XERBLA.
*> \endverbatim
*>
*> \param[out] INFO
*> \verbatim
*>          INFO is INTEGER
*>          = 0:  successful exit
*>          < 0:  if INFO = -i, the i-th argument had an illegal value
*>          > 0:  if INFO = i and JOBZ = 'N', then the algorithm failed
*>                to converge; i off-diagonal elements of an intermediate
*>                tridiagonal form did not converge to zero;
*>                if INFO = i and JOBZ = 'V', then the algorithm failed
*>                to compute an eigenvalue while working on the submatrix
*>                lying in rows and columns INFO/(N+1) through
*>                mod(INFO,N+1).
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup doubleSYeigen
*
*> \par Contributors:
*  ==================
*>
*> Jeff Rutter, Computer Science Division, University of California
*> at Berkeley, USA \n
*>  Modified by Francoise Tisseur, University of Tennessee \n
*>  Modified description of INFO. Sven, 16 Feb 05. \n


*>
*  =====================================================================
!!      SUBROUTINE DSYEVD( JOBZ, UPLO, N, A, LDA, W, WORK, LWORK, IWORK,
!!     $                   LIWORK, INFO )
      SUBROUTINE dDSYEVD( JOBZ, UPLO, N, Ah, Al, LDA, Wh, Wl,
     $                    WORKh, WORKl, LWORK, IWORK,
     $                    LIWORK, INFO )
*
*  -- LAPACK driver routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      CHARACTER          JOBZ, UPLO
      INTEGER            INFO, LDA, LIWORK, LWORK, N
*     ..
*     .. Array Arguments ..
      INTEGER            IWORK( * )
!!      DOUBLE PRECISION   A( LDA, * ), W( * ), WORK( * )
      double precision Ah(LDA,*), Al(LDA,*)
      double precision Wh(*), Wl(*)
      double precision WORKh(*), WORKl(*)
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
!!      DOUBLE PRECISION   ZERO, ONE
!!      PARAMETER          ( ZERO = 0.0D+0, ONE = 1.0D+0 )
      double precision ZEROh, ZEROl
      double precision ONEh, ONEl
*     ..
*     .. Local Scalars ..
*
      LOGICAL            LOWER, LQUERY, WANTZ
      INTEGER            IINFO, INDE, INDTAU, INDWK2, INDWRK, ISCALE,
     $                   LIOPT, LIWMIN, LLWORK, LLWRK2, LOPT, LWMIN
!!      DOUBLE PRECISION   ANRM, BIGNUM, EPS, RMAX, RMIN, SAFMIN, SIGMA,
!!     $                   SMLNUM
      double precision ANRMh, ANRMl
      double precision BIGNUMh, BIGNUMl
      double precision EPSh, EPSl
      double precision RMAXh, RMAXl
      double precision RMINh, RMINl
      double precision SAFMINh, SAFMINl
      double precision SIGMAh, SIGMAl
      double precision SMLNUMh, SMLNUMl
*     ..
*     .. External Functions ..
      LOGICAL            LSAME
      INTEGER            ILAENV
!!      DOUBLE PRECISION   DLAMCH, DLANSY
!!      EXTERNAL           LSAME, DLAMCH, DLANSY, ILAENV
      EXTERNAL           LSAME, ILAENV
*     ..
*     .. External Subroutines ..
!!      EXTERNAL           DLACPY, DLASCL, DORMTR, DSCAL, DSTEDC, DSTERF,
!!     $                   DSYTRD, XERBLA
      EXTERNAL           dDLACPY, dDLASCL, dDORMTR, dDSCAL, dDSTEDC,
     $                   dDSTERF,
     $                   dDSYTRD, XERBLA, dDLAMCH, dDLANSY
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX, SQRT
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l
      double precision tmp1h, tmp1l
*     ..
*     .. Executable Statements ..
      ZEROh = 0.0d+0
      ZEROl = 0.0d+0
      ONEh = 1.0d+0
      ONEl = 0.0d+0
*
*     Test the input parameters.
*
      WANTZ = LSAME( JOBZ, 'V' )
      LOWER = LSAME( UPLO, 'L' )
      LQUERY = ( LWORK.EQ.-1 .OR. LIWORK.EQ.-1 )
*
      INFO = 0
      IF( .NOT.( WANTZ .OR. LSAME( JOBZ, 'N' ) ) ) THEN
         INFO = -1
      ELSE IF( .NOT.( LOWER .OR. LSAME( UPLO, 'U' ) ) ) THEN
         INFO = -2
      ELSE IF( N.LT.0 ) THEN
         INFO = -3
      ELSE IF( LDA.LT.MAX( 1, N ) ) THEN
         INFO = -5
      END IF
*
      IF( INFO.EQ.0 ) THEN
         IF( N.LE.1 ) THEN
            LIWMIN = 1
            LWMIN = 1
            LOPT = LWMIN
            LIOPT = LIWMIN
         ELSE
            IF( WANTZ ) THEN
               LIWMIN = 3 + 5*N
               LWMIN = 1 + 6*N + 2*N**2
            ELSE
               LIWMIN = 1
               LWMIN = 2*N + 1
            END IF
            LOPT = MAX( LWMIN, 2*N +
     $                  ILAENV( 1, 'DSYTRD', UPLO, N, -1, -1, -1 ) )
            LIOPT = LIWMIN
         END IF
!!         WORK( 1 ) = LOPT
      WORKh(1) = LOPT
      WORKl(1) = 0.0d+0
         IWORK( 1 ) = LIOPT
*
         IF( LWORK.LT.LWMIN .AND. .NOT.LQUERY ) THEN
            INFO = -8
         ELSE IF( LIWORK.LT.LIWMIN .AND. .NOT.LQUERY ) THEN
            INFO = -10
         END IF
      END IF
*
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'DSYEVD', -INFO )
         RETURN
      ELSE IF( LQUERY ) THEN
         RETURN
      END IF
*
*     Quick return if possible
*
      IF( N.EQ.0 )
     $   RETURN
*
      IF( N.EQ.1 ) THEN
!!         W( 1 ) = A( 1, 1 )
      Wh(1) = Ah(1,1)
      Wl(1) = Al(1,1)
!!         IF( WANTZ )
!!     $      A( 1, 1 ) = ONE
         IF( WANTZ ) THEN
      Ah(1,1) = ONEh
      Al(1,1) = ONEl
         END IF
         RETURN
      END IF
*
*     Get machine constants.
*
!!      SAFMIN = DLAMCH( 'Safe minimum' )
      CALL dDLAMCH( 'Safe minimum', SAFMINh, SAFMINl )
!!      EPS = DLAMCH( 'Precision' )
      CALL dDLAMCH( 'Precision', EPSh, EPSl )
!!      SMLNUM = SAFMIN / EPS
!<<<  SMLNUM = SAFMIN / EPS
      T_q1 = SAFMINh / EPSh
      T_ch = EPSh * T_q1
      T_u1 = 134217729.0 * EPSh
      T_p1 = T_u1
     +         - (T_u1 - EPSh)
      T_p2 = EPSh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (EPSl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = SAFMINh - T_ch
      T_r1 = T_s1 - SAFMINh
      T_s2 = (SAFMINh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + SAFMINl
      T_q2 = (T_s1 + T_s2)
     +         / EPSh
      SMLNUMh = T_q1 + T_q2
      SMLNUMl = T_q2
     +      - (SMLNUMh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  SMLNUM = SAFMIN / EPS
!!      BIGNUM = ONE / SMLNUM
!<<<  BIGNUM = ONE / SMLNUM
      T_q1 = ONEh / SMLNUMh
      T_ch = SMLNUMh * T_q1
      T_u1 = 134217729.0 * SMLNUMh
      T_p1 = T_u1
     +         - (T_u1 - SMLNUMh)
      T_p2 = SMLNUMh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (SMLNUMl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = ONEh - T_ch
      T_r1 = T_s1 - ONEh
      T_s2 = (ONEh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + ONEl
      T_q2 = (T_s1 + T_s2)
     +         / SMLNUMh
      BIGNUMh = T_q1 + T_q2
      BIGNUMl = T_q2
     +      - (BIGNUMh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  BIGNUM = ONE / SMLNUM
!!      RMIN = SQRT( SMLNUM )
!<<<  RMIN = sqrt(SMLNUM)
      if (SMLNUMh.eq.0.0D+0.and.SMLNUMl.eq.0.0D+0) then
        RMINh = 0.0d+0
        RMINl = 0.0d+0
      else
      if (SMLNUMh.lt.0.0d+0.or.
     +   (SMLNUMh.eq.0.0d+0.and.SMLNUMl.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        RMINh = 0.0d+0
        RMINl = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(SMLNUMh)
      T_ax = SMLNUMh * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           SMLNUMh,SMLNUMl,T_z1,T_z2)
          T_p1 = SMLNUMh - T_z1
          T_p2 = T_p1 - SMLNUMh
          T_t2l = (SMLNUMh
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + SMLNUMl - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          RMINh = T_t1h
          RMINl = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  RMIN = sqrt(SMLNUM)
!!      RMAX = SQRT( BIGNUM )
!<<<  RMAX = sqrt(BIGNUM)
      if (BIGNUMh.eq.0.0D+0.and.BIGNUMl.eq.0.0D+0) then
        RMAXh = 0.0d+0
        RMAXl = 0.0d+0
      else
      if (BIGNUMh.lt.0.0d+0.or.
     +   (BIGNUMh.eq.0.0d+0.and.BIGNUMl.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        RMAXh = 0.0d+0
        RMAXl = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(BIGNUMh)
      T_ax = BIGNUMh * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           BIGNUMh,BIGNUMl,T_z1,T_z2)
          T_p1 = BIGNUMh - T_z1
          T_p2 = T_p1 - BIGNUMh
          T_t2l = (BIGNUMh
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + BIGNUMl - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          RMAXh = T_t1h
          RMAXl = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  RMAX = sqrt(BIGNUM)
*
*     Scale matrix to allowable range, if necessary.
*
!!      ANRM = DLANSY( 'M', UPLO, N, A, LDA, WORK )
      CALL dDLANSY( 'M', UPLO, N, Ah, Al, LDA, WORKh, WORKl,
     $               ANRMh, ANRMl )
      ISCALE = 0
!!      IF( ANRM.GT.ZERO .AND. ANRM.LT.RMIN ) THEN
      IF(
!       (ANRM .gt. 0.0)
     + ((ANRMh .gt. 0.0d+0) .or.
     +  ((ANRMh .eq. 0.0d+0) .and.
     +   (ANRMl .gt. 0.0d+0)))
     $      .AND.
!       (ANRM .lt. RMIN)
     + ((ANRMh .lt. RMINh) .or.
     +  ((ANRMh .eq. RMINh) .and.
     +   (ANRMl .lt. RMINl)))
     $  ) THEN
         ISCALE = 1
!!         SIGMA = RMIN / ANRM
!<<<  SIGMA = RMIN / ANRM
      T_q1 = RMINh / ANRMh
      T_ch = ANRMh * T_q1
      T_u1 = 134217729.0 * ANRMh
      T_p1 = T_u1
     +         - (T_u1 - ANRMh)
      T_p2 = ANRMh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (ANRMl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = RMINh - T_ch
      T_r1 = T_s1 - RMINh
      T_s2 = (RMINh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + RMINl
      T_q2 = (T_s1 + T_s2)
     +         / ANRMh
      SIGMAh = T_q1 + T_q2
      SIGMAl = T_q2
     +      - (SIGMAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  SIGMA = RMIN / ANRM
!!      ELSE IF( ANRM.GT.RMAX ) THEN
      ELSE IF(
!       (ANRM .gt. RMAX)
     + ((ANRMh .gt. RMAXh) .or.
     +  ((ANRMh .eq. RMAXh) .and.
     +   (ANRMl .gt. RMAXl)))
     $       ) THEN
         ISCALE = 1
!!         SIGMA = RMAX / ANRM
!<<<  SIGMA = RMAX / ANRM
      T_q1 = RMAXh / ANRMh
      T_ch = ANRMh * T_q1
      T_u1 = 134217729.0 * ANRMh
      T_p1 = T_u1
     +         - (T_u1 - ANRMh)
      T_p2 = ANRMh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (ANRMl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = RMAXh - T_ch
      T_r1 = T_s1 - RMAXh
      T_s2 = (RMAXh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + RMAXl
      T_q2 = (T_s1 + T_s2)
     +         / ANRMh
      SIGMAh = T_q1 + T_q2
      SIGMAl = T_q2
     +      - (SIGMAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  SIGMA = RMAX / ANRM
      END IF
!!      IF( ISCALE.EQ.1 )
!!     $   CALL DLASCL( UPLO, 0, 0, ONE, SIGMA, N, N, A, LDA, INFO )
      IF( ISCALE.EQ.1 ) THEN
         CALL dDLASCL( UPLO, 0, 0, ONEh, ONEl, SIGMAh, SIGMAl, N, N,
     $                 Ah, Al, LDA, INFO )
      END IF
*
*     Call DSYTRD to reduce symmetric matrix to tridiagonal form.
*
      INDE = 1
      INDTAU = INDE + N
      INDWRK = INDTAU + N
      LLWORK = LWORK - INDWRK + 1
      INDWK2 = INDWRK + N*N
      LLWRK2 = LWORK - INDWK2 + 1
*
!!      CALL DSYTRD( UPLO, N, A, LDA, W, WORK( INDE ), WORK( INDTAU ),
!!     $             WORK( INDWRK ), LLWORK, IINFO )
      CALL dDSYTRD( UPLO, N, Ah, Al, LDA, Wh, Wl,
     $              WORKh( INDE ), WORKl( INDE ),
     $              WORKh( INDTAU ), WORKl( INDTAU ),
     $              WORKh( INDWRK ), WORKl( INDWRK ), LLWORK, IINFO )
!!      LOPT = 2*N + WORK( INDWRK )
      LOPT = 2*N + WORKh( INDWRK )
*
*     For eigenvalues only, call DSTERF.  For eigenvectors, first call
*     DSTEDC to generate the eigenvector matrix, WORK(INDWRK), of the
*     tridiagonal matrix, then call DORMTR to multiply it by the
*     Householder transformations stored in A.
*
      IF( .NOT.WANTZ ) THEN
!!         CALL DSTERF( N, W, WORK( INDE ), INFO )
         CALL dDSTERF( N, Wh, Wl, WORKh( INDE ), WORKl( INDE ), INFO )
      ELSE
!!         CALL DSTEDC( 'I', N, W, WORK( INDE ), WORK( INDWRK ), N,
!!     $                WORK( INDWK2 ), LLWRK2, IWORK, LIWORK, INFO )
         CALL dDSTEDC( 'I', N, Wh, Wl, WORKh( INDE ), WORKl( INDE ),
     $                 WORKh( INDWRK ), WORKl( INDWRK ), N,
     $                 WORKh( INDWK2 ), WORKl( INDWK2 ),
     $                 LLWRK2, IWORK, LIWORK, INFO )
!!         CALL DORMTR( 'L', UPLO, 'N', N, N, A, LDA, WORK( INDTAU ),
!!     $                WORK( INDWRK ), N, WORK( INDWK2 ), LLWRK2, IINFO )
         CALL dDORMTR( 'L', UPLO, 'N', N, N, Ah, Al, LDA,
     $                 WORKh( INDTAU ), WORKl( INDTAU ),
     $                 WORKh( INDWRK ), WORKl( INDWRK ), N,
     $                 WORKh( INDWK2 ), WORKl( INDWK2 ), LLWRK2, IINFO )
!!         CALL DLACPY( 'A', N, N, WORK( INDWRK ), N, A, LDA )
         CALL dDLACPY( 'A', N, N, WORKh( INDWRK ), WORKl( INDWRK ), N,
     $                 Ah, Al, LDA )
         LOPT = MAX( LOPT, 1+6*N+2*N**2 )
      END IF
*
*     If matrix was scaled, then rescale eigenvalues appropriately.
*
!!      IF( ISCALE.EQ.1 )
!!     $   CALL DSCAL( N, ONE / SIGMA, W, 1 )
      IF( ISCALE.EQ.1 ) THEN
!<<<  tmp1 = ONE / SIGMA
      T_q1 = ONEh / SIGMAh
      T_ch = SIGMAh * T_q1
      T_u1 = 134217729.0 * SIGMAh
      T_p1 = T_u1
     +         - (T_u1 - SIGMAh)
      T_p2 = SIGMAh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (SIGMAl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = ONEh - T_ch
      T_r1 = T_s1 - ONEh
      T_s2 = (ONEh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + ONEl
      T_q2 = (T_s1 + T_s2)
     +         / SIGMAh
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = ONE / SIGMA
         CALL dDSCAL( N, tmp1h, tmp1l, Wh, Wl, 1 )
      END IF
*
!!      WORK( 1 ) = LOPT
      WORKh(1) = LOPT
      WORKl(1) = 0.0d+0
      IWORK( 1 ) = LIOPT
*
      RETURN
*
*     End of DSYEVD
*
      END
