#include "Bdef.h"

/*** Quadruple precision ***/
#if( _F2C_CALL_ == _F2C_UPCASE )
#define MPI_Allreduce_dd_ MPI_ALLREDUCE_DD
#define MPI_Reduce_dd_    MPI_REDUCE_DD
#endif
#if( _F2C_CALL_ == _F2C_NOCHANGE )
#define MPI_Allreduce_dd_ mpi_allreduce_dd
#define MPI_Reduce_dd_    mpi_reduce_dd
#endif
#ifdef __STDC__
void MPI_Allreduce_dd_(double * sh, double * sl, double * rh, double * rl,
		       int * count, int * datatype, int * op, int * comm, int * ierr);
void MPI_Reduce_dd_(double * sh, double * sl, double * rh, double * rl,
		    int * count, int * datatype, int * op, int * root, int * comm, int * ierr);
#else
void MPI_Allreduce_dd_();
void MPI_Reduce_dd_();
#endif
/***************************/

#if (INTFACE == C_CALL)
void Cddgsum2d(int ConTxt, char *scope, char *top, int m, int n, double *Ah, double *Al,
	       int lda, int rdest, int cdest)
#else
F_VOID_FUNC ddgsum2d_(int *ConTxt, F_CHAR scope, F_CHAR top, int *m, int *n,
		      double *Ah, double *Al, int *lda, int *rdest, int *cdest)
#endif
/*
 *  -- V1.1 BLACS routine --
 *  University of Tennessee, May 1, 1996
 *  Written by Clint Whaley.
 *
 *  Purpose
 *  =======
 *  Combine sum operation for double precision rectangular matrices.
 *
 *  Arguments
 *  =========
 *
 *  ConTxt  (input) Ptr to int
 *          Index into MyConTxts00 (my contexts array).
 *
 *  SCOPE   (input) Ptr to char
 *          Limit the scope of the operation.
 *          = 'R' :   Operation is performed by a process row.
 *          = 'C' :   Operation is performed by a process column.
 *          = 'A' :   Operation is performed by all processes in grid.
 *
 *  TOP     (input) Ptr to char
 *          Controls fashion in which messages flow within the operation.
 *
 *  M       (input) Ptr to int
 *          The number of rows of the matrix A.  M >= 0.
 *
 *  N       (input) Ptr to int
 *          The number of columns of the matrix A.  N >= 0.
 *
 *  A       (output) Ptr to double precision two dimensional array
 *          The m by n matrix A.  Fortran77 (column-major) storage
 *          assumed.
 *
 *  LDA     (input) Ptr to int
 *          The leading dimension of the array A.  LDA >= M.
 *
 *  RDEST   (input) Ptr to int
 *          The process row of the destination of the sum.
 *          If rdest == -1, then result is left on all processes in scope.
 *
 *  CDEST   (input) Ptr to int
 *          The process column of the destination of the sum.
 *          If rdest == -1, then CDEST ignored.
 *
 * ------------------------------------------------------------------------
 */
{
   void BI_ArgCheck(int, int, char *, char, char, char, int, int, int, int,
                    int *, int *);
   void BI_UpdateBuffs(BLACBUFF *);
   BLACBUFF *BI_GetBuff(int);
   int BI_BuffIsFree(BLACBUFF *, int);
   MPI_Datatype BI_GetMpiGeType(BLACSCONTEXT *, int, int, int,
                                   MPI_Datatype, int *);
   BLACBUFF *BI_Pack(BLACSCONTEXT *, BVOID *, BLACBUFF *, MPI_Datatype);
   MPI_Datatype BI_GetMpiGeType(BLACSCONTEXT *, int, int, int,
                                   MPI_Datatype, int *);
   void BI_Unpack(BLACSCONTEXT *, BVOID *, BLACBUFF *, MPI_Datatype);
   void BI_MringComb(BLACSCONTEXT *, BLACBUFF *, BLACBUFF *, int, VVFUNPTR,
                     int, int);
   void BI_TreeComb(BLACSCONTEXT *, BLACBUFF *, BLACBUFF *, int, VVFUNPTR,
                    int, int);
   void BI_BeComb(BLACSCONTEXT *, BLACBUFF *, BLACBUFF *, int, VVFUNPTR);
   void BI_dvvsum(int, char *, char *);
/*
 *  Variable Declarations
 */
   /*** Quadruple precision ***/
   BLACBUFF *bhp, *bhp2;
   BLACBUFF *blp, *blp2;
   /***************************/
   BLACSCONTEXT *ctxt;
   char ttop, tscope;
   int N, length, dest, tlda, trdest, ierr;
   extern BLACBUFF *BI_ActiveQ;
   //extern BLACBUFF BI_AuxBuff;
   extern BLACBUFF BI_AuxBuffH;
   extern BLACBUFF BI_AuxBuffL;

   MGetConTxt(Mpval(ConTxt), ctxt);
   ttop = F2C_CharTrans(top);
   ttop = Mlowcase(ttop);
   tscope = F2C_CharTrans(scope);
   tscope = Mlowcase(tscope);
/*
 *  If the user has set the default combine topology, use it instead of
 *  BLACS default
 */
#ifdef DefCombTop
   if (ttop == ' ') ttop = DefCombTop;
#endif
   if (Mpval(cdest) == -1) trdest = -1;
   else trdest = Mpval(rdest);
#if (BlacsDebugLvl > 0)
   BI_ArgCheck(Mpval(ConTxt), RT_COMB, __FILE__, tscope, 'u', 'u', Mpval(m),
               Mpval(n), Mpval(lda), 1, &trdest, Mpaddress(cdest));
#endif
   if (Mpval(lda) >= Mpval(m)) tlda = Mpval(lda);
   else tlda = Mpval(m);
   switch(tscope)
   {
   case 'r':
      ctxt->scp = &ctxt->rscp;
      if (trdest == -1) dest = -1;
      else dest = Mpval(cdest);
      break;
   case 'c':
      ctxt->scp = &ctxt->cscp;
      dest = trdest;
      break;
   case 'a':
      ctxt->scp = &ctxt->ascp;
      if (trdest == -1) dest = -1;
      else dest = Mvkpnum(ctxt, trdest, Mpval(cdest));
      break;
   default:
      BI_BlacsErr(Mpval(ConTxt), __LINE__, __FILE__, "Unknown scope '%c'",
                  tscope);
   }


/*
 * It's not defined how MPI reacts to 0 element reductions, so use BLACS 1-tree
 * topology if we've got one.  Also, we can't use MPI functions if we need to
 * guarantee repeatability.
 */
   if (ttop == ' ')
      if ( (Mpval(m) < 1) || (Mpval(n) < 1) || (ctxt->TopsRepeat) ) ttop = '1';
   N = Mpval(m) * Mpval(n);
   length = N * sizeof(double);
/*
 * If A is contiguous, we can use it as one of the buffers
 */
   if ( (Mpval(m) == tlda) || (Mpval(n) == 1) )
   {
     /*** Quadruple precision ***/
      bhp = &BI_AuxBuffH;
      bhp->Buff = (char *) Ah;
      bhp2 = BI_GetBuff(length);

      blp = &BI_AuxBuffL;
      blp->Buff = (char *) Al;
      blp2 = BI_GetBuff(length);
      /***************************/
   }
/*
 * Otherwise, we must allocate both buffers
 */
   else
   {
     /*** Quadruple precision ***/
      bhp = BI_GetBuff(length*2);
      bhp2 = &BI_AuxBuffH;
      bhp2->Buff = &bhp->Buff[length];
      BI_dmvcopy(Mpval(m), Mpval(n), Ah, tlda, bhp->Buff);

      blp = BI_GetBuff(length*2);
      blp2 = &BI_AuxBuffL;
      blp2->Buff = &blp->Buff[length];
      BI_dmvcopy(Mpval(m), Mpval(n), Al, tlda, blp->Buff);
      /***************************/
   }
   /*** Quadruple precision ***/
   bhp->dtype = bhp2->dtype = MPI_DOUBLE;
   bhp->N = bhp2->N = N;

   blp->dtype = blp2->dtype = MPI_DOUBLE;
   blp->N = blp2->N = N;
   /***************************/

   switch(ttop)
   {
   case ' ':         /* use MPI's reduction by default */
      if (dest != -1)
      {
	/*** Quadruple precision ***/
	mpi_reduce_dd_(
		       (double *)(bhp->Buff),  (double *)(blp->Buff), 
		       (double *)(bhp2->Buff), (double *)(blp2->Buff), 
		       &(bhp->N), 
		       &(bhp->dtype), 
		       (int *) MPI_SUM,
		       &dest,
		       &(ctxt->scp->comm),
		       &ierr
		       );
	if (ctxt->scp->Iam == dest){
	  BI_dvmcopy(Mpval(m), Mpval(n), Ah, tlda, bhp2->Buff);
	  BI_dvmcopy(Mpval(m), Mpval(n), Al, tlda, blp2->Buff);
	}
	/***************************/
      }
      else
      {
	/*** Quadruple precision ***/
	mpi_allreduce_dd_((double *)(bhp->Buff),  (double *)(blp->Buff),
			  (double *)(bhp2->Buff), (double *)(blp2->Buff),
			  &(bhp->N),
			  &(bhp->dtype),
			  (int *) MPI_SUM,
			  &(ctxt->scp->comm),
			  &ierr
			  );
	BI_dvmcopy(Mpval(m), Mpval(n), Ah, tlda, bhp2->Buff);
	BI_dvmcopy(Mpval(m), Mpval(n), Al, tlda, blp2->Buff);
	/***************************/
      }
      if (BI_ActiveQ) BI_UpdateBuffs(NULL);
      return;
      break;
/*
 *   case 'i':
 *      BI_MringComb(ctxt, bp, bp2, N, BI_dvvsum, dest, 1);
 *      break;
 *   case 'd':
 *      BI_MringComb(ctxt, bp, bp2, N, BI_dvvsum, dest, -1);
 *      break;
 *   case 's':
 *      BI_MringComb(ctxt, bp, bp2, N, BI_dvvsum, dest, 2);
 *      break;
 *   case 'm':
 *      BI_MringComb(ctxt, bp, bp2, N, BI_dvvsum, dest, ctxt->Nr_co);
 *      break;
 *   case '1':
 *   case '2':
 *   case '3':
 *   case '4':
 *   case '5':
 *   case '6':
 *   case '7':
 *   case '8':
 *   case '9':
 *      BI_TreeComb(ctxt, bp, bp2, N, BI_dvvsum, dest, ttop-47);
 *      break;
 *   case 'f':
 *      BI_TreeComb(ctxt, bp, bp2, N, BI_dvvsum, dest, FULLCON);
 *      break;
 *   case 't':
 *      BI_TreeComb(ctxt, bp, bp2, N, BI_dvvsum, dest, ctxt->Nb_co);
 *      break;
 *   case 'h':
 */
/*
 *    Use bidirectional exchange if everyone wants answer
 */
/*
 *      if ( (trdest == -1) && !(ctxt->TopsCohrnt) )
 *         BI_BeComb(ctxt, bp, bp2, N, BI_dvvsum);
 *      else
 *         BI_TreeComb(ctxt, bp, bp2, N, BI_dvvsum, dest, 2);
 *      break;
 */
   default :
      BI_BlacsErr(Mpval(ConTxt), __LINE__, __FILE__, "Unknown topology '%c'",
                  ttop);
   }

/*
 * If I am selected to receive answer
 */
   /*** Quadruple precision ***/
   if (bhp != &BI_AuxBuffH)
   {
      if ( (ctxt->scp->Iam == dest) || (dest == -1) )
         BI_dvmcopy(Mpval(m), Mpval(n), Ah, tlda, bhp->Buff);
      BI_UpdateBuffs(bhp);
   }
   else
   {
      if (BI_ActiveQ) BI_UpdateBuffs(NULL);
      BI_BuffIsFree(bhp, 1);
   }
   if (blp != &BI_AuxBuffL)
   {
      if ( (ctxt->scp->Iam == dest) || (dest == -1) )
         BI_dvmcopy(Mpval(m), Mpval(n), Al, tlda, blp->Buff);
      BI_UpdateBuffs(blp);
   }
   else
   {
      if (BI_ActiveQ) BI_UpdateBuffs(NULL);
      BI_BuffIsFree(blp, 1);
   }
   /***************************/
}
