!      SUBROUTINE DMMDDACT( M, N, ALPHA, A, LDA, BETA, B, LDB )
      SUBROUTINE DDMMDDACT(
     &     M, N,
     &     ALPHAh, ALPHAl,
     &     Ah, Al, LDA,
     &     BETAh, BETAl,
     &     Bh, Bl, LDB
     &     )
*
*  -- PBLAS auxiliary routine (version 2.0) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     April 1, 1998
*
*     .. Scalar Arguments ..
      INTEGER            LDA, LDB, M, N
!      DOUBLE PRECISION   ALPHA, BETA
      double precision ALPHAh, ALPHAl
      double precision BETAh, BETAl
*     ..
*     .. Array Arguments ..
!      DOUBLE PRECISION   A( LDA, * ), B( LDB, * )
      double precision Ah(LDA,*), Al(LDA,*)
      double precision Bh(LDB,*), Bl(LDB,*)
*     ..
*
*  Purpose
*  =======
*
*  DMMDDACT performs the following operation:
*
*     A := alpha * A + beta * B',
*
*  where alpha, beta are scalars; A is an m by n matrix and B is an n by
*  m matrix.
*
*  Arguments
*  =========
*
*  M       (local input) INTEGER
*          On entry, M  specifies the number of rows of A and the number
*          of columns of B. M must be at least zero.
*
*  N       (local input) INTEGER
*          On entry, N  specifies the number of rows of B and the number
*          of columns of A. N must be at least zero.
*
*  ALPHA   (local input) DOUBLE PRECISION
*          On entry,  ALPHA  specifies the scalar alpha. When  ALPHA  is
*          supplied as zero then the local entries of the array  A  need
*          not be set on input.
*
*  A       (local input/local output) DOUBLE PRECISION array
*          On entry, A is an array of dimension ( LDA, N ). On exit, the
*          leading n by m part of B has been added into the leading m by
*          n part of A.
*
*  LDA     (local input) INTEGER
*          On entry, LDA specifies the leading dimension of the array A.
*          LDA must be at least max( 1, M ).
*
*  BETA    (local input) DOUBLE PRECISION
*          On entry,  BETA  specifies the scalar beta. When BETA is sup-
*          plied as zero then the local entries of the array B need  not
*          be set on input.
*
*  B       (local input) DOUBLE PRECISION array
*          On entry, B is an array of dimension ( LDB, M ).
*
*  LDB     (local input) INTEGER
*          On entry, LDB specifies the leading dimension of the array B.
*          LDB must be at least max( 1, N ).
*
*  -- Written on April 1, 1998 by
*     Antoine Petitet, University  of  Tennessee, Knoxville 37996, USA.
*
*  =====================================================================
*
*     .. Parameters ..
!      DOUBLE PRECISION   ONE, ZERO
!      PARAMETER          ( ONE = 1.0D+0, ZERO = 0.0D+0 )
      double precision ONEh, ONEl
      double precision ZEROh, ZEROl
      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l
*     ..
*     .. Local Scalars ..
      INTEGER            I, J
*     ..
*     .. External Subroutines ..
!      EXTERNAL           DAXPY, DCOPY, DSCAL
      EXTERNAL           DDAXPY, DDCOPY, DDSCAL
*
      ZEROh = 0.0d+0
      ZEROl = 0.0d+0
      ONEh = 1.0d+0
      ONEl = 0.0d+0
*     ..
*     .. Executable Statements ..
*
      IF( M.GE.N ) THEN
         IF(
!            BETA.EQ.ONE
!       (BETA .eq. ONE)
     + ((BETAh .eq. ONEh) .and.
     +  (BETAl .eq. ONEl))
     &      ) THEN
            IF(
!               ALPHA.EQ.ZERO
!       (ALPHA .eq. ZERO)
     + ((ALPHAh .eq. ZEROh) .and.
     +  (ALPHAl .eq. ZEROl))
     &          ) THEN
               DO 20 J = 1, N
!                  CALL DCOPY( M, B( J, 1 ), LDB, A( 1, J ), 1 )
                  CALL DDCOPY( M,
     &                 Bh( J, 1 ), Bl( J, 1 ), LDB,
     &                 Ah( 1, J ), Ah( 1, J ), 1 )
*                 DO 10 I = 1, M
*                    A( I, J ) = B( J, I )
*  10             CONTINUE
   20          CONTINUE
            ELSE IF(
!                    ALPHA.NE.ONE
!       (ALPHA .ne. ONE)
     + ((ALPHAh .ne. ONEh) .or.
     +  (ALPHAl .ne. ONEl))
     &               ) THEN
               DO 40 J = 1, N
                  DO 30 I = 1, M
!                     A( I, J ) = B( J, I ) + ALPHA * A( I, J )
!<<<  tmp1 = ALPHA * A(I,J)
      tmp1h = ALPHAh * Ah(I,J)
      T_u1 = 134217729.0 * ALPHAh
      T_p1 = T_u1
     +         - (T_u1 - ALPHAh)
      T_p2 = ALPHAh - T_p1
      T_u1 = 134217729.0 * Ah(I,J)
      T_r1 = T_u1
     +         - (T_u1 - Ah(I,J))
      T_r2 = Ah(I,J) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (ALPHAh*Al(I,J)
     +      + ALPHAl*Ah(I,J))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = ALPHA * A(I,J)
!<<<  A(I,J) = B(J,I) + tmp1
      T_p1 = Bh(J,I) + tmp1h
      T_p2 = T_p1 - Bh(J,I)
      Ah(I,J) = T_p1
      Al(I,J) = (Bh(J,I) - (T_p1 - T_p2))
     +      + (tmp1h - T_p2)
      Al(I,J) = Al(I,J) + Bl(J,I) + tmp1l
      T_s1 = Ah(I,J) + Al(I,J)
      Al(I,J) = Al(I,J) - (T_s1 - Ah(I,J))
      Ah(I,J) = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  A(I,J) = B(J,I) + tmp1
   30             CONTINUE
   40          CONTINUE
            ELSE
               DO 60 J = 1, N
!                  CALL DAXPY( M, ONE, B( J, 1 ), LDB, A( 1, J ), 1 )
                  CALL DDAXPY( M,
     &                 ONE,ZERO,
     &                 Bh( J, 1 ), Bl( J, 1 ), LDB,
     &                 Ah( 1, J ), Al( 1, J ), 1 )
*                 DO 50 I = 1, M
*                    A( I, J ) = B( J, I ) + A( I, J )
*  50             CONTINUE
   60          CONTINUE
            END IF
         ELSE IF(
!                 BETA.NE.ZERO
!       (BETA .ne. ZERO)
     + ((BETAh .ne. ZEROh) .or.
     +  (BETAl .ne. ZEROl))
     &           ) THEN
            IF( 
!               ALPHA.EQ.ZERO
!       (ALPHA .eq. ZERO)
     + ((ALPHAh .eq. ZEROh) .and.
     +  (ALPHAl .eq. ZEROl))
     &          ) THEN
               DO 80 J = 1, N
                  DO 70 I = 1, M
!                     A( I, J ) = BETA * B( J, I )
!<<<  A(I,J) = BETA * B(J,I)
      Ah(I,J) = BETAh * Bh(J,I)
      T_u1 = 134217729.0 * BETAh
      T_p1 = T_u1
     +         - (T_u1 - BETAh)
      T_p2 = BETAh - T_p1
      T_u1 = 134217729.0 * Bh(J,I)
      T_r1 = T_u1
     +         - (T_u1 - Bh(J,I))
      T_r2 = Bh(J,I) - T_r1
      Al(I,J) = ((T_p1*T_r1
     +        - Ah(I,J))
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      Al(I,J) = Al(I,J)
     +      + (BETAh*Bl(J,I)
     +      + BETAl*Bh(J,I))
      T_s1 = Ah(I,J) + Al(I,J)
      Al(I,J) = Al(I,J) - (T_s1 - Ah(I,J))
      Ah(I,J) = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  A(I,J) = BETA * B(J,I)
   70             CONTINUE
   80          CONTINUE
            ELSE IF(
!                    ALPHA.NE.ONE
!       (ALPHA .ne. ONE)
     + ((ALPHAh .ne. ONEh) .or.
     +  (ALPHAl .ne. ONEl))
     &               ) THEN
               DO 100 J = 1, N
                  DO 90 I = 1, M
!                     A( I, J ) = BETA * B( J, I ) + ALPHA * A( I, J )
!<<<  tmp1 = BETA * B(J,I)
      tmp1h = BETAh * Bh(J,I)
      T_u1 = 134217729.0 * BETAh
      T_p1 = T_u1
     +         - (T_u1 - BETAh)
      T_p2 = BETAh - T_p1
      T_u1 = 134217729.0 * Bh(J,I)
      T_r1 = T_u1
     +         - (T_u1 - Bh(J,I))
      T_r2 = Bh(J,I) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (BETAh*Bl(J,I)
     +      + BETAl*Bh(J,I))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = BETA * B(J,I)
!<<<  tmp2 = ALPHA * A(I,J)
      tmp2h = ALPHAh * Ah(I,J)
      T_u1 = 134217729.0 * ALPHAh
      T_p1 = T_u1
     +         - (T_u1 - ALPHAh)
      T_p2 = ALPHAh - T_p1
      T_u1 = 134217729.0 * Ah(I,J)
      T_r1 = T_u1
     +         - (T_u1 - Ah(I,J))
      T_r2 = Ah(I,J) - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (ALPHAh*Al(I,J)
     +      + ALPHAl*Ah(I,J))
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = ALPHA * A(I,J)
!<<<  A(I,J) = tmp1 + tmp2
      T_p1 = tmp1h + tmp2h
      T_p2 = T_p1 - tmp1h
      Ah(I,J) = T_p1
      Al(I,J) = (tmp1h - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      Al(I,J) = Al(I,J) + tmp1l + tmp2l
      T_s1 = Ah(I,J) + Al(I,J)
      Al(I,J) = Al(I,J) - (T_s1 - Ah(I,J))
      Ah(I,J) = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  A(I,J) = tmp1 + tmp2
   90             CONTINUE
  100          CONTINUE
            ELSE
               DO 120 J = 1, N
!                  CALL DAXPY( M, BETA, B( J, 1 ), LDB, A( 1, J ), 1 )
                  CALL DDAXPY( M,
     &                 BETAh, BETAl,
     &                 Bh( J, 1 ), Bl( J, 1 ), LDB,
     &                 Ah( 1, J ), Al( 1, J ), 1 )
*                 DO 110 I = 1, M
*                    A( I, J ) = BETA * B( J, I ) + A( I, J )
* 110             CONTINUE
  120          CONTINUE
            END IF
         ELSE
            IF(
!               ALPHA.EQ.ZERO
!       (ALPHA .eq. ZERO)
     + ((ALPHAh .eq. ZEROh) .and.
     +  (ALPHAl .eq. ZEROl))
     &         ) THEN
               DO 140 J = 1, N
                  DO 130 I = 1, M
!                     A( I, J ) = ZERO
      Ah(I,J) = ZEROh
      Al(I,J) = ZEROl
  130             CONTINUE
  140          CONTINUE
            ELSE IF(
!                    ALPHA.NE.ONE
!       (ALPHA .ne. ONE)
     + ((ALPHAh .ne. ONEh) .or.
     +  (ALPHAl .ne. ONEl))
     &              ) THEN
               DO 160 J = 1, N
!                  CALL DSCAL( M, ALPHA, A( 1, J ), 1 )
                  CALL DDSCAL( M,
     &                 ALPHAh, ALPHAl,
     &                 Ah( 1, J ), Al( 1, J ), 1 )
*                 DO 150 I = 1, M
*                    A( I, J ) = ALPHA * A( I, J )
* 150             CONTINUE
  160          CONTINUE
            END IF
         END IF
      ELSE
         IF(
!            BETA.EQ.ONE
!       (BETA .eq. ONE)
     + ((BETAh .eq. ONEh) .and.
     +  (BETAl .eq. ONEl))
     &       ) THEN
            IF(
!               ALPHA.EQ.ZERO
!       (ALPHA .eq. ZERO)
     + ((ALPHAh .eq. ZEROh) .and.
     +  (ALPHAl .eq. ZEROl))
     &         ) THEN
               DO 180 J = 1, M
!                  CALL DCOPY( N, B( 1, J ), 1, A( J, 1 ), LDA )
                  CALL DDCOPY( N,
     &                 Bh( 1, J ), Bl( 1, J ), 1,
     &                 Ah( J, 1 ), Al( J, 1 ), LDA )
*                 DO 170 I = 1, N
*                    A( J, I ) = B( I, J )
* 170             CONTINUE
  180          CONTINUE
            ELSE IF(
!                    ALPHA.NE.ONE
!       (ALPHA .ne. ONE)
     + ((ALPHAh .ne. ONEh) .or.
     +  (ALPHAl .ne. ONEl))
     &               ) THEN
               DO 200 J = 1, M
                  DO 190 I = 1, N
!                     A( J, I ) = B( I, J ) + ALPHA * A( J, I )
!<<<  tmp1 = ALPHA * A(J,I)
      tmp1h = ALPHAh * Ah(J,I)
      T_u1 = 134217729.0 * ALPHAh
      T_p1 = T_u1
     +         - (T_u1 - ALPHAh)
      T_p2 = ALPHAh - T_p1
      T_u1 = 134217729.0 * Ah(J,I)
      T_r1 = T_u1
     +         - (T_u1 - Ah(J,I))
      T_r2 = Ah(J,I) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (ALPHAh*Al(J,I)
     +      + ALPHAl*Ah(J,I))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = ALPHA * A(J,I)
!<<<  A(J,I) = B(I,J) + tmp1
      T_p1 = Bh(I,J) + tmp1h
      T_p2 = T_p1 - Bh(I,J)
      Ah(J,I) = T_p1
      Al(J,I) = (Bh(I,J) - (T_p1 - T_p2))
     +      + (tmp1h - T_p2)
      Al(J,I) = Al(J,I) + Bl(I,J) + tmp1l
      T_s1 = Ah(J,I) + Al(J,I)
      Al(J,I) = Al(J,I) - (T_s1 - Ah(J,I))
      Ah(J,I) = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  A(J,I) = B(I,J) + tmp1
  190             CONTINUE
  200          CONTINUE
            ELSE
               DO 220 J = 1, M
!                  CALL DAXPY( N, ONE, B( 1, J ), 1, A( J, 1 ), LDA )
                  CALL DDAXPY( N, 
     &                 ONE, ZERO, 
     &                 Bh( 1, J ), Bl( 1, J ), 1, 
     &                 Ah( J, 1 ), Al( J, 1 ), LDA )
*                 DO 210 I = 1, N
*                    A( J, I ) = B( I, J ) + A( J, I )
* 210             CONTINUE
  220          CONTINUE
            END IF
         ELSE IF(
!                 BETA.NE.ZERO
!       (BETA .ne. ZERO)
     + ((BETAh .ne. ZEROh) .or.
     +  (BETAl .ne. ZEROl))
     &            ) THEN
            IF(
!               ALPHA.EQ.ZERO
!       (ALPHA .eq. ZERO)
     + ((ALPHAh .eq. ZEROh) .and.
     +  (ALPHAl .eq. ZEROl))
     &         ) THEN
               DO 240 J = 1, M
                  DO 230 I = 1, N
!                     A( J, I ) = BETA * B( I, J )
!<<<  A(J,I) = BETA * B(I,J)
      Ah(J,I) = BETAh * Bh(I,J)
      T_u1 = 134217729.0 * BETAh
      T_p1 = T_u1
     +         - (T_u1 - BETAh)
      T_p2 = BETAh - T_p1
      T_u1 = 134217729.0 * Bh(I,J)
      T_r1 = T_u1
     +         - (T_u1 - Bh(I,J))
      T_r2 = Bh(I,J) - T_r1
      Al(J,I) = ((T_p1*T_r1
     +        - Ah(J,I))
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      Al(J,I) = Al(J,I)
     +      + (BETAh*Bl(I,J)
     +      + BETAl*Bh(I,J))
      T_s1 = Ah(J,I) + Al(J,I)
      Al(J,I) = Al(J,I) - (T_s1 - Ah(J,I))
      Ah(J,I) = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  A(J,I) = BETA * B(I,J)
  230             CONTINUE
  240          CONTINUE
            ELSE IF(
!                    ALPHA.NE.ONE
!       (ALPHA .ne. ONE)
     + ((ALPHAh .ne. ONEh) .or.
     +  (ALPHAl .ne. ONEl))
     &               ) THEN
               DO 260 J = 1, M
                  DO 250 I = 1, N
!                     A( J, I ) = BETA * B( I, J ) + ALPHA * A( J, I )
!<<<  tmp1 = BETA * B(I,J)
      tmp1h = BETAh * Bh(I,J)
      T_u1 = 134217729.0 * BETAh
      T_p1 = T_u1
     +         - (T_u1 - BETAh)
      T_p2 = BETAh - T_p1
      T_u1 = 134217729.0 * Bh(I,J)
      T_r1 = T_u1
     +         - (T_u1 - Bh(I,J))
      T_r2 = Bh(I,J) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (BETAh*Bl(I,J)
     +      + BETAl*Bh(I,J))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = BETA * B(I,J)
!<<<  tmp2 = ALPHA * A(J,I)
      tmp2h = ALPHAh * Ah(J,I)
      T_u1 = 134217729.0 * ALPHAh
      T_p1 = T_u1
     +         - (T_u1 - ALPHAh)
      T_p2 = ALPHAh - T_p1
      T_u1 = 134217729.0 * Ah(J,I)
      T_r1 = T_u1
     +         - (T_u1 - Ah(J,I))
      T_r2 = Ah(J,I) - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (ALPHAh*Al(J,I)
     +      + ALPHAl*Ah(J,I))
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = ALPHA * A(J,I)
!<<<  A(J,I) = tmp1 + tmp2
      T_p1 = tmp1h + tmp2h
      T_p2 = T_p1 - tmp1h
      Ah(J,I) = T_p1
      Al(J,I) = (tmp1h - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      Al(J,I) = Al(J,I) + tmp1l + tmp2l
      T_s1 = Ah(J,I) + Al(J,I)
      Al(J,I) = Al(J,I) - (T_s1 - Ah(J,I))
      Ah(J,I) = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  A(J,I) = tmp1 + tmp2
  250             CONTINUE
  260          CONTINUE
            ELSE
               DO 280 J = 1, M
!                  CALL DAXPY( N, BETA, B( 1, J ), 1, A( J, 1 ), LDA )
                  CALL DDAXPY( N,
     &                 BETAh, BETAl, 
     &                 Bh( 1, J ), Bl( 1, J ), 1,
     &                 Ah( J, 1 ), Al( J, 1 ), LDA )
*                 DO 270 I = 1, N
*                    A( J, I ) = BETA * B( I, J ) + A( J, I )
* 270             CONTINUE
  280          CONTINUE
            END IF
         ELSE
            IF(
!               ALPHA.EQ.ZERO
!       (ALPHA .eq. ZERO)
     + ((ALPHAh .eq. ZEROh) .and.
     +  (ALPHAl .eq. ZEROl))
     &         ) THEN
               DO 300 J = 1, N
                  DO 290 I = 1, M
!                     A( I, J ) = ZERO
      Ah(I,J) = ZEROh
      Al(I,J) = ZEROl
  290             CONTINUE
  300          CONTINUE
            ELSE IF(
!                    ALPHA.NE.ONE
!       (ALPHA .ne. ONE)
     + ((ALPHAh .ne. ONEh) .or.
     +  (ALPHAl .ne. ONEl))
     &               ) THEN
               DO 320 J = 1, N
!                  CALL DSCAL( M, ALPHA, A( 1, J ), 1 )
                  CALL DDSCAL( M,
     &                 ALPHAh, ALPHAl,
     &                 Ah( 1, J ), Al( 1, J ), 1 )
*                 DO 310 I = 1, M
*                    A( I, J ) = ALPHA * A( I, J )
* 310             CONTINUE
  320          CONTINUE
            END IF
         END IF
      END IF
*
      RETURN
*
*     End of DMMDDACT
*
      END
