      SUBROUTINE DDMMDDAT(
     &     M, N, 
     &     ALPHAh, ALPHAl, 
     &     Ah, Al, LDA, 
     &     BETAh,  BETAl, 
     &     Bh, Bl, LDB )
*
*  -- PBLAS auxiliary routine (version 2.0) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     April 1, 1998
*
*     .. Scalar Arguments ..
      INTEGER            LDA, LDB, M, N
!     *** Quadruple precision ***
      DOUBLE PRECISION ALPHAh, BETAh
      DOUBLE PRECISION ALPHAl, BETAl
!     ***************************
*     ..
*     .. Array Arguments ..
!     *** Quadruple precision ***
      DOUBLE PRECISION  Ah( LDA, * ), Bh( LDB, * )
      DOUBLE PRECISION  Al( LDA, * ), Bl( LDB, * )
!     ***************************
*     ..
*
*  Purpose
*  =======
*
*  DMMDDAT performs the following operation:
*
*     A := alpha * A + beta * B',
*
*  where alpha, beta are scalars; A is an m by n matrix and B is an n by
*  m matrix.
*
*  Arguments
*  =========
*
*  M       (local input) INTEGER
*          On entry, M  specifies the number of rows of A and the number
*          of columns of B. M must be at least zero.
*
*  N       (local input) INTEGER
*          On entry, N  specifies the number of rows of B and the number
*          of columns of A. N must be at least zero.
*
*  ALPHA   (local input) DOUBLE PRECISION
*          On entry,  ALPHA  specifies the scalar alpha. When  ALPHA  is
*          supplied as zero then the local entries of the array  A  need
*          not be set on input.
*
*  A       (local input/local output) DOUBLE PRECISION array
*          On entry, A is an array of dimension ( LDA, N ). On exit, the
*          leading n by m part of B has been added to the leading m by n
*          part of A.
*
*  LDA     (local input) INTEGER
*          On entry, LDA specifies the leading dimension of the array A.
*          LDA must be at least max( 1, M ).
*
*  BETA    (local input) DOUBLE PRECISION
*          On entry,  BETA  specifies the scalar beta. When BETA is sup-
*          plied as zero then the local entries of the array B need  not
*          be set on input.
*
*  B       (local input) DOUBLE PRECISION array
*          On entry, B is an array of dimension ( LDB, M ).
*
*  LDB     (local input) INTEGER
*          On entry, LDB specifies the leading dimension of the array B.
*          LDB must be at least max( 1, N ).
*
*  -- Written on April 1, 1998 by
*     Antoine Petitet, University  of  Tennessee, Knoxville 37996, USA.
*
*  =====================================================================
*
*     .. Parameters ..
      DOUBLE PRECISION   ONE, ZERO
      PARAMETER          ( ONE = 1.0D+0, ZERO = 0.0D+0 )
*     ..
*     .. Local Scalars ..
      INTEGER            I, J
!     *** Quadruple precision ***
      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
!     ***************************
*     ..
*     .. External Subroutines ..
      EXTERNAL           DAXPY, DCOPY, DSCAL
*     ..
*     .. Executable Statements ..
*
      IF( M.GE.N ) THEN
         IF( (BETAh.EQ.ONE).AND.(BETAl.EQ.ZERO) ) THEN
            IF( (ALPHAh.EQ.ZERO).AND.(ALPHAl.EQ.ZERO) ) THEN
!              *** Quadruple precision ***
               DO 20 J = 1, N
                  CALL DDCOPY( M,
     &                 Bh( J, 1 ), Bl( J, 1 ), LDB, 
     &                 Ah( 1, J ), Bl( 1, J ), 1 )
*                 DO 10 I = 1, M
*                    A( I, J ) = B( J, I )
*  10             CONTINUE
   20          CONTINUE
!              ***************************
            ELSE IF( (ALPHAh.NE.ONE).OR.(ALPHAl.NE.ZERO) ) THEN
!              *** Quadruple precision *** ???
               DO 40 J = 1, N
                  DO 30 I = 1, M
                     Ah( I, J ) = Bh( J, I ) + ALPHAh * Ah( I, J )
   30             CONTINUE
   40          CONTINUE
!              ***************************
            ELSE
!              *** Quadruple precision ***
               DO 60 J = 1, N
                  CALL DDAXPY( M,
     &                 ONE, ZERO, 
     &                 Bh( J, 1 ), Bl( J, 1 ), LDB,
     &                 Ah( 1, J ), Al( 1, J ), 1 )
*                 DO 50 I = 1, M
*                    A( I, J ) = B( J, I ) + A( I, J )
*  50             CONTINUE
   60          CONTINUE
!              ***************************
            END IF
         ELSE IF( (BETAh.NE.ZERO).OR.(BETAl.NE.ZERO) ) THEN
            IF( (ALPHAh.EQ.ZERO).AND.(ALPHAl.EQ.ZERO) ) THEN
!              *** Quadruple precision *** ???
               DO 80 J = 1, N
                  DO 70 I = 1, M
                     Ah( I, J ) = BETAh * Bh( J, I )
   70             CONTINUE
   80          CONTINUE
!              ***************************
            ELSE IF( (ALPHAh.NE.ONE).OR.(ALPHAl.NE.ZERO) ) THEN
!              *** Quadruple precision *** ???
               DO 100 J = 1, N
                  DO 90 I = 1, M
                     Ah( I, J ) = BETAh*Bh( J, I ) + ALPHAh*Ah( I, J )
   90             CONTINUE
  100          CONTINUE
!              ***************************
            ELSE
!              *** Quadruple precision ***
               DO 120 J = 1, N
                  CALL DDAXPY( M, 
     &                 BETAh, BETAl, 
     &                 Bh( J, 1 ), Bl( J, 1 ), LDB, 
     &                 Ah( 1, J ), Al( 1, J ), 1 )
*                 DO 110 I = 1, M
*                    A( I, J ) = BETA * B( J, I ) + A( I, J )
* 110             CONTINUE
  120          CONTINUE
!              ***************************
            END IF
         ELSE
            IF( (ALPHAh.EQ.ZERO).AND.(ALPHAl.EQ.ZERO) ) THEN
!              *** Quadruple precision ***
               DO 140 J = 1, N
                  DO 130 I = 1, M
                     Ah( I, J ) = ZERO
                     Al( I, J ) = ZERO
  130             CONTINUE
  140          CONTINUE
!              ***************************
            ELSE IF( (ALPHAh.NE.ONE).OR.(ALPHAl.NE.ZERO) ) THEN
!              *** Quadruple precision ***
               DO 160 J = 1, N
                  CALL DDSCAL( M,
     &                 ALPHAh, ALPHAl, 
     &                 Ah( 1, J ), Al( 1, J ), 1 )
*                 DO 150 I = 1, M
*                    A( I, J ) = ALPHA * A( I, J )
* 150             CONTINUE
  160          CONTINUE
!              ***************************
            END IF
         END IF
      ELSE
         IF( (BETAh.EQ.ONE).AND.(BETAl.EQ.ZERO) ) THEN
            IF( (ALPHAh.EQ.ZERO).AND.(ALPHAl.EQ.ZERO) ) THEN
!              *** Quadruple precision ***
               DO 180 J = 1, M
                  CALL DDCOPY( N,
     &                 Bh( 1, J ), Bl( 1, J ), 1,
     &                 Ah( J, 1 ), Al( J, 1 ), LDA )
*                 DO 170 I = 1, N
*                    A( J, I ) = B( I, J )
* 170             CONTINUE
  180          CONTINUE
!              ***************************
            ELSE IF( (ALPHAh.NE.ONE).OR.(ALPHAl.NE.ZERO) ) THEN
!              *** Quadruple precision *** ???
               DO 200 J = 1, M
                  DO 190 I = 1, N
                     Ah( J, I ) = Bh( I, J ) + ALPHAh * Ah( J, I )
  190             CONTINUE
  200          CONTINUE
!              ***************************
            ELSE
!              *** Quadruple precision ***
               DO 220 J = 1, M
                  CALL DDAXPY( N,
     &                 ONE, ZERO, 
     &                 Bh( 1, J ), Bl( 1, J ), 1,
     &                 Ah( J, 1 ), Al( J, 1 ), LDA )
*                 DO 210 I = 1, N
*                    A( J, I ) = B( I, J ) + A( J, I )
* 210             CONTINUE
  220          CONTINUE
!              ***************************
            END IF
         ELSE IF( (BETAh.NE.ZERO).OR.(BETAl.NE.ZERO) ) THEN
            IF( (ALPHAh.EQ.ZERO).AND.(ALPHAl.EQ.ZERO) ) THEN
!              *** Quadruple precision *** ???
               DO 240 J = 1, M
                  DO 230 I = 1, N
                     Ah( J, I ) = BETAh * Bh( I, J )
  230             CONTINUE
  240          CONTINUE
!              ***************************
            ELSE IF( (ALPHAh.NE.ONE).OR.(ALPHAl.NE.ZERO) ) THEN
!              *** Quadruple precision *** ???
               DO 260 J = 1, M
                  DO 250 I = 1, N
                     Ah( J, I ) = BETAh*Bh( I, J ) + ALPHAh*Ah( J, I )
  250             CONTINUE
  260          CONTINUE
!              ***************************
            ELSE
!              *** Quadruple precision ***
               DO 280 J = 1, M
                  CALL DDAXPY( N, 
     &                 BETAh, BETAl, 
     &                 Bh( 1, J ), Bl( 1, J ), 1, 
     &                 Ah( J, 1 ), Al( J, 1 ), LDA )
*                 DO 270 I = 1, N
*                    A( J, I ) = BETA * B( I, J ) + A( J, I )
* 270             CONTINUE
  280          CONTINUE
!              ***************************
            END IF
         ELSE
            IF( (ALPHAh.EQ.ZERO).AND.(ALPHAl.EQ.ZERO) ) THEN
!              *** Quadruple precision ***
               DO 300 J = 1, N
                  DO 290 I = 1, M
                     Ah( I, J ) = ZERO
                     Al( I, J ) = ZERO
  290             CONTINUE
  300          CONTINUE
!              ***************************
            ELSE IF( (ALPHAh.NE.ONE).OR.(ALPHAl.NE.ZERO) ) THEN
!              *** Quadruple precision ***
               DO 320 J = 1, N
                  CALL DDSCAL( M,
     &                 ALPHAh, ALPHAl, 
     &                 Ah( 1, J ), Al( 1, J ), 1 )
*                 DO 310 I = 1, M
*                    A( I, J ) = ALPHA * A( I, J )
* 310             CONTINUE
  320          CONTINUE
!              ***************************
            END IF
         END IF
      END IF
*
      RETURN
*
*     End of DMMDDAT
*
      END
