!      SUBROUTINE PDLAEDZ( N, N1, ID, Q, IQ, JQ, LDQ, DESCQ, Z, WORK )
      SUBROUTINE PDDLAEDZ( N, N1, ID, Qh, Ql, IQ, JQ, LDQ, DESCQ, 
     &                      Zh, Zl, WORKh, WORKl )
*
*  -- ScaLAPACK auxiliary routine (version 1.7) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     December 31, 1998
*
*     .. Scalar Arguments ..
      INTEGER            ID, IQ, JQ, LDQ, N, N1
*     ..
*     .. Array Arguments ..
      INTEGER            DESCQ( * )
!      DOUBLE PRECISION   Q( LDQ, * ), WORK( * ), Z( * )
      double precision Qh(LDQ,*), Ql(LDQ,*)
      double precision WORKh(*), WORKl(*)
      double precision Zh(*), Zl(*)
!... MPI ...
#ifdef COMM_MPI
      include 'mpif.h'
      integer ireq(2),ierr
      integer DEST,TAG,COMM
      integer myrank
      integer status(MPI_STATUS_SIZE)
      double precision DBUFF(2*N),SBUFF(2*N)
#endif
!.............
*     ..
*
*  Purpose
*  =======
*
*  PDLAEDZ Form the z-vector which consists of the last row of Q_1
*  and the first row of Q_2.
*  =====================================================================
*
*     .. Parameters ..
*
      INTEGER            BLOCK_CYCLIC_2D, DLEN_, DTYPE_, CTXT_, M_, N_,
     $                   MB_, NB_, RSRC_, CSRC_, LLD_
      PARAMETER          ( BLOCK_CYCLIC_2D = 1, DLEN_ = 9, DTYPE_ = 1,
     $                   CTXT_ = 2, M_ = 3, N_ = 4, MB_ = 5, NB_ = 6,
     $                   RSRC_ = 7, CSRC_ = 8, LLD_ = 9 )
*     ..
*     .. Local Scalars ..
*
      INTEGER            COL, I, IBUF, ICTXT, IIQ, IIZ1, IIZ2, IQCOL,
     $                   IQROW, IZ, IZ1, IZ1COL, IZ1ROW, IZ2, IZ2COL,
     $                   IZ2ROW, J, JJQ, JJZ1, JJZ2, MYCOL, MYROW, N2,
     $                   NB, NBLOC, NPCOL, NPROW, NQ1, NQ2, ZSIZ
#ifdef COMM_BLACS2
      double precision Z(2*N)
      double precision WORK(2*N)
#endif
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MIN, MOD
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_GRIDINFO, DDCOPY, DGEBR2D, DGEBS2D,
     $                   DGERV2D, DGESD2D, INFOG2L
*     ..
*     .. External Functions ..
      INTEGER            NUMROC
      EXTERNAL           NUMROC
!... MPI ...
#ifdef COMM_MPI
      EXTERNAL           MakeMpiInfo
#endif
!.............
*     ..
*     .. Executable Statements ..
*
!... MPI ...
#ifdef COMM_MPI
      Call mpi_comm_rank(MPI_COMM_WORLD,myrank,ierr)
#endif
!...........
*       This is just to keep ftnchek and toolpack/1 happy
      IF( BLOCK_CYCLIC_2D*CSRC_*CTXT_*DLEN_*DTYPE_*LLD_*MB_*M_*NB_*N_*
     $    RSRC_.LT.0 )RETURN
*
      ICTXT = DESCQ( CTXT_ )
      NB = DESCQ( NB_ )
      CALL BLACS_GRIDINFO( ICTXT, NPROW, NPCOL, MYROW, MYCOL )
      CALL INFOG2L( ID, ID, DESCQ, NPROW, NPCOL, MYROW, MYCOL, IIQ, JJQ,
     $              IQROW, IQCOL )
      N2 = N - N1
*
*     Form z1 which consist of the last row of Q1
*
      CALL INFOG2L( IQ-1+( ID+N1-1 ), JQ-1+ID, DESCQ, NPROW, NPCOL,
     $              MYROW, MYCOL, IIZ1, JJZ1, IZ1ROW, IZ1COL )
      NQ1 = NUMROC( N1, NB, MYCOL, IZ1COL, NPCOL )
      IF( ( MYROW.EQ.IZ1ROW ) .AND. ( NQ1.NE.0 ) ) THEN
!         CALL DCOPY( NQ1, Q( IIZ1, JJZ1 ), LDQ, WORK, 1 )
         CALL DDCOPY( NQ1, Qh( IIZ1, JJZ1 ), Ql( IIZ1, JJZ1 ), LDQ, 
     &                WORKh, WORKl, 1 )
         IF( MYROW.NE.IQROW .OR. MYCOL.NE.IQCOL ) THEN
!     $      CALL DGESD2D( ICTXT, NQ1, 1, WORK, NQ1, IQROW, IQCOL )
!... MPI ...
#ifdef COMM_MPI
            CALL MakeMpiInfo('A',ICTXT,IQROW,IQCOL,DEST,COMM,TAG)
            CALL MPI_Isend_dd(WORKh,WORKl,SBUFF,NQ1,
     $           MPI_DOUBLE_PRECISION,DEST,1,COMM,ireq,ierr)
            CALL MPI_Wait_dd(SBUFF,ireq,status,ierr)
#elif defined COMM_BLACS2
            DO l = 1, NQ1
               WORK(l)     = WORKh(l)
               WORK(l+NQ1) = WORKl(l)
            END DO
            CALL DGESD2D( ICTXT, 2*NQ1, 1, WORK, 2*NQ1, IQROW, IQCOL )
#else
            CALL DGESD2D( ICTXT, NQ1, 1, WORKh, NQ1, IQROW, IQCOL )
            CALL DGESD2D( ICTXT, NQ1, 1, WORKl, NQ1, IQROW, IQCOL )
#endif
!...........
        END IF
      END IF
*
*     Proc (IQROW, IQCOL) receive the parts of z1
*
      IF( MYROW.EQ.IQROW .AND. MYCOL.EQ.IQCOL ) THEN
         COL = IZ1COL
         DO 20 I = 0, NPCOL - 1
            NQ1 = NUMROC( N1, NB, COL, IZ1COL, NPCOL )
            IF( NQ1.GT.0 ) THEN
               IF( IZ1ROW.NE.IQROW .OR. COL.NE.IQCOL ) THEN
                  IBUF = N1 + 1
!                  CALL DGERV2D( ICTXT, NQ1, 1, WORK( IBUF ), NQ1,
!     $                          IZ1ROW, COL )
!... MPI ...
#ifdef COMM_MPI
                  CALL MakeMpiInfo('A',ICTXT,IZ1ROW,COL,DEST,COMM,TAG)
                  DBUFF(1:NQ1)       = WORKh(IBUF:IBUF+NQ1-1)
                  DBUFF(NQ1+1:2*NQ1) = WORKl(IBUF:IBUF+NQ1-1)
                  CALL MPI_Irecv_dd(DBUFF,NQ1,
     $                 MPI_DOUBLE_PRECISION,DEST,1,COMM,ireq,ierr)
                  CALL MPI_WaitIrecv_dd(WORKh(IBUF),WORKl(IBUF),
     $                 DBUFF,NQ1,ireq,status,ierr)
#elif defined COMM_BLACS2
                  CALL DGERV2D( ICTXT, 2*NQ1, 1, WORK( IBUF ), 2*NQ1,
     $                          IZ1ROW, COL )
                  Do l = IBUF,IBUF+NQ1-1
                     WORKh(l) = WORK(l)
                     WORKl(l) = WORK(l+NQ1)
                  END Do
#else
                  CALL DGERV2D( ICTXT, NQ1, 1, WORKh( IBUF ), NQ1,
     $                          IZ1ROW, COL )
                  CALL DGERV2D( ICTXT, NQ1, 1, WORKl( IBUF ), NQ1,
     $                          IZ1ROW, COL )
#endif
!...........
               ELSE
                  IBUF = 1
               END IF
               IZ1 = 0
               IZ = I*NB + 1
               NBLOC = ( NQ1-1 ) / NB + 1
               DO 10 J = 1, NBLOC
                  ZSIZ = MIN( NB, NQ1-IZ1 )
!                  CALL DCOPY( ZSIZ, WORK( IBUF+IZ1 ), 1, Z( IZ ), 1 )
                  CALL DDCOPY( ZSIZ, WORKh( IBUF+IZ1 ), 
     &                   WORKl( IBUF+IZ1 ), 1, Zh( IZ ), Zl( IZ ), 1 )
                  IZ1 = IZ1 + NB
                  IZ = IZ + NB*NPCOL
   10          CONTINUE
            END IF
            COL = MOD( COL+1, NPCOL )
   20    CONTINUE
      END IF
*
*     Form z2 which consist of the first row of Q2
*
      CALL INFOG2L( IQ-1+( ID+N1 ), JQ-1+( ID+N1 ), DESCQ, NPROW, NPCOL,
     $              MYROW, MYCOL, IIZ2, JJZ2, IZ2ROW, IZ2COL )
      NQ2 = NUMROC( N2, NB, MYCOL, IZ2COL, NPCOL )
      IF( ( MYROW.EQ.IZ2ROW ) .AND. ( NQ2.NE.0 ) ) THEN
!         CALL DCOPY( NQ2, Q( IIZ2, JJZ2 ), LDQ, WORK, 1 )
         CALL DDCOPY( NQ2, Qh( IIZ2, JJZ2 ), Ql( IIZ2, JJZ2 ), 
     &                LDQ, WORKh, WORKl, 1 )
         IF( MYROW.NE.IQROW .OR. MYCOL.NE.IQCOL ) THEN
!     $      CALL DGESD2D( ICTXT, NQ2, 1, WORK, NQ2, IQROW, IQCOL )
!... MPI ...
#ifdef COMM_MPI
            CALL MakeMpiInfo('A',ICTXT,IQROW,IQCOL,DEST,COMM,TAG)
            CALL MPI_Isend_dd(WORKh,WORKl,SBUFF,NQ2,
     $           MPI_DOUBLE_PRECISION,DEST,1,COMM,ireq,ierr)
            CALL MPI_Wait_dd(SBUFF,ireq,status,ierr)
#elif defined COMM_BLACS2
            DO l = 1,NQ2
               WORK(l)     = WORKh(l)
               WORK(l+NQ2) = WORKl(l)
            END DO
            CALL DGESD2D( ICTXT, 2*NQ2, 1, WORK, 2*NQ2, IQROW, IQCOL )
#else
            CALL DGESD2D( ICTXT, NQ2, 1, WORKh, NQ2, IQROW, IQCOL )
            CALL DGESD2D( ICTXT, NQ2, 1, WORKl, NQ2, IQROW, IQCOL )
#endif
!...........
         END IF
      END IF
*
*     Proc (IQROW, IQCOL) receive the parts of z2
*
      IF( MYROW.EQ.IQROW .AND. MYCOL.EQ.IQCOL ) THEN
         COL = IZ2COL
         DO 40 I = 0, NPCOL - 1
            NQ2 = NUMROC( N2, NB, COL, IZ2COL, NPCOL )
            IF( NQ2.GT.0 ) THEN
               IF( IQROW.NE.IZ2ROW .OR. IQCOL.NE.COL ) THEN
                  IBUF = 1 + N2
!                  CALL DGERV2D( ICTXT, NQ2, 1, WORK( IBUF ), NQ2,
!     $                          IZ2ROW, COL )
!... MPI ...
#ifdef COMM_MPI
                  CALL MakeMpiInfo('A',ICTXT,IZ2ROW,COL,DEST,COMM,TAG)
                  DBUFF(1:NQ2)       = WORKh(IBUF:IBUF+NQ2-1)
                  DBUFF(1+NQ2:2*NQ2) = WORKl(IBUF:IBUF+NQ2-1)
                  CALL MPI_Irecv_dd(DBUFF,NQ2,
     $                 MPI_DOUBLE_PRECISION,DEST,1,COMM,ireq,ierr)
                  CALL MPI_WaitIrecv_dd(WORKh(IBUF),WORKl(IBUF),
     $                 DBUFF,NQ2,ireq,status,ierr)
#elif defined COMM_BLACS2
                  CALL DGERV2D( ICTXT, 2*NQ2, 1, WORK, 2*NQ2,
     $                          IZ2ROW, COL )
                  Do l = 1, NQ2
                     J = l+IBUF-1
                     WORKh(J) = WORK(l)
                     WORKl(J) = WORK(l+NQ2)
                  END DO
#else
                  CALL DGERV2D( ICTXT, NQ2, 1, WORKh( IBUF ), NQ2,
     $                 IZ2ROW, COL )
                  CALL DGERV2D( ICTXT, NQ2, 1, WORKl( IBUF ), NQ2,
     $                 IZ2ROW, COL )
#endif
!...........
               ELSE
                  IBUF = 1
               END IF
               IZ2 = 0
               IZ = NB*I + N1 + 1
               NBLOC = ( NQ2-1 ) / NB + 1
               DO 30 J = 1, NBLOC
                  ZSIZ = MIN( NB, NQ2-IZ2 )
!                  CALL DCOPY( ZSIZ, WORK( IBUF+IZ2 ), 1, Z( IZ ), 1 )
                  CALL DDCOPY( ZSIZ, WORKh(IBUF+IZ2), WORKl(IBUF+IZ2),
     &                         1, Zh( IZ ), Zl( IZ ), 1 )
                  IZ2 = IZ2 + NB
                  IZ = IZ + NB*NPCOL
   30          CONTINUE
            END IF
            COL = MOD( COL+1, NPCOL )
   40    CONTINUE
      END IF
*
*     proc(IQROW,IQCOL) broadcast Z=(Z1,Z2)
*
      IF( MYROW.EQ.IQROW .AND. MYCOL.EQ.IQCOL ) THEN
!         CALL DGEBS2D( ICTXT, 'All', ' ', N, 1, Z, N )
!... MPI ...
#ifdef COMM_MPI
         CALL MakeMpiInfo('S',ICTXT,0,0,DEST,COMM,TAG)
         CALL MPI_Bcast_dd(Zh,Zl,N,
     $        MPI_DOUBLE_PRECISION,DEST,COMM,ierr)
#elif defined COMM_BLACS2
         DO l = 1,N
            Z(l)   = Zh(l)
            Z(l+N) = Zl(l)
         END DO
         CALL DGEBS2D( ICTXT, 'All', ' ', 2*N, 1, Z, 2*N )
#else
         CALL DGEBS2D( ICTXT, 'All', ' ', N, 1, Zh, N )
         CALL DGEBS2D( ICTXT, 'All', ' ', N, 1, Zl, N )
#endif
!...........
      ELSE
!         CALL DGEBR2D( ICTXT, 'All', ' ', N, 1, Z, N, IQROW, IQCOL )
!... MPI ...
#ifdef COMM_MPI
         CALL MakeMpiInfo('B',ICTXT,IQROW,IQCOL,DEST,COMM,TAG)
         CALL MPI_Bcast_dd(Zh,Zl,N,
     $        MPI_DOUBLE_PRECISION,DEST,COMM,ierr)
#elif defined COMM_BLACS2
         CALL DGEBR2D( ICTXT, 'All', ' ', 2*N, 1, Z, 2*N, IQROW, IQCOL )
         DO l = 1,N
            Zh(l) = Z(l)
            Zl(l) = Z(l+N)
         END DO
#else
         CALL DGEBR2D( ICTXT, 'All', ' ', N, 1, Zh, N, IQROW, IQCOL )
         CALL DGEBR2D( ICTXT, 'All', ' ', N, 1, Zl, N, IQROW, IQCOL )
#endif
!...........
      END IF
*
      RETURN
*
*     End of PDLAEDZ
*
      END
