C
C     Sample program for calculating the eigenvalues 
C                         of the Frank mattix of order 96
C
      use ddcommunication_sx, only : eigen_init
     &                             , eigen_free
     &                             , translate_l2g

      implicit double precision (a-h,o-z)

      real(8),allocatable :: ah(:,:), al(:,:)
      real(8), pointer ::
     &           bh(:),zh(:),dh(:),eh(:),wh(:),
     &           bl(:),zl(:),dl(:),el(:),wl(:)

      integer, parameter :: NDIM = 2
      logical iexist

      include 'mpif.h'
      include 'trd.h'

      real(16)::whl16,wth16,wth16_rmax,nrm16
      real(16)::whl16_rmax
      real(4)::ratio,ratio_rmax
      character(64)::txt,form16,form32
      real(8),allocatable :: ax(:,:),wx(:,:)

      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      Real(16)     qtmp
      Real(16)     qtmp2h,qtmp2l
      real(8) temp1,temp2

C     Initialize MPI
      call mpi_init(ierr)
      call mpi_comm_rank(mpi_comm_world,i$inod,ierr)
      call mpi_comm_size(mpi_comm_world,i$nnod,ierr)

C     Initialize Eigen and Memoris of matrix
      n=96
      m=32
      allocate(ah(n,n),al(n,n))
      call eigen_init(NDIM)
      NPROW = size_of_col
      NPCOL = size_of_row
      nx = ((n-1)/NPROW+1)
      call CSTAB_get_optdim(nx, 6, 16*2, 16*4, nm)
      call eigen_free(0)

      NB  = 64+32
      nmz = ((n-1)/NPROW+1)
      nmz = ((nmz-1)/NB+1)*NB+1
      nmw = ((n-1)/NPCOL+1)
      nmw = ((nmw-1)/NB+1)*NB+1
      nme = ((n-1)/2+1)*2
      nh = (n-1)/4+1
      call CSTAB_get_optdim(nh, 4, 16*2, 16*4, nnh)
      nnh = 4 * nnh

      n1x = ((n-1)/nprocs+1)
      larray = MAX(MAX(nmz,nm,nnh)*nmw, n*n1x)

      allocate(
     &         bh(larray), zh(larray),
     &         dh(nme), eh(2*nme), wh(nme),
     &         bl(larray), zl(larray),
     &         dl(nme), el(2*nme), wl(nme),
     &         stat=istat)
      if(istat.ne.0) then
         print*,"Memory exhausted"
         call flush(6)
         stop
      endif

C     Frank matrix
       do i=1,n
          do j=1,n
             qtmp=(n-max(i,j)+1)*1.0q0
             tmp2h = qtmp
             qtmp2h = tmp2h
             qtmp2l = qtmp-qtmp2h
             tmp2l = qtmp2l
             ah(j,i) = tmp2h
             al(j,i) = tmp2l
          enddo
       enddo

C     Adjust matrix
      call ddmatrix_adjust_sx(n,ah,al,bh,bl,nm)

C     Calculating eigen values and vectors
      call ddeigen_sx(n,bh,bl,nm,wh,wl,zh,zl,dh,dl,eh,el,nme,m,0)

      if(myrank==1)then 
        do k=1,n
          write(*,*) 'EigenValue=',wh(k),wl(k)
        enddo
      endif 

C     Free
      deallocate(ah,al)
      deallocate(bh,bl)
      deallocate(zh,zl)
      deallocate(dh,dl)
      deallocate(eh,el)
      deallocate(wh,wl)

C     Finalizing MPI
      call MPI_Finalize(ierr)
      end

