C
C     Soubroutine for check the result compared 
C                                   with eigen values and vectors
C
      subroutine dd_check_result(rwh,rwl,arh,arl,aih,ail,
     &                             wh,wl,zrh,zrl,zih,zil,n,nm)

      use ddcommunication_sx, only : eigen_init
     &                            , eigen_free

       integer  :: n, nm
       real(8)  :: rwh(*),arh(*), aih(*), wh(*), zrh(*), zih(*)
       real(8)  :: rwl(*),arl(*), ail(*), wl(*), zrl(*), zil(*)
       integer, parameter :: NDIM = 2

       include 'mpif.h'
       include 'trd.h'

C      Initalize eigen
       call eigen_init(NDIM)
       call MPI_Barrier(MPI_COMM_WORLD,ierr)

C      Checking eigen values and vectors
       call dd_compare_v_m(rwh(1),rwl(1),arh(1),arl(1),
     &                      wh(1),wl(1),zrh(1),zrl(1),
     &                      n,nm)

C      Finarilze eigen
       call MPI_Barrier(MPI_COMM_WORLD,ierr)
       call eigen_free(0)

       return
       end

C
C     Soubroutine for comparetion with eigen values and vectors
C
      subroutine dd_compare_v_m(rwh,rwl,arh,arl,
     &                            wh,wl,zrh,zrl,n,nm)

      use ddcommunication_sx, only : dddatacast_dbl 
     &                             , get_loop_start
     &                             , translate_g2l
     &                             , translate_l2g
     &                             , get_loop_end

      integer                :: n, nm
      real(8)                :: rwh(1:n), rwl(1:n)
      real(8)                :: arh(*), arl(*)
      real(8)                :: zrh(*), zrl(*)
      real(8)                :: wh(1:n), wl(1:n)

      double precision temp_h(n) , temp_l(n)
      double precision temp1_h   , temp1_l
      double precision error_h(n), error_l(n)
      double precision norm_h(n) , norm_l(n)
      double precision max_error_ramda_h, max_error_ramda_l
      double precision ave_error_ramda_h, ave_error_ramda_l
      double precision max_norm_h, max_norm_l
      double precision dbl_n_h,dbl_n_l
      integer i_2,i_3,j_2,j_3,i_start,i_end,j_start,j_end
      integer i_g
      double precision temp2_h,temp2_l,temp3_h,temp3_l
      integer ierr
      double precision sgn
      integer i,j

       include 'mpif.h'
       include 'trd.h'

       max_error_ramda_h=0.0d+0
       max_error_ramda_l=0.0d+0
       norm_h(1:n)= 0.0d+0
       norm_l(1:n)= 0.0d+0

C      Eigen values
       do j=1,n
         call dd_sub(wh(j), wl(j),
     &               rwh(j), rwl(j),
     &               error_h(j),error_l(j))
         call dd_abs(error_h(j),error_l(j),temp1_h,temp1_l)
         error_h(j)=temp1_h
         error_l(j)=temp1_l

         if( (max_error_ramda_h .lt. error_h(j)) .or.
     &      ((max_error_ramda_h .eq. 0.0d+0) .and.
     &       (max_error_ramda_l .lt. error_l(j)))
     &     ) then
           max_error_ramda_h = error_h(j)
           max_error_ramda_l = error_l(j)
         endif

         call dd_add(ave_error_ramda_h,ave_error_ramda_l,
     &               error_h(j),error_l(j),
     &               temp1_h,temp1_l)
         ave_error_ramda_h = temp1_h
         ave_error_ramda_l = temp1_l

       enddo

C      Eigen vectors
       dbl_n_h = dble(n)
       dbl_n_l = 0.0d+0
       call dd_divide(ave_error_ramda_h,ave_error_ramda_l,
     &                dbl_n_h,dbl_n_l,
     &                temp1_h,temp1_l)
         ave_error_ramda_h = temp1_h
         ave_error_ramda_l = temp1_l

       max_norm_h = 0.0d+0
       max_norm_l = 0.0d+0

       i_2 = get_loop_start(1, size_of_row,my_row)
       i_3 = get_loop_end  (n, size_of_row,my_row)
       j_2 = get_loop_start(1, size_of_col,my_col)
       j_3 = get_loop_end  (n, size_of_col,my_col)

       i_start = i_2
       i_end = i_3
       j_start = j_2
       j_end = j_3 

       do i=i_start,i_end
         i_g = translate_l2g(i, size_of_row, my_row)
         do j=j_start,j_end
           sgn = sign(1d0,arh(1+(i-1)*nm)) * sign(1d0,zrh(1+(i-1)*nm))
           call dd_sub(arh(j+(i-1)*nm), arl(j+(i-1)*nm),
     &                  sgn * zrh(j+(i-1)*nm), sgn * zrl(j+(i-1)*nm),
     &                  temp1_h,temp1_l)
           call dd_multiple(temp1_h, temp1_l, temp1_h, temp1_l,
     &          temp2_h,temp2_l)
           temp3_h = norm_h(i_g)
           temp3_l = norm_l(i_g)
           call dd_add(temp2_h,temp2_l,temp3_h,temp3_l,
     &          norm_h(i_g),norm_l(i_g))
         end do
       end do

C      Transfar results
       call MPI_Allreduce_dd(norm_h,norm_l,temp_h,temp_l,n,
     &      MPI_DOUBLE_PRECISION,MPI_SUM,MPI_COMM_WORLD,ierr)
       
       do i=1,n
          call dd_sqrt(temp_h(i),temp_l(i),norm_h(i),norm_l(i))
       end do

       do i=1,n
         if( (max_norm_h .lt. norm_h(i)) .or.
     &      ((max_norm_h .eq. norm_h(i)) .and.
     &       (max_norm_l .lt. norm_l(i)))
     &     ) then
           max_norm_h = norm_h(i)
           max_norm_l = norm_l(i)
         endif
       enddo

C      Show results
       if(myrank==1)then
         print*,"max |r(QPEigen)-r(define)|    = ",
     &           max_error_ramda_h,max_error_ramda_l
         print*,"ave |r(QPEigen)-r(define)|    = ",
     &           ave_error_ramda_h,ave_error_ramda_l
         print*,"max||xi(QPEigen)-xi(define)|| = ",max_norm_h,max_norm_l
       endif

       return
       end
