C
C     Verifing program for calculating the eigenvalues 
C                         of orthogonal matrix
C
      use ddcommunication_sx, only : eigen_init
     &                             , eigen_free
     &                             , translate_l2g

      implicit double precision (a-h,o-z)

      real(8),allocatable :: ah(:,:), al(:,:)
      real(8),allocatable :: xh(:,:), xl(:,:)
      real(8),allocatable :: rh(:,:), rl(:,:)
      real(8),allocatable :: sh(:,:), sl(:,:)
      real(8),allocatable :: ash(:,:), asl(:,:)
      real(8),allocatable :: asih(:,:), asil(:,:)
      real(8),allocatable ::
     &           bh(:),zh(:),dh(:),eh(:),wh(:),
     &           bl(:),zl(:),dl(:),el(:),wl(:),
     &           ch(:),cl(:),rwh(:),rwl(:)

      real(8) 	alphah ,alphal , betah, betal
      integer, parameter :: NDIM = 2

      real(8)                :: hs0, hs1

      include 'mpif.h'
      include 'trd.h'

C     Initialize MPI
      call mpi_init(ierr)
      call mpi_comm_rank(mpi_comm_world,i$inod,ierr)
      call mpi_comm_size(mpi_comm_world,i$nnod,ierr)

C     Initialize Eigen and Memoris of matrix
      n=96
      m=32
      allocate(ah(n,n),al(n,n))
      allocate(xh(n,n),xl(n,n))
      allocate(rh(n,n),rl(n,n))
      allocate(sh(n,n),sl(n,n))
      allocate(ash(n,n),asl(n,n))

      call eigen_init(NDIM)
      NPROW = size_of_col
      NPCOL = size_of_row
      nx = ((n-1)/NPROW+1)
      call CSTAB_get_optdim(nx, 6, 16*2, 16*4, nm)
      call eigen_free(0)

      NB  = 64+32
      nmz = ((n-1)/NPROW+1)
      nmz = ((nmz-1)/NB+1)*NB+1
      nmw = ((n-1)/NPCOL+1)
      nmw = ((nmw-1)/NB+1)*NB+1
      nme = ((n-1)/2+1)*2

      nh = (n-1)/4+1
      call CSTAB_get_optdim(nh, 4, 16*2, 16*4, nnh)
      nnh = 4 * nnh

      n1x = ((n-1)/nprocs+1)
      larray = MAX(MAX(nmz,nm,nnh)*nmw, n*n1x)
      allocate(
     &         bh(larray), zh(larray),
     &         dh(nme), eh(2*nme), wh(nme),
     &         bl(larray), zl(larray),
     &         dl(nme), el(2*nme), wl(nme),
     &         ch(larray),cl(larray),
     &         rwh(nme),rwl(nme),
     &         stat=istat)
      if(istat.ne.0) then
         print*,"Memory exhausted"
         call flush(6)
         stop
      endif

C     Set matrix
      call ddmatrix_orthogonal(n,xh,xl)
      call ddmatrix_diagonal(n, rh, rl, rwh, rwl)

	alphah = 1.0
	alphal = 0.0
	betah  = 0.0
        betal  = 0.0

      call ddgemm("t","n",n,n,n,alphah,alphal,xh,xl,n,rh,rl,n,
     +            betah,betal,sh,sl,n)

      call ddgemm("n","n",n,n,n,alphah,alphal,sh,sl,n,xh,xl,n,
     +            betah,betal,ash,asl,n)

      call t_matrix(n, ash, asl)
      call ddmatrix_adjust_sx(n,ash,asl,bh,bl,nm)

C     Calculating eigen values and vectors
      hs0 = mpi_wtime()
      call ddeigen_sx(n,bh,bl,nm,wh,wl,zh,zl,dh,dl,eh,el,nme,m,0)

      hs1 = mpi_wtime()
#ifdef TIMER
      if(myrank==1)then
         print*," "
         print*,"Total Execution Time of eigen_sx = ",hs1-hs0,"(sec)"
      endif
      call flush(6)
#endif

C     Sort eigen valus and vectors on variables
      call sort_matrix(n, rwh, rwl, xh, xl)

C     Check resuls
      call ddmatrix_adjust_sx(n,ash,asl,bh,bl,nm)
      call ddev_test_2D(n, bh, bl, nm, wh, wl, zh, zl, nm, t)

C     Verify result
      call ddmatrix_adjust_sx(n,xh,xl,ch,cl,nm)
      call dd_check_result(rwh,rwl,ch,cl,cih,cil,
     &                      wh,wl,zh,zl,bih,bil,n,nm)

C     Free memory
      deallocate(ah,al)
      deallocate(bh,bl)
      deallocate(ch,cl)
      deallocate(zh,zl)
      deallocate(dh,dl)
      deallocate(eh,el)
      deallocate(wh,wl)

C     Finalizing MPI
      call MPI_Finalize(ierr)
      end

      subroutine sort_matrix(n, rwh, rwl, xh, xl)
      double precision rwh(n), rwl(n)
      double precision xh(n,n), xl(n,n)
      integer          n

      double precision temp_h,temp_l
      double precision temp1_h(n), temp1_l(n)
      double precision temp2_h(n,n), temp2_l(n,n)
      integer          buffer

      do i=1,n
          temp_h=rwh(1)
          temp_l=rwl(1)
          buffer=1
             do j=1,n
               if( (temp_h .lt. rwh(j)).or.
     &            ((temp_h .eq. rwh(j)).and.(temp_l .lt. rwl(j)))) then
                   temp_h=rwh(j)
                   temp_l=rwl(j)
                   buffer=j
               endif

             enddo
             temp1_h(n-i+1) = rwh(buffer)
             temp1_l(n-i+1) = rwl(buffer)
             do j=1,n
               temp2_h(j,n-i+1) = xh(buffer,j)
               temp2_l(j,n-i+1) = xl(buffer,j)
             enddo
             rwh(buffer)=0.0d+0
             rwl(buffer)=0.0d+0

       enddo


	rwh = temp1_h
        rwl = temp1_l
	xh = temp2_h
        Xl = temp2_l      
      end subroutine

      subroutine t_matrix(n, xh, xl)
      double precision xh(n,n), xl(n,n)
      integer          n

      double precision temp_h(n,n), temp_l(n,n)
      integer   i,j

      temp_h = xh
      temp_l = xl

	do i=1,n
	   do j=1,n
              xh(i,j)=temp_h(j,i)
              xl(i,j)=temp_l(j,i)
           end do
         end do
      end subroutine

